#!/usr/bin/perl -w


use lib ('./blib','./lib','../blib','../lib')
use CGI::Minimal

my $do_tests = \$: 1..2

my $test_subs = \%: 
    1 => \(%:  code => &test_calling_parms_table, desc => 'generate calling parms table               ' )
    2 => \(%:  code => &test_rfc1123_date,        desc => 'generate RFC 1123 date                     ' )
    

run_tests($test_subs,$do_tests)

exit

###########################################################################################

sub reset_form
    env::var('QUERY_STRING'      ) = 'hello=testing;hello2=SGML+encoded+FORM;submit+button=submit'
    env::var('CONTENT_LENGTH'    ) = length(env::var('QUERY_STRING'))
    env::var('CONTENT_TYPE'      ) = 'application/sgml-form-urlencoded'
    env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
    env::var('REQUEST_METHOD'    ) = 'GET'
    CGI::Minimal::reset_globals


###########################################################################################

sub test_calling_parms_table
    my $calling_parms_table = try {
        reset_form();
        env::var('QUERY_STRING' ) = '';
        env::var('CONTENT_LENGTH'    ) = length(env::var('QUERY_STRING'));
        my $cgi = CGI::Minimal->new;
        return $cgi->calling_parms_table;
    }
    if ($^EVAL_ERROR)
        return "unexpected failure $($^EVAL_ERROR->message)"
    
    if ($calling_parms_table eq '') { return 'failed to generate calling parms table with no parms for decoding'; };

    $calling_parms_table = try {
        reset_form();
        my $cgi = CGI::Minimal->new;
        return $cgi->calling_parms_table;
    }
    if ($^EVAL_ERROR)
        return "unexpected failure $($^EVAL_ERROR->message . $^EVAL_ERROR->stacktrace)"
    
    if ($calling_parms_table eq '') { return 'failed to generate calling parms table'; };

    $calling_parms_table = try {
        reset_form();
        my $cgi = generate_and_read_multipart_form();;
        return $cgi->calling_parms_table;
    }
    if ($^EVAL_ERROR)
        return "unexpected failure $($^EVAL_ERROR->message . $^EVAL_ERROR->stacktrace)"
    
    if ($calling_parms_table eq '') { return 'failed to generate calling parms table for multipart form'; };

    return ''


###########################################################################################

sub test_rfc1123_date
    my $rfc_date = try {
        reset_form();
        return CGI::Minimal->date_rfc1123(0);
    }
    if ($^EVAL_ERROR)
        return "unexpected failure $($^EVAL_ERROR->message)"
    
    unless ($rfc_date eq 'Thu, 01 Jan 1970 00:00:00 GMT')
        return "Generated unexpected date of $rfc_date for epoch date '0'"
    
    return ''


###########################################################################################

sub generate_and_read_multipart_form
    local $^WARNING = 0

    my $basic_boundary = 'lkjsdlkjsd'

    my $data = multipart_data($basic_boundary)

    env::var('CONTENT_LENGTH'    ) = length($data)
    env::var('CONTENT_TYPE'      ) = "multipart/form-data; boundary=---------------------------$basic_boundary"
    env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
    env::var('REQUEST_METHOD'    ) = 'POST'

    my $test_file = "test-data.$^PID.data"
    open (my $testfh, ">","$test_file") || return ("failed : could not open test file $test_file for writing: $^OS_ERROR")
    binmode ($testfh)
    print $testfh, $data
    close ($testfh)

    # "Bad evil naughty Zoot"
    CGI::Minimal::reset_globals
    open ($^STDIN, "<",$test_file) || return ("failed : could not open test file $test_file for reading: $^OS_ERROR")
    my $cgi = CGI::Minimal->new
    close ($^STDIN)
    unlink $test_file

    return $cgi


######################################################
# multipart test data                                #
######################################################

sub multipart_data($boundary)

    my $data =<<"EOD"
-----------------------------$boundary
Content-Disposition: form-data; name="hello"; filename="hello1.txt"

testing
-----------------------------$boundary
Content-Disposition: form-data; name="hello"; filename="hello1.xml"
Content-Type: application/xml 

<data>also testing</data>
-----------------------------$boundary
Content-Disposition: form-data; name="hello2"; filename="example"
Content-Type: text/html

testing2
-----------------------------$boundary
Content-Disposition: form-data; name="submit button"

submit
-----------------------------$boundary--
EOD
    $data =~ s/\012/\015\012/gs
    return $data


###########################################################################################

sub run_tests($test_subs,$do_tests)

    print $^STDOUT, < $do_tests->[[(@: 0)]],'..', <$do_tests->[[(@: (nelems($do_tests->@)-1))]],"\n"
    print $^STDERR, "\n"
    my $n_failures = 0
    foreach my $test ( $do_tests->@)
        my $sub  = $test_subs->{$test}->{?code}
        my $desc = $test_subs->{$test}->{?desc}
        my $failure = ''
        $failure = $sub->();

        if ($failure)
            print $^STDOUT, "not ok $test\n"
            print $^STDERR, "    $desc - $failure\n"
            $n_failures++
        else
            print $^STDOUT, "ok $test\n"
            print $^STDERR, "    $desc - ok\n"

    print $^STDOUT, "END\n"


