#!/usr/bin/perl -w


use lib ('./blib','./lib','../blib','../lib')
use CGI::Minimal
use Test::More

plan tests => 6

my $do_tests = \$: 1..4

my $test_subs = %: 
    1 => (%:  code => &test_param_mod,           desc => 'set standard parameter values              ' )
    2 => (%:  code => &test_delete_parms,        desc => 'delete standard parameters                 ' )
    3 => (%:  code => &test_multi_param_mod,     desc => 'set multipart parameter values             ' )
    4 => (%:  code => &test_multi_delete_parms,  desc => 'delete multipart parameters                ' )
    

run_tests($test_subs,$do_tests)

exit

###########################################################################################

######################################################
# Test changing parameter values                     #
######################################################

sub test_param_mod
    env::var('QUERY_STRING'      ) = 'hello=testing;hello2=SGML+encoded+FORM;submit+button=submit'
    env::var('CONTENT_LENGTH'    ) = length(env::var('QUERY_STRING'))
    env::var('CONTENT_TYPE'      ) = 'application/sgml-form-urlencoded'
    env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
    env::var('REQUEST_METHOD'    ) = 'GET'

    CGI::Minimal::reset_globals

    my $cgi = CGI::Minimal->new

    my $string_pairs = %:  'hello' => 'testing'
                           'hello2' => 'SGML encoded FORM'
                           'submit button' => 'submit'
        
    my @form_keys   = keys $string_pairs
    my @param_keys  = $cgi->param_as_list
    if ((nelems(@form_keys)-1) !=( nelems(@param_keys)-1))
        fail 'Expected 3 parameters SGML form, found ' . ((nelems(@param_keys)-1) + 1)
    

    do
        $string_pairs{+'hello'} = 'changed'
        $cgi->param('hello' => 'changed')
        my %form_keys_hash = %+: map { %: $_ => $string_pairs{?$_} }, @form_keys
        foreach my $key_item ( @param_keys)
            if (! defined %form_keys_hash{?$key_item})
                return 'failed : Parameter names did not match'
            
            my $item_value = $cgi->param($key_item)
            if (%form_keys_hash{?$key_item} ne $item_value)
                return 'failed : Parameter values did not match'
            
        
    

    do
        $string_pairs{+'hello'} = 'changed2'
        $cgi->param((%: 'hello' => 'changed2'))
        my %form_keys_hash = %+: map { %: $_ => $string_pairs{?$_} }, @form_keys
        foreach my $key_item ( @param_keys)
            if (! defined %form_keys_hash{?$key_item})
                return 'failed : Parameter names did not match after anon hash form change'
            
            my $item_value = $cgi->param($key_item)
            if (%form_keys_hash{?$key_item} ne $item_value)
                return 'failed : Parameter values did not match'
            
        
    

    # Multivalue parameters
    $cgi->param('a' => (@: 'b','c'))

    # Bad parameter mods
    dies_like( {
            $cgi->param('a','b','c'); # Odd number of parameters is bad
        }, qr/Odd number/ )

    # Bad parameter types
    dies_like({
            $cgi->param('a' => (%:  'a' => 'b' )); # only arrays and scalars allowed
        }, qr/illegal data type/)

    # Success is an empty string (no error message ;) )
    return ''


######################################################
# Test parameter deletion                            #
######################################################

sub test_delete_parms
    env::var('QUERY_STRING'      ) = 'hello=testing&hello2=standard+encoded+FORM&submit+button=submit'
    env::var('CONTENT_LENGTH'    ) = length(env::var('QUERY_STRING'))
    env::var('CONTENT_TYPE'      ) = 'application/x-www-form-urlencoded'
    env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
    env::var('REQUEST_METHOD'    ) = 'GET'

    CGI::Minimal::reset_globals

    my $cgi = CGI::Minimal->new

    my $string_pairs = %:  'hello' => 'testing'
                           'hello2' => 'standard encoded FORM'
                           'submit button' => 'submit'
        
    my @form_keys   = keys $string_pairs
    my @param_keys  = $cgi->param_as_list
    if ((nelems(@form_keys)-1) !=( nelems(@param_keys)-1))
        return 'failed : Expected 3 parameters in x-www-form-urlencoded, found ' . ((nelems(@param_keys)-1) + 1)
    

    delete $string_pairs{'hello'}
    delete $string_pairs{'hello2'}
    $cgi->delete('hello', 'hello2')
    @param_keys  = $cgi->param_as_list
    my %form_keys_hash = %+: map { %: $_ => $string_pairs{?$_} }, @form_keys
    foreach my $key_item ( @param_keys)
        if (! defined %form_keys_hash{?$key_item})
            return 'failed : Parameter names did not match'
        
        my $item_value = $cgi->param($key_item)
        if (%form_keys_hash{?$key_item} ne $item_value)
            return 'failed : Parameter values did not match'
        
    


    $cgi->delete_all
    my @parms = $cgi->param
    unless (0 == nelems(@parms))
        return 'failed : delete_all failed to remove all parameters'
    

    # Success is an empty string (no error message ;) )
    return ''


######################################################
# Test multiparm form parameter changes              #
######################################################

sub test_multi_param_mod(?$mode)
    $mode = '' unless (defined $mode)
    local $^WARNING = 0

    my $basic_boundary = 'lkjsdlkjsd'
    my @boundaries_list = $@
    my $boundary_test_code = $%
    for my $count (0..127)
        next if ((10 == $count) or (13 == $count) or (26 == $count))
        my $test_boundary = chr($count) . $basic_boundary
        push (@boundaries_list,$test_boundary)
        $boundary_test_code{+$test_boundary} = $count
    

    foreach my $boundary ( @boundaries_list)
        my $data = multipart_data($boundary)

        env::var('CONTENT_LENGTH'    ) = length($data)
        if ($mode eq 'truncate') { env::var('CONTENT_LENGTH'  ) = length($data) + 1; }
        env::var('CONTENT_TYPE'      ) = "multipart/form-data; boundary=---------------------------$boundary"
        env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
        env::var('REQUEST_METHOD'    ) = 'POST'

        my $test_file = "test-data.$^PID.data"
        open (my $testfh, ">","$test_file") || return ("failed : could not open test file $test_file for writing: $^OS_ERROR")
        binmode ($testfh)
        print $testfh, $data
        close ($testfh)

        # "Bad evil naughty Zoot"
        CGI::Minimal::reset_globals
        open ($^STDIN, "<",$test_file) || return ("failed : could not open test file $test_file for reading: $^OS_ERROR")
        my $cgi = CGI::Minimal->new
        close ($^STDIN)
        unlink $test_file

        if ($mode eq 'truncate')
            unless ($cgi->truncated) { return 'failed: did not detect truncated form'; }
        else
            if ($cgi->truncated) { return 'failed: form falsely appeared truncated for boundary char ' . $boundary_test_code->{?$boundary}; }
        
        $cgi->param('hello' => 'other')
        my $string_pairs = %:  'hello' => 'other'
                               'hello2' => 'testing2'
                               'submit button' => 'submit'
            
        my @form_keys   = keys $string_pairs
        my @param_keys  = $cgi->param_as_list
        if ((nelems(@form_keys)-1) !=( nelems(@param_keys)-1))
            return 'failed : Expected 3 parameters in multipart form, found '
                . ((nelems(@param_keys)-1) + 1)
                . ". testing codepoint " . $boundary_test_code->{?$boundary}
                . " "
                . " for boundary $boundary $data"
        

        my %form_keys_hash = %+: map { %: $_ => $string_pairs{?$_} }, @form_keys
        foreach my $key_item ( @param_keys)
            if (! defined %form_keys_hash{?$key_item})
                return 'failed : Parameter names did not match'
            
            my $item_value = $cgi->param($key_item)
            if (%form_keys_hash{?$key_item} ne $item_value)
                return 'failed : Parameter values did not match'
            
        

    

    # Success is an empty string (no error message ;) )
    return ''

######################################################
# Test multiparm form parameter deletions            #
######################################################

sub test_multi_delete_parms(?$mode)
    $mode = '' unless (defined $mode)
    local $^WARNING = 0

    my $basic_boundary = 'lkjsdlkjsd'
    my @boundaries_list = $@
    my $boundary_test_code = $%
    for my $count (0..127)
        next if ((10 == $count) or (13 == $count))
        my $test_boundary = chr($count) . $basic_boundary
        push (@boundaries_list,$test_boundary)
        $boundary_test_code{+$test_boundary} = $count
    

    foreach my $boundary ( @boundaries_list)
        my $data = multipart_data($boundary)

        env::var('CONTENT_LENGTH'    ) = length($data)
        if ($mode eq 'truncate') { env::var('CONTENT_LENGTH'  ) = length($data) + 1; }
        env::var('CONTENT_TYPE'      ) = "multipart/form-data; boundary=---------------------------$boundary"
        env::var('GATEWAY_INTERFACE' ) = 'CGI/1.1'
        env::var('REQUEST_METHOD'    ) = 'POST'

        my $test_file = "test-data.$^PID.data"
        open (my $testfh, ">","$test_file") || return ("failed : could not open test file $test_file for writing: $^OS_ERROR")
        binmode ($testfh)
        print $testfh, $data
        close ($testfh)

        # "Bad evil naughty Zoot"
        CGI::Minimal::reset_globals
        open ($^STDIN, "<",$test_file) || return ("failed : could not open test file $test_file for reading: $^OS_ERROR")
        my $cgi = CGI::Minimal->new
        close ($^STDIN)
        unlink $test_file

        if ($mode eq 'truncate')
            unless ($cgi->truncated) { return 'failed: did not detect truncated form'; }
        else
            if ($cgi->truncated) { return 'failed: form falsely appeared truncated'; }
        
        $cgi->delete('hello2','submit button')
        my $string_pairs = %:  'hello' => 'testing'
            
        my @form_keys   = keys $string_pairs
        my @param_keys  = $cgi->param_as_list
        if ((nelems(@form_keys)-1) !=( nelems(@param_keys)-1))
            return 'failed : Expected 3 parameters in multipart form, found '
                . ((nelems(@param_keys)-1) + 1)
                . ". testing codepoint " . $boundary_test_code->{?$boundary}
                . " "
                . " for boundary $boundary $data"
        

        my %form_keys_hash = %+: map { %: $_ => $string_pairs{?$_} }, @form_keys
        foreach my $key_item ( @param_keys)
            if (! defined %form_keys_hash{?$key_item})
                return 'failed : Parameter names did not match'
            
            my $item_value = $cgi->param($key_item)
            if (%form_keys_hash{?$key_item} ne $item_value)
                return 'failed : Parameter values did not match'
            
        

        $cgi->delete_all
        my @parms = $cgi->param
        unless (0 == nelems(@parms))
            return 'failed : delete_all failed to remove all parameters'
        

    

    # Success is an empty string (no error message ;) )
    return ''


######################################################
# multipart test data                                #
######################################################

sub multipart_data($boundary)

    my $data =<<"EOD"
-----------------------------$boundary
Content-Disposition: form-data; name="hello"

testing
-----------------------------$boundary
Content-Disposition: form-data; name="hello2"

testing2
-----------------------------$boundary
Content-Disposition: form-data; name="submit button"

submit
-----------------------------$boundary--
EOD
    $data =~ s/\012/\015\012/gs
    return $data


###########################################################################################

sub run_tests($test_subs,$do_tests)

    my $n_failures = 0
    foreach my $test ( $do_tests->@)
        my $sub  = $test_subs{$test}{?code}
        my $desc = $test_subs{$test}{?desc}
        my $failure = ''
        $failure = $sub->()
        ok(! $failure, $desc) or diag $failure
    

    print $^STDOUT, "END\n"


