package CPAN::Complete;

sub gnu_cpl($text, $line, $start, $end)
    my(@perlret) = cpl($text, $line, $start);
    # find longest common match. Can anybody show me how to peruse
    # T::R::Gnu to have this done automatically? Seems expensive.
    return () unless nelems(@perlret);
    my($newtext) = $text;
    my $i = length($text)+1;
    while (1) {
        last unless length(@perlret[0]) && length(@perlret[0]) +>= $i;
        my $try = substr(@perlret[0],0,$i);
        my @tries = grep {substr($_,0,$i) eq $try}, @perlret;
        # warn "try[$try]tries[@tries]";
        if (nelems(@tries) == nelems(@perlret)) {
            $newtext = $try;
        } else {
            last;
        }
        $i++;
    }
    return @: $newtext,@perlret

#-> sub CPAN::Complete::cpl ;
sub cpl($word,$line,$pos)
    $word ||= "";
    $line ||= "";
    $pos ||= 0;
    CPAN->debug("word [$word] line[$line] pos[$pos]") if $CPAN::DEBUG;
    $line =~ s/^\s*//;
    if ($line =~ s/^((?:notest|f?force)\s*)//) {
        $pos -= length($1);
    }
    my @return;
    if ($pos == 0 || $line =~ m/^(?:h(?:elp)?|\?)\s/) {
        @return = grep m/^\Q$word\E/, @CPAN::Complete::COMMANDS;
    } elsif ( $line !~ m/^[\!abcdghimorutl]/ )
        @return = $@
    elsif ($line =~ m/^(a|ls)\s/)
        @return = cplx('CPAN::Author',uc($word))
    elsif ($line =~ m/^b\s/)
        CPAN::Shell->local_bundles
        @return = cplx('CPAN::Bundle',$word)
    elsif ($line =~ m/^d\s/) {
        @return = cplx('CPAN::Distribution',$word);
    } elsif ($line =~ m/^(
                          [mru]|make|clean|dump|get|test|install|readme|look|cvs_import|perldoc|recent
                         )\s/x ) {
        if ($word =~ m/^Bundle::/) {
            CPAN::Shell->local_bundles;
        }
        @return = cplx('CPAN::Module',$word) +@+ cplx('CPAN::Bundle',$word)
    } elsif ($line =~ m/^i\s/) {
        @return = cpl_any($word);
    } elsif ($line =~ m/^reload\s/) {
        @return = cpl_reload($word,$line,$pos);
    } elsif ($line =~ m/^o\s/) {
        @return = cpl_option($word,$line,$pos);
    } elsif ($line =~ m/^\S+\s/ )
        # fallback for future commands and what we have forgotten above
        @return = cplx('CPAN::Module',$word) +@+ cplx('CPAN::Bundle',$word)
    else
        @return = $@

    return @return;

#-> sub CPAN::Complete::cplx ;
sub cplx($class, $word)
    if (CPAN::_sqlite_running) {
        $CPAN::SQLite->search($class, "^\Q$word\E");
    }
    sort grep { m/^\Q$word\E/ }, map { $_->id }, $CPAN::META->all_objects($class);

#-> sub CPAN::Complete::cpl_any ;
sub cpl_any($word)
    return @:
            cplx('CPAN::Author',$word)
            cplx('CPAN::Bundle',$word)
            cplx('CPAN::Distribution',$word)
            cplx('CPAN::Module',$word)

#-> sub CPAN::Complete::cpl_reload ;
sub cpl_reload($word,$line,$pos)
    $word ||= "";
    my(@words) = split " ", $line;
    CPAN->debug("word[$word] line[$line] pos[$pos]") if $CPAN::DEBUG;
    my(@ok) = qw(cpan index);
    return @ok if nelems(@words) == 1;
    return grep m/^\Q$word\E/, @ok if nelems(@words) == 2 && $word;

#-> sub CPAN::Complete::cpl_option ;
sub cpl_option($word,$line,$pos)
    $word ||= "";
    my(@words) = split " ", $line;
    CPAN->debug("word[$word] line[$line] pos[$pos]") if $CPAN::DEBUG;
    my(@ok) = qw(conf debug);
    return @ok if nelems(@words) == 1;
    return grep m/^\Q$word\E/, @ok if nelems(@words) == 2 && length($word);
    if (0) {
    } elsif (@words[1] eq 'index') {
        return ();
    } elsif (@words[1] eq 'conf') {
        return CPAN::HandleConfig::cpl(< @_);
    } elsif (@words[1] eq 'debug')
        return sort grep m/^\Q$word\E/i,
            sort keys %CPAN::DEBUG, 'all'

1
