package CPAN::Module


# Accessors
#-> sub CPAN::Module::userid
sub userid
    my $self = shift
    my $ro = $self->ro
    return unless $ro
    return $ro->{?userid} || $ro->{?CPAN_USERID}

#-> sub CPAN::Module::description
sub description
    my $self = shift
    my $ro = $self->ro or return ""
    $ro->{?description}


#-> sub CPAN::Module::distribution
sub distribution($self)
    CPAN::Shell->expand("Distribution", $self->cpan_file)


#-> sub CPAN::Module::undelay
sub undelay
    my $self = shift
    delete $self->{later}
    if ( my $dist = CPAN::Shell->expand("Distribution", $self->cpan_file) )
        $dist->undelay
    


# mark as dirty/clean
#-> sub CPAN::Module::color_cmd_tmps ;
sub color_cmd_tmps
    my(@: $self) =@:  shift
    my(@: $depth) = shift || 0
    my(@: $color) = shift || 0
    my(@: $ancestors) = shift || \$@
    # a module needs to recurse to its cpan_file

    return if exists $self->{incommandcolor}
                && $color==1
      && $self->{?incommandcolor}==$color
    return if $color==0 && !$self->{?incommandcolor}
    if ($color+>=1)
        if ( $self->uptodate )
            $self->{+incommandcolor} = $color
            return
        elsif (my $have_version = $self->available_version)
            # maybe what we have is good enough
            if (nelems($ancestors->@))
                my $who_asked_for_me = $ancestors->[-1]
                my $obj = CPAN::Shell->expandany($who_asked_for_me)
                if (0) {
                }elsif ($obj->isa("CPAN::Bundle"))
                    # bundles cannot specify a minimum version
                    return
                elsif ($obj->isa("CPAN::Distribution"))
                    if (my $prereq_pm = $obj->prereq_pm)
                        for my $k (keys $prereq_pm->%)
                            if (my $want_version = $prereq_pm->{$k}->{?$self->id})
                                if (CPAN::Version->vcmp($have_version,$want_version) +>= 0)
                                    $self->{+incommandcolor} = $color
                                    return
                                
                            
                        
                    
                
            
        
    else
        $self->{+incommandcolor} = $color # set me before recursion,
    # so we can break it
    
    if ($depth+>=$CPAN::MAX_RECURSION)
        die( <CPAN::Exception::RecursiveDependency->new($ancestors))
    
    # warn "color_cmd_tmps $depth $color " . $self->id; # sleep 1;

    if ( my $dist = CPAN::Shell->expand("Distribution", < $self->cpan_file) )
        $dist->color_cmd_tmps($depth+1,$color,\(@: < $ancestors->@, < $self->id))
    
    # unreached code?
    # if ($color==0) {
    #    delete $self->{badtestcnt};
    # }
    $self->{+incommandcolor} = $color


#-> sub CPAN::Module::as_glimpse ;
sub as_glimpse($self)
    my(@m)
    my $class = ref($self)
    $class =~ s/^CPAN:://
    my $color_on = ""
    my $color_off = ""
    if (
          $CPAN::Shell::COLOR_REGISTERED
          &&
        $CPAN::META->has_inst("Term::ANSIColor")
          &&
        $self->description
        )
        $color_on = Term::ANSIColor::color("green")
        $color_off = Term::ANSIColor::color("reset")
    
    my $uptodateness = " "
    if ($class eq "Bundle") {
    }elsif ($self->uptodate)
        $uptodateness = "="
    elsif ($self->inst_version)
        $uptodateness = "<"
    
    push @m, sprintf("\%-7s \%1s \%s\%-22s\%s (\%s)\n",
        $class,
        $uptodateness,
        $color_on, <
        $self->id,
        $color_off,
        ($self->distribution ?? <
         $self->distribution->pretty_id !! <
         $self->cpan_userid
         ),
        )
    join "", @m


#-> sub CPAN::Module::dslip_status
sub dslip_status($self)
    my($stat)
    # development status
    $stat->{D}->{[qw,i c a b R M S,]}     =  qw,idea
                                              pre-alpha alpha beta released
                                              mature standard,
    # support level
    $stat->{S}->{[qw,m d u n a,]}         =  qw,mailing-list
                                              developer comp.lang.perl.*
                                              none abandoned,
    # language
    $stat->{L}->{[qw,p c + o h,]}         =  qw,perl C C++ other hybrid,
    # interface
    $stat->{I}->{[qw,f r O p h n,]}       =  qw,functions
                                              references+ties
                                              object-oriented pragma
                                              hybrid none,
    # public licence
    $stat->{P}->{[qw,p g l b a 2 o d r n,]} =  qw,Standard-Perl
                                              GPL LGPL
                                              BSD Artistic Artistic_2
                                              open-source
                                              distribution_allowed
                                              restricted_distribution
                                              no_licence,
    for my $x (qw(d s l i p))
        $stat->{$x}->{+' '} = 'unknown'
        $stat->{$x}->{+'?'} = 'unknown'
    
    my $ro = $self->ro
    return \$% unless $ro && $ro->{?statd}
    return \(%: 
        D  => $ro->{?statd}
        S  => $ro->{?stats}
        L  => $ro->{?statl}
        I  => $ro->{?stati}
        P  => $ro->{?statp}
        DV => $stat->{D}->{?$ro->{?statd}}
        SV => $stat->{S}->{?$ro->{?stats}}
        LV => $stat->{L}->{?$ro->{?statl}}
        IV => $stat->{I}->{?$ro->{?stati}}
        PV => $stat->{P}->{?$ro->{?statp}}
        )


#-> sub CPAN::Module::as_string ;
sub as_string($self)
    my(@m)
    CPAN->debug("$self entering as_string") if $CPAN::DEBUG
    my $class = ref($self)
    $class =~ s/^CPAN:://
    local($^WARNING) = 0
    push @m, $class, " id = $self->{?ID}\n"
    my $sprintf = "    \%-12s \%s\n"
    push @m, sprintf($sprintf, 'DESCRIPTION', < $self->description)
        if $self->description
    my $sprintf2 = "    \%-12s \%s (\%s)\n"
    my($userid)
    $userid = $self->userid
    if ( $userid )
        my $author
        if ($author = CPAN::Shell->expand('Author',$userid))
            my $email = ""
            my $m # old perls
            if ($m = $author->email)
                $email = " <$m>"
            
            push @m, sprintf(
                $sprintf2,
                'CPAN_USERID',
                $userid,
                $author->fullname . $email
                )
        
    
    push @m, sprintf($sprintf, 'CPAN_VERSION', < $self->cpan_version)
        if $self->cpan_version
    if (my $cpan_file = $self->cpan_file)
        push @m, sprintf($sprintf, 'CPAN_FILE', $cpan_file)
        if (my $dist = CPAN::Shell->expand("Distribution",$cpan_file))
            my $upload_date = $dist->upload_date
            if ($upload_date)
                push @m, sprintf($sprintf, 'UPLOAD_DATE', $upload_date)
            
        
    
    my $sprintf3 = "    \%-12s \%1s\%1s\%1s\%1s\%1s (\%s,\%s,\%s,\%s,\%s)\n"
    my $dslip = $self->dslip_status
    push @m, sprintf(
        $sprintf3,
        'DSLIP_STATUS', <
        $dslip->{[qw(D S L I P DV SV LV IV PV)]},
        ) if $dslip->{?D}
    my $local_file = $self->inst_file
    unless ($self->{?MANPAGE})
        my $manpage
        if ($local_file)
            $manpage = $self->manpage_headline($local_file)
        else
            # If we have already untarred it, we should look there
            my $dist = $CPAN::META->instance('CPAN::Distribution', <
                $self->cpan_file)
            # warn "dist[$dist]";
            # mff=manifest file; mfh=manifest handle
            my($mff,$mfh)
            if (
                $dist->{?build_dir}
                  and
                (-f  ($mff = File::Spec->catfile($dist->{?build_dir}, "MANIFEST")))
                  and
                $mfh = FileHandle->new($mff)
                )
                CPAN->debug("mff[$mff]") if $CPAN::DEBUG
                my $lfre = $self->id # local file RE
                $lfre =~ s/::/./g
                $lfre .= "\\.pm\$"
                my($lfl) # local file file
                local $^INPUT_RECORD_SEPARATOR = "\n"
                my(@: @mflines) =@:  (@:  ~< $mfh )
                for ( @mflines)
                    s/^\s+//
                    s/\s.*//s
                
                while (length($lfre)+>5 and !$lfl)
                    ($lfl) = < grep { m/$lfre/ }, @mflines
                    CPAN->debug("lfl[$lfl]lfre[$lfre]") if $CPAN::DEBUG
                    $lfre =~ s/.+?\.//
                
                $lfl =~ s/\s.*// # remove comments
                $lfl =~ s/\s+//g # chomp would maybe be too system-specific
                my $lfl_abs = File::Spec->catfile($dist->{?build_dir},$lfl)
                # warn "lfl_abs[$lfl_abs]";
                if (-f $lfl_abs)
                    $manpage = $self->manpage_headline($lfl_abs)
                
            
        
        $self->{+MANPAGE} = $manpage if $manpage
    
    for my $item (qw/MANPAGE/)
        push @m, sprintf($sprintf, $item, $self->{?$item})
            if exists $self->{$item}
    
    for my $item (qw/CONTAINS/)
        push @m, sprintf($sprintf, $item, join(" ", $self->{?$item}->@))
            if exists $self->{$item} && nelems($self->{?$item}->@)
    
    push @m, sprintf($sprintf, 'INST_FILE',
        $local_file || "(not installed)")
    push @m, sprintf($sprintf, 'INST_VERSION', <
        $self->inst_version) if $local_file
    join "", (@:  < @m, "\n")


#-> sub CPAN::Module::manpage_headline
sub manpage_headline($self,$local_file)
    my(@: @local_file) =@:  (@:  $local_file )
    $local_file =~ s/\.pm(?!\n)\Z/.pod/
    push @local_file, $local_file
    my(@result)
    for my $locf ( @local_file)
        next unless -f $locf
        my $fh = FileHandle->new($locf)
            or $Carp::Frontend->mydie("Couldn't open $locf: $^OS_ERROR")
        my $inpod = 0
        local $^INPUT_RECORD_SEPARATOR = "\n"
        while ( ~< $fh)
            $inpod = m/^=(?!head1\s+NAME\s*$)/ ?? 0 !!
                m/^=head1\s+NAME\s*$/ ?? 1 !! $inpod
            next unless $inpod
            next if m/^=/
            next if m/^\s+$/
            chomp
            push @result, $_
        
        close $fh
        last if nelems(@result)
    
    for ( @result)
        s/^\s+//
        s/\s+$//
    
    join " ", @result


#-> sub CPAN::Module::cpan_file ;
# Note: also inherited by CPAN::Bundle
sub cpan_file
    my $self = shift
    # CPAN->debug(sprintf "id[%s]", $self->id) if $CPAN::DEBUG;
    unless ($self->ro)
        CPAN::Index->reload
    
    my $ro = $self->ro
    if ($ro && defined $ro->{?CPAN_FILE})
        return $ro->{?CPAN_FILE}
    else
        my $userid = $self->userid
        if ( $userid )
            if ($CPAN::META->exists("CPAN::Author",$userid))
                my $author = $CPAN::META->instance("CPAN::Author",
                    $userid)
                my $fullname = $author->fullname
                my $email = $author->email
                unless (defined $fullname && defined $email)
                    return sprintf("Contact Author \%s",
                        $userid,
                        )
                
                return "Contact Author $fullname <$email>"
            else
                return "Contact Author $userid (Email address not available)"
            
        else
            return "N/A"
        
    


#-> sub CPAN::Module::cpan_version ;
sub cpan_version
    my $self = shift

    my $ro = $self->ro
    unless ($ro)
        # Can happen with modules that are not on CPAN
        $ro = \$%
    
    $ro->{+CPAN_VERSION} = 'undef'
        unless defined $ro->{?CPAN_VERSION}
    $ro->{?CPAN_VERSION}


#-> sub CPAN::Module::force ;
sub force($self)
    $self->{+force_update} = 1


#-> sub CPAN::Module::fforce ;
sub fforce($self)
    $self->{+force_update} = 2


#-> sub CPAN::Module::notest ;
sub notest($self)
    # $CPAN::Frontend->mywarn("XDEBUG: set notest for Module");
    $self->{+notest}++


#-> sub CPAN::Module::rematein ;
sub rematein($self,$meth)
    $CPAN::Frontend->myprint(sprintf("Running \%s for module '\%s'\n",
        $meth, <
        $self->id))
    my $cpan_file = $self->cpan_file
    if ($cpan_file eq "N/A" || $cpan_file =~ m/^Contact Author/)
        $CPAN::Frontend->mywarn(sprintf qq{
  The module \%s isn\'t available on CPAN.

  Either the module has not yet been uploaded to CPAN, or it is
  temporary unavailable. Please contact the author to find out
  more about the status. Try 'i \%s'.
}, <
            $self->id, <
            $self->id,
            )
        return
    
    my $pack = $CPAN::META->instance('CPAN::Distribution',$cpan_file)
    $pack->called_for( <$self->id)
    if (exists $self->{force_update})
        if ($self->{?force_update} == 2)
            $pack->fforce($meth)
        else
            $pack->force($meth)
        
    
    $pack->notest($meth) if exists $self->{notest} && $self->{?notest}

    $pack->{+reqtype} ||= ""
    CPAN->debug("dist-reqtype[$pack->{?reqtype}]".
        "self-reqtype[$self->{?reqtype}]") if $CPAN::DEBUG
    if ($pack->{?reqtype})
        if ($pack->{?reqtype} eq "b" && $self->{?reqtype} =~ m/^[rc]$/)
            $pack->{+reqtype} = $self->{?reqtype}
            if (
                  exists $pack->{install}
                  &&
                (
                     UNIVERSAL::can($pack->{?install},"failed") ??
                     $pack->{install}->failed !!
                     $pack->{?install} =~ m/^NO/
                     )
                )
                delete $pack->{install}
                $CPAN::Frontend->mywarn
                    ("Promoting $pack->{?ID} from 'build_requires' to 'requires'")
            
        
    else
        $pack->{+reqtype} = $self->{?reqtype}
    

    my $success = try {
        $pack->?$meth();
    }
    my $err = $^EVAL_ERROR
    $pack->unforce if $pack->can("unforce") && exists $self->{force_update}
    $pack->unnotest if $pack->can("unnotest") && exists $self->{notest}
    delete $self->{force_update}
    delete $self->{notest}
    if ($err)
        die $err
    
    return $success


#-> sub CPAN::Module::perldoc ;
sub perldoc { shift->rematein('perldoc') }
#-> sub CPAN::Module::readme ;
sub readme  { shift->rematein('readme') }
#-> sub CPAN::Module::look ;
sub look    { shift->rematein('look') }
#-> sub CPAN::Module::cvs_import ;
sub cvs_import { shift->rematein('cvs_import') }
#-> sub CPAN::Module::get ;
sub get     { shift->rematein('get',< @_) }
#-> sub CPAN::Module::make ;
sub make    { shift->rematein('make') }
#-> sub CPAN::Module::test ;
sub test
    my $self = shift
    # $self->{badtestcnt} ||= 0;
    $self->rematein('test',< @_)

#-> sub CPAN::Module::uptodate ;
sub uptodate($self)
    local($_) # protect against a bug in MakeMaker 6.17
    my $latest = $self->cpan_version
    $latest ||= 0
    my $inst_file = $self->inst_file
    my($have) = 0
    if (defined $inst_file)
        $have = $self->inst_version
    
    local($^WARNING)=0
    if ($inst_file
          &&
        ! CPAN::Version->vgt($latest, $have)
        )
        CPAN->debug("returning uptodate. inst_file[$inst_file] ".
            "latest[$latest] have[$have]") if $CPAN::DEBUG
        return 1
    
    return

#-> sub CPAN::Module::install ;
sub install($self)
    my($doit) = 0
    if ($self->uptodate
        &&
        not exists $self->{force_update}
        )
        $CPAN::Frontend->myprint(sprintf("\%s is up to date (\%s).\n", <
            $self->id, <
            $self->inst_version,
            ))
    else
        $doit = 1
    
    my $ro = $self->ro
    if ($ro && $ro->{?stats} && $ro->{?stats} eq "a")
        $CPAN::Frontend->mywarn(qq{
\n\n\n     ***WARNING***
     The module $self->{?ID} has no active maintainer.\n\n\n
})
        $CPAN::Frontend->mysleep(5)
    
    $self->rematein('install') if $doit

#-> sub CPAN::Module::clean ;
sub clean  { shift->rematein('clean') }

#-> sub CPAN::Module::inst_file ;
sub inst_file($self)
    $self->_file_in_path(\ $^INCLUDE_PATH)


#-> sub CPAN::Module::available_file ;
sub available_file($self)
    my $sep = config_value("path_sep")
    my $perllib = env::var('PERL5LIB')
    $perllib = env::var('PERLLIB') unless defined $perllib
    my @perllib = split(m/$sep/,$perllib) if defined $perllib
    $self->_file_in_path(\(@: < @perllib,< $^INCLUDE_PATH))


#-> sub CPAN::Module::file_in_path ;
sub _file_in_path($self,$path)
    my(@packpath)
    @packpath = split m/::/, $self->{?ID}
    @packpath[-1] .= ".pm"
    if (nelems(@packpath) == 1 && @packpath[0] eq "readline.pm")
        unshift @packpath, "Term", "ReadLine" # historical reasons
    
    foreach my $dir ( $path->@)
        my $pmfile = File::Spec->catfile($dir,< @packpath)
        if (-f $pmfile)
            return $pmfile
        
    
    return


#-> sub CPAN::Module::xs_file ;
sub xs_file($self)
    my(@packpath)
    @packpath = split m/::/, $self->{?ID}
    push @packpath, @packpath[-1]
    @packpath[-1] .= "." . config_value('dlext')
    foreach my $dir ( $^INCLUDE_PATH)
        my $xsfile = File::Spec->catfile($dir,'auto',< @packpath)
        if (-f $xsfile)
            return $xsfile
        
    
    return


#-> sub CPAN::Module::inst_version ;
sub inst_version($self)
    my $parsefile = $self->inst_file or return
    my $have = $self->parse_version($parsefile)
    $have


#-> sub CPAN::Module::inst_version ;
sub available_version($self)
    my $parsefile = $self->available_file or return
    my $have = $self->parse_version($parsefile)
    $have


#-> sub CPAN::Module::parse_version ;
sub parse_version($self,$parsefile)
    my $have = MM->parse_version($parsefile)
    $have = "undef" unless defined $have && length $have
    $have =~ s/^ // # since the %vd hack these two lines here are needed
    $have =~ s/ $// # trailing whitespace happens all the time

    $have = CPAN::Version->readable($have)

    $have =~ s/\s*//g # stringify to float around floating point issues
    $have # no stringify needed, \s* above matches always


#-> sub CPAN::Module::reports
sub reports($self)
    $self->distribution->reports


1
