my $count

use Test::More
use File::Spec
sub _f ($file)
    File::Spec->catfile( <split m/\//, $file)

use File::Copy < qw(cp)
cp _f("t/CPAN/TestConfig.pm"), _f("t/CPAN/MyConfig.pm")
    or die "Could not cp t/CPAN/TestConfig.pm over t/CPAN/MyConfig.pm: $^OS_ERROR"
unshift $^INCLUDE_PATH, "t"
require CPAN::MyConfig
require CPAN
require CPAN::Kwalify
require CPAN::HandleConfig
do
    try {CPAN::rtlpr()}
    like $^EVAL_ERROR->message, qr/Undefined subroutine &CPAN::rtlpr called/,q/subroutine not found/
    BEGIN{$count++}

do
    my $rdep = CPAN::Exception::RecursiveDependency->new(\qw(foo bar baz foo))
    like $rdep->as_string, qr/foo.+=>.+bar.+=>.+baz.+=>.+foo/s, "circular dependency"
    BEGIN{$count++}

do
    my $a = CPAN::Shell->expand("Module",
        "CPAN::Test::Dummy::Perl5::Make"
        )[-1]->distribution()[-1]->author()[-1]->as_string
    like $a, qr/Andreas/, "found Andreas in CPAN::Test::Dummy::Perl5::Make"
    BEGIN{$count++}

do

    do
        package S
        for my $m (qw(myprint mydie mywarn mysleep))
            Symbol::fetch_glob($m)->* = sub (@< @_)
                return

    $CPAN::Frontend = $CPAN::Frontend = "S"
    $_ = "Fcntl"
    my $m = CPAN::Shell->expand(Module => $_)[-1]
    $m->uptodate
    is($_,"Fcntl","\$_ is properly localized")
    BEGIN{$count++}

do
    my @s
    BEGIN
        @s=@:
            '"a"'
            '["a"]'
            '{a=>"b"}'
            '{"a;"=>"b"}'
            '"\\"'

        $count+=nelems(@s)

    for (0..(nelems(@s)-1))
        my $x = eval @s[$_]
        my $y = CPAN::HandleConfig->neatvalue($x)
        my $z = eval $y
        is_deeply($z,$x,"s[$_]")


do
    my $this_block_count
    BEGIN { $count += $this_block_count = 2; }
    try { require Kwalify; require YAML; } # most of the kwalify
    # stuff does not work
    # without yaml
    if ($^EVAL_ERROR || (($YAML::VERSION||$YAML::VERSION||0) +< 0.62)) # silence 5.005_04
        for (1..$this_block_count)
            ok(1)

    else
        my $data = \%:
            "match" => \ %:
                "distribution" => "^(ABW|ADAMK)/Template-Toolkit-2.16"
            "pl" => \ %:
                "args" => \ @:
                                        "TT_EXTRAS=no"
                "expect" => \ @:
                                          "Do you want to build the XS Stash module"
                                          "n\n"
                                          "Do you want to install these components"
                                          "n\n"
                                          "Installation directory"
                                          "\n"
                                          "URL base for TT2 images"
                                          "\n"
                barth => '1984'

        try {CPAN::Kwalify::_validate("distroprefs",
                                      $data, <
                                      _f("t/12cpan.t"),
                                      0)}
        ok($^EVAL_ERROR,"no kwalify [$^EVAL_ERROR]")
        delete $data->{pl}->{barth}
        CPAN::Kwalify::_clear_cache()
        try {CPAN::Kwalify::_validate("distroprefs",
                                      $data, <
                                      _f("t/12cpan.t"),
                                      0)}
        ok(!$^EVAL_ERROR,"kwalify ok")



BEGIN{plan tests => $count}

# Local Variables:
# mode: cperl
# cperl-indent-level: 4
# End:
