#!/usr/bin/perl

use warnings


use Test::More tests => 10

BEGIN { use_ok( 'Carp::Assert::More' ); }

local $^EVAL_ERROR = undef
$^EVAL_ERROR = ''

# one element in arrayref
try {
    assert_in('one', \(@:  'one' ) );
}
is( $^EVAL_ERROR, '' )

# separate string, two elements
try {
    my $string = 'B';
    assert_in( $string, \(@:  'A', 'B' )  );
}
is( $^EVAL_ERROR, '' )

# separate string and manual arrayref
try {
    my $string = 'delta';
    my @array = (@: 'alpha','beta','delta');
    assert_in( $string, \@array );
}
is( $^EVAL_ERROR, '' )

# separate string and arrayref
try {
    my $string = 'tres';
    my $ref = \(@:  'uno', 'dos', 'tres', 'quatro' );
    assert_in( $string, $ref  );
}
is( $^EVAL_ERROR, '' )

# not found fails
try {
    assert_in( 'F', \(@:  'A', 'B', 'C', 'D', 'E' ) );
}
like( $^EVAL_ERROR->{?description}, qr/Assertion.*failed/ )

# undef string fa6yyils
try {
    assert_in( undef, \(@:  'fail' ) );
}
like( $^EVAL_ERROR->{?description}, qr/Assertion.*failed/ )

# empty array fails
try {
    assert_in( 'empty', \$@ );
}
like( $^EVAL_ERROR->{?description}, qr/Assertion.*failed/ )

# undef for the arrayref fails
try {
    my $string = "zippo";
    assert_in( $string, undef );
}
like( $^EVAL_ERROR->{?description}, qr/Assertion.*failed/ )

# A bad reference should also fail.
try {
    my $string = "nil";
    my $ref = \$string;
    assert_in( $string, $ref );
}
like( $^EVAL_ERROR->{?description}, qr/Assertion.*failed/ )
