#!perl
use Test::More tests => 34

for my $class (qw(Class::Accessor Class::Accessor::Fast Class::Accessor::Faster))
    require_ok($class)
    my $silly = "Silly::$class"
    do
        Symbol::fetch_glob("$($silly)::ISA")->*->@ = @: $class
        Symbol::fetch_glob("$($silly)::car")->* = sub (@< @_) { shift->_car_accessor(< @_); }
        Symbol::fetch_glob("$($silly)::mar")->* = sub (@< @_) { return "Overloaded"; }
        $silly->mk_accessors( <qw( foo bar yar car mar ))
        $silly->mk_ro_accessors( <qw(static unchanged))
        $silly->mk_wo_accessors( <qw(sekret double_sekret))
    

    my $test = $silly->new(\(%: 
        static       => "variable"
        unchanged    => "dynamic"
        ))

    $test->foo(42)
    $test->bar('Meep')

    is($test->foo, 42, "foo accessor")
    is($test->{foo}, 42, "foo hash element") unless $class eq 'Class::Accessor::Faster'

    is($test->static, 'variable', 'ro accessor')
    try { $test->static('foo'); }
    like($^EVAL_ERROR->message,
         qr/^'main' cannot alter the value of 'static' on objects of class '$silly'/,
         'ro accessor write protection')

    $test->double_sekret(1001001)
    is( $test->{double_sekret}, 1001001, 'wo accessor') unless $class eq 'Class::Accessor::Faster'
    try { $test->double_sekret; }
    like($^EVAL_ERROR->message,
         qr/^'main' cannot access the value of 'double_sekret' on objects of class '$silly'/,
         'wo accessor read protection' )

    is($test->_foo_accessor, 42, 'accessor alias')

    $test->car("AMC Javalin")
    is($test->car, 'AMC Javalin', 'internal override access')
    is($test->mar, 'Overloaded', 'internal override constant')

    # Make sure bogus accessors die.
    try { $test->gargle() }
    ok($^EVAL_ERROR, 'bad accessor')

    do
        my $eeek
        local $^WARN_HOOK = sub (@< @_) { $eeek = shift->message }
        $silly->mk_accessors( <qw(DESTROY))
        like($eeek,
             qr/a data accessor named DESTROY/i,
             'mk DESTROY accessor warning')
    ;


