
use Test::More < qw(no_plan)

use lib 't/lib'
use Foo

our $out = ""

ok( Foo->add_trigger(
       name     => 'before_foo',
       callback => sub (@< @_) { $out .= "before_foo\n" }
       ),
       'add_trigger in Foo'
       )
ok( Foo->add_trigger(
       callback => sub (@< @_) { $out .= "after_foo\n" },
       name     => 'after_foo', # change the param order to test from hash -> list
       ),
       'add_trigger in foo'
       )

my $foo = Foo->new

do
    $out = ""
    $foo->foo
    is $out, "before_foo\nfoo\nafter_foo\n"


ok( Foo->add_trigger(
       name     => 'after_foo',
       callback => sub (@< @_) { $out .= "after_foo2\n" }
       ),
       'add_trigger in Foo'
       )

do
    $out = ""
    $foo->foo
    is $out, "before_foo\nfoo\nafter_foo\nafter_foo2\n"


ok( Foo->add_trigger(
       name     => 'after_foo',
       callback => sub (@< @_) { $out .= ref @_[0] }
       ),
       'add_trigger in Foo'
       )

do
    $out = ""
    $foo->foo
    is $out, "before_foo\nfoo\nafter_foo\nafter_foo2\nFoo", 'class name'

ok( Foo->add_trigger(
       name      => 'after_foo',
       callback  => sub (@< @_) { $out .= "\ngets_here"; return 'YAY'; },
       abortable => 1
       ),
       'add_trigger in Foo'
       )

do
    $out = ""
    is( $foo->foo, 4, "Success returned" )
    is_deeply ($foo->last_trigger_results->[-1], \(@: 'YAY'))
    is $out,
       "before_foo\nfoo\nafter_foo\nafter_foo2\nFoo\ngets_here",
       'class name'


ok( Foo->add_trigger(
       name      => 'after_foo',
       callback  => sub (@< @_) { $out .= "\nstopping_after"; return undef; },
       abortable => 1

       ),
       'add_trigger in Foo'
       )
ok( Foo->add_trigger(
       name     => 'after_foo',
       callback => sub (@< @_) { $out .= "should not get here" }
       ),
       'add_trigger in Foo'
       )

do
    $out = ""
    is( $foo->foo, undef, "The lat thing we ran was 'stopping_after', then returned failure " )
    is $out,
       "before_foo\nfoo\nafter_foo\nafter_foo2\nFoo\ngets_here\nstopping_after",
       'class name'
    unlike( $out, qr/should not get here/ )


# coverage tests

do

    # pass a non-code ref and catch the carp

    dies_like( sub()
                   Foo->add_trigger(
                                    name     => 'wrong_type',
                                    callback => \$@
                                    )
             , qr(add_trigger[(][)] needs coderef),
             'check for right callback param'
             )

