package MakeUtil 
package main 


use File::Copy

my $VERSION = '1.0'


BEGIN

    try { require File::Spec::Functions ; File::Spec::Functions->import() } 
    if ($^EVAL_ERROR)
        *catfile = sub (@< @_) { return "@_[0]/@_[1]" }
    


require VMS::Filespec if $^OS_NAME eq 'VMS'

sub MY::libscan
    my $self = shift
    my $path = shift

    return undef
        if $path =~ m/(~|\.bak|_bak)$/ ||
      $path =~ m/\..*\.sw(o|p)$/  ||
      $path =~ m/\B\.svn\b/

    return $path


sub MY::postamble
    my @files = getPerlFiles('MANIFEST')

    # Note: Once you remove all the layers of shell/makefile escaping
    # the regular expression below reads
    #
    #    /^\s*local\s*\(\s*\$^W\s*\)/
    #
    my $postamble = '

MyTrebleCheck:
	@echo Checking for $$^W in files: '. " $(join ' ', @files)" . q|
	@perl -ne '						\
	    exit 1 if /^\s*local\s*\(\s*\$$\^W\s*\)/;		\
         ' | . "  $(join ' ', @files) || " . '				\
	(echo found unexpected $$^W ; exit 1)
	@echo All is ok.

'

    return $postamble


sub getPerlFiles(@< @manifests)
    my @files = ()

    for my $manifest (@manifests)
        my $prefix = './'

        $prefix = $1
            if $manifest =~ m#^(.*/)#

        open my $m_fh, "<", "$manifest"
            or die "Cannot open '$manifest': $^OS_ERROR\n"
        while ( ~< $m_fh)
            chomp 
            next if m/^\s*#/ || m/^\s*$/ 

            s/^\s+//
            s/\s+$//

            m/^(\S+)\s*(.*)$/

            my (@: $file, $rest) = @: $1, $2

            if ($file =~ m/\.(pm|pl|t)$/ and $file !~ m/MakeUtil.pm/)
                push @files, "$prefix$file"
            elsif ($rest =~ m/perl/i)
                push @files, "$prefix$file"
            

        
        close $m_fh
    

    return @files


sub FindBrokenDependencies
    my $version = shift 
    my %thisModule = %+: map { %: $_ => 1 }, @_

    my @modules = qw(
                    IO::Compress::Base
                    IO::Compress::Base::Common
                    IO::Uncompress::Base

                    Compress::Raw::Zlib
                    Compress::Raw::Bzip2

                    IO::Compress::RawDeflate
                    IO::Uncompress::RawInflate
                    IO::Compress::Deflate
                    IO::Uncompress::Inflate
                    IO::Compress::Gzip
                    IO::Compress::Gzip::Constants
                    IO::Uncompress::Gunzip
                    IO::Compress::Zip
                    IO::Uncompress::Unzip

                    IO::Compress::Bzip2
                    IO::Uncompress::Bunzip2

                    IO::Compress::Lzf
                    IO::Uncompress::UnLzf

                    IO::Compress::Lzop
                    IO::Uncompress::UnLzop

                    Compress::Zlib
                    )

    my @broken = ()

    foreach my $module ( < grep { ! %thisModule{?$_} }, @modules)
        my $hasVersion = getInstalledVersion($module)

        # No need to upgrade if the module isn't installed at all
        next
            if ! defined $hasVersion

        # If already have C::Z version 1, then an upgrade to any of the
        # IO::Compress modules will not break it.
        next
            if $module eq 'Compress::Zlib' && $hasVersion +< 2

        if ($hasVersion +< $version)
            push @broken, $module
        
    

    return @broken


sub getInstalledVersion
    my $module = shift
    my $version

    eval " require $module; "

    if ($^EVAL_ERROR eq '')

        $version = {Symbol::fetch_glob( $module . "::VERSION")}->*->$
        $version = 0
    

    return $version


package MakeUtil 

1


