
use lib < qw(t t/compress)

use warnings
use bytes

use Test::More 
use CompTestUtils
use Symbol

BEGIN 

    # use Test::NoWarnings, if available
    my $extra = 0 
    $extra = 1
        if try { require Test::NoWarnings ;  Test::NoWarnings->import(); 1 }

    my $count = 40 


    plan tests => $count + $extra 

    use_ok('Compress::Zlib', 2) 



my $hello = <<EOM 
hello world
this is a test
EOM

my $len   = length $hello 

# Check zlib_version and ZLIB_VERSION are the same.
is Compress::Zlib::zlib_version, ZLIB_VERSION,
   "ZLIB_VERSION matches Compress::Zlib::zlib_version" 

# generate a long random string
my $contents = '' 
foreach (1 .. 5000)
    { $contents .= chr int rand 256 }

my $x 
my $fil

# compress/uncompress tests
# =========================

try { compress(\(@: 1)); }
ok $^EVAL_ERROR->{?description} =~ m#not a scalar reference#
    or print $^STDOUT, "# $^EVAL_ERROR\n" 

try { uncompress(\(@: 1)); }
ok $^EVAL_ERROR->{?description} =~ m#not a scalar reference#
    or print $^STDOUT, "# $^EVAL_ERROR\n" 

$hello = "hello mum" 
my $keep_hello = $hello 

my $compr = compress($hello) 
ok $compr ne "" 

my $keep_compr = $compr 

my $uncompr = uncompress ($compr) 

ok $hello eq $uncompr 

ok $hello eq $keep_hello 
ok $compr eq $keep_compr 

# compress a number
$hello = 7890 
$keep_hello = $hello 

$compr = compress($hello) 
ok $compr ne "" 

$keep_compr = $compr 

$uncompr = uncompress ($compr) 

ok $hello eq $uncompr 

ok $hello eq $keep_hello 
ok $compr eq $keep_compr 

# bigger compress

$compr = compress ($contents) 
ok $compr ne "" 

$uncompr = uncompress ($compr) 

ok $contents eq $uncompr 

# buffer reference

$compr = compress(\$hello) 
ok $compr ne "" 


$uncompr = uncompress (\$compr) 
ok $hello eq $uncompr 

# bad level
$compr = compress($hello, 1000) 
ok ! defined $compr

# change level
$compr = compress($hello, Z_BEST_COMPRESSION) 
ok defined $compr
$uncompr = uncompress (\$compr) 
ok $hello eq $uncompr 

# corrupt data
$compr = compress(\$hello) 
ok $compr ne "" 

substr($compr,0, 1, "\x[FF]")
ok !defined uncompress (\$compr) 

# deflate/inflate - small buffer
# ==============================

$hello = "I am a HAL 9000 computer" 
my @hello = split('', $hello)
my ($err, $X, $status)

title 'memGzip & memGunzip'
do
    my $buffer = <<EOM
some sample 
text

EOM

    my $len = length $buffer 
    my ($x, $uncomp) 


    # create an in-memory gzip file
    my $dest = Compress::Zlib::memGzip($buffer) 
    ok length $dest 

    # now check that memGunzip can deal with it.
    my $ungzip = Compress::Zlib::memGunzip($dest) 
    ok defined $ungzip 
    ok $buffer eq $ungzip 

    # now do the same but use a reference

    $dest = Compress::Zlib::memGzip(\$buffer) 
    ok length $dest 

    # now check that memGunzip can deal with it.
    my $keep = $dest
    $ungzip = Compress::Zlib::memGunzip(\$dest) 
    ok defined $ungzip 
    ok $buffer eq $ungzip 

    # check corrupt header -- too short
    $dest = "x" 
    my $result = Compress::Zlib::memGunzip($dest) 
    ok !defined $result 

    # check corrupt header -- full of junk
    $dest = "x" x 200 
    $result = Compress::Zlib::memGunzip($dest) 
    ok !defined $result 

    # corrupt header - 1st byte wrong
    my $bad = $keep 
    substr($bad, 0, 1, "\x[FF]") 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 

    # corrupt header - 2st byte wrong
    $bad = $keep 
    substr($bad, 1, 1, "\x[FF]") 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 

    # corrupt header - method not deflated
    $bad = $keep 
    substr($bad, 2, 1, "\x[FF]") 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 

    # corrupt header - reserverd bits used
    $bad = $keep 
    substr($bad, 3, 1, "\x[FF]") 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 

    # corrupt trailer - length wrong
    $bad = $keep 
    substr($bad, -8, 4, "\x[FF]" x 4) 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 

    # corrupt trailer - CRC wrong
    $bad = $keep 
    substr($bad, -4, 4, "\x[FF]" x 4) 
    $ungzip = Compress::Zlib::memGunzip(\$bad) 
    ok ! defined $ungzip 


do
    title 'CRC32' 

    # CRC32 of this data should have the high bit set
    # value in ascii is ZgRNtjgSUW
    my $data = "\x5a\x67\x52\x4e\x74\x6a\x67\x53\x55\x57"
    my $expected_crc = 0xCF707A2B  # 3480255019

    my $crc = crc32($data) 
    is $crc, $expected_crc


do
    title 'Adler32' 

    # adler of this data should have the high bit set
    # value in ascii is lpscOVsAJiUfNComkOfWYBcPhHZ[bT
    my $data = "\x6c\x70\x73\x63\x4f\x56\x73\x41\x4a\x69\x55\x66" .
        "\x4e\x43\x6f\x6d\x6b\x4f\x66\x57\x59\x42\x63\x50" .
        "\x68\x48\x5a\x5b\x62\x54"
    my $expected_crc = 0xAAD60AC7  # 2866154183
    my $crc = adler32($data) 
    is $crc, $expected_crc


sub slurp
    my $name = shift 

    my $input
    my $fil = gzopen($name, "rb") 
    ok $fil , "opened $name"
    cmp_ok $fil->gzread($input, 50000), "+>", 0, "read more than zero bytes"
    ok ! $fil->gzclose(), "closed ok"

    return $input


sub trickle
    my $name = shift 

    my $got
    my $input
    $fil = gzopen($name, "rb") 
    ok $fil, "opened ok"
    while ($fil->gzread($input, 50000) +> 0)
        $got .= $input
        $input = ''
    
    ok ! $fil->gzclose(), "closed ok"

    return $got

    return $input

