#!/usr/local/bin/perl

use Test::More

my ($i, $j, $test_data)

eval "use Crypt::Rijndael"
if ($^EVAL_ERROR)
    plan skip_all => "Crypt::Rijndael not installed\n"


plan tests => 59

sub pad($s,$decrypt)
    if ($decrypt eq 'd')
        $s =~ s/10*$//s
    else
        $s .= '1' . ('0' x (16 - length($s) % 16 - 1) )
    
    return $s


$test_data = <<END
Mary had a little lamb,
Its fleece was black as coal,
And everywere that Mary went,
That lamb would dig a hole.
END
    

eval "use Crypt::CBC"

ok(!$^EVAL_ERROR,"Couldn't load module")
ok(($i = Crypt::CBC->new(key         => 'a' x 16,
    cipher      => 'Rijndael',
    iv          => 'f' x 16,
    literal_key => 1,
    add_header  => 0,
    padding     => 'oneandzeroes'
    )),
   "Couldn't create new object")
ok(($j = Crypt::Rijndael->new('a' x 16, Crypt::Rijndael->MODE_CBC)),
   "Couldn't create new object")
ok($j->set_iv('f' x 16))

ok($i->decrypt($i->encrypt($test_data)) eq $j->decrypt($j->encrypt($test_data)),"Decrypt doesn't match")

ok($i->decrypt($j->encrypt($test_data)) eq $test_data,"Crypt::CBC can't decrypt Rijndael encryption")

ok($j->decrypt($i->encrypt($test_data)) eq $test_data,"Rijndael can't decrypt Crypt::CBC encryption")

# now try various truncations of the whole
my $t = $test_data
for my $c (1..7)
    substr($t,-$c, undef, '')  # truncate
    ok($t eq pad($i->decrypt($j->encrypt(pad($t,'e'))),'d'),"Crypt::CBC can't decrypt Rijndael encryption")


$t = $test_data
for my $c (1..7)
    substr($t,-$c, undef, '')  # truncate
    is(pad($j->decrypt($i->encrypt(pad($t,'e'))),'d'), $t, "Rijndael can't decrypt Crypt::CBC encryption")


# now try various short strings
for my $c (0..18)
    my $t = 'i' x $c
    ok($t eq pad($j->decrypt($i->encrypt(pad($t,'e'))),'d'),"Rijndael can't decrypt Crypt::CBC encryption")


# now try various short strings
for my $c (0..18)
    my $t = 'i' x $c
    ok($t eq pad($j->decrypt($i->encrypt(pad($t,'e'))),'d'),"Rijndael can't decrypt Crypt::CBC encryption")


