
use Test::More tests => 8

use Net::SSL

my $sock
try {
    $sock = Net::SSL->new(
        PeerAddr => '127.0.0.1',
        PeerPort => 443,
        Timeout  => 3,
        );
}

my $test_name = 'Net::SSL->new'
if ($^EVAL_ERROR)
    my $fail = $^EVAL_ERROR->message
    if ($fail =~ m/\AConnect failed: Connection refused/i)
        pass( "$test_name - expected failure" )
    else
        fail( "$test_name" )
        diag( $fail )
    
else
    ok( defined $sock, $test_name )


SKIP: do
    skip( "nothing listening on localhost:443", 7 )
        unless defined $sock

    is( ref($sock), 'Net::SSL', 'blessed socket' )

    try { $sock->accept }
    like ($^EVAL_ERROR->{?description}, qr(\Aaccept not implemented for Net::SSL sockets),
          'accept() not implemented'
          )

    try { $sock->getc }
    like ($^EVAL_ERROR->{?description}, qr(\Agetc not implemented for Net::SSL sockets),
          'getc() not implemented'
          )

    try { $sock->ungetc }
    like ($^EVAL_ERROR->{?description}, qr(\Aungetc not implemented for Net::SSL sockets),
          'ungetc() not implemented'
          )

    try { $sock->getlines }
    like ($^EVAL_ERROR->{?description}, qr(\Agetlines not implemented for Net::SSL sockets),
          'getlines() not implemented'
          )

    is( $sock->blocking, 1, 'socket is blocking' )
    $sock->blocking(0)
    is( $sock->blocking, 0, 'socket is now non-blocking' )

