#!/usr/local/bin/perl
#
#   $Id: 40blobs.t 9633 2007-06-08 12:54:26Z capttofu $
#
#   This is a test for correct handling of BLOBS; namely $dbh->quote
#   is expected to work correctly.
#


#
#   Make -w happy
#
our $test_dsn = ''
our $test_user = ''
our $test_password = ''


#
#   Include lib.pl
#
require DBI
require bytes
our $mdriver = ""
foreach my $file ((@: "lib.pl", "t/lib.pl"))
    evalfile $file; if ($^EVAL_ERROR) { print $^STDERR, "Error while executing lib.pl: $^EVAL_ERROR\n";
        exit 10;
    }
    if ($mdriver ne '')
        last
    


sub ServerError()
    print $^STDERR, ("Cannot connect: ", DBI::errstr(), "\n",
                     "\tEither your server is not up and running or you have no\n",
                     "\tpermissions for acessing the DSN $test_dsn.\n",
                     "\tThis test requires a running server and write permissions.\n",
                     "\tPlease make sure your server is running and you have\n",
                     "\tpermissions, then retry.\n")
    exit 10



sub ShowBlob($blob)
    for my $i (0..7)
        if (defined($blob)  &&  length($blob) +> $i)
            $b = substr($blob, $i*32)
        else
            $b = ""
        
        printf($^STDOUT, "\%08lx \%s\n", $i*32, unpack("H64", $b))
    



#
#   Main loop; leave this untouched, put tests after creating
#   the new table.
#
while (Testing())
    #
    #   Connect to the database
    our ($state, $dbh)

    Test($state or $dbh = DBI->connect($test_dsn, $test_user, $test_password))
        or ServerError()

    my($def)
    my $table='t1'
    foreach my $size ((@: 128))

        Test($state or $dbh->do("DROP TABLE IF EXISTS $table"))
            or DbiError($dbh->err, $dbh->errstr)
        #
        #   Create a new table
        #
        if (!$state)
            $def = TableDefinition($table,
                                   \(@: "id",   "INTEGER",      4, 0),
                                   \(@: "name", "BLOB",     $size, 0))
            print $^STDOUT, "Creating table:\n$def\n"
        
        Test($state or $dbh->do($def))
            or DbiError($dbh->err, $dbh->errstr)


        #
        #  Create a blob
        #
        my (@: $blob, $qblob) = @: "", undef
        if (!$state)
            my $b = ""
            for my $j (0..255)
                $b .= bytes::chr($j)
            
            for my $i (0..$size-1)
                $blob .= $b
            
            $qblob = $dbh->quote($blob)
        

        #
        #   Insert a row into the test table.......
        #
        my($query)
        if (!$state)
            $query = "INSERT INTO $table VALUES(1, $qblob)"
            if (env::var('SHOW_BLOBS')  &&  open(my $out, ">", "" . $(env::var('SHOW_BLOBS'))))
                print $out, $query
                close($out)
            
        
        Test($state or $dbh->do($query))
            or DbiError($dbh->err, $dbh->errstr)

        #
        #   Now, try SELECT'ing the row out.
        #
        our $sth
        Test($state or $sth = $dbh->prepare("SELECT * FROM $table"
             . " WHERE id = 1"))
            or DbiError($dbh->err, $dbh->errstr)

        Test($state or $sth->execute)
            or DbiError($dbh->err, $dbh->errstr)

        our $row
        Test($state or (defined($row = $sth->fetchrow_arrayref)))
            or DbiError($sth->err, $sth->errstr)

        Test($state or (nelems($row->@) == 2  &&  $row->[0] == 1  &&  $row->[1] eq $blob))
            or (ShowBlob($blob),
                ShowBlob(defined($row->[1]) ?? $row->[1] !! ""))

        Test($state or $sth->finish)
            or DbiError($sth->err, $sth->errstr)

        Test($state or undef $sth || 1)
            or DbiError($sth->err, $sth->errstr)

        #
        #   Finally drop the test table.
        #
        Test($state or $dbh->do("DROP TABLE $table"))
            or DbiError($dbh->err, $dbh->errstr)
    

