#!/usr/local/bin/perl
#
#   $Id: 40numrows.t 9633 2007-06-08 12:54:26Z capttofu $
#
#   This tests, whether the number of rows can be retrieved.
#


#
#   Make -w happy
#
our $test_dsn = ''
our $test_user = ''
our $test_password = ''


#
#   Include lib.pl
#
use DBI
our $mdriver = ""
foreach my $file (@: "lib.pl", "t/lib.pl", "DBD-mysql/t/lib.pl")
    evalfile $file; if ($^EVAL_ERROR) { print $^STDERR, "Error while executing lib.pl: $^EVAL_ERROR\n";
        exit 10;
    }
    if ($mdriver ne '')
        last
    


sub ServerError()
    print $^STDERR, ("Cannot connect: ", DBI::errstr, "\n",
                     "\tEither your server is not up and running or you have no\n",
                     "\tpermissions for acessing the DSN $test_dsn.\n",
                     "\tThis test requires a running server and write permissions.\n",
                     "\tPlease make sure your server is running and you have\n",
                     "\tpermissions, then retry.\n")
    exit 10



sub TrueRows($sth)
    my $count = 0
    while ($sth->fetchrow_arrayref)
        ++$count
    
    $count



#
#   Main loop; leave this untouched, put tests after creating
#   the new table.
#
while (Testing())
    #
    #   Connect to the database
    our ($state, $dbh)
    Test($state or ($dbh = DBI->connect($test_dsn, $test_user,
                    $test_password)))
        or ServerError()

    my $table= 't1'

    Test($state or $dbh->do("DROP TABLE IF EXISTS $table"))
        or DbiError($dbh->err, $dbh->errstr)
    #
    #   Create a new table; EDIT THIS!
    #
    my $def
    Test($state or (($def = TableDefinition($table,
                                            \(@: "id",   "INTEGER",  4, 0),
                                            \(@: "name", "CHAR",    64, 0))),
                    $dbh->do($def)))
        or DbiError($dbh->err, $dbh->errstr)


    #
    #   This section should exercise the sth->rows
    #   method by preparing a statement, then finding the
    #   number of rows within it.
    #   Prior to execution, this should fail. After execution, the
    #   number of rows affected by the statement will be returned.
    #
    Test($state or $dbh->do("INSERT INTO $table"
         . " VALUES( 1, 'Alligator Descartes' )"))
        or DbiError($dbh->err, $dbh->errstr)

    my $sth
    Test($state or ($sth = $dbh->prepare("SELECT * FROM $table"
                    . " WHERE id = 1")))
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or $sth->execute)
        or DbiError($dbh->err, $dbh->errstr)

    my $numrows
    Test($state or ($numrows = $sth->rows) == 1  or  ($numrows == -1))
        or ErrMsgF("Expected 1 rows, got \%s.\n", $numrows)

    Test($state or ($numrows = TrueRows($sth)) == 1)
        or ErrMsgF("Expected to fetch 1 rows, got \%s.\n", $numrows)

    Test($state or $sth->finish)
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or undef $sth or 1)

    Test($state or $dbh->do("INSERT INTO $table"
         . " VALUES( 2, 'Jochen Wiedmann' )"))
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or ($sth = $dbh->prepare("SELECT * FROM $table"
                    . " WHERE id >= 1")))
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or $sth->execute)
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or ($numrows = $sth->rows) == 2  or  ($numrows == -1))
        or ErrMsgF("Expected 2 rows, got \%s.\n", $numrows)

    Test($state or ($numrows = TrueRows($sth)) == 2)
        or ErrMsgF("Expected to fetch 2 rows, got \%s.\n", $numrows)

    Test($state or $sth->finish)
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or undef $sth or 1)

    Test($state or $dbh->do("INSERT INTO $table"
         . " VALUES(3, 'Tim Bunce')"))
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or ($sth = $dbh->prepare("SELECT * FROM $table"
                    . " WHERE id >= 2")))
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or $sth->execute)
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or ($numrows = $sth->rows) == 2  or  ($numrows == -1))
        or ErrMsgF("Expected 2 rows, got \%s.\n", $numrows)

    Test($state or ($numrows = TrueRows($sth)) == 2)
        or ErrMsgF("Expected to fetch 2 rows, got \%s.\n", $numrows)

    Test($state or $sth->finish)
        or DbiError($dbh->err, $dbh->errstr)

    Test($state or undef $sth or 1)

    #
    #   Finally drop the test table.
    #
    Test($state or $dbh->do("DROP TABLE $table"))
        or DbiError($dbh->err, $dbh->errstr)


