#!/usr/local/bin/perl
#
#   $Id: 60leaks.t 9633 2007-06-08 12:54:26Z capttofu $
#
#   This is a skeleton test. For writing new tests, take this file
#   and modify/extend it.
#

my $COUNT_CONNECT = 4000   # Number of connect/disconnect iterations
my $COUNT_PREPARE = 10000  # Number of prepare/execute/finish iterations


my $haveStorable

if (!env::var('SLOW_TESTS'))
    print $^STDOUT, "1..0 # Skip \$ENV\{SLOW_TESTS\} is not set\n"
    exit 0

try { require Proc::ProcessTable; }
if ($^EVAL_ERROR)
    print $^STDOUT, "1..0 # Skip Proc::ProcessTable not installed \n"
    exit 0

try { require Storable }
$haveStorable = $^EVAL_ERROR ?? 0 !! 1

sub size
    my $pt = Proc::ProcessTable->new('cache_ttys' => $haveStorable)
    foreach my $p ( $pt->table()->@)
        if ($p->pid() == $^PID)
            return $p->size()
        
    
    die "Cannot find my own process?!?\n"
    exit 0


#
#   Make -w happy
#

our ($test_dsn, $test_user, $test_password)

#
#   Include lib.pl
#
require DBI
our $mdriver = ""
foreach my $file ((@: "lib.pl", "t/lib.pl"))
    evalfile $file; if ($^EVAL_ERROR) { print $^STDERR, "Error while executing lib.pl: $^EVAL_ERROR\n";
        exit 10;
    }
    if ($mdriver ne '')
        last
    

if ($mdriver eq 'whatever')
    print $^STDOUT, "1..0\n"
    exit 0



#
#   Main loop; leave this untouched, put tests after creating
#   the new table.
#
while (Testing())
    #
    #   Connect to the database
    our ($state, $dbh)
    Test($state or ($dbh = DBI->connect($test_dsn, $test_user,
                    $test_password)),
         undef,
         "Attempting to connect.\n")
        or ErrMsgF("Cannot connect: Error \%s.\n\n"
                   . "Make sure, your database server is up and running.\n"
                   . "Check that '$test_dsn' references a valid database"
                   . " name.\nDBI error message: DBI::errstr")

    my $table= 't1'

    Test($state or < $dbh->do("DROP TABLE IF EXISTS $table"))
        or ErrMsgF("Cannot DROP test table $table: \%s.\n", <
                   $dbh->errstr)
    #
    #   Create a new table; EDIT THIS!
    #
    my $def
    Test($state or (($def = TableDefinition($table,
                                            \(@: "id",   "INTEGER",  4, 0),
                                            \(@: "name", "CHAR",    64, 0))), <
                    $dbh->do($def)))
        or ErrMsgF("Cannot create table: Error \%s.\n", <
                   $dbh->errstr)


    my($size, $prevSize, $ok, $notOk, $dbh2, $msg)
    if (!$state)
        print $^STDOUT, "Testing memory leaks in connect/disconnect\n"
        $msg = "Possible memory leak in connect/disconnect detected"

        $ok = 0
        $notOk = 0
        for my $i (0..$COUNT_CONNECT-1)
            if (!($dbh2 = DBI->connect($test_dsn, $test_user,
                  $test_password)))
                $ok = 0
                $msg = "Cannot connect: DBI::errstr\n"
                last
            
            $dbh2->disconnect()
            undef $dbh2
            if ($i % 100  ==  99)
                $size = size()
                if (defined($prevSize)  &&  $size == $prevSize)
                    ++$ok
                else
                    ++$notOk
                
                $prevSize = $size
            
        
    
    Test($state or ($ok +> $notOk))
        or print $^STDOUT, "$msg\n"


    if (!$state)
        print $^STDOUT, "Testing memory leaks in prepare/execute/finish\n"
        $msg = "Possible memory leak in prepare/execute/finish detected"

        $ok = 0
        $notOk = 0
        undef $prevSize
        for my $i (0..$COUNT_PREPARE-1)
            my $sth = $dbh->prepare("SELECT * FROM $table")
            $sth->execute()
            $sth->finish()
            undef $sth

            if ($i % 100  ==  99)
                $size = size()
                if (defined($prevSize)  &&  $size == $prevSize)
                    ++$ok
                else
                    ++$notOk
                
                $prevSize = $size
            
        
    
    Test($state or ($ok +> $notOk))
        or print $^STDOUT, "$msg\n"


    if (!$state)
        print $^STDOUT, "Testing memory leaks in fetchrow_arrayref\n"
        $msg = "Possible memory leak in fetchrow_arrayref detected"

        # Insert some records into the test table
        foreach my $row ((@: \(@: 1, 'Jochen Wiedmann')
                             \(@: 2, 'Andreas Knig')
                             \(@: 3, 'Tim Bunce')
                             \(@: 4, 'Alligator Descartes')
                             \(@: 5, 'Jonathan Leffler')))
            $dbh->do(sprintf("INSERT INTO $table VALUES (\%d, \%s)",
                $row->[0], < $dbh->quote($row->[1])))
        

        $ok = 0
        $notOk = 0
        undef $prevSize
        for my $i (0..$COUNT_PREPARE-1)
            do
                my $sth = $dbh->prepare("SELECT * FROM $table")
                $sth->execute()
                my $row
                while ($row = $sth->fetchrow_arrayref()) {
                }
                $sth->finish()
            

            if ($i % 100  ==  99)
                $size = size()
                if (defined($prevSize)  &&  $size == $prevSize)
                    ++$ok
                else
                    ++$notOk
                
                $prevSize = $size
            
        
    
    Test($state or ($ok +> $notOk))
        or print $^STDOUT, "$msg\n"


    if (!$state)
        print $^STDOUT, "Testing memory leaks in fetchrow_hashref\n"
        $msg = "Possible memory leak in fetchrow_hashref detected"

        $ok = 0
        $notOk = 0
        undef $prevSize
        for my $i (0..$COUNT_PREPARE-1)
            do
                my $sth = $dbh->prepare("SELECT * FROM $table")
                $sth->execute()
                my $row
                while ($row = $sth->fetchrow_hashref()) {
                }
                $sth->finish()
            

            if ($i % 100  ==  99)
                $size = size()
                if (defined($prevSize)  &&  $size == $prevSize)
                    ++$ok
                else
                    ++$notOk
                
                $prevSize = $size
            
        
    
    Test($state or ($ok +> $notOk))
        or print $^STDOUT, "$msg\n"


    #
    #   Finally drop the test table.
    #
    Test($state or < $dbh->do("DROP TABLE $table"))
        or ErrMsgF("Cannot DROP test table $table: \%s.\n", <
                   $dbh->errstr)
    Test($state or < $dbh->disconnect)

