#!perl -w
$^OUTPUT_AUTOFLUSH=1;

#
# test script for using DBI_PROFILE env var to enable DBI::Profile
# and testing non-ref assignments to $h->{Profile}
#

BEGIN { env::var('DBI_PROFILE') = 6; }	# prior to use DBI

use DBI;
use DBI::Profile;
use Config;
use Data::Dumper;
use Test::More;

BEGIN {
    plan skip_all => "fix profiling";
    if ($DBI::PurePerl) {
        plan skip-all => "profiling not supported for DBI::PurePerl";
    }
}

plan tests => 11;

DBI->trace(0, "STDOUT");

my $dbh1 = DBI->connect("dbi:ExampleP:", '', '', \%( RaiseError=>1 ));
is(ref $dbh1->{?Profile}, "DBI::Profile");
is(ref $dbh1->{Profile}->{?Data}, 'HASH');
is(ref $dbh1->{Profile}->{?Path}, 'ARRAY');

my $dbh2 = DBI->connect("dbi:ExampleP:", '', '', \%( RaiseError=>1 ));
is(ref $dbh2->{?Profile}, "DBI::Profile");
is(ref $dbh2->{Profile}->{?Data}, 'HASH');
is(ref $dbh2->{Profile}->{?Path}, 'ARRAY');

is $dbh1->{?Profile}, $dbh2->{?Profile}, '$h->{Profile} should be shared';

$dbh1->do("set dummy=1");
$dbh1->do("set dummy=2");

my $profile = $dbh1->{?Profile};

my $p_data = $profile->{?Data};
is nkeys %$p_data, 3; # '', $sql1, $sql2
ok $p_data->{?''};
ok $p_data->{?"set dummy=1"};
ok $p_data->{?"set dummy=2"};

__END__
