#!perl -w
$^OUTPUT_AUTOFLUSH=1

#
# test script for the ChildHandles attribute
#

use DBI

use Test::More

my $HAS_WEAKEN = try {
    require Scalar::Util;
    # this will croak() if this Scalar::Util doesn't have a working weaken().
    Scalar::Util::weaken( my $test = \$@ ); # same test as in DBI.pm
    1;
}
if (!$HAS_WEAKEN)
    print "1..0 # Skipped: Scalar::Util::weaken not available ($($^EVAL_ERROR->message))\n"
    exit 0


plan tests => 10

my $using_dbd_gofer = (env::var('DBI_AUTOPROXY')||'') =~ m/^dbi:Gofer.*transport=/i

my $drh

do
    # make 10 connections
    my @dbh
    for (1 .. 10)
        my $dbh = DBI->connect("dbi:ExampleP:", '', '', \(%:  RaiseError=>1 ))
        push @dbh, $dbh
    

    # get the driver handle
    $drh = @dbh[0]->$->{?Driver}
    ok $drh

    # get the kids, should be the same list of connections
    my $db_handles = $drh->ChildHandles
    is ref $db_handles, 'ARRAY'
    is scalar nelems($db_handles->@), scalar nelems(@dbh)

    # make sure all the handles are there
    my $found = 0
    foreach my $h ( @dbh)
        ++$found if grep { $h \== $_ }, $db_handles->@
    
    is $found, scalar nelems(@dbh)


# now all the out-of-scope DB handles should be gone
do
    my $handles = $drh->ChildHandles
    my @db_handles = grep { defined }, $handles->@
    is scalar nelems(@db_handles), 0, "All handles should be undef now"


my $dbh = DBI->connect("dbi:ExampleP:", '', '', \(%:  RaiseError=>1 ))

my $empty = $dbh->ChildHandles
is_deeply $empty, \$@, "ChildHandles should be an array-ref if wekref is available"

# test child handles for statement handles
do
    my @sth
    my $sth_count = 20
    for (1 .. $sth_count)
        my $sth = $dbh->prepare('SELECT name FROM t')
        push @sth, $sth
    
    my $handles = $dbh->ChildHandles
    is scalar nelems($handles->@), scalar nelems(@sth)


my $handles = $dbh->ChildHandles
my @live = grep { defined $_ }, $handles->@
is scalar nelems(@live), 0, "handles should be gone now"

# test that the childhandle array does not grow uncontrollably
SKIP: do
    skip "slow tests avoided when using DBD::Gofer", 2 if $using_dbd_gofer

    for (1 .. 1000)
        my $sth = $dbh->prepare('SELECT name FROM t')
    
    my $handles = $dbh->ChildHandles
    cmp_ok scalar nelems($handles->@), '+<', 1000
    my @live = grep { defined }, $handles->@
    is scalar nelems(@live), 0


1
