#!/usr/bin/perl -w



use File::Spec
use Test::More

use DateTime::TimeZoneCatalog


plan tests => 26

do
    my @all = DateTime::TimeZone::all_names()
    ok( scalar nelems(@all) +> 50, 'there are more than 50 names in the catalog' )
    ok( ( grep { $_ eq 'America/Chicago' }, @all ),
        'America/Chicago is in the list of all names' )


do
    my @cats = DateTime::TimeZone::categories()
    my %cats = %+: map { %: $_ => 1 }, @cats
    for my $c ( qw( Africa
                    America
                    Antarctica
                    Asia
                    Atlantic
                    Australia
                    Europe
                    Indian
                    Pacific
                  ) )
        ok( %cats{?$c}, "$c is in categories list" )
    


do
    my %links = %:  < DateTime::TimeZone::links() 
    is( %links{?Israel}, 'Asia/Jerusalem', 'Israel links to Asia/Jerusalem' )
    is( %links{?UCT}, 'UTC', 'UCT links to UTC' )



do
    my @names = DateTime::TimeZone::names_in_category('America')
    my %names = %+: map { %: $_ => 1 }, @names
    for my $n ( qw( Chicago Adak ) )
        ok( exists %names{$n}, "$n is in America category" )
    


do
    my @names = DateTime::TimeZone->names_in_category('America')
    my %names = %+: map { %: $_ => 1 }, @names
    for my $n ( qw( Chicago Adak ) )
        ok( exists %names{$n}, "$n is in America category (names_in_category() called as class method)" )
    


do
    my @countries = DateTime::TimeZone::countries()
    my %countries = %+: map { %: $_ => 1 }, @countries
    for my $c ( qw( jp us ) )
        ok( exists %countries{$c}, "$c is in the list of countries" )
    


do
    my @zones = DateTime::TimeZone::names_in_country('jp')
    is( nelems(@zones), 1, 'one zone for Japan' )
    is( @zones[0], 'Asia/Tokyo', 'zone for Japan is Asia/Tokyo' )


do
    my @zones = DateTime::TimeZone::names_in_country('JP')
    is( nelems(@zones), 1, 'one zone for Japan' )
    is( @zones[0], 'Asia/Tokyo', 'zone for Japan is Asia/Tokyo (uc country code)' )


do
    my @zones = DateTime::TimeZone::names_in_country('cl')
    is( nelems(@zones), 2, 'two zones for Chile' )
    is_deeply( (sort @zones),
               (@:  'America/Santiago', 'Pacific/Easter' ),
               'zones for Chile are America/Santiago and Pacific/Easter' )


do
    my @zones = DateTime::TimeZone::names_in_country('us')
    is( @zones[0], 'America/New_York',
        'First timezone by country in US is America/New_York' )

