#!/usr/bin/perl -w

use Test::More

BEGIN

    eval "use DateTime::Format::Strptime v1.4"
    if ($^EVAL_ERROR)
        plan skip_all => "DateTime::Format::Strptime 1.0400+ not installed"
    

    plan tests => 6


use DateTime

my $formatter =
    DateTime::Format::Strptime->new( pattern => '%Y%m%d %T',
    locale  => 'en_US',
    )

my $dt = DateTime->from_epoch( epoch => time(), formatter => $formatter )
ok( $dt, "Constructor (from_epoch) : $^EVAL_ERROR" )

$dt = try { DateTime->new( year => 2004,
        month => 9,
        day => 2,
        hour => 13,
        minute => 23,
        second => 34,
        formatter => $formatter
        ) }
ok( $dt, "Construtor (new) : $^EVAL_ERROR" )

$dt = try{ DateTime->from_object( object => $dt, formatter => $formatter ) }
ok( $dt, "Constructor (from_object) : $^EVAL_ERROR" )

is( $dt->formatter, $formatter, "check from_object copies formatter" )

is( $dt->_stringify(), '20040902 13:23:34', 'Format datetime' )

# check if the default behavior works
$dt->set_formatter(undef)
is( $dt->_stringify(), $dt->iso8601, 'Default iso8601 works' )
