package Devel::Symdump

use Carp ()

our ($Defaults, $VERSION, $MAX_RECURSION)

$VERSION = '2.07'
$MAX_RECURSION = 97

$Defaults = \%:
    'RECURS'   => 0
    'AUTOLOAD' => \ %:
        'packages'	=> 1
        'scalars'	=> 1
        'arrays'	=> 1
        'hashes'	=> 1
        'functions'	=> 1
        'ios'	=> 1
        'unknowns'	=> 1
    'SEEN' => \$%


sub rnew($class,@< @packages)

    my $self = bless \(%: < Symbol::fetch_glob("$class\::Defaults")->*->$->%), $class
    $self->{+RECURS}++
    $self->_doit(< @packages)


sub new($class,@< @packages)

    my $self = bless \(%: < Symbol::fetch_glob("$class\::Defaults")->*->$->%), $class
    $self->_doit(< @packages)


sub _doit($self,@< @packages)
    @packages = (@: "main") unless nelems(@packages)
    $self->{+RESULT} = $self->_symdump(< @packages)
    return $self


sub _symdump($self,@< @packages)
    my($num,@todo,$tmp)
    my $result = $%
    foreach my $pack ( @packages)

        my $stash = Symbol::fetch_glob("$pack\::")
        for my $key (keys $stash->*->%)
            my $valref = \$stash->*->{+$key}
            my $gotone = 0
            #### SCALAR ####
            if (defined $valref->*{SCALAR})
                $result{+$pack}{+SCALARS}{+$key}++
                $gotone++

            #### ARRAY ####
            if (defined $valref->*{ARRAY})
                $result{+$pack}{+ARRAYS}{+$key}++
                $gotone++

            #### HASH ####
            if (defined $valref->*{HASH} && $key !~ m/::/)
                $result->{+$pack}{+HASHES}{+$key}++
                $gotone++

            #### PACKAGE ####
            if (defined $valref->*{HASH} && $key =~ m/::$/ &&
                $key ne "main::" && $key ne "<none>::")
                my $p = $pack ne "main" ?? "$pack\::" !! ""
                ($p .= $key) =~ s/::$//
                $result{+$pack}{+PACKAGES}{+$p}++
                $gotone++
                if (++$self->{SEEN}->{+Symbol::glob_name($valref->*)} +> $Devel::Symdump::MAX_RECURSION)
                    next

                push @todo, $p

            #### FUNCTION ####
            if (defined $valref->*{CODE})
                $result{+$pack}{+FUNCTIONS}{+$key}++
                $gotone++


            #### IO #### had to change after 5.003_10
            if (defined $valref->*{IO}) # fileno and telldir...
                $result{+$pack}{+IOS}{+$key}++
                $gotone++


            #### SOMETHING ELSE ####
            unless ($gotone)
                $result{+$pack}{+UNKNOWNS}{+$key}++




    return (nelems(@todo) && $self->{?RECURS})
        ?? ( $result +%+ $self->_symdump(< @todo) )
        !! $result


sub _partdump($self,$part)
    my (@result)
    my $prepend = ""
    foreach my $pack (keys $self->{?RESULT})
        $prepend = "$pack\::" unless $part eq 'PACKAGES'
        push @result, < map {"$prepend$_"}, keys($self->{RESULT}{$pack}{?$part} || $%)

    return @result


sub as_string
    my $self = shift
    my(@m)
    for my $type (sort keys $self->{?'AUTOLOAD'}->%)
        push @m, $type
        push @m, "\t" . join "\n\t", map {
                s/([\000-\037\177])/$( '^' . pack('c', ord($1) ^^^ 64) )/g;
                $_;
            }, sort $self->_partdump(uc $type)

    return join "\n", @m


sub as_HTML
    my $self = shift
    my(@m)
    push @m, "<TABLE>"
    for my $type (sort keys $self->{?'AUTOLOAD'}->%)
        push @m, "<TR><TD valign=top><B>$type</B></TD>"
        push @m, "<TD>" . join ", ", map {
                s/([\000-\037\177])/$( '^' .
                pack('c', ord($1) ^^^ 64)
                )/g; $_;
            }, sort $self->_partdump(uc $type)
        push @m, "</TD></TR>"

    push @m, "</TABLE>"
    return join "\n", @m


sub diff($self,$second)
    my(@m)
    for my $type (sort keys $self->{?'AUTOLOAD'}->%)
        my(%first,%second,%all)
        foreach my $symbol ( $self->_partdump(uc $type))
            %first{+$symbol}++
            %all{+$symbol}++

        foreach my $symbol ( $second->_partdump(uc $type))
            %second{+$symbol}++
            %all{+$symbol}++

        my(@typediff)
        foreach my $symbol (sort keys %all)
            next if %first{?$symbol} && %second{?$symbol}
            push @typediff, "- $symbol" unless %second{?$symbol}
            push @typediff, "+ $symbol" unless %first{?$symbol}

        foreach ( @typediff)
            s/([\000-\037\177])/$( '^' . pack('c', ord($1) ^^^ 64) )/g

        push @m, $type, < @typediff if nelems(@typediff)

    return join "\n", @m


sub inh_tree($self)
    return $self->{?INHTREE} if ref $self && defined $self->{?INHTREE}
    my $inherited_by = \$%
    my($m)=""
    my(@: @isa) =@:  grep { m/\bISA$/ }, Devel::Symdump->rnew->arrays
    foreach my $isa (sort @isa)
        $isa =~ s/::ISA$//
        foreach my $isaisa ( Symbol::fetch_glob("$isa\::ISA")->*->@)
            $inherited_by->{$isaisa}->{+$isa}++


    foreach my $item (sort keys $inherited_by->%)
        $m .= "$item\n"
        $m .= _inh_tree($item,$inherited_by)

    $self->{+INHTREE} = $m if ref $self
    $m


sub _inh_tree($package,$href,$depth)
    return unless defined $href
    $depth ||= 0
    $depth++
    if ($depth +> 100)
        warn "Deep recursion in ISA\n"
        return

    my($m) = ""
    # print "DEBUG: package[$package]depth[$depth]\n";
    foreach my $i (sort keys $href->{?$package}->%)
        $m .= qq{\t} x $depth
        $m .= qq{$i\n}
        $m .= _inh_tree($i,$href,$depth)

    $m


sub isa_tree($self)
    return $self->{?ISATREE} if ref $self && defined $self->{?ISATREE}
    my(@: @isa) =@:  grep { m/\bISA$/ }, Devel::Symdump->rnew->arrays
    my($m) = ""
    foreach my $isa (sort @isa)
        $isa =~ s/::ISA$//
        $m .= qq{$isa\n}
        $m .= _isa_tree($isa)

    $self->{+ISATREE} = $m if ref $self
    $m


sub _isa_tree($package,$depth)
    $depth ||= 0
    $depth++
    if ($depth +> 100)
        warn "Deep recursion in ISA\n"
        return

    my($m) = ""
    # print "DEBUG: package[$package]depth[$depth]\n";
    foreach my $isaisa ( Symbol::fetch_glob("$package\::ISA")->*->@)
        $m .= qq{\t} x $depth
        $m .= qq{$isaisa\n}
        $m .= _isa_tree($isaisa,$depth)

    $m


for my $name ((@:  <qw|filehandles dirhandles|, < keys $Defaults->{?'AUTOLOAD'}->%))
    Symbol::fetch_glob($name)->* =
        sub (@< @_)
            my(@: $self,@< @packages) =  @_
            unless (ref $self)
                $self = $self->new(< @packages)


            (my $xname = $name) =~ s/(file|dir)handles/ios/
            my $compat = $1

            my @syms = $self->_partdump(uc $xname)
            return @syms # make sure now it gets context right



1

__END__

=head1 NAME

Devel::Symdump - dump symbol names or the symbol table

=head1 SYNOPSIS

    # Constructor
    require Devel::Symdump;
    @packs = qw(some_package another_package);
    $obj = Devel::Symdump->new(@packs);        # no recursion
    $obj = Devel::Symdump->rnew(@packs);       # with recursion

    # Methods
    @array = $obj->packages;
    @array = $obj->scalars;
    @array = $obj->arrays;
    @array = $obj->hashes;
    @array = $obj->functions;
    @array = $obj->filehandles;  # deprecated, use ios instead
    @array = $obj->dirhandles;   # deprecated, use ios instead
    @array = $obj->ios;
    @array = $obj->unknowns;     # only perl version < 5.003 had some

    $string = $obj->as_string;
    $string = $obj->as_HTML;
    $string = $obj1->diff($obj2);

    $string = Devel::Symdump->isa_tree;    # or $obj->isa_tree
    $string = Devel::Symdump->inh_tree;    # or $obj->inh_tree

    # Methods with autogenerated objects
    # all of those call new(@packs) internally
    @array = Devel::Symdump->packages(@packs);
    @array = Devel::Symdump->scalars(@packs);
    @array = Devel::Symdump->arrays(@packs);
    @array = Devel::Symdump->hashes(@packs);
    @array = Devel::Symdump->functions(@packs);
    @array = Devel::Symdump->ios(@packs);
    @array = Devel::Symdump->unknowns(@packs);

=head1 DESCRIPTION

This little package serves to access the symbol table of perl.

=over 4

=item C<Devel::Symdump-E<gt>rnew(@packages)>

returns a symbol table object for all subtrees below @packages.
Nested Modules are analyzed recursively. If no package is given as
argument, it defaults to C<main>. That means to get the whole symbol
table, just do a C<rnew> without arguments.

The global variable $Devel::Symdump::MAX_RECURSION limits the
recursion to prevent contention. The default value is set to 97, just
low enough to survive the test suite without a warning about deep
recursion.

=item C<Devel::Symdump-E<gt>new(@packages)>

does not go into recursion and only analyzes the packages that are
given as arguments.

=item packages, scalars, arrays, hashes, functions, ios

The methods packages(), scalars(), arrays(), hashes(), functions(),
ios(), and (for older perls) unknowns() each return an array of fully
qualified symbols of the specified type in all packages that are held
within a Devel::Symdump object, but without the leading C<$>, C<@> or
C<%>. In a scalar context, they will return the number of such
symbols. Unknown symbols are usually either formats or variables that
haven't yet got a defined value.

=item as_string

=item as_HTML

As_string() and as_HTML() return a simple string/HTML representations
of the object.

=item diff

Diff() prints the difference between two Devel::Symdump objects in
human readable form. The format is similar to the one used by the
as_string method.

=item isa_tree

=item inh_tree

Isa_tree() and inh_tree() both return a simple string representation
of the current inheritance tree. The difference between the two
methods is the direction from which the tree is viewed: top-down or
bottom-up. As I'm sure, many users will have different expectation
about what is top and what is bottom, I'll provide an example what
happens when the Socket module is loaded:

=item % print Devel::Symdump-E<gt>inh_tree

    AutoLoader
            DynaLoader
                    Socket
    DynaLoader
            Socket
    Exporter
            Carp
            Config
            Socket

The inh_tree method shows on the left hand side a package name and
indented to the right the packages that use the former.

=item % print Devel::Symdump-E<gt>isa_tree

    Carp
            Exporter
    Config
            Exporter
    DynaLoader
            AutoLoader
    Socket
            Exporter
            DynaLoader
                    AutoLoader

The isa_tree method displays from left to right ISA relationships, so
Socket IS A DynaLoader and DynaLoader IS A AutoLoader. (Actually, they
were at the time this manpage was written)

=back

You may call both methods, isa_tree() and inh_tree(), with an
object. If you do that, the object will store the output and retrieve
it when you call the same method again later. The typical usage would
be to use them as class methods directly though.

=head1 SUBCLASSING

The design of this package is intentionally primitive and allows it to
be subclassed easily. An example of a (maybe) useful subclass is
Devel::Symdump::Export, a package which exports all methods of the
Devel::Symdump package and turns them into functions.

=head1 AUTHORS

Andreas Koenig F<< <andk@cpan.org> >> and Tom Christiansen
F<< <tchrist@perl.com> >>. Based on the old F<dumpvar.pl> by Larry
Wall.

=head1 COPYRIGHT, LICENSE

This module is

Copyright (c) 1995, 1997, 2000, 2002, 2005, 2006 Andreas Koenig C<< <andk@cpan.org> >>.

All rights reserved.

This library is free software;
you may use, redistribute and/or modify it under the same
terms as Perl itself.

=cut


# Local Variables:
# mode: cperl
# cperl-indent-level: 4
# End:
