#!perl -w


use Config 'config_value'
use ExtUtils::MakeMaker

my $PERL_CORE = grep { $_ eq "PERL_CORE=1" }, @ARGV

my @extra
@extra = (@: DEFINE => "-DU32_ALIGNMENT_REQUIRED") unless free_u32_alignment()

if ($^OS_NAME eq 'VMS')
    if (defined(config_value("ccname")))
        if (grep( {m/VMS_VAX/ }, $^INCLUDE_PATH) && (config_value("ccname") eq 'DEC'))
            # VAX compiler optimizer even as late as v6.4 gets stuck
            push(@extra, OPTIMIZE => "/Optimize=(NODISJOINT)")
        
    


push(@extra, 'INSTALLDIRS'  => 'perl')

WriteMakefile(
    'NAME'	   => 'Digest::MD5',
    'VERSION_FROM' => 'MD5.pm',
    'PREREQ_PM'    => (%:  'File::Spec' => 0
                           'Digest::base' => '1.00'
    ),
    < @extra,
    'dist'         => (%:  COMPRESS => 'gzip -9f', SUFFIX => 'gz', ),
    )



sub free_u32_alignment
    $^OUTPUT_AUTOFLUSH=1
    if (defined config_value("d_u32align"))
        print $^STDOUT, "Perl's config says that U32 access must "
        print $^STDOUT, "not " unless config_value("d_u32align")
        print $^STDOUT, "be aligned.\n"
        return ! config_value("d_u32align")
    

    if ($^OS_NAME eq 'VMS' || $^OS_NAME eq 'MSWin32')
        print $^STDOUT, "Assumes that $^OS_NAME implies free alignment for U32 access.\n"
        return 1
    

    if ($^OS_NAME eq 'hpux' && config_value("osvers") +< 11.0)
        print $^STDOUT, "Will not test for free alignment on older HP-UX.\n"
        return 0
    

    print $^STDOUT, "Testing alignment requirements for U32... "
    open(my $align_test, ">", "u32align.c") or die "$^OS_ERROR"
    print $align_test, <<'EOT'; close($align_test)
/*--------------------------------------------------------------*/
/*  This program allocates a buffer of U8 (char) and then tries */
/*  to access it through a U32 pointer at every offset.  The    */
/*  program  is expected to die with a bus error/seg fault for  */
/*  machines that do not support unaligned integer read/write   */
/*--------------------------------------------------------------*/

#include <stdio.h>
#include "EXTERN.h"
#include "perl.h"

#ifdef printf
 #undef printf
#endif

int main(int argc, char** argv, char** env)
{
#if BYTEORDER == 0x1234 || BYTEORDER == 0x4321
    U8 buf[] = "\0\0\0\1\0\0\0\0";
    U32 *up;
    int i;

    if (sizeof(U32) != 4) {
	printf("sizeof(U32) is not 4, but %d\n", sizeof(U32));
	exit(1);
    }

    fflush(stdout);

    for (i = 0; i < 4; i++) {
	up = (U32*)(buf + i);
	if (! ((*up == 1 << (8*i)) ||   /* big-endian */
	       (*up == 1 << (8*(3-i)))  /* little-endian */
	      )
	   )
	{
	    printf("read failed (%x)\n", *up);
	    exit(2);
	}
    }

    /* write test */
    for (i = 0; i < 4; i++) {
	up = (U32*)(buf + i);
	*up = 0xBeef;
	if (*up != 0xBeef) {
	    printf("write failed (%x)\n", *up);
	    exit(3);
	}
    }

    printf("no restrictions\n");
    exit(0);
#else
    printf("unusual byteorder, playing safe\n");
    exit(1);
#endif
    return 0;
}
/*--------------------------------------------------------------*/
EOT

    my $cc_cmd = config_value("cc") . " " . config_value("ccflags") . " -I" . config_value("archlibexp") . "/CORE"
    my $exe = "u32align" . config_value("exe_ext")
    $cc_cmd .= " -o $exe"
    my $rc
    $rc = system("$cc_cmd $(config_value('ldflags')) u32align.c $(config_value('libs'))")
    if ($rc)
        print $^STDOUT, "Can't compile test program.  Will ensure alignment to play safe.\n\n"
        unlink("u32align.c", $exe, "u32align$(config_value('obj_ext'))")
        return 0
    

    $rc = system("./$exe")
    unlink("u32align.c", $exe, "u32align$(config_value('obj_ext'))")

    return 1 unless $rc

    if ($rc +> 0x80)
        (my $cp = $rc) >>= 8
        print $^STDOUT, "Test program exit status was $cp\n"
    
    if ($rc ^&^ 0x80)
        $rc ^&^= ^~^0x80
        unlink("core") && print $^STDOUT, "Core dump deleted\n"
    
    print $^STDOUT, "signal $rc\n" if $rc && $rc +< 0x80
    return 0

