
print $^STDOUT, "1..4\n"

use Digest::MD5 < qw(md5 md5_hex md5_base64)

# To update the EBCDIC section even on a Latin 1 platform,
# run this script with $ENV{EBCDIC_MD5SUM} set to a true value.
# (You'll need to have Perl 5.7.3 or later, to have the Encode installed.)
# (And remember that under the Perl core distribution you should
#  also have the $ENV{PERL_CORE} set to a true value.)
# Similarly, to update MacOS section, run with $ENV{MAC_MD5SUM} set.

my $EXPECT
if ("\n" eq "\015") # MacOS
    $EXPECT = <<EOT
628699b88b6a803225678802d2470067  Changes
c95549c6c5e1e1c078b27042f1dc850f  README
315f43d139c2ead64060d03b13b39ec5  MD5.xs
754b9db19f79dbc4992f7166eb0f37ce  rfc1321.txt
EOT
else
    # This is the output of: 'md5sum Changes README MD5.pm MD5.xs rfc1321.txt'
    $EXPECT = <<EOT
2fdc25c326960308f5334c967455d1f5  Changes
c95549c6c5e1e1c078b27042f1dc850f  README
4841575733dfd8a8169d52246ce57104  MD5.xs
754b9db19f79dbc4992f7166eb0f37ce  rfc1321.txt
EOT


if (!(-f "README") && -f "../README")
    chdir("..") or die "Can't chdir: $^OS_ERROR"


my $testno = 0

my $B64 = 1
try { require MIME::Base64; }
if ($^EVAL_ERROR)
    print $^STDOUT, "# $^EVAL_ERROR: Will not test base64 methods\n"
    $B64 = 0


for (split m/^/, $EXPECT)
    my(@: $md5hex, $file) =  split ' '
    my $base = $file
    #     print "# $base\n";
    #     print "# file = $file\n";
    unless (-f $file)
        warn "No such file: $file\n"
        next
    
    if (env::var('MAC_MD5SUM'))
        require Encode
        my $data = cat_file($file)
        Encode::from_to($data, 'latin1', 'MacRoman')
        print $^STDOUT, < md5_hex($data), "  $base\n"
        next
    
    my $md5bin = pack("H*", $md5hex)
    my $md5b64
    if ($B64)
        $md5b64 = MIME::Base64::encode($md5bin, "")
        chop($md5b64); chop($md5b64)   # remove padding
    
    my $failed
    my $got

    if (digest_file($file, 'digest') ne $md5bin)
        print $^STDOUT, "$file: Bad digest\n"
        $failed++
    

    if (($got = digest_file($file, 'hexdigest')) ne $md5hex)
        print $^STDOUT, "$file: Bad hexdigest: got $got expected $md5hex\n"
        $failed++
    

    if ($B64 && digest_file($file, 'b64digest') ne $md5b64)
        print $^STDOUT, "$file: Bad b64digest\n"
        $failed++
    

    my $data = cat_file($file)
    if (md5($data) ne $md5bin)
        print $^STDOUT, "$file: md5() failed\n"
        $failed++
    
    if (md5_hex($data) ne $md5hex)
        print $^STDOUT, "$file: md5_hex() failed\n"
        $failed++
    
    if ($B64 && md5_base64($data) ne $md5b64)
        print $^STDOUT, "$file: md5_base64() failed\n"
        $failed++
    

    if (Digest::MD5->new->add($data)->digest ne $md5bin)
        print $^STDOUT, "$file: MD5->new->add(...)->digest failed\n"
        $failed++
    
    if (Digest::MD5->new->add($data)->hexdigest ne $md5hex)
        print $^STDOUT, "$file: MD5->new->add(...)->hexdigest failed\n"
        $failed++
    
    if ($B64 && Digest::MD5->new->add($data)->b64digest ne $md5b64)
        print $^STDOUT, "$file: MD5->new->add(...)->b64digest failed\n"
        $failed++
    

    my @data = split m//, $data
    if (md5(< @data) ne $md5bin)
        print $^STDOUT, "$file: md5(\@data) failed\n"
        $failed++
    
    if (Digest::MD5->new->add(< @data)->digest ne $md5bin)
        print $^STDOUT, "$file: MD5->new->add(\@data)->digest failed\n"
        $failed++
    
    my $md5 = Digest::MD5->new
    for ( @data)
        $md5->add($_)
    
    if ($md5->digest ne $md5bin)
        print $^STDOUT, "$file: \$md5->add()-loop failed\n"
        $failed++
    

    print $^STDOUT, "not " if $failed
    print $^STDOUT, "ok ", ++$testno, "\n"



sub digest_file($file, $method)
    $method ||= "digest"
    #print "$file $method\n";

    open(my $fh, "<", $file) or die "Can't open $file: $^OS_ERROR"
    my $digest = Digest::MD5->new->addfile($fh->*)->?$method()
    close($fh)

    $digest


sub cat_file($file)
    local $^INPUT_RECORD_SEPARATOR = undef  # slurp
    open(my $fh, "<", $file) or die "Can't open $file: $^OS_ERROR"

    # For PerlIO in case of UTF-8 locales.
    eval 'binmode($fh, ":bytes")'

    my $tmp = ~< $fh->*
    close($fh)
    $tmp


