
use Test::More
use IO::File

my $MODULE

BEGIN 
    $MODULE = (-e "SHA.pm") ?? "Digest::SHA" !! "Digest::SHA::PurePerl"
    eval "require $MODULE" || die $^EVAL_ERROR
    $MODULE->import()



my @out = @: 
 "ungWv48Bz+pBQUDeXa4iI7ADYaOWF3qctBD/YfIAFa0"
 "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1"
    

plan tests => 6 + nelems @out

# attempt to use an invalid algorithm, and check for failure

my $NSA = "SHA-42"	# No Such Algorithm
ok( ! $MODULE->new($NSA) )

my $tempfile = "methods.tmp"
END { 1 while unlink $tempfile }

# test OO methods using first two SHA-256 vectors from NIST

my $fh = IO::File->new($tempfile, "w")
binmode($fh)
print $fh, "bcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
$fh->close

my $sha = $MODULE->new()->reset("SHA-256")->new()
$sha->add_bits("a", 5)->add_bits("001")

my $rsp = shift(@out)
is($sha->clone->add("b", "c")->b64digest, $rsp)

$rsp = shift(@out)

# test addfile with bareword filehandle

open(my $file, "<", "$tempfile")
binmode($file)
is($sha->clone->addfile(\$file->*)->hexdigest, $rsp)
close($file)

# test addfile with indirect filehandle

$fh = IO::File->new($tempfile, "r")
binmode($fh)
is($sha->clone->addfile($fh)->hexdigest, $rsp)
$fh->close

# test addfile using file name instead of handle

is($sha->addfile($tempfile, "b")->hexdigest, $rsp)

# test addfile portable mode

$fh = IO::File->new($tempfile, "w")
binmode($fh)
print $fh, "abc\012" x 2048		# using UNIX newline
$fh->close

is($sha->new(1)->addfile($tempfile, "p")->hexdigest,
   "d449e19c1b0b0c191294c8dc9fa2e4a6ff77fc51")

$fh = IO::File->new($tempfile, "w")
binmode($fh)
print $fh, "abc\015\012" x 2048		# using DOS/Windows newline
$fh->close

is($sha->new(1)->addfile($tempfile, "p")->hexdigest,
   "d449e19c1b0b0c191294c8dc9fa2e4a6ff77fc51")

$fh = IO::File->new($tempfile, "w")
binmode($fh)
print $fh, "abc\015" x 2048		# using early-Mac newline
$fh->close

is($sha->new(1)->addfile($tempfile, "p")->hexdigest,
   "d449e19c1b0b0c191294c8dc9fa2e4a6ff77fc51")
