package Encode::JP::JIS7
use warnings
our $VERSION = do { my @r = (@:  q$Revision: 2.3 $ =~ m/\d+/g ); sprintf "\%d." . "\%02d" x( nelems(@r)-1), < @r }

use Encode < qw(:fallbacks)

for my $name ((@:  '7bit-jis', 'iso-2022-jp', 'iso-2022-jp-1') )
    my $h2z     = ( $name eq '7bit-jis' )    ?? 0 !! 1
    my $jis0212 = ( $name eq 'iso-2022-jp' ) ?? 0 !! 1

    %Encode::Encoding{+$name} = bless \(%: 
        Name    => $name
        h2z     => $h2z
        jis0212 => $jis0212
        ) => __PACKAGE__


use base < qw(Encode::Encoding)

# we override this to 1 so PerlIO works
sub needs_lines { 1 }

use Encode::CJKConstants < qw(:all)

#
# decode is identical for all 2022 variants
#

sub decode
    my (@:  $obj, $str, $chk ) =  @_
    my $residue = ''
    if ($chk)
        $str =~ s/([^\x[00]-\x[7f]].*)$//so and $residue = $1
    
    $residue .= jis_euc( \$str )
    @_[1] = $residue if $chk
    return Encode::decode( 'euc-jp', $str, FB_PERLQQ )


#
# encode is different
#

sub encode
    require Encode::JP::H2Z
    my (@:  $obj, $utf8, $chk ) =  @_

    # empty the input string in the stack so perlio is ok
    @_[1] = '' if $chk
    my (@:  $h2z, $jis0212 ) =  $obj->{[qw(h2z jis0212)]}
    my $octet = Encode::encode( 'euc-jp', $utf8, $chk )
    $h2z and Encode::JP::H2Z::h2z( \$octet )
    euc_jis( \$octet, $jis0212 )
    return $octet


#
# cat_decode
#
my $re_scan_jis_g = qr{
   \G ( (%RE{?JIS_0212}) |  %RE{?JIS_0208}  |
        (%RE{?ISO_ASC})  | (%RE{?JIS_KANA}) | )
      ([^\e]*)
}x

sub cat_decode    # ($obj, $dst, $src, $pos, $trm, $chk)
    my (@:  $obj, _, _, $pos, $trm ) =  @_    # currently ignores $chk
    my (@:  $rdst, $rsrc, $rpos ) = \ <@_[[(@:  1, 2, 3) ]]
    local $^ENCODING = undef
    use bytes;
    my $opos = pos($rsrc->$)
    pos($rsrc->$, $pos)
    while ( $rsrc->$ =~ m/$re_scan_jis_g/gc )
        my (@:  $esc, $esc_0212, $esc_asc, $esc_kana, $chunk ) =
            @:  $1, $2, $3, $4, $5 

        unless ($chunk) { $esc or last; next; }

        if ( $esc && !$esc_asc )
            use bytes
            $chunk =~ s/([\x[21]-\x[7e]])/$( chr(ord($1)+0x80) )/g
            if ($esc_kana)
                $chunk =~ s/([\x[a1]-\x[df]])/\x[8e]$1/og
            elsif ($esc_0212)
                $chunk =~ s/([\x[a1]-\x[fe]][\x[a1]-\x[fe]])/\x[8f]$1/og
            
            $chunk = Encode::decode( 'euc-jp', $chunk, 0 )
        elsif ( ( my $npos = index( $chunk, $trm ) ) +>= 0 )
            $rdst->$ .= substr( $chunk, 0, $npos + length($trm) )
            $rpos->$ += length($esc) + $npos + length($trm)
            pos($rsrc->$, $opos)
            return 1
        
        $rdst->$ .= $chunk
        $rpos->$ = pos($rsrc->$)
    
    $rpos->$ = pos($rsrc->$)
    pos($rsrc->$, $opos)
    return ''


# JIS<->EUC
my $re_scan_jis = qr{
   (?:(%RE{?JIS_0212})|%RE{?JIS_0208}|(%RE{?ISO_ASC})|(%RE{?JIS_KANA}))([^\e]*)
}x

sub jis_euc
    local $^ENCODING = undef
    my $r_str = shift
    $r_str->$ =~ s($re_scan_jis)
     {$( do {
        my (@: $esc_0212, $esc_asc, $esc_kana, $chunk) =
        (@: $1, $2, $3, $4);
        if (!$esc_asc) {
            use bytes;
            $chunk =~ s/([\x[21]-\x[7e]])/$( chr(ord($1)+0x80) )/g;
            if ($esc_kana) {
                $chunk =~ s/([\x[a1]-\x[df]])/\x[8e]$1/og;
            }
            elsif ($esc_0212) {
                $chunk =~ s/([\x[a1]-\x[fe]][\x[a1]-\x[fe]])/\x[8f]$1/og;
            }
        }
        $chunk;
     })}gox
    my (@: $residue) = @:  $r_str->$ =~ s/(\e.*)$//so 
    return $residue


sub euc_jis
    no warnings < qw(uninitialized)
    use bytes
    my $r_str   = shift
    my $jis0212 = shift
    $r_str->$ =~ s{
    ((?:%RE{?EUC_C})+|(?:%RE{?EUC_KANA})+|(?:%RE{?EUC_0212})+)
    }{$( do {
        my $chunk = $1;
        my $esc =
        ( $chunk =~ s/\x[8E]//g ) ?? %ESC{?KANA} !!
        ( $chunk =~ s/\x[8F]//g ) ?? %ESC{?JIS_0212} !!
        %ESC{?JIS_0208};
        if ($esc eq %ESC{?JIS_0212} && !$jis0212){
            # fallback to '?'
            $chunk =~ s/[\x[A1]-\x[FE]]/[\x[3F]]/g;
        }else{
            $chunk =~ s/([\x[A1]-\x[FE]])/$( chr(ord($1)-0x80) )/g;
        }
        $esc . $chunk . %ESC{?ASC};
    })}gox
    $r_str->$ =~ s/\Q%ESC{?ASC}\E
        (\Q%ESC{?KANA}\E|\Q%ESC{?JIS_0212}\E|\Q%ESC{?JIS_0208}\E)/$1/gox
    $r_str->$


1
__END__


=head1 NAME

Encode::JP::JIS7 -- internally used by Encode::JP

=cut
