package Encode::KR::2022_KR
use warnings
our $VERSION = do { my @r = (@:  q$Revision: 2.2 $ =~ m/\d+/g ); sprintf "\%d." . "\%02d" x( nelems(@r)-1), < @r }

use Encode < qw(:fallbacks)

use base < qw(Encode::Encoding)
__PACKAGE__->Define('iso-2022-kr')

sub needs_lines { 1 }

sub perlio_ok
    return 0    # for the time being


sub decode
    my (@:  $obj, $str, $chk ) =  @_
    my $res     = $str
    my $residue = iso_euc( \$res )

    # This is for PerlIO
    @_[1] = $residue if $chk
    return Encode::decode( 'euc-kr', $res, FB_PERLQQ )


sub encode
    my (@:  $obj, $utf8, $chk ) =  @_

    # empty the input string in the stack so perlio is ok
    @_[1] = '' if $chk
    my $octet = Encode::encode( 'euc-kr', $utf8, FB_PERLQQ )
    euc_iso( \$octet )
    return $octet


use Encode::CJKConstants < qw(:all)

# ISO<->EUC

sub iso_euc
    my $r_str = shift
    $r_str->$ =~ s/%RE{?'2022_KR'}//gox    # remove the designator
    $r_str->$ =~ s{                      # replace characters in GL
     \x[0e]                              # between SO(\x[0e]) and SI(\x[0f])
     ([^\x[0f]]*)                        # with characters in GR
     \x[0f]
     }
     {$( do {
        my $out= $1;
        use bytes;
        $out =~ s/([\x[21]-\x[7e]])/$( chr(ord($1)+0x80) )/g;
        $out;
     })}gox
    my (@: $residue) = @:  $r_str->$ =~ s/(\e.*)$//so 
    return $residue


sub euc_iso
    no warnings < qw(uninitialized)
    my $r_str = shift
    substr( $r_str->$, 0, 0,
        %ESC{?'2022_KR'} )    # put the designator at the beg.
    $r_str->$ =~
        s{                         # move KS X 1001 characters in GR to GL
        (%RE{?EUC_C}+)                     # and enclose them with SO and SI
        }{$( do {
        my $str = $1;
        use bytes;
        $str =~ s/([\x[A1]-\x[FE]])/$( chr(ord($1)-0x80) )/g;
        "\x[0e]" . $str . "\x[0f]";
    })}gox
    $r_str->$


1
__END__

=head1 NAME

Encode::KR::2022_KR -- internally used by Encode::KR

=cut
