package Encode::MIME::Header::ISO_2022_JP

use warnings

use base < qw(Encode::MIME::Header)

%Encode::Encoding{+'MIME-Header-ISO_2022_JP'} =
    bless \(%:  encode => 'B', bpl => 76, Name => 'MIME-Header-ISO_2022_JP' ) =>
    __PACKAGE__

use constant HEAD => '=?ISO-2022-JP?B?'
use constant TAIL => '?='

use Encode::CJKConstants < qw(%RE)

our $VERSION = do { my @r = (@:  q$Revision: 1.3 $ =~ m/\d+/g ); sprintf "\%d." . "\%02d" x( nelems(@r)-1), < @r }

# I owe the below codes totally to
#   Jcode by Dan Kogai & http://www.din.or.jp/~ohzaki/perl.htm#JP_Base64

sub encode
    my $self = shift
    my $str  = shift

    Encode::from_to( $str, 'utf8', 'euc-jp' )

    my (@: ?$trailing_crlf) = @:  $str =~ m/(\n|\r|\x0d\x0a)$/o 

    $str = _mime_unstructured_header( $str, $self->{?bpl} )

    not $trailing_crlf and $str =~ s/(\n|\r|\x0d\x0a)$//o

    return $str


sub _mime_unstructured_header( $oldheader, $bpl)
    my $crlf = $oldheader =~ m/\n$/
    my $header = ''
    my (@words, @wordstmp, $i)

    $oldheader =~ s/\s+$//

    @wordstmp = split m/\s+/, $oldheader

    for my $i ( 0 .. nelems(@wordstmp)-2 )
        if (    @wordstmp[$i] !~ m/^[\x21-\x7E]+$/
            and @wordstmp[ $i + 1 ] !~ m/^[\x21-\x7E]+$/ )
            @wordstmp[ $i + 1 ] = "@wordstmp[$i] @wordstmp[$i + 1]"
        else
            push( @words, @wordstmp[$i] )
        
    

    push( @words, @wordstmp[-1] )

    for my $word ( @words)
        if ( $word =~ m/^[\x21-\x7E]+$/ )
            $header =~ m/(?:.*\n)*(.*)/
            if ( length($1) + length($word) +> $bpl )
                $header .= "\n $word"
            else
                $header .= $word
            
        else
            $header = _add_encoded_word( $word, $header, $bpl )
        

        $header =~ m/(?:.*\n)*(.*)/

        if ( length($1) == $bpl )
            $header .= "\n "
        else
            $header .= ' '
        
    

    $header =~ s/\n? $//mg

    $crlf ?? "$header\n" !! $header


sub _add_encoded_word( $str, $line, $bpl)
    my $result = ''

    while ( length($str) )
        my $target = $str
        $str = ''

        if (
              length($line) + 22 +
            ( $target =~ m/^(?:%RE{?EUC_0212}|%RE{?EUC_C})/o ) * 8 +> $bpl )
            $line =~ s/[ \t\n\r]*$/\n/
            $result .= $line
            $line = ' '
        

        while (1)
            my $iso_2022_jp = $target
            Encode::from_to( $iso_2022_jp, 'euc-jp', 'iso-2022-jp' )

            my $encoded =
                HEAD . MIME::Base64::encode_base64( $iso_2022_jp, '' ) . TAIL

            if ( length($encoded) + length($line) +> $bpl )
                $target =~
                    s/(%RE{?EUC_0212}|%RE{?EUC_KANA}|%RE{?EUC_C}|%RE{?ASCII})$//o
                $str = $1 . $str
            else
                $line .= $encoded
                last
            
        

    

    $result . $line


1
__END__

