BEGIN 
    $^OUTPUT_AUTOFLUSH = 1


use utf8
use Test::More tests => 778
use Encode
use utf8
use Encode::GSM0338

# The specification of GSM 03.38 is not awfully clear.
# (http://www.unicode.org/Public/MAPPINGS/ETSI/GSM0338.TXT)
# The various combinations of 0x00 and 0x1B as leading bytes
# are unclear, as is the semantics of those bytes as standalone
# or as final single bytes.


my $chk = Encode::LEAVE_SRC()

# escapes
# see http://www.csoft.co.uk/sms/character_sets/gsm.htm
my %esc_seq = %: 
    "\x{20ac}" => "\x[1b65]"
    "\x{0c}"     => "\x[1b0A]"
    "["        => "\x[1b3C]"
    "\\"       => "\x[1b2F]"
    "]"        => "\x[1b3E]"
    "^"        => "\x[1b14]"
    "\{"        => "\x[1b28]"
    "|"        => "\x[1b40]"
    "\}"        => "\x[1b29]"
    "~"        => "\x[1b3D]"
    

my %unesc_seq = %:  < reverse @:< %esc_seq


sub eu
    @_[0] =~ m/[\x00-\x1f]/ ??
        sprintf("\\x\{\%04X\}", ord(@_[0])) !! encode_utf8(@_[0])



for my $c ( map { chr }, 0 .. 127 )
    my $u = %Encode::GSM0338::GSM2UNI{?$c}

    # default character set
    is decode( "gsm0338", $c, $chk ), $u,
       sprintf( "decode \\x\%02X", ord($c) )
    try { decode( "gsm0338", $c . "\x{ff}", $chk ) }
    ok( $^EVAL_ERROR, "error with decode" )
    is encode( "gsm0338", $u, $chk ), $c, sprintf( "encode \%s", eu($u) )
    try { encode( "gsm0338", $u . "\x{3000}", $chk ) }
    ok( $^EVAL_ERROR, "error with encode" )

    # nasty atmark
    if ( $c eq "\x[00]" )
        is decode( "gsm0338", "\x[00]" . $c, $chk ), "\x[00]",
           sprintf( '@@ =>: \x00+\x%02X', ord($c) )
    else
        is decode( "gsm0338", "\x[00]" . $c ), '@' . decode( "gsm0338", $c ),
           sprintf( '@: decode \x00+\x%02X', ord($c) )
    

    # escape seq.
    my $ecs = "\x{1b}" . $c
    if ( %unesc_seq{?$ecs} )
        is decode( "gsm0338", $ecs, $chk ), %unesc_seq{?$ecs},
           sprintf( "ESC: decode ESC+\\x\%02X", ord($c) )
        is encode( "gsm0338", %unesc_seq{?$ecs}, $chk ), $ecs,
           sprintf( "ESC: encode \%s ", eu( %unesc_seq{?$ecs} ) )
    else
        is decode( "gsm0338", $ecs, $chk ),
           "\x{A0}" . decode( "gsm0338", $c ),
           sprintf( "decode ESC+\\x\%02X", ord($c) )
    


__END__
for my $c (map { chr } 0..127){
    my $b = "\x1b$c";
    my $u =  $Encode::GSM0338::GSM2UNI{$b};
    next unless $u;
    $u ||= "\xA0" . $Encode::GSM0338::GSM2UNI{$c};
    is decode("gsm0338", $b), $u, sprintf("decode ESC+\\x%02X", ord($c) );
}

__END__
# old test follows
ub t { is(decode("gsm0338", my $t = $_[0]), $_[1]) }

# t("\x00",     "\x00"); # ???

# "Round-trip".
t("\x41",     "\x41");

t("\x01",     "\xA3");
t("\x02",     "\x24");
t("\x03",     "\xA5");
t("\x09",     "\xE7");

t("\x00\x00", "\x00\x00"); # Maybe?
t("\x00\x1B", "\x40\xA0"); # Maybe?
t("\x00\x41", "\x40\x41");

# t("\x1B",     "\x1B"); # ???

# Escape with no special second byte is just a NBSP.
t("\x1B\x41", "\xA0\x41");

t("\x1B\x00", "\xA0\x40"); # Maybe?

# Special escape characters.
t("\x1B\x0A", "\x0C");
t("\x1B\x14", "\x5E");
t("\x1B\x28", "\x7B");
t("\x1B\x29", "\x7D");
t("\x1B\x2F", "\x5C");
t("\x1B\x3C", "\x5B");
t("\x1B\x3D", "\x7E");
t("\x1B\x3E", "\x5D");
t("\x1B\x40", "\x7C");
t("\x1B\x40", "\x7C");
t("\x1B\x65", "\x{20AC}");
