# Check that undeffing a pending event wont remove it from the event queue. It
# should remain scheduled. But then also check that this 'dangling' event
# is properly cleaned up.

package MyEvent
use Event::Lib
use base < qw/Event::Lib::event/

our @DESTROYS

sub new
    my $class = shift
    bless event_new(< @_) => $class


do
    my $count = 0
    sub DESTROY
        my $self = shift
        push @DESTROYS, ++$count
        no warnings 'misc';
        $self->SUPER::DESTROY
    


package MyTimer
use Event::Lib
use base < qw/Event::Lib::timer/

our @DESTROYS

sub new
    my $class = shift
    bless timer_new(< @_) => $class


do
    my $count = 0
    sub DESTROY
        my $self = shift
        push @DESTROYS, ++$count
        no warnings 'misc';
        $self->SUPER::DESTROY
    


package MySignal
use Event::Lib
use base < qw/Event::Lib::signal/

our @DESTROYS

sub new
    my $class = shift
    bless signal_new(< @_) => $class


do
    my $count = 0
    sub DESTROY
        my $self = shift
        push @DESTROYS, ++$count
        no warnings 'misc';
        $self->SUPER::DESTROY
    


package main

use IO::Socket::UNIX
use Event::Lib
use Test::More
use POSIX < qw/SIGHUP/

BEGIN { plan tests => 4 + 3 + 3};

do   # fh-event
    my $PATH = "t/sock-$^PID"
    if (fork)
        # PARENT
        my $sock = IO::Socket::UNIX->new(
            Local => $PATH,
            Listen => 1,
            ) or die "$^EVAL_ERROR"
        $sock->listen or die $^OS_ERROR

        my $e = MyEvent->new($sock, EV_READ, sub (@< @_) { ok(my $h = shift->fh->accept); ok(1); $h->close; })
        $e->add
        undef $e

        is(@MyEvent::DESTROYS[0], 1)
        event_mainloop
        is(@MyEvent::DESTROYS[1], 2)

        unlink $PATH
        wait
    else
        # CHILD
        sleep 2
        my $real_client = IO::Socket::UNIX->new(Peer => $PATH) or die $^EVAL_ERROR
        $real_client->close
        exit
    


do   # timer-event
    my $e = MyTimer->new(sub (@< @_) { ok(1); })
    $e->add(1)
    undef $e

    is(@MyTimer::DESTROYS[0], 1)
    event_mainloop
    is(@MyTimer::DESTROYS[1], 2)


do   # signal-event
    if (my $pid = fork)
        sleep 1
        kill SIGHUP => $pid
        wait
    else
        # CHILD
        my $e = MySignal->new(SIGHUP, sub (@< @_) { ok(1); shift->remove; })
        $e->add
        undef $e

        is(@MyTimer::DESTROYS[0], 1)
        event_mainloop
        is(@MyTimer::DESTROYS[1], 2)
        exit
    
    Test::Builder->new->current_test += 3

