# Objective:
# ----------
#
# Simply run and don't segfault.
# Previously events hidden in objects
# passed as additional arguments caused
# event-suicide.

use warnings

use IO::Socket::UNIX
use Event::Lib
use Test::More

BEGIN 
    plan tests => 8


my $PATH = "t/sock-$^PID"

my $pid = fork
die "couldn't fork: $^OS_ERROR" unless defined $pid

unless ($pid)
    # CHILD
    sleep 1
    for my $try (1..2)
        my $real_client = IO::Socket::UNIX->new(Peer => $PATH) or die $^EVAL_ERROR
        $real_client->autoflush(1)

        for (1..2)
            $real_client->syswrite("foobar")
            my $buf
            my $read = $real_client->sysread(\$buf, 1024)
            select undef, undef, undef, 0.5
        
        $real_client->syswrite("done", 4, 0)
        select undef, undef, undef, 0.5
    
    exit
else
    # PARENT
    my $sock = IO::Socket::UNIX->new(
        Local     => $PATH,
        Listen    => 1,
        ) or die "$^OS_ERROR"

    $sock->listen() or die $^OS_ERROR

    my $ctx = \%:  sock => $sock 

    my $e = event_new($sock, EV_READ^|^EV_PERSIST, &handle_incoming, $ctx)
    $ctx->{+incoming_event} = $e
    $e->add
    event_register_except_handler(&event_exception_handler)
    event_mainloop()
    wait


sub handle_incoming($e, $e_type, $ctx)

    $ctx->{+fh} = $ctx->{sock}->accept() or die $^OS_ERROR
    $ctx->{+read_event}  = event_new($ctx->{?fh}, EV_READ,  &do_read,  $ctx)
    $ctx->{+write_event} = event_new($ctx->{?fh}, EV_WRITE, &do_write, $ctx)

    $ctx->{read_event}->add


sub do_read($e, $e_type, $ctx)
    my $read = sysread($ctx->{?fh}, my $buf, 1024)
    if ($buf eq 'done')
        ok(1)
        die "please no more!"
    else
        is($buf, "foobar")
        $ctx->{write_event}->add
    


sub do_write($e, $e_type, $ctx)
    my $sent = syswrite($ctx->{?fh}, "ok", 2, 0)
    $ctx->{read_event}->add


my $cnt = 0
sub event_exception_handler($e, $err, $e_type, $ctx)
    $cnt++
    ok($err->message =~ m/please no more!/)
    if ($cnt == 2)
        unlink $PATH or die $^OS_ERROR
        exit
    

