# A read event kept in a member of an object and readded accordingly.  This
# test is essentially the same as t/62 only that the read-event is added
# directly and not via a timer event.

package A

use Test::More
use Event::Lib
use constant DELAY => 0.5

BEGIN 
    plan tests => 5


sub read_one_byte($event, _, $self)
    read($self->fh, my $char, 1)
    is($char, 'a')
    $self->count( $self->count - 1)
    if ($self->count)
        $self->my_event_add(EV_READ, &read_one_byte, $self->reader)
    


sub new($class, $fh, $iters)
    my $self = bless \(%: 
        _count  => undef
        _reader => undef
        _fh     => $fh
        ) => $class
    $self->reader( event_new($self->fh, EV_READ, &read_one_byte, $self) )
    $self->count( $iters )
    return $self


sub my_event_add($self, $e_type, $e_cb, $e)
    $e->add


sub run
    my $self = shift
    $self->my_event_add(EV_READ, &read_one_byte, $self->reader)
    event_mainloop


sub count
    my $self = shift
    $self->{+_count} = shift if nelems(@_)
    return $self->{?_count}


sub fh
    my $self = shift
    $self->{+_fh} = shift if nelems(@_)
    return $self->{?_fh}


sub reader
    my $self = shift
    $self->{+_reader} = shift if nelems(@_)
    return $self->{?_reader}


package main

use Event::Lib
use IO::Socket::UNIX

use constant CHARACTER_COUNT => 5

my $PATH = "t/sock-$^PID"

unless (fork)
    sleep 1
    my $client = IO::Socket::UNIX->new(Peer => $PATH) or die $^EVAL_ERROR
    $client->autoflush(1)
    for (1 .. CHARACTER_COUNT)
        $client->write('a')
else
    my $server = IO::Socket::UNIX->new(
        Local => $PATH,
        Listen => 1,
        ) or die $^EVAL_ERROR
    $server->listen or die $^OS_ERROR
    my $client = $server->accept
    my $proxy = A->new($client, CHARACTER_COUNT)->run
    $client->close
    unlink $PATH
    wait

    

