# Objective:
# ----------
#
# This script will trigger the destructor of still
# pending events and they should indeed be deleted and
# cleaned up.

use warnings

use IO::Socket::UNIX
use Event::Lib
use Test::More
use signals

BEGIN 
    plan tests => 6


my $PATH = "t/sock-$^PID"

my $pid = fork
die "couldn't fork: $^OS_ERROR" unless defined $pid

signals::handler("PIPE") = 'IGNORE'

unless ($pid)
    # CHILD
    sleep 1
    for my $try (1..2)
        my $real_client = IO::Socket::UNIX->new(Peer => $PATH, Blocking  => 0)
            or die "child: Can't open $PATH: $^EVAL_ERROR"
        $real_client->autoflush(1)

        for (1..3)
            $real_client->syswrite("foobar")
            my $buf
            my $read = $real_client->sysread(\$buf, 1024)
        
        $real_client->close
        select undef, undef, undef, 0.5
    
    exit
else
    # PARENT
    my $sock = IO::Socket::UNIX->new(
        Blocking  => 0,
        Local     => $PATH,
        Listen    => 1,
        ) or die "parent: Can't open $PATH: $^EVAL_ERROR"

    $sock->listen() or die $^OS_ERROR

    my $ctx = \%:  sock => $sock 

    my $e = event_new($sock, EV_READ^|^EV_PERSIST, &handle_incoming, $ctx)
    $ctx->{+incoming_event} = $e
    $e->add
    event_mainloop()
    wait


sub handle_incoming($e, $e_type, $ctx)

    $ctx->{fh}->close if exists $ctx->{fh}

    $ctx->{+fh} = $ctx->{sock}->accept() or die $^OS_ERROR

    $ctx->{+read_event}  = event_new($ctx->{?fh}, EV_READ,  &do_read,  $ctx)
    $ctx->{+write_event} = event_new($ctx->{?fh}, EV_WRITE, &do_write, $ctx)

    $ctx->{read_event}->add


my $ok = 0
sub do_read($e, $e_type, $ctx)
    my $buf
    my $read = sysread($ctx->{?fh}, $buf, 1024)
    if (defined $read)
        if ($read)
            $ok++
            is($buf, "foobar")
            $ctx->{write_event}->add
            exit if $ok == 6
        else
            return
        
    else
        return
    


sub do_write($e, $e_type, $ctx)
    my $sent = syswrite($ctx->{?fh}, "ok", 2, 0)
    unless (defined $sent)
        return
    
    $ctx->{read_event}->add


END 
    unlink $PATH

