# Objective:
# ----------
#
# Test Event::Lib::base::args() and ::args_del()

use Test::More
BEGIN { plan tests => 13 }
use Event::Lib

do # read-only access
    sub handler1($ev, $evtype, @< @args)
        is(($ev->args)[0], @args[0])
        is(($ev->args)[9_999], @args[-1])
    

    my $e = timer_new(&handler1, < 1 .. 10_000)
    is(($e->args)[0], 1)
    is(($e->args)[9_999], 10_000)
    $e->add(0.5)

    event_mainloop



do # write access: deleting args
    sub handler2($ev, $evtype, @< @args)
        is_deeply($ev->args, $@)
        is(nelems(@args), 0)
    

    my $e = timer_new(&handler2, < 1 .. 10_000)
    $e->args_del
    is_deeply($e->args, $@)
    $e->add(0.5)

    event_mainloop


do # write access: replace args
    sub handler3($ev, $evtype, @< @args)
        is( join("", $ev->args), join("", 23 .. 42) )
        is( join("", $ev->args), join("", @args) )
    

    my $e = timer_new(&handler3, < 1 .. 10_000)
    $e->args( < 23 .. 42 )
    is( join("", $e->args), join("", 23 .. 42 ) )
    $e->add(0.5)

    event_mainloop


do # write access: delete args (Lib.xs wont yet free the array of SV*'s),

    my $e = timer_new(&handler3, < 1 .. 10_000)
    $e->args_del
    $e->args( < 23 .. 42)
    is( join("", (@:  < $e->args)), join("", 23 .. 42 ) )
    $e->add(0.5)

    event_mainloop

