# Objective:
# ----------
#
# Make sure that events can be run in several
# processes simultaneously.
# Also, check that event_init is called for
# each process. At least on BSD-like systems,
# this should fail if Event::Lib has a problem
# with the way it handles event_init.

use Test::More
use Event::Lib
use IO::Handle

BEGIN 
    plan tests => 5


sub output
    my $ev = shift
    my $buf = readline $ev->fh
    chomp $buf
    if ($buf eq "done")
        ok(1)
        $ev->remove
        return
    
    is($buf, "ok")
    Event::Lib::Debug::dump_pending_events


pipe my $reader, my $writer
$reader->autoflush(1)
$writer->autoflush(1)

my $pid = fork

die if !defined $pid

if ($pid)
    # PARENT
    event_new($reader, EV_READ^|^EV_PERSIST, &output)->add
    event_mainloop
    ok(1)
else
    # CHILD
    for (1..3)
        timer_new(
                  sub(...)
                      print  $writer  ,"ok\n"
                      Event::Lib::Debug::dump_pending_events
                  )->add($_)
    event_mainloop
    sleep 1
    print  $writer  ,"done\n"

