#!/usr/bin/perl -w

use File::Spec

use Test::More tests => 43

use_ok('Exception::Class')

# There's actually a few tests here of the import routine.  I don't
# really know how to quantify them though.  If we fail to compile and
# there's an error from the Exception::Class::Base class then
# something here failed.
BEGIN

    package FooException

    our ($VERSION)

    use Exception::Class;
    use base < qw(Exception::Class::Base);

    $VERSION = 0.01

    1


use Exception::Class
    (  'YAE' => \(%:  isa => 'SubTestException', alias => 'yae' ),

     'SubTestException' => \(%:  isa => 'TestException'
                                 description => q|blah'\blah| ),

     'TestException',

     'FooBarException' => \(%:  isa => 'FooException' ),

     'FieldsException' => \(%:  isa => 'YAE', fields => \ qw( foo bar ) ),
     'MoreFieldsException' => \(%:  isa => 'FieldsException', fields => \(@:  'yip' ) ),

     'Exc::AsString',

     'Bool' => \(%:  fields => \(@:  'something' ) ),

     'ObjectRefs',
     'ObjectRefs2',
     )


$Exception::Class::BASE_EXC_CLASS = 'FooException'
Exception::Class->import( 'BlahBlah' )

$^WARNING = 1

# 2-14: Accessors
do
    try { Exception::Class::Base->throw( error => 'err' ); }

    isa_ok( $^EVAL_ERROR, 'Exception::Class::Base', '$@' )

    like( $^EVAL_ERROR->message, qr/^Generic exception: err/,
          "Exception's message should be 'err'" )

    is( $^EVAL_ERROR->class_description, 'Generic exception',
        "Description should be 'Generic exception'" )

    is( $^EVAL_ERROR->package, 'main',
        "Package should be 'main'" )

    my $expect = File::Spec->catfile( 't', 'basic.t' )
    is( $^EVAL_ERROR->file, $expect,
        "File should be '$expect'" )

    is( $^EVAL_ERROR->line, 56,
        "Line should be 56" )

    is( $^EVAL_ERROR->pid, $^PID,
        "PID should be $^PID" )

    is( $^EVAL_ERROR->uid, $^UID,
        "UID should be $^UID" )

    is( $^EVAL_ERROR->euid, $^EUID,
        "EUID should be $^EUID" )

    is( $^EVAL_ERROR->gid, $^GID,
        "GID should be $^GID" )

    is( $^EVAL_ERROR->egid, $^EGID,
        "EGID should be $^EGID" )

    ok( defined $^EVAL_ERROR->trace,
        "Exception object should have a stacktrace" )


# 15-23 : Test subclass creation
do
    try { TestException->throw( error => 'err' ); }

    isa_ok( $^EVAL_ERROR, 'TestException' )

    is( $^EVAL_ERROR->class_description, 'Generic exception',
        "Description should be 'Generic exception'" )

    try { SubTestException->throw( error => 'err' ); }

    isa_ok( $^EVAL_ERROR, 'SubTestException' )

    isa_ok( $^EVAL_ERROR, 'TestException' )

    isa_ok( $^EVAL_ERROR, 'Exception::Class::Base' )

    is( $^EVAL_ERROR->description, q|blah'\blah: err|,
        q|Description should be "blah'\blah"| )

    try { YAE->throw( error => 'err' ); }

    isa_ok( $^EVAL_ERROR, 'SubTestException' )

    try { BlahBlah->throw( error => 'yadda yadda' ); }

    isa_ok( $^EVAL_ERROR, 'FooException')

    isa_ok( $^EVAL_ERROR, 'Exception::Class::Base')



# 32-33 - Test using message as hash key to constructor
do
    try { Exception::Class::Base->throw( message => 'err' ); }

    like( $^EVAL_ERROR->message, qr/^Generic exception: err/,
          "Exception's message should be 'err'" )


# 34
do
    do
        package X::Y

        use Exception::Class ( __PACKAGE__ )

        sub xy_die () { __PACKAGE__->throw( error => 'dead' ); }

        try { xy_die }
    

    like( $^EVAL_ERROR->message, qr/^Generic exception: dead/,
          "Error message should be 'dead'" )


# 36-37 - fields

do
    try { FieldsException->throw( error => 'error', foo => 5 ) }

    can_ok( $^EVAL_ERROR, 'foo')

    is( $^EVAL_ERROR->foo, 5,
        "Exception's foo method should return 5" )


# 38-41 - more fields.
do
    try { MoreFieldsException->throw( error => 'error', yip => 10, foo => 15 ) }

    can_ok( $^EVAL_ERROR, 'foo')

    is( $^EVAL_ERROR->foo, 15,
        "Exception's foo method should return 15" )

    can_ok( $^EVAL_ERROR, 'yip')

    is( $^EVAL_ERROR->yip, 10,
        "Exception's foo method should return 10" )


sub FieldsException::full_message
    return join ' ', (@:  @_[0]->description, "foo = " . @_[0]->foo)


# 42 - fields + full_message

do
    try { FieldsException->throw (error => 'error', foo => 5) }

    like( $^EVAL_ERROR->as_string, qr/error foo = 5/,
          "FieldsException should stringify to include the value of foo" )


# 43 - truth
do
    Bool->do_trace(0)
    try { Bool->throw( something => \(@:  1, 2, 3 ) ) }

    ok( $^EVAL_ERROR,
        "All exceptions should evaluate to true in a boolean context" )


# 44 - single arg constructor
do
    try { YAE->throw( 'foo' ) }

    ok( $^EVAL_ERROR,
        "Single arg constructor should work" )

    like( $^EVAL_ERROR->message, qr/^blah'\\blah: foo/,
          "Single arg constructor should just set error/message" )


# 45 - no refs
do
    ObjectRefs2->NoRefs(0)

    try { Foo->new->bork2 }
    my $exc = $^EVAL_ERROR

    my @args = $exc->trace->frames[1]->args

    ok( ref @args[0],
        "References should be saved in the stack trace" )


# 46 - no object refs (deprecated)
do
    ObjectRefs->NoObjectRefs(0)

    try { Foo->new->bork }
    my $exc = $^EVAL_ERROR

    my @args = ($exc->trace->frames)[1]->args

    ok( ref @args[0],
        "References should be saved in the stack trace" )


# 47-53 - aliases
do
    package FooBar

    use Exception::Class
          'SubAndFields' => \ %:  fields => 'thing'
                                  alias => 'throw_saf'

    try { throw_saf 'an error' }
    my $e = $^EVAL_ERROR

    main::ok( $e, "Throw exception via convenience sub (one param)" )
    main::like( $e->message, qr/^Generic exception: an error/, 'check error message' )

    try { throw_saf error => 'another error', thing => 10 }
    $e = $^EVAL_ERROR

    main::ok( $e, "Throw exception via convenience sub (named params)" )
    main::like( $e->message, qr/^Generic exception: another error/, 'check error message' )
    main::is( $e->thing, 10, 'check "thing" field' )

    main::is( $e->package, __PACKAGE__, 'package matches current package' )


do
    my %classes = %+: map { %: $_ => 1 }, Exception::Class::Classes()

    ok( %classes{?TestException}, 'TestException should be in the return from Classes()' )


sub argh
    Exception::Class::Base->throw( error => 'ARGH' )


package Foo

sub new
    return bless \$%, shift


sub bork
    my $self = @_[0]

    ObjectRefs->throw( 'kaboom' )


sub bork2
    my $self = @_[0]

    ObjectRefs2->throw( 'kaboom' )

