#!/usr/bin/perl -w

use Test::More tests => 4

package Foo

sub foo
    my $class = shift

    TestException->throw(< @_) unless $class eq 'Foo'

    Bar->bar(< @_)


package Bar

sub bar
    shift
    Baz->baz(< @_)


package Baz

our (@ISA)
@ISA = @:  < qw(Foo) 

sub baz
    shift->foo(< @_)


package main

use Exception::Class < qw(TestException)


sub check_trace( $trace, $unwanted_pkg, $unwanted_class)

    my($bad_frame)
    while ( my $frame = $trace->next_frame )
        if ( (grep { $frame->package eq $_ }, (@:  < $unwanted_pkg->@))
             || (grep { UNIVERSAL::isa( $frame->package, $_ ) }, (@:  < $unwanted_class->@)))
            $bad_frame = $frame
            last
        
    

    ok( ! $bad_frame, "Check for unwanted frames" )
    diag( "Unwanted frame found: $bad_frame" )
        if $bad_frame


try { Foo->foo() }
my $e = $^EVAL_ERROR

check_trace( $e->trace, \$@, \$@ )

try { Foo->foo( ignore_package => \(@:  'Baz' ) ) }
$e = $^EVAL_ERROR

check_trace( $e->trace, \(@:  'Baz' ), \$@ )

try { Foo->foo( ignore_class => \(@:  'Foo' ) ) }
$e = $^EVAL_ERROR

check_trace( $e->trace, \$@, \(@:  'Foo' ) )

try { Foo->foo( ignore_package => \(@:  'Foo', 'Baz' ) ) }
$e = $^EVAL_ERROR

check_trace($e->trace, \(@:  'Foo', 'Baz' ), \$@)

