#!/usr/bin/perl -w

use lib < qw(t/lib)

use Test::More tests => 28

BEGIN { use_ok('Exporter::Lite'); }
can_ok(__PACKAGE__, 'import')

do
    package Test1
    use Dummy

    main::can_ok('Dummy', 'import')
    main::ok( exists &foo,     '@EXPORT' )
    main::is( foo, 42,          '    in one piece' )
    main::is( $foo, 'foofer',   '    and variables' )


do
    package YATest1
    use Dummy < qw(foo)

    main::ok( exists &foo,     '@EXPORT with explicit request' )
    main::is( foo, 42,          '    in one piece' )


do
    package Test2
    use Dummy ()

    main::ok( !exists &foo,    'import with ()' )


do
    package Test3
    try { Dummy->import('car') }
    main::like( $^EVAL_ERROR->{?description}, '/"car" is not exported by the Dummy module/',
                'importing non-expoted function' )



do
    package Test4
    use Dummy < qw(bar)
    main::ok( exists &bar,     '@EXPORT_OK' )
    main::ok( !exists &foo,    '    overrides @EXPORT' )
    main::ok( !exists &my_sum, '    only what we asked for from @EXPORT_OK' )
    main::is( bar, 23,          '    not damaged in transport' )


do
    package YATest4
    use Dummy < qw(bar $bar)
    main::ok( exists &bar,     '@EXPORT_OK' )
    main::ok( !exists &foo,    '    overrides @EXPORT' )
    main::ok( !exists &my_sum, '    only what we asked for from @EXPORT_OK' )
    main::is( bar, 23,          '    not damaged in transport' )
    main::is( $bar, 'barfer',   '    $bar exported' )


do
    package Test5

    my $warning = ''
    local $^WARN_HOOK = sub (@< @_) { $warning = join '', @_ }
    eval 'use Dummy < qw(bar)'
    eval 'use Dummy < qw(&bar)'
    main::ok( exists &bar,     'importing multiple times' )
    main::is( $^EVAL_ERROR, '',           '   no errors' )
    main::is( $warning, '',     '   no warnings' )


do
    package Test6

    my $warning = ''
    local $^WARN_HOOK = sub (@< @_) { $warning = join '', @_ }
    eval 'use Dummy < qw(bar &bar bar bar &bar bar)'
    main::ok( exists &bar,     'importing duplicates' )
    main::is( $^EVAL_ERROR, '',           '   no errors' )
    main::is( $warning, '',     '   no warnings' )


do
    package Test7
    use Dummy < qw(my_sum bar foo)

    my @list =  qw(1 2 3 4)
    my $sum = my_sum sub (@< @_) { @_[0] + @_[1] }, < @list
    main::is( $sum, 10, '    and it works' )



do
    package Test8
    eval "use Dummy v0.5"
    main::is( $^EVAL_ERROR, '',         'use Foo VERSION' )

    eval "use Dummy v99"
    main::like( $^EVAL_ERROR->{?description}, '/Dummy version v99.* required/',
                'use with version check' )

