#!/usr/bin/perl


BEGIN 
    $^OUTPUT_AUTOFLUSH = 1
    $^WARNING = 1


use Test::More <   qw(no_plan) # tests => 2;
use File::Remove < qw{ remove trash }





# Set up the tests

my @dirs = @: "$^PROGRAM_NAME.tmp", < map { "$^PROGRAM_NAME.tmp/$_" }, qw(a a/b c c/d e e/f g)

for my $path (reverse @dirs)
    if ( -e $path )
        ok( rmdir($path), "rmdir: $path" )
        ok( !-e $path,    "!-e: $path"   )
    


for my $path ( @dirs )
    ok( ! -e $path,   "!-e: $path"   )
    ok( mkdir($path), "mkdir: $path" )
    chmod 0777, $path
    ok( -e $path,     "-e: $path"    )


for my $path (reverse @dirs)
    ok( -e $path,     "-e: $path"    )
    ok( rmdir($path), "rmdir: $path" )
    ok( !-e $path,    "!-e: $path"   )


for my $path ( @dirs )
    ok( ! -e $path,   "!-e: $path"   )
    ok( mkdir($path), "mkdir: $path" )
    chmod 0777, $path
    ok( -e $path,     "-e: $path"    )


for my $path (reverse @dirs)
    ok( -e $path,          "-e: $path"         )
    ok( remove(\1, $path), "remove \\1: $path" )
    ok( !-e $path,         "!-e: $path"        )


for my $path (@dirs)
    ok( !-e $path,    "!-e: $path"   )
    ok( mkdir($path), "mkdir: $path" )
    chmod 0777, $path
    ok( -e $path,     "-e: $path"    )


for my $path (reverse @dirs)
    ok( -e $path,      "-e: $path"     )
    ok( remove($path), "remove: $path" )
    ok( !-e $path,     "!-e: $path"    )


for my $path (reverse @dirs)
    ok( !-e $path, "-e: $path" )
    if (-e _)
        ok( rmdir($path), "rmdir: $path" )
        ok( !-e $path,    "!-e: $path"   )
    


SKIP: do
    if ($^OS_NAME eq 'darwin')
        eval 'use Mac::Glue ();'
        skip "Undelete support requires Mac::Glue", 0 if length $^EVAL_ERROR
        eval 'use Mac::Glue::Finder ();'
        skip "Undelete support requires Mac::Glue::Finder", 0 if length $^EVAL_ERROR
    elsif ($^OS_NAME eq 'cygwin' || $^OS_NAME =~ m/^MSWin/)
        eval 'use Win32::FileOp::Recycle;'
        skip "Undelete support requires Win32::FileOp::Recycle", 0 if length $^EVAL_ERROR
    else
        skip "Undelete support not available by default", 0
    

    for my $path (@dirs)
        ok( !-e $path,    "!-e: $path"   )
        ok( mkdir($path), "mkdir: $path" )
        chmod 0777, $path
        ok( -e $path,     "-e: $path"    )
    

    for my $path (reverse @dirs)
        ok( -e $path,              "-e: $path"    )
        ok( try { trash($path) }, "trash: $path" )
        is( $^EVAL_ERROR, '',                "trash: \$@"   )
        ok( !-e $path,             "!-e: $path"   )
    

    for my $path (reverse @dirs)
        ok( !-e $path, "-e: $path" )
        if (-e _)
            ok( rmdir($path), "rmdir: $path" )
            ok( !-e $path,    "!-e: $path"   )
        
    

    for my $path (@dirs)
        ok( !-e $path,    "!-e: $path"   )
        ok( mkdir($path), "mkdir: $path" )
        chmod 0777, $path
        ok( -e $path,     "-e: $path"    )
    

    for my $path (reverse @dirs)
        ok( -e $path,      "-e: $path"     )
        ok( remove($path), "remove: $path" )
        ok( !-e $path,     "!-e: $path"    )
    

    for my $path (reverse @dirs)
        ok( !-e $path, "-e: $path" )
        if (-e _)
            ok( rmdir($path), "rmdir: $path" )
            ok( !-e $path,    "!-e: $path"   )
        
    

    for my $path (@dirs)
        ok( !-e $path,    "!-e: $path"   )
        ok( mkdir($path), "mkdir: $path" )
        chmod 0777, $path
        ok( -e $path,     "-e: $path"    )
    

    for my $path (reverse @dirs)
        ok( -e $path, "-e: $path"        )
        ok(
           trash(\(%:  'rmdir' => { 1 }, 'unlink' => { 1 } ), $path),
           "trash: $path",
           )
        ok( -e $path, "-e: $path"        )
        ok( rmdir($path), "rmdir: $path" )
        ok( !-e $path, "!-e: $path"      )
    


1
