#!perl -w

use HTML::Parser
use utf8
require bytes
use Test::More tests => 89


my @warn
$^WARN_HOOK = sub (@< @_)
    push(@warn, @_[0])


my @parsed
my $p = HTML::Parser->new(
    api_version => 3,
    default_h => \(@: \@parsed, 'event, text, dtext, offset, length, offset_end, column, tokenpos, attr'),
    )

my $doc = "<title>\x{263A}</title><h1 id=\x{2600} f>Smile &#x263a</h1>\x{0420}"
is(length($doc), 46)

$p->parse($doc)->eof

#use Data::Dump; Data::Dump::dump(@parsed);

is(nelems(@parsed), 9)
is(@parsed[0]->[0], "start_document")

is(@parsed[1]->[0], "start")
is(@parsed[1]->[1], "<title>")
is(@parsed[1]->[3], 0)
is(@parsed[1]->[4], 7)

is(@parsed[2]->[0], "text")
is(ord(@parsed[2]->[1]), 0x263A)
is(@parsed[2]->[2], chr(0x263A))
is(@parsed[2]->[3], 7)
is(@parsed[2]->[4], 3)
is(@parsed[2]->[5], 10)
is(@parsed[2]->[6], 7)

is(@parsed[3]->[0], "end")
is(@parsed[3]->[1], "</title>")
is(@parsed[3]->[3], 10)
is(@parsed[3]->[6], 10)

is(@parsed[4]->[0], "start")
is(@parsed[4]->[1], "<h1 id=\x{2600} f>")
is(join("|", @parsed[4]->[7]->@), "1|2|4|2|7|3|11|1|0|0")
is(@parsed[4]->[8]->{?id}, "\x{2600}")

is(@parsed[5]->[0], "text")
is(@parsed[5]->[1], "Smile &#x263a")
is(@parsed[5]->[2], "Smile \x{263A}")

is(@parsed[7]->[0], "text")
is(@parsed[7]->[1], "\x{0420}")

is(@parsed[8]->[0], "end_document")
is(@parsed[8]->[3], bytes::length($doc))
is(@parsed[8]->[5], bytes::length($doc))
is(@parsed[8]->[6], bytes::length($doc))
is(nelems(@warn), 0)

# Try to parse it as an UTF8 encoded string
use Encode
$doc = Encode::decode('latin1', $doc)
is(utf8::length($doc), 51)

@parsed = $@
$p->parse($doc)->eof

#use Data::Dump; Data::Dump::dump(@parsed);

is(nelems(@parsed), 9)
is(@parsed[0]->[0], "start_document")

is(@parsed[1]->[0], "start")
is(@parsed[1]->[1], "<title>")
is(@parsed[1]->[3], 0)
is(@parsed[1]->[4], 7)

is(@parsed[2]->[0], "text")
is(ord(@parsed[2]->[1]), 226)
is(@parsed[2]->[1], "\x{E2}\x{98}\x{BA}")
is(@parsed[2]->[2], "\x{E2}\x{98}\x{BA}")
is(@parsed[2]->[3], 7)
is(@parsed[2]->[4], 6)
is(@parsed[2]->[5], 13)
is(@parsed[2]->[6], 7)

is(@parsed[3]->[0], "end")
is(@parsed[3]->[1], "</title>")
is(@parsed[3]->[3], 13)
is(@parsed[3]->[6], 13)

is(@parsed[4]->[0], "start")
is(@parsed[4]->[1], "<h1 id=\x{E2}\x{98}\x{80} f>")
is(join("|", @parsed[4]->[7]->@), "1|2|4|2|7|6|14|1|0|0")
is(@parsed[4]->[8]->{?id}, "\x{E2}\x{98}\x{80}")

is(@parsed[5]->[0], "text")
is(@parsed[5]->[1], "Smile &#x263a")
is(@parsed[5]->[2], "Smile \x{263A}")

is(@parsed[7]->[0], "text")
is(@parsed[7]->[1], "\x[c3]\x[90]\x[c2]\x[a0]")

is(@parsed[8]->[0], "end_document")
is(@parsed[8]->[3], bytes::length($doc))
is(@parsed[8]->[5], bytes::length($doc))
is(@parsed[8]->[6], bytes::length($doc))

is(nelems(@warn), 0)

my $file = "test-$^PID.html"
open(my $fh, ">:utf8", $file) || die
print $fh, <<EOT
\x{FEFF}
<title>\x{263A} Love! </title>
<h1 id=&hearts;\x{2665}>&hearts; Love \x{2665}<h1>
EOT
close($fh) || die

@warn = $@
@parsed = $@
$p->parse_file($file)
is(nelems(@parsed), "11")
is(@parsed[6]->[0], "start")
is(@parsed[6]->[8]->{?id}, "\x{2665}\x[E2]\x[99]\x[A5]")
is(@parsed[7]->[0], "text")
is(@parsed[7]->[1], "&hearts; Love \x[E2]\x[99]\x[A5]")
is(@parsed[7]->[2], "\x{2665} Love \x[E2]\x[99]\x[A5]")  # expected garbage
is(@parsed[10]->[3], -s $file)
is(nelems(@warn), 0)

@warn = $@
@parsed = $@
open($fh, "<:raw:utf8", $file) || die
$p->parse_file($fh)
is(nelems(@parsed), "11")
is(@parsed[6]->[0], "start")
is(@parsed[6]->[8]->{?id}, "\x{2665}\x{2665}")
is(@parsed[7]->[0], "text")
is(@parsed[7]->[1], "&hearts; Love \x{2665}")
is(@parsed[7]->[2], "\x{2665} Love \x{2665}")
is(@parsed[10]->[3], (-s $file))
is(nelems(@warn), 0)

@warn = $@
@parsed = $@
open($fh, "<:raw", $file) || die
$p->parse_file($fh)
is(nelems(@parsed), "11")
is(@parsed[6]->[0], "start")
is(@parsed[6]->[8]->{?id}, "\x[E2]\x[99]\x[A5]\x[E2]\x[99]\x[A5]")
is(@parsed[7]->[0], "text")
is(@parsed[7]->[1], "&hearts; Love \x[E2]\x[99]\x[A5]")
is(@parsed[7]->[2], "\x[E2]\x[99]\x[A5] Love \x[E2]\x[99]\x[A5]")
is(@parsed[10]->[3], -s $file)
is(nelems(@warn), 0)

unlink($file)
