#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#ifdef SVf_IVisUV
#  define slu_sv_value(sv) (SvIOK(sv)) ? (SvIOK_UV(sv)) ? (NV)(SvUVX(sv)) : (NV)(SvIV(sv)) : (SvNV(sv))
#else
#  define slu_sv_value(sv) (SvIOK(sv)) ? (NV)(SvIVX(sv)) : (SvNV(sv))
#endif

#ifndef Drand01
#    define Drand01()           ((rand() & 0x7FFF) / (double) ((unsigned long)1 << 15))
#endif

#ifndef PTR2UV
#  define PTR2UV(ptr) (UV)(ptr)
#endif

#ifndef SvPV_nolen
    STRLEN N_A;
#   define SvPV_nolen(sv) SvPV(sv, N_A)
#endif

#define WARN_OFF \
    SV *oldwarn = PL_curcop->cop_warnings; \
    PL_curcop->cop_warnings = pWARN_NONE;

#define WARN_ON \
    PL_curcop->cop_warnings = oldwarn;

#define EACH_ARRAY_BODY \
    register int i;                                                     \
    arrayeach_args * args;                                              \
    HV *stash = gv_stashpv("List::MoreUtils_ea", TRUE);                 \
    CV *closure = newXS(NULL, XS_List__MoreUtils__array_iterator, __FILE__); \
                                                                        \
    New(0, args, 1, arrayeach_args);                                    \
    New(0, args->avs, items, AV*);                                      \
    args->navs = items;                                                 \
    args->curidx = 0;                                                   \
                                                                        \
    for (i = 0; i < items; i++) {                                       \
        if ( ! SvAVOK(ST(i)) )                                       \
            Perl_croak(aTHX_ "Expected a ARRAY got %s", Ddesc(ST(i))); \
        args->avs[i] = svTav(ST(i));                          \
	AvREFCNT_inc(args->avs[i]);                                     \
    }                                                                   \
                                                                        \
    CvXSUBANY(closure).any_ptr = args;                                  \
    RETVAL = (SV*)closure; 

#define BOOL_CALL_SV \
        bool bool_res; \
        ENTER; \
        SAVETMPS; \
        PUSHMARK(SP); \
        XPUSHs(args[i]); \
        PUTBACK; \
        bool_res = SvTRUE( call_sv(code, G_SCALAR) ); \
        SPAGAIN; \
        FREETMPS; \
        LEAVE;

/* #include "dhash.h" */

/* need this one for array_each() */
typedef struct {
    AV **avs;	    /* arrays over which to iterate in parallel */
    int navs;	    /* number of arrays */
    int curidx;	    /* the current index of the iterator */
} arrayeach_args;

/* used for natatime */
typedef struct {
    SV **svs;
    int nsvs;
    int curidx;
    int natatime;
} natatime_args;

void
insert_after (int idx, SV *what, AV *av) {
    register int i, len;
    av_extend(av, (len = av_len(av) + 1));
    
    for (i = len; i > idx+1; i--) {
	SV **sv = av_fetch(av, i-1, FALSE);
	SvREFCNT_inc(*sv);
	av_store(av, i, *sv);
    }
    if (!av_store(av, idx+1, what))
	SvREFCNT_dec(what);

}
   
MODULE = List::MoreUtils		PACKAGE = List::MoreUtils		

void
any (code,...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    HV *stash;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	XSRETURN_UNDEF;

    for(i = 1 ; i < items ; ++i) {
        BOOL_CALL_SV;
	if (bool_res) {
	    XSRETURN_YES;
	}
    }
    XSRETURN_NO;
}

void
all (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	XSRETURN_UNDEF;

    for(i = 1 ; i < items ; i++) {
        bool res;
        ENTER;
        SAVETMPS;
        PUSHMARK(SP);
        XPUSHs(args[i]);
        PUTBACK;
        res = SvTRUE( call_sv(code, G_SCALAR) );
        SPAGAIN;
        FREETMPS;
        LEAVE;
	if (!res) {
	    XSRETURN_NO;
	}
    }
    XSRETURN_YES;
}


void
none (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	XSRETURN_UNDEF;

    for(i = 1 ; i < items ; ++i) {
        BOOL_CALL_SV;
	if (bool_res) {
	    XSRETURN_NO;
	}
    }
    XSRETURN_YES;
}

void
notall (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	XSRETURN_UNDEF;

    for(i = 1 ; i < items ; ++i) {
        bool res;
        ENTER;
        SAVETMPS;
        PUSHMARK(SP);
        XPUSHs(args[i]);
        PUTBACK;
        res = SvTRUE( call_sv(code, G_SCALAR) );
        SPAGAIN;
        FREETMPS;
        LEAVE;
	if (!res) {
	    XSRETURN_YES;
	}
    }
    XSRETURN_NO;
}

int
true (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    I32 count = 0;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	goto done;

    for(i = 1 ; i < items ; ++i) {
        BOOL_CALL_SV;
	if (bool_res)
	    count++;
    }

    done:
    RETVAL = count;
}
OUTPUT:
    RETVAL

int
false (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    I32 count = 0;
    SV **args = &PL_stack_base[ax];

    if (items <= 1)
	goto done;

    for(i = 1 ; i < items ; ++i) {
        BOOL_CALL_SV;
	if (!bool_res)
	    count++;
    }

    done:
    RETVAL = count;
}
OUTPUT:
    RETVAL

int
firstidx (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    RETVAL = -1;
    
    if (items > 1) {
	for (i = 1 ; i < items ; ++i) {
            BOOL_CALL_SV;
	    if (bool_res) {
		RETVAL = i-1;
		break;
	    }
	}
    }
}
OUTPUT:
    RETVAL

int
lastidx (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    RETVAL = -1;
    
    if (items > 1) {
	for (i = items-1 ; i > 0 ; --i) {
            BOOL_CALL_SV;
	    if (bool_res) {
		RETVAL = i-1;
		break;
	    }
	}
    }
}
OUTPUT:
    RETVAL

int
insert_after (code, val, avref)
    SV *code;
    SV *val;
    SV *avref;
PROTOTYPE: &$\@
CODE:
{
    register int i;
    AV* av;
    int len;

    if ( ! SvRVOK(avref) )
        Perl_croak(aTHX_ "Expected a ARRAY REF got %s", Ddesc(avref));
    if ( ! SvAVOK(SvRV(avref)) )
        Perl_croak(aTHX_ "Expected a ARRAY REF got %s REF", Ddesc(SvRV(avref)));
    av = svTav(SvRV(avref));
    len = av_len(av);
    RETVAL = 0;
    
    for (i = 0; i <= len ; ++i) {
        bool res;
        ENTER;
        PUSHMARK(SP);
        XPUSHs(*av_fetch(av, i, FALSE));
        PUTBACK;
        res = SvTRUE(call_sv(code, G_SCALAR));
        LEAVE;
	if (res) {
	    RETVAL = 1;
	    break;
	}
    }
    
    if (RETVAL) {
	SvREFCNT_inc(val);
	insert_after(i, val, av);
    }
}
OUTPUT:
    RETVAL

int
insert_after_string (string, val, avref)
	SV *string;
	SV *val;
	SV *avref;
    PROTOTYPE: $$\@
    CODE:
    {
	register int i;
	AV *av;
	int len;
	register SV **sv;
	STRLEN slen = 0, alen;
	register char *str;
	register char *astr;
	RETVAL = 0;
	
        if ( ! SvRVOK(avref) )
            Perl_croak(aTHX_ "Expected a ARRAY REF got %s", Ddesc(avref));
        if ( ! SvAVOK(SvRV(avref)) )
            Perl_croak(aTHX_ "Expected a ARRAY REF got %s REF", Ddesc(SvRV(avref)));
        av = svTav(SvRV(avref));
        len = av_len(av);

	if (SvTRUE(string))
	    str = SvPV(string, slen);
	else 
	    str = NULL;
	    
	for (i = 0; i <= len ; i++) {
	    sv = av_fetch(av, i, FALSE);
	    if (SvTRUE(*sv))
		astr = SvPV(*sv, alen); 
	    else {
		astr = NULL;
		alen = 0;
	    }
	    if (slen == alen && memcmp(astr, str, slen) == 0) {
		RETVAL = 1;
		break;
	    }
	}
	if (RETVAL) {
	    SvREFCNT_inc(val);
	    insert_after(i, val, av);
	}

    }
    OUTPUT:
	RETVAL

void
after (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i, j;
    SV **args = &PL_stack_base[ax];
    AV* res;

    if (items <= 1)
	XSRETURN_EMPTY;

    for (i = 1; i < items; i++) {
        BOOL_CALL_SV;
	if (bool_res) {
	    break;
	}
    }

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));
    for (j = i + 1; j < items; ++j)
	av_push(res, SvREFCNT_inc(args[j]));

    XSRETURN(1);
}

void
after_incl (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i, j;
    SV **args = &PL_stack_base[ax];
    AV *res;

    if (items <= 1)
	XSRETURN_EMPTY;

    for (i = 1; i < items; i++) {
        BOOL_CALL_SV;
	if (bool_res) {
	    break;
	}
    }

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));
    for (j = i; j < items; j++)
	av_push(res, SvREFCNT_inc(args[j]));

    XSRETURN(1);
}

void
before (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];
    AV *res;
    
    if (items <= 1)
	XSRETURN_EMPTY;

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));

    for (i = 1; i < items; i++) {
        BOOL_CALL_SV;
	if (bool_res) {
	    break;
	}
        av_push(res, SvREFCNT_inc(args[i]));
    }

    XSRETURN(1);
}

void
before_incl (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];
    AV *res;

    if (items <= 1)
	XSRETURN_EMPTY;

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));

    for (i = 1; i < items; ++i) {
        BOOL_CALL_SV;
        av_push(res, SvREFCNT_inc(args[i]));
	if (bool_res) {
	    ++i;
	    break;
	}
    }

    XSRETURN(1);
}

void
indexes (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];
    AV *res;

    if (items <= 1)
	XSRETURN_EMPTY;

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));

    for (i = 1; i < items; i++) {
        BOOL_CALL_SV;
	if (bool_res) {
            av_push(res, newSViv(i-1));
	}
    }
    
    XSRETURN(1);
}

SV *
lastval (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];	

    RETVAL = &PL_sv_undef;
    
    if (items > 1) {
	for (i = items-1 ; i > 0 ; --i) {
            BOOL_CALL_SV;
	    if (bool_res) {
		/* see comment in indexes() */
		SvREFCNT_inc(RETVAL = args[i]);
		break;
	    }
	}
    }
}
OUTPUT:
    RETVAL

SV *
firstval (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i;
    SV **args = &PL_stack_base[ax];

    RETVAL = &PL_sv_undef;
    
    if (items > 1) {
	for (i = 1; i < items; ++i) {
            BOOL_CALL_SV;
	    if (bool_res) {
		/* see comment in indexes() */
		SvREFCNT_inc(RETVAL = args[i]);
		break;
	    }
	}
    }
}
OUTPUT:
    RETVAL

void
_array_iterator (method = "")
    char *method;
    PROTOTYPE: ;$
    CODE:
    {
	register int i;
	int exhausted = 1;
        AV* res;
	
	/* 'cv' is the hidden argument with which XS_List__MoreUtils__array_iterator (this XSUB)
	 * is called. The closure_arg struct is stored in this CV. */
#define ME_MYSELF_AND_I cv
	
	arrayeach_args *args = (arrayeach_args*)CvXSUBANY(ME_MYSELF_AND_I).any_ptr;
	
	if (strEQ(method, "index")) {
	    EXTEND(SP, 1);
	    ST(0) = args->curidx > 0 ? sv_2mortal(newSViv(args->curidx-1)) : &PL_sv_undef;
	    XSRETURN(1);
	}
    
        res = newAV();
        mXPUSHs(avTsv(res));

	for (i = 0; i < args->navs; i++) {
	    AV *av = args->avs[i];
	    if (args->curidx <= av_len(av)) {
                av_push(res, newSVsv(*av_fetch(av, args->curidx, FALSE)));
		exhausted = 0;
		continue;
	    }
            av_push(res, SvREFCNT_inc(&PL_sv_undef));
	}
	
	if (exhausted) 
	    XSRETURN_EMPTY;

	args->curidx++;
	XSRETURN(1);
    }

SV *
each_array (...)
    PROTOTYPE: \@;\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@
    CODE:
    {
	EACH_ARRAY_BODY;
    }
    OUTPUT:
	RETVAL

SV *
each_arrayref (...)
    CODE:
    {
	EACH_ARRAY_BODY;
    }
    OUTPUT:
	RETVAL

void
pairwise (code, ...)
	SV *code;
    PROTOTYPE: &\@\@
    PPCODE:
    {
#define av_items(a) (av_len(a)+1)
	
	register int i;
	AV *avs[2];
	SV **oldsp;
        AV *res;
	
	int nitems = 0, maxitems = 0;

        res = av_2mortal(newAV());

	/* deref AV's for convenience and 
	 * get maximum items */
        if ( ! SvAVOK(ST(1)) )
            Perl_croak(aTHX_ "Expected array got %s", Ddesc(ST(1)));
        if ( ! SvAVOK(ST(2)) )
            Perl_croak(aTHX_ "Expected array got %s", Ddesc(ST(2)));
	avs[0] = svTav(ST(1));
	avs[1] = svTav(ST(2));
	maxitems = av_items(avs[0]);
	if (av_items(avs[1]) > maxitems)
	    maxitems = av_items(avs[1]);
	
        SAVESPTR(PL_firstgv);
        SAVESPTR(PL_secondgv);
        GVcpREPLACE(PL_firstgv, gv_fetchpv("a", TRUE, SVt_PV));
        GVcpREPLACE(PL_secondgv, gv_fetchpv("b", TRUE, SVt_PV));
        SAVESPTR(GvSV(PL_firstgv));
        SAVESPTR(GvSV(PL_secondgv));
	
	oldsp = PL_stack_base;
	ENTER;
	for (i = 0; i < maxitems; i++) {
	    SV* ret;
	    SV **svp = av_fetch(avs[0], i, FALSE);
	    SVcpREPLACE( GvSV(PL_firstgv), svp ? *svp : &PL_sv_undef );
	    svp = av_fetch(avs[1], i, FALSE);
	    SVcpREPLACE( GvSV(PL_secondgv), svp ? *svp : &PL_sv_undef );
	    PUSHMARK(SP);
	    PUTBACK;
	    ret = call_sv(code, G_SCALAR);
	    SPAGAIN;
            av_push(res, newSVsv(ret));
	}
	PL_stack_base = oldsp;
	LEAVE;
        ST(0) = avTsv(res);
	XSRETURN(1);
    }

void
_natatime_iterator ()
    PROTOTYPE:
    CODE:
    {
	register int i;
	int nret;
        AV *res;

	/* 'cv' is the hidden argument with which XS_List__MoreUtils__array_iterator (this XSUB)
	 * is called. The closure_arg struct is stored in this CV. */
#define ME_MYSELF_AND_I cv
	
	natatime_args *args = (natatime_args*)CvXSUBANY(ME_MYSELF_AND_I).any_ptr;
	
        res = newAV();
        mXPUSHs(avTsv(res));

	for (i = 0; i < args->natatime; i++) {
	    if (args->nsvs) {
                av_push(res, newSVsv(args->svs[args->curidx++]));
		args->nsvs--;
	    }
	    else {
		XSRETURN(1);
	    }
	}

	XSRETURN(1);
    }

SV *
natatime (n, ...)
    int n;
    PROTOTYPE: $@
    CODE:
    {
	register int i;
	natatime_args * args;
	HV *stash = gv_stashpv("List::MoreUtils_na", TRUE);

	CV *closure = newXS(NULL, XS_List__MoreUtils__natatime_iterator, __FILE__);

	/* must NOT set prototype on iterator:
	 * otherwise one cannot write: &$it */
	/* !! sv_setpv((SV*)closure, ""); !! */

	New(0, args, 1, natatime_args);
	New(0, args->svs, items-1, SV*);
	args->nsvs = items-1;
	args->curidx = 0;
	args->natatime = n;

	for (i = 1; i < items; i++) 
	    SvREFCNT_inc(args->svs[i-1] = ST(i));
	
	CvXSUBANY(closure).any_ptr = args;
	RETVAL = newRV_noinc((SV*)closure);

	/* in order to allow proper cleanup in DESTROY-handler */
	sv_bless(RETVAL, stash);    
    }
    OUTPUT:
	RETVAL

void
mesh (...)
    PROTOTYPE: \@\@;\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@
    CODE:
    {
	register int i, j, maxidx = -1;
	AV **avs;
        AV *res;
	New(0, avs, items, AV*);
	
	for (i = 0; i < items; i++) {
            if ( ! SvAVOK(ST(i)) )
                Perl_croak(aTHX_ "mesh expected a ARRAY got %s", Ddesc(ST(i)));
	    avs[i] = svTav(ST(i));
	    if (av_len(avs[i]) > maxidx)
		maxidx = av_len(avs[i]);
	}

        res = newAV();
        ST(0) = sv_2mortal(avTsv(res));

	for (i = 0; i <= maxidx; i++) 
	    for (j = 0; j < items; j++) {
		SV **svp = av_fetch(avs[j], i, FALSE);
		av_push(res, svp ? newSVsv(*svp) : SvREFCNT_inc(&PL_sv_undef));
	    }

	Safefree(avs);
	XSRETURN(1);
    }

void
uniq (...)
    PROTOTYPE: @
    CODE:
    {
	register int i, count = 0;
	HV *hv = hv_2mortal(newHV());
        AV *res;
	
        res = newAV();

	/* list context: populate SP with mortal copies */
	for (i = 0; i < items; i++) {
	    if (!hv_exists_ent(hv, ST(i), 0)) {
		av_push(res, newSVsv(ST(i)));
		count++;
		hv_store_ent(hv, ST(i), &PL_sv_yes, 0);
	    }
	}
        ST(0) = sv_2mortal(avTsv(res));
	XSRETURN(1);
    }

void
minmax (...)
    PROTOTYPE: @
    CODE:
    {
	register int i;
	register SV *minsv, *maxsv, *asv, *bsv;
	register double min, max, a, b;
        AV* res;
	
	if (!items)
	    XSRETURN_EMPTY;

	minsv = maxsv = ST(0);
	min = max = slu_sv_value(minsv);

	for (i = 1; i < items; i += 2) {
	    asv = ST(i-1);
	    bsv = ST(i);
	    a = slu_sv_value(asv);
	    b = slu_sv_value(bsv);
	    if (a <= b) {
		if (min > a) {
		    min = a;
		    minsv = asv;
		}
		if (max < b) {
		    max = b;
		    maxsv = bsv;
		}
	    } else {
		if (min > b) {
		    min = b;
		    minsv = bsv;
		}
		if (max < a) {
		    max = a;
		    maxsv = asv;
		}
	    }
	}

	if (items & 1) {
	    asv = ST(items-2);
	    bsv = ST(items-1);
	    a = slu_sv_value(asv);
	    b = slu_sv_value(bsv);
	    if (a <= b) {
		if (min > a) {
		    min = a;
		    minsv = asv;
		}
		if (max < b) {
		    max = b;
		    maxsv = bsv;
		}
	    } else {
		if (min > b) {
		    min = b;
		    minsv = bsv;
		}
		if (max < a) {
		    max = a;
		    maxsv = asv;
		}
	    }
	}
        res = newAV();
	ST(0) = sv_2mortal(avTsv(res));
        av_push(res, SvREFCNT_inc(minsv));
        av_push(res, SvREFCNT_inc(maxsv));

	XSRETURN(1);
    }

void
part (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    register int i, j;
    HV *stash;
    GV *gv;
    I32 gimme = G_SCALAR;
    I32 count = 0;
    SV **args = &PL_stack_base[ax];
    CV *cv;
    
    AV **tmp = NULL;
    int last = 0;
    AV *res;
    
    if (items == 1)
	XSRETURN_EMPTY;

    cv = sv_2cv(code, &gv, 0);
    ENTER;

    for(i = 1 ; i < items ; ++i) {
	int idx;
        PUSHMARK(SP);
        mXPUSHs(SvREFCNT_inc(args[i]));
        PUTBACK;
        idx = SvIV( call_sv(cvTsv(cv), G_SCALAR) );
        SPAGAIN;

	if (idx < 0 && (idx += last) < 0)
	    croak("Modification of non-creatable array value attempted, subscript %i", idx);

	if (idx >= last) {
	    int oldlast = last;
	    last = idx + 1;
	    Renew(tmp, last, AV*);
	    Zero(tmp + oldlast, last - oldlast, AV*);
	}
	if (!tmp[idx])
	    tmp[idx] = newAV();
	av_push(tmp[idx], args[i]);
	SvREFCNT_inc(args[i]);
    }
    LEAVE;

    res = newAV();
    ST(0) = sv_2mortal(avTsv(res));

    for (i = 0; i < last; ++i) {
	if (!tmp[i]) {
	    av_push(res, &PL_sv_undef);
	    continue;
	}
	av_push(res, newRV_noinc((SV*)tmp[i]));
    }
    
    Safefree(tmp);
    XSRETURN(1);
}

#if 0
void
part_dhash (code, ...)
    SV *code;
PROTOTYPE: &@
CODE:
{
    /* We might want to keep this dhash-implementation.
     * It is currently slower than the above but it uses less
     * memory for sparse parts such as 
     *   @part = part { 10_000_000 } 1 .. 100_000;
     * Maybe there's a way to optimize dhash.h to get more speed
     * from it.
     */
    dMULTICALL;
    register int i, j, lastidx = -1;
    int max;
    HV *stash;
    GV *gv;
    I32 gimme = G_SCALAR;
    I32 count = 0;
    SV **args = &PL_stack_base[ax];
    CV *cv;

    dhash_t *h = dhash_init();

    if (items == 1)
	XSRETURN_EMPTY;

    cv = sv_2cv(code, &gv, 0);
    PUSH_MULTICALL(cv);
    SAVESPTR(GvSV(PL_defgv));

    for(i = 1 ; i < items ; ++i) {
	int idx;
        SVcpREPLACE(GvSV(PL_defgv), args[i]);
	MULTICALL;
	idx = SvIV(*PL_stack_sp);

	if (idx < 0 && (idx += h->max) < 0)
	    croak("Modification of non-creatable array value attempted, subscript %i", idx);

	dhash_store(h, idx, args[i]);
    }
    POP_MULTICALL;

    dhash_sort_final(h);
    
    EXTEND(SP, max = h->max+1);
    i = 0;
    lastidx = -1;
    while (i < h->count) {
	int retidx = h->ary[i].key;
	int fill = retidx - lastidx - 1;
	for (j = 0; j < fill; j++) {
	    ST(retidx - j - 1) = &PL_sv_undef;
	}
	ST(retidx) = newRV_noinc((SV*)h->ary[i].val);
	i++;
	lastidx = retidx;
    }
    
    dhash_destroy(h);
    XSRETURN(max);
}

#endif

void
_XScompiled ()
    CODE:
	XSRETURN_YES;


MODULE = List::MoreUtils                PACKAGE = List::MoreUtils_ea

void
DESTROY(sv)
    SV *sv;
    CODE:
    {
	register int i;
	CV *code = (CV*)SvRV(sv);
	arrayeach_args *args = CvXSUBANY(code).any_ptr;
	if (args) {
	    for (i = 0; i < args->navs; ++i)
		AvREFCNT_dec(args->avs[i]);
	    Safefree(args->avs);
	    Safefree(args);
	    CvXSUBANY(code).any_ptr = NULL;
	}
    }


MODULE = List::MoreUtils                PACKAGE = List::MoreUtils_na

void
DESTROY(sv)
    SV *sv;
    CODE:
    {
	register int i;
	CV *code = (CV*)SvRV(sv);
	natatime_args *args = CvXSUBANY(code).any_ptr;
	if (args) {
	    for (i = 0; i < args->nsvs; ++i)
		SvREFCNT_dec(args->svs[i]);
	    Safefree(args->svs);
	    Safefree(args);
	    CvXSUBANY(code).any_ptr = NULL;
	}
    }

