#!/usr/bin/perl -w

use warnings
use Test::More
use File::Spec

use MIME::Tools
use MIME::Decoder

#------------------------------------------------------------
# BEGIN
#------------------------------------------------------------

# Is gzip available?  Quick and dirty test:
my $has_gzip
foreach (split $^OS_NAME eq "MSWin32" ?? ';' !! ':', env::var('PATH'))
    last if ($has_gzip = -x "$_/gzip")

if ($has_gzip)
    require MIME::Decoder::Gzip64
    MIME::Decoder::Gzip64->install( 'x-gzip64')


# Get list of encodings we think we provide:
my @encodings = @: 'base64'
                   'quoted-printable'
                   '7bit'
                   '8bit'
                   'binary'
                   ($has_gzip ?? 'x-gzip64' !! ())
                   'x-uuencode'
                   'binhex'

plan( tests => scalar nelems(@encodings))

# Report what tests we may be skipping:
diag($has_gzip
     ?? "Using gzip: $has_gzip"
     !! "No gzip: skipping x-gzip64 test")

# Test each encoding in turn:
my $eno = 0
foreach my $e ( @encodings)
    ++$eno
    my $decoder = MIME::Decoder->new($e)
    unless(defined($decoder))
        pass("Encoding/decoding of $e not supported -- skipping test")
        next
    

    my $infile  = File::Spec->catfile('.', 'testin', 'fun.txt')
    my $encfile = File::Spec->catfile('.', 'testout', "fun.en$eno")
    my $decfile = File::Spec->catfile('.', 'testout', "fun.de$eno")

    # Encode:
    open my $in, "<", "$infile" or die "open $infile: $^OS_ERROR"
    open my $out, ">", "$encfile" or die "open $encfile: $^OS_ERROR"
    binmode $in; binmode $out
    $decoder->encode(\$in->*, \$out->*) or next
    close $out
    close $in

    # Decode:
    open $in, "<", "$encfile" or die "open $encfile: $^OS_ERROR"
    open $out, ">", "$decfile" or die "open $decfile: $^OS_ERROR"
    binmode $in; binmode $out
    $decoder->decode(\$in->*, \$out->*) or next
    close $out
    close $in

    # Can we compare?
    if ($e =~ m/^(binhex|base64|quoted-printable|binary|x-gzip64|x-uuencode)$/i)
        is(-s $infile, -s $decfile, "Encoding/decoding of $e: size of $infile == size of $decfile")
    else
        pass("Encoding/decoding of $e: size not comparable, marking pass anyway")
    

