#!/usr/bin/perl

use lib '.'
use lib 't'
our ($in_fh);
use SATest; sa_t_init("line_endings")
use Test::More tests => 26

# ---------------------------------------------------------------------------

# Use a slightly modified gtube ...
my $origtest = 'data/spam/gtube.eml'
my $test = 'log/report_safe.eml'
my $test2 = 'log/report_safe2.eml'
my $original = ''
my ($otest_fh, $test_fh, $test2_fh)
if (open($otest_fh, "<", $origtest) && open($test_fh, ">", "$test") && open($test2_fh, ">", "$test2"))
    binmode $otest_fh
    binmode $test_fh
    binmode $test2_fh

    local $^INPUT_RECORD_SEPARATOR=undef
    $original .= "X-Spam-Prev-Subject: this is a test\n"
    $original .= "X-Spam-Status: No, this should fail horribly!\n"
    $original .= ~< $otest_fh->*
    $original =~ s/\r*\n/\r\n/gs; print $test_fh, $original
    $original =~ s/\r*\n/\n/gs; print $test2_fh, $original

    close($test_fh)
    close($test2_fh)
    close($otest_fh)
else
    die "can't open input files: $^OS_ERROR"


my $description = 'original message before SpamAssassin'
my $disposition = 'inline'
my $message

my $resulttext
my $count_crnl
my $count_nl

our %patterns = %: 'X-Spam-Status: Yes' => 'XSS_Yes'
our %anti_patterns = %: $original => 'P_1', 'X-Spam-Status: No' => 'XSS_No', q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 0\n")
sarun ("-L < $test", &my_patterns_run)
ok_all_patterns()

# support one *or* the other, depending on platform
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)


$message = safe(our $boundary, '', 'message/rfc822', $description, 'inline')
our %patterns = %: $message => 'P_2', 'X-Spam-Status: Yes' => 'XSS_Yes'
our %anti_patterns = %:  q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 1\n")
sarun ("-L < $test", &my_patterns_run)
ok_all_patterns()
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)


$message = safe($boundary, '', 'text/plain', $description, 'inline')
%patterns = %: $message => 'P_3', 'X-Spam-Status: Yes' => 'XSS_Yes'
%anti_patterns = %:  q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 2\n")
sarun ("-L < $test", &my_patterns_run)
ok_all_patterns()
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)


# now with the other line-ending style...

%patterns = %: 'X-Spam-Status: Yes' => 'XSS_Yes'
%anti_patterns = %: $original => 'P_1', 'X-Spam-Status: No' => 'XSS_No', q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 0\n")
sarun ("-L < $test2", &my_patterns_run)
ok_all_patterns()

# support one *or* the other, depending on platform
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)


$message = safe($boundary, '', 'message/rfc822', $description, 'inline')
%patterns = %: $message => 'P_2', 'X-Spam-Status: Yes' => 'XSS_Yes'
%anti_patterns = %:  q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 1\n")
sarun ("-L < $test2", &my_patterns_run)
ok_all_patterns()
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)


$message = safe($boundary, '', 'text/plain', $description, 'inline')
%patterns = %: $message => 'P_3', 'X-Spam-Status: Yes' => 'XSS_Yes'
%anti_patterns = %:  q{ MISSING_HB_SEP }, 'hb_sep'
tstprefs ("report_safe 2\n")
sarun ("-L < $test2", &my_patterns_run)
ok_all_patterns()
count_line_endings($resulttext)
if ($count_crnl)
    ok ($count_crnl!=0 && $count_nl==0)
else
    ok ($count_crnl==0 && $count_nl!=0)

exit

# ---------------------------------------------------------------------------

# unfortunately, we cannot use a regexp since quotemeta is used on patterns
sub safe($boundary, $report_charset, $type, $description, $disposition)

    return <<"EOM"
Content-Type: $type; x-spam-type=original
Content-Description: $description
Content-Disposition: $disposition
Content-Transfer-Encoding: 8bit

$original
EOM


sub my_patterns_run
    $resulttext = join ('', (@:  ~< $in_fh->*))
    return patterns_run_cb($resulttext)


sub count_line_endings
    $count_crnl = 0
    $count_nl = 0
    foreach my $line (split(m/\n/s, shift))
        if ($line =~ m/\r$/)
            $count_crnl++
        else
            $count_nl++
        
    
    print $^STDOUT, "line endings found: NL=$count_nl CRNL=$count_crnl\n"

