#!/usr/bin/perl

use lib '.'; use lib 't';
use SATest; sa_t_init("spamd_prefork_stress_4");
use Test::More;
use signals;

our $RUN_THIS_TEST;
our ($testname, $spamd_stderr);

BEGIN {
  $RUN_THIS_TEST = conf_bool('run_spamd_prefork_stress_test');
  plan skip_all => 'skip spamd tests' if our $SKIP_SPAMD_TESTS;
  plan skip_all => 'do not run this test' unless our $RUN_THIS_TEST;
  plan tests => 43;
};

print $^STDOUT, "NOTE: this test requires 'run_spamd_prefork_stress_test' set to 'y'.\n";

# ---------------------------------------------------------------------------

# tstlocalrules ('
        # loadplugin myTestPlugin ../../data/testplugin.pm
        # header PLUGIN_SLEEP eval:sleep_based_on_header()
# ');


our %patterns = %(

q{ X-Spam-Status: Yes, score=}, 'status',
q{ X-Spam-Flag: YES}, 'flag',
q{ X-Spam-Level: **********}, 'stars',
q{ TEST_ENDSNUMS}, 'endsinnums',
q{ TEST_NOREALNAME}, 'noreal',

);

my $tmpnum = 0;
start_spamd("-L -m10");
ok (our $spamd_pid +> 1);

srand ($^PID); print $^STDOUT, "srand: $^PID\n";

ok (spamcrun ("< data/spam/001", \&patterns_run_cb));
ok_all_patterns();

test_spamc(); ok_all_patterns();

my @bgpids = @( () );

signals::handler("INT") = sub {
  kill 15, < @bgpids;
  die "interrupted";
};

foreach my $i (0 .. 20) {
  my $pid = fork();

  if ($pid) {
    push @bgpids, $pid;
    print $^STDOUT, "forked $pid\n";

  } else {
    $tmpnum = ($^PID * 100);
    $testname .= ".pid$^PID";

    my $nummsgs = 5 + ($^PID % 9);
    
    print $^STDOUT, "pid $^PID: starting, will send $nummsgs msgs\n";

    for my $j (1 .. $nummsgs) {
      select(undef, undef, undef, 0.25 + rand(10));
      if (!test_without_ok()) {
        die "pid $^PID failed on spamc run";
      }
      if (!ok_all_patterns(1)) {
        die "pid $^PID failed on results check";
      }
    }

    print $^STDOUT, "pid $^PID: done\n";
    exit 0;
  }
}

foreach my $pid ( @bgpids) {
  print $^STDOUT, "wait for $pid\n";
  waitpid($pid, 0);
  my $ex = ($^CHILD_ERROR >> 8);
  print $^STDOUT, "$pid exited with status: ".$ex."\n";
  ok ($ex == 0);
}

test_spamc(); ok_all_patterns();
ok (stop_spamd());

# now search for errors
my $failed = 0;

ok (open (my $in_fh, "<", "$($spamd_stderr)"));
while ( ~< *$in_fh) {
  m/prefork: ordered child \S+ to accept, but/ and $failed++;
  m/prefork: killing failed child/ and $failed++;
  m/syswrite.. to parent failed/ and $failed++;
}
close $in_fh;

ok (!$failed);



sub test_without_ok {
  clear_pattern_counters();
  print $^STDOUT, "pid $^PID: running spamc\n";
  return (spamcrun ("<data/spam/001", \&patterns_run_cb));
}

sub test_spamc {
  clear_pattern_counters();
  ok (spamcrun ("<data/spam/001", \&patterns_run_cb));
}



