#!/usr/bin/perl -w

# test rounding, accuracy, precicion and fallback, round_mode and mixing
# of classes

use Test::More

BEGIN
  
    $^OUTPUT_AUTOFLUSH = 1
    # to locate the testing files
    my $location = $^PROGRAM_NAME; $location =~ s/mbimbf.t//i
    if (-d 't')
        chdir 't'
        require File::Spec
        unshift $^INCLUDE_PATH, File::Spec->catdir(File::Spec->updir, $location)
    else
        unshift $^INCLUDE_PATH, $location
    
    print $^STDOUT, "# INC = $(join ' ',$^INCLUDE_PATH)\n"

    plan tests => 697


use Math::BigInt v1.70
use Math::BigFloat v1.43

our ($mbi, $mbf)

$mbi = 'Math::BigInt'
$mbf = 'Math::BigFloat'

require 'mbimbf.inc'

# some tests that won't work with subclasses, since the things are only
# garantied in the Math::BigInt/BigFloat (unless subclass chooses to support
# this)

Math::BigInt->round_mode('even')	# reset for tests
Math::BigFloat->round_mode('even')	# reset for tests

my $x = eval '$mbi->round_mode("huhmbi");'
print $^STDOUT, "# Got '$^EVAL_ERROR'\n" unless
    ok ($^EVAL_ERROR->{?description} =~ m/^Unknown round mode 'huhmbi' at/)

$x = eval '$mbf->round_mode("huhmbf");'
print $^STDOUT, "# Got '$^EVAL_ERROR'\n" unless
    ok ($^EVAL_ERROR->{?description} =~ m/^Unknown round mode 'huhmbf' at/)

# see if accessor also changes old variable
$mbi->round_mode('odd')
$mbf->round_mode('odd')

foreach my $class (qw/Math::BigInt Math::BigFloat/)
    is ($class->accuracy(5),5)		# set A
    is ($class->precision(), undef)	# and now P must be cleared
    is ($class->precision(5),5)		# set P
    is ($class->accuracy(), undef)	# and now A must be cleared


foreach my $class (qw/Math::BigInt Math::BigFloat/)
    $class->accuracy(42)
    my $x = $class->new(123)	# $x gets A of 42, too!
    is ($x->accuracy(),42)	# really?
    is ($x->accuracy(undef),42)	# $x has no A, but the
    # global is still in effect for $x
    # so the return value of that operation should
    # be 42, not undef
    is ($x->accuracy(),42)	# so $x should still have A = 42
    $class->accuracy(undef)	# reset for further tests
    $class->precision(undef)

# bug with flog(Math::BigFloat,Math::BigInt)
$x = Math::BigFloat->new(100)
$x = $x->blog(Math::BigInt->new(10))

is ($x->bstr,2)
