#!/usr/bin/perl -w

# test rounding with non-integer A and P parameters

use Test::More

BEGIN
  
    $^OUTPUT_AUTOFLUSH = 1
    # to locate the testing files
    my $location = $^PROGRAM_NAME; $location =~ s/round.t//i
    unshift $^INCLUDE_PATH, < qw(../lib)
    if (-d 't')
        chdir 't'
        require File::Spec
        unshift $^INCLUDE_PATH, File::Spec->catdir(File::Spec->updir, $location)
    else
        unshift $^INCLUDE_PATH, $location
    
    print $^STDOUT, "# INC = $(join ' ',$^INCLUDE_PATH)\n"

    plan tests => 95


use Math::BigFloat

my $cf = 'Math::BigFloat'
my $ci = 'Math::BigInt'

my $x = $cf->new('123456.123456')

# unary ops with A
_do_a($x, 'round', 3, '123000')
_do_a($x, 'bfround', 3, '123500')
_do_a($x, 'bfround', 2, '123460')
_do_a($x, 'bfround', -2, '123456.12')
_do_a($x, 'bfround', -3, '123456.123')

_do_a($x, 'bround', 4, '123500')
_do_a($x, 'bround', 3, '123000')
_do_a($x, 'bround', 2, '120000')

_do_a($x, 'bsqrt', 4, '351.4')
_do_a($x, 'bsqrt', 3, '351')
_do_a($x, 'bsqrt', 2, '350')

# setting P
_do_p($x, 'bsqrt', 2,  '350')
_do_p($x, 'bsqrt', -2, '351.36')

# binary ops
_do_2_a($x, 'bdiv', 2, 6, '61728.1')
_do_2_a($x, 'bdiv', 2, 4, '61730')
_do_2_a($x, 'bdiv', 2, 3, '61700')

_do_2_p($x, 'bdiv', 2, -6, '61728.061728')
_do_2_p($x, 'bdiv', 2, -4, '61728.0617')
_do_2_p($x, 'bdiv', 2, -3, '61728.062')

# all tests done

#############################################################################

sub _do_a($x, $method, $A, $result)

    is ($x->copy->?$method($A)->bstr, $result, "$method($A)")
    is ($x->copy->?$method($A.'.1')->bstr, $result, "$method($($A).1)")
    is ($x->copy->?$method($A.'.5')->bstr, $result, "$method($($A).5)")
    is ($x->copy->?$method($A.'.6')->bstr, $result, "$method($($A).6)")
    is ($x->copy->?$method($A.'.9')->bstr, $result, "$method($($A).9)")


sub _do_p($x, $method, $P, $result)

    is ($x->copy->?$method(undef,$P)->bstr, $result, "$method(undef,$P)")
    is ($x->copy->?$method(undef,$P.'.1')->bstr, $result, "$method(undef,$($P).1)")
    is ($x->copy->?$method(undef,$P.'.5')->bstr, $result, "$method(undef.$($P).5)")
    is ($x->copy->?$method(undef,$P.'.6')->bstr, $result, "$method(undef,$($P).6)")
    is ($x->copy->?$method(undef,$P.'.9')->bstr, $result, "$method(undef,$($P).9)")


sub _do_2_a($x, $method, $y, $A, $result)

    my $cy = $cf->new($y)

    is ($x->copy->?$method($cy,$A)->bstr, $result, "$method(\$cy,$A)")
    is ($x->copy->?$method($cy,$A.'.1')->bstr, $result, "$method(\$cy,$($A).1)")
    is ($x->copy->?$method($cy,$A.'.5')->bstr, $result, "$method(\$cy,$($A).5)")
    is ($x->copy->?$method($cy,$A.'.6')->bstr, $result, "$method(\$cy,$($A).6)")
    is ($x->copy->?$method($cy,$A.'.9')->bstr, $result, "$method(\$cy,$($A).9)")


sub _do_2_p($x, $method, $y, $P, $result)

    my $cy = $cf->new($y)

    is ($x->copy->?$method($cy,undef,$P)->bstr, $result, "$method(undef,$P)")
    is ($x->copy->?$method($cy,undef,$P.'.1')->bstr, $result,
        "$method(\$cy,undef,$($P).1)")
    is ($x->copy->?$method($cy,undef,$P.'.5')->bstr, $result,
        "$method(\$cy,undef.$($P).5)")
    is ($x->copy->?$method($cy,undef,$P.'.6')->bstr, $result,
        "$method(\$cy,undef,$($P).6)")
    is ($x->copy->?$method($cy,undef,$P.'.9')->bstr, $result,
        "$method(\$cy,undef,$($P).9)")


