package Module::Build::PodParser

our (@ISA)

sub new
    # Perl is so fun.
    my $package = shift

    my $self

    # Try using Pod::Parser first
    if (try{ require Pod::Parser; 1; })
        @ISA = qw(Pod::Parser)
        $self = $package->SUPER::new(< @_)
        $self->{+have_pod_parser} = 1
    else
        @ISA = $@
        *parse_from_filehandle = &_myparse_from_filehandle
        $self = bless \(%: have_pod_parser => 0, < @_), $package
    

    unless ($self->{?fh})
        die "No 'file' or 'fh' parameter given" unless $self->{?file}
        $self->{+fh} = IO::File->new($self->{file}) or die "Couldn't open $self->{?file}: $^OS_ERROR"
    

    return $self


sub _myparse_from_filehandle($self, $fh)

    local $_ = undef
    while ( ~< $fh)
        next unless m/^=(?!cut)/ .. m/^=cut/  # in POD
        last if ($self->{+abstract}) = m/^  (?:  [a-z:]+  \s+ - \s+  )  (.*\S)  /ix

    my @author
    my $in_authors
    while ( ~< $fh)
        $in_authors &&= m/^=/
        $in_authors ||= m/^=head1\s+AUTHORS?/
        next unless $in_authors
        next if m/^=/
        push @author, $_ if m/\@/
    
    return unless (nelems @author)
    foreach (@author)
        s/^\s+|\s+$//g

    $self->{+author} = \@author

    return


sub get_abstract
    my $self = shift
    return $self->{?abstract} if defined $self->{?abstract}

    $self->parse_from_filehandle($self->{fh})

    return $self->{?abstract}


sub get_author
    my $self = shift
    return $self->{?author} if defined $self->{?author}

    $self->parse_from_filehandle($self->{fh})

    return $self->{?author} || $@


################## Pod::Parser overrides ###########
sub initialize
    my $self = shift
    $self->{+_head} = ''
    $self->SUPER::initialize()


sub command($self, $cmd, $text, ...)
    if ( $cmd eq 'head1' )
        $text =~ s/^\s+//
        $text =~ s/\s+$//
        $self->{+_head} = $text
    


sub textblock($self, $text, ...)
    $text =~ s/^\s+//
    $text =~ s/\s+$//
    if ($self->{?_head} eq 'NAME')
        my (@: $name, $abstract) =  split( m/\s+-\s+/, $text, 2 )
        $self->{+abstract} = $abstract
    elsif ($self->{?_head} =~ m/^AUTHORS?$/)
        push $self->{+author}, $text if $text =~ m/\@/
    


sub verbatim {}
sub interior_sequence {}

1
