#!/usr/bin/perl -w


use lib env::var('PERL_CORE') ?? '../lib/Module/Build/t/lib' !! 't/lib'
use MBTest tests => 48

use Cwd ()
my $cwd = Cwd::cwd
my $tmp = File::Spec->catdir( $cwd, 't', '_tmp' )

use DistGen
my $dist = DistGen->new( dir => $tmp )
$dist->regen

chdir( $dist->dirname ) or die "Can't chdir to '$($dist->dirname)': $^OS_ERROR"

#########################


use_ok 'Module::Build'

SKIP: do
    skip "no blib in core", 1 if env::var('PERL_CORE')
    like $^INCLUDED{?'Module/Build.pm'}, qr/\bblib\b/, "Make sure Module::Build was loaded from blib/"



# Test object creation
do
    my $mb = Module::Build->new( module_name => $dist->name )
    ok $mb
    is $mb->module_name, $dist->name
    is $mb->build_class, 'Module::Build'
    is $mb->dist_name, $dist->name

    $mb = Module::Build->new( dist_name => $dist->name, dist_version => 7 )
    ok $mb
    ok ! $mb->module_name  # Make sure it's defined
    is $mb->dist_name, $dist->name


# Make sure actions are defined, and known_actions works as class method
do
    my %actions = %:  < @+: map { @: $_, 1}, keys Module::Build->known_actions->% 
    ok %actions{?clean}
    ok %actions{?distdir}


# Test prerequisite checking
do
    local $^INCLUDE_PATH = @: File::Spec->catdir( $dist->dirname, 'lib' ), < $^INCLUDE_PATH
    my $flagged = 0
    local $^WARN_HOOK = sub (@< @_) { $flagged = 1 if @_[0]->message =~ m/$($dist->name)/}
    my $mb = Module::Build->new(
        module_name => $dist->name,
        requires    => \(%: $dist->name => 0),
        )
    ok ! $flagged
    ok ! $mb->prereq_failures
    $mb->dispatch('realclean')
    $dist->clean

    $flagged = 0
    $mb = Module::Build->new(
        module_name => $dist->name,
        requires    => \(%: $dist->name => 3.14159265),
        )
    ok $flagged
    ok $mb->prereq_failures
    ok $mb->prereq_failures{requires}{?$dist->name}
    is $mb->prereq_failures{requires}{$dist->name}->{?have}, 0.01
    is $mb->prereq_failures{requires}{$dist->name}->{?need}, 3.14159265

    $mb->dispatch('realclean')
    $dist->clean

    # Make sure check_installed_status() works as a class method
    my $info = Module::Build->check_installed_status('File::Spec', 0)
    ok $info->{?ok}
    is $info->{?have}, $File::Spec::VERSION

    # Make sure check_installed_status() works with an advanced spec
    $info = Module::Build->check_installed_status('File::Spec', '+> 0')
    ok $info->{?ok}

    # Use 2 lines for this, to avoid a "used only once" warning
    local $Foo::Module::VERSION = undef
    $Foo::Module::VERSION = '1.01_02'

    $info = Module::Build->check_installed_status('Foo::Module', '1.01_02')
    ok $info->{?ok} or diag($info->{?message})


do
    # Make sure the correct warning message is generated when an
    # optional prereq isn't installed
    my $flagged = 0
    local $^WARN_HOOK = sub (@< @_) { $flagged = 1 if @_[0]->message =~ m/ModuleBuildNonExistent is not installed/}

    my $mb = Module::Build->new(
        module_name => $dist->name,
        recommends  => \(%: ModuleBuildNonExistent => 3),
        )
    ok $flagged
    $dist->clean


# Test verbosity
do
    my $mb = Module::Build->new(module_name => $dist->name)

    $mb->add_to_cleanup('save_out')
    # Use uc() so we don't confuse the current test output
    like uc(stdout_of( sub (@< @_) {$mb->dispatch('test', verbose => 1)} )), qr/^OK \d/m
    like uc(stdout_of( sub (@< @_) {$mb->dispatch('test', verbose => 0)} )), qr/\.\.OK/

    $mb->dispatch('realclean')
    $dist->clean


# Make sure 'config' entries are respected on the command line, and that
# Getopt::Long specs work as expected.
do
    use Config
    $dist->change_file( 'Build.PL', <<"---" )
use Module::Build
my \$build = Module::Build->new(
  module_name => '$($dist->name)',
  license     => 'perl',
  get_options => \\\%(: foo => \\\$\%,
                   bar => \\\%(: type    => '+'  ),
                   bat => \\\%(: type    => '=s' ),
                   dee => \\\%(: type    => '=s',
                            default => 'goo'
                          ),
                 )
  )

\$build->create_build_script
---

    $dist->regen
    Module::Build->run_perl_script('Build.PL', \$@,
        \(@: '--nouse-rcfile', '--config', "foocakes=barcakes"
             '--foo', '--bar', '--bar', '-bat=hello', 'gee=whiz'
             '--any', 'hey', '--destdir', 'yo', '--verbose', '1'))

    my $mb = Module::Build->resume
    ok $mb->valid_property('config')

    is $mb->config('cc'), config_value("cc")
    is $mb->config('foocakes'), 'barcakes'

    # Test args().
    is $mb->args('foo'), 1
    is $mb->args('bar'), 2, 'bar'
    is $mb->args('bat'), 'hello', 'bat'
    is $mb->args('gee'), 'whiz'
    is $mb->args('any'), 'hey'
    is $mb->args('dee'), 'goo'
    is $mb->destdir, 'yo'
    my %runtime = %:  < $mb->runtime_params 
    is_deeply \%runtime,
            \%: 
            verbose => 1
            destdir => 'yo'
            use_rcfile => 0
            config => \(%:  foocakes => 'barcakes' )
            

    ok(my $args = $mb->args)
    is $args{?foo}, 1

    # revert test distribution to pristine state because we modified a file
    chdir( $cwd ) or die "Can''t chdir to '$cwd': $^OS_ERROR"
    $dist->remove
    $dist = DistGen->new( dir => $tmp )
    $dist->regen
    chdir( $dist->dirname ) or die "Can't chdir to '$(join ' ',(\$dist->dirname)->@)': $^OS_ERROR"


# Test author stuff
do
    my $mb = Module::Build->new(
        module_name => $dist->name,
        dist_author => 'Foo Meister <foo@example.com>',
        build_class => 'My::Big::Fat::Builder',
        )
    ok $mb
    ok ref($mb->dist_author), 'dist_author converted to array if simple string'
    is $mb->dist_author->[0], 'Foo Meister <foo@example.com>'
    is $mb->build_class, 'My::Big::Fat::Builder'


# Test conversion of shell strings
do
    my $mb = Module::Build->new(
        module_name => $dist->name,
        dist_author => 'Foo Meister <foo@example.com>',
        extra_compiler_flags => '-I/foo -I/bar',
        extra_linker_flags => '-L/foo -L/bar',
        )
    ok $mb
    is_deeply $mb->extra_compiler_flags, \(@: '-I/foo', '-I/bar'), "Should split shell string into list"
    is_deeply $mb->extra_linker_flags,   \(@: '-L/foo', '-L/bar'), "Should split shell string into list"

    # Try again with command-line args
    Module::Build->run_perl_script('Build.PL', \$@, \(@: '--extra_compiler_flags', '-I/foo -I/bar'
                                                         '--extra_linker_flags', '-L/foo -L/bar'))
    $mb = Module::Build->resume
    ok $mb
    is_deeply $mb->extra_compiler_flags, \(@: '-I/foo', '-I/bar'), "Should split shell string into list"
    is_deeply $mb->extra_linker_flags,   \(@: '-L/foo', '-L/bar'), "Should split shell string into list"



# cleanup
chdir( $cwd ) or die "Can''t chdir to '$cwd': $^OS_ERROR"
$dist->remove

use File::Path
rmtree( $tmp )
