package MBTest


use File::Spec

BEGIN 
    # Make sure none of our tests load the users ~/.modulebuildrc file
    env::var('MODULEBUILDRC' ) = 'NONE'

    # In case the test wants to use Test::More or our other bundled
    # modules, make sure they can be loaded.  They'll still do "use
    # Test::More" in the test script.
    my $t_lib = File::Spec->catdir('t', 'bundled')

    if (!env::var('PERL_CORE'))
        push $^INCLUDE_PATH, $t_lib # Let user's installed version override
    else
        # We change directories, so expand $^INCLUDE_PATH to absolute paths
        # Also add .
        $^INCLUDE_PATH = (@: < map( { <File::Spec->rel2abs($_) }, $^INCLUDE_PATH), ".")

        # we are in 't', go up a level so we don't create t/t/_tmp
        chdir '..' or die "Couldn't chdir to .."

        push $^INCLUDE_PATH, < File::Spec->catdir( <qw/lib Module Build/, $t_lib)

        # make sure children get $^INCLUDE_PATH pointing to uninstalled files
        require Cwd
        env::var('PERL5LIB' ) = File::Spec->catdir( <Cwd::cwd(), 'lib')
    


use Exporter
use Test::More
use Config

# We pass everything through to Test::More
our ($VERSION, @ISA, @EXPORT, %EXPORT_TAGS, $TODO)
$VERSION = 0.01
@ISA = qw(Test::More) # Test::More isa Exporter
@EXPORT = @Test::More::EXPORT
%EXPORT_TAGS = (%:  < %Test::More::EXPORT_TAGS )

# We have a few extra exports, but Test::More has a special import()
# that won't take extra additions.
my @extra_exports = qw(stdout_of stderr_of slurp find_in_path check_compiler have_module)
push @EXPORT, < @extra_exports
__PACKAGE__->export(scalar caller, < @extra_exports)


sub save_handle($handle, $subr)
    my $outfile = 'save_out'

    local *SAVEOUT
    open \*SAVEOUT, ">&", "" . fileno($handle) or die "Can't save output handle: $^OS_ERROR"
    open $handle, ">", "$outfile" or die "Can't create $outfile: $^OS_ERROR"

    try {$subr->()}
    open $handle, ">&", \*SAVEOUT or die "Can't restore output: $^OS_ERROR"
    die $^EVAL_ERROR if $^EVAL_ERROR

    my $ret = slurp($outfile)
    1 while unlink $outfile
    return $ret


sub stdout_of { save_handle($^STDOUT, < @_) }
sub stderr_of { save_handle($^STDERR, < @_) }

sub slurp
    my $fh = IO::File->new(@_[0]) or die "Can't open @_[0]: $^OS_ERROR"
    local $^INPUT_RECORD_SEPARATOR = undef
    return scalar ~< $fh


sub exe_exts
    # Some extensions we should know about if we're looking for executables

    if ($^OS_NAME eq 'MSWin32')
        return split(config_value("path_sep"), env::var('PATHEXT') || '.com;.exe;.bat')
    
    if ($^OS_NAME eq 'os2')
        return qw(.exe .com .pl .cmd .bat .sh .ksh)
    
    return


sub find_in_path
    my $thing = shift

    my @path = split config_value("path_sep"), env::var('PATH')
    my @exe_ext = exe_exts()
    foreach ( @path)
        my $fullpath = File::Spec->catfile($_, $thing)
        foreach my $ext ((@:  '', < @exe_ext) )
            return "$fullpath$ext" if -e "$fullpath$ext"
        
    
    return


# returns ($have_c_compiler, $C_support_feature);
sub check_compiler
    return ( (@: 1,1 )) if env::var('PERL_CORE')

    local $^WARN_HOOK = sub {}

    my $mb = Module::Build->current
    $mb->verbose( 0 )

    my $have_c_compiler
    stderr_of( sub (@< @_) {$have_c_compiler = $mb->have_c_compiler} )

    return ( (@: $have_c_compiler, $mb->feature('C_support') ))


sub have_module
    my $module = shift
    return eval "use $module; 1"


1
