#!/usr/bin/perl -w

# Test ~ expansion from command line arguments.


use lib env::var('PERL_CORE') ?? '../lib/Module/Build/t/lib' !! 't/lib'
use MBTest tests => 14

use Cwd ()
my $cwd = Cwd::cwd
my $tmp = File::Spec->catdir( $cwd, 't', '_tmp' )

use DistGen
my $dist = DistGen->new( dir => $tmp )
$dist->regen

chdir( $dist->dirname ) or die "Can't chdir to '$(join ' ',(\$dist->dirname)->@)': $^OS_ERROR"


use Module::Build

sub run_sample
    my @args = @_

    local $Test::Builder::Level = $Test::Builder::Level + 1

    $dist->clean

    my $mb
    stdout_of( sub (@< @_)
        $mb = Module::Build->new_from_context( < @args )
     )

    return $mb



do
    local env::var('HOME' ) = 'home'

    my $mb

    $mb = run_sample( install_base => '~' )
    is( $mb->install_base,      env::var('HOME') )

    $mb = run_sample( install_base => '~/foo' )
    is( $mb->install_base,      "$(env::var('HOME'))/foo" )

    $mb = run_sample( install_base => '~~' )
    is( $mb->install_base,      '~~' )

    TODO: do
        local $TODO = "Not handling spaces in _detildefy() properly yet"

        $mb = run_sample( install_base => '~ foo' )
        is( $mb->install_base,      '~ foo' )

        # glob() doesn't work on non-existent paths with spaces
        $mb = run_sample( install_base => '~/ foo' )
        is( $mb->install_base,      "$(env::var('HOME'))/ foo" )

        $mb = run_sample( install_base => '~/fo o' )
        is( $mb->install_base,      "$(env::var('HOME'))/fo o" )
    

    $mb = run_sample( install_base => 'foo~' )
    is( $mb->install_base,      'foo~' )

    $mb = run_sample( prefix => '~' )
    is( $mb->prefix,            env::var('HOME') )

    $mb = run_sample( install_path => \%:  html => '~/html'
                                           lib  => '~/lib'   
                      )
    is( $mb->install_destination('lib'),  "$(env::var('HOME'))/lib" )
    # 'html' is translated to 'binhtml' & 'libhtml'
    is( $mb->install_destination('binhtml'), "$(env::var('HOME'))/html" )
    is( $mb->install_destination('libhtml'), "$(env::var('HOME'))/html" )

    $mb = run_sample( install_path => \(%:  lib => '~/lib' ) )
    is( $mb->install_destination('lib'),  "$(env::var('HOME'))/lib" )

    $mb = run_sample( destdir => '~' )
    is( $mb->destdir,           env::var('HOME') )

    $mb->install_base('~')
    is( $mb->install_base,      '~', 'API does not expand tildes' )



# cleanup
chdir( $cwd ) or die "Can''t chdir to '$cwd': $^OS_ERROR"
$dist->remove

use File::Path
rmtree( $tmp )
