
use Test::More;

BEGIN
  require Socket
  if (my $port = getservbyname('echo', 'tcp'))
    socket(*ECHO, Socket::PF_INET(), Socket::SOCK_STREAM(), (@: getprotobyname 'tcp')[2]);
    unless (connect(*ECHO, Socket::sockaddr_in($port, Socket::inet_aton("localhost"))))
      plan skip_all => "loopback tcp echo service is off ($^OS_ERROR)"
    close (*ECHO)
  else
      plan skip_all => "no echo port"

# Test of stream protocol using loopback interface.
#
# NOTE:
#   The echo service must be enabled on localhost
#   to really test the stream protocol ping.  See
#   the end of this document on how to enable it.

use Net::Ping;
plan tests => 22;

my $p = Net::Ping->new( "stream");

# new() worked?
ok !!$p;

# Attempt to connect to the echo port
ok ($p -> ping("localhost"));

# Try several pings while it is connected
for (1..20) {
  select (undef,undef,undef,0.1);
  ok $p -> ping("localhost");
}

__END__

A simple xinetd configuration to enable the echo service can easily be made.
Just create the following file before restarting xinetd:

/etc/xinetd.d/echo:

# description: An echo server.
service echo
{
        type            = INTERNAL
        id              = echo-stream
        socket_type     = stream
        protocol        = tcp
        user            = root
        wait            = no
        disable         = no
}


Or if you are using inetd, before restarting, add
this line to your /etc/inetd.conf:

echo   stream  tcp     nowait  root    internal
