package PPI::Util

# Provides some common utility functions that can be imported

use Digest::MD5  ()
use Params::Util < qw{ _INSTANCE _SCALAR0 _ARRAY0 }

our ($VERSION, @ISA, @EXPORT_OK)
BEGIN 
    $VERSION = '1.204_01'

    require Exporter
    @ISA       = qw{ Exporter         }
    @EXPORT_OK = qw{ _Document _slurp }


# Alarms are used to catch unexpectedly slow and complex documents
use constant HAVE_ALARM   => !  ( $^OS_NAME eq 'MSWin32' or $^OS_NAME eq 'cygwin' )

use constant HAVE_UNICODE => 1

# Down here so we don't get into circular troubles
require PPI::Document





#####################################################################
# Functions

# Allows a sub that takes a L<PPI::Document> to handle the full range
# of different things, including file names, SCALAR source, etc.
sub _Document
    shift if nelems(@_) +> 1
    return undef unless defined @_[0]
    return PPI::Document->new( shift ) unless ref @_[0]
    return PPI::Document->new( shift ) if _SCALAR0(@_[0])
    return PPI::Document->new( shift ) if _ARRAY0(@_[0])
    return shift if _INSTANCE(@_[0], 'PPI::Document')
    return undef


# Provide a simple _slurp implementation
sub _slurp
    my $file = shift
    local $^INPUT_RECORD_SEPARATOR = undef
    open( my $fh, '<', $file ) or return "open($file) failed: $^OS_ERROR"
    my $source = ~< $fh->*
    close( $fh ) or return "close($file) failed: $^OS_ERROR"
    return \$source


# Provides a version of Digest::MD5's md5hex that explicitly
# works on the unix-newlined version of the content.
sub md5hex
    my $string = shift
    $string =~ s/(?:\015{1,2}\012|\015|\012)/\015/gs
    Digest::MD5::md5_hex($string)


# As above but slurps and calculates the id for a file by name
sub md5hex_file
    my $file    = shift
    my $content = _slurp($file)
    return undef unless ref $content
    $content->$ =~ s/(?:\015{1,2}\012|\015|\012)/\n/gs
    md5hex($content->$)


1
