#!/usr/bin/perl

# Test compatibility with Storable

use File::Spec::Functions ':ALL'
BEGIN 
    $^OUTPUT_AUTOFLUSH = 1
    $PPI::XS_DISABLE = 1
    $PPI::XS_DISABLE = 1 # Prevent warning

use PPI
use PPI::Document ()
use Test::More    ()
use Scalar::Util  'refaddr'

# Is Storable installed?
try { require Storable; }
if ( $^EVAL_ERROR )
    Test::More::plan( 'skip_all' )
    exit(0)

Test::More::plan( tests => 9 )




#####################################################################
# Test freeze/thaw of PPI::Document objects

do
    # Create a document with various example package statements
    my $Document = PPI::Lexer->lex_source( <<'END_PERL' )
package Foo;
@ISA = (qw/File::Spec/);

1;
END_PERL
    Test::More::isa_ok( $Document, 'PPI::Document' )
    do
        my $isa = $Document->find_first(sub (@< @_) { @_[1]->content eq '@ISA'; })
        Test::More::ok( $isa, "Found ISA var")
        Test::More::is( $isa->parent->content, q|@ISA = (qw/File::Spec/);|, "Got parent ok")
    
    my $clone = Storable::dclone($Document)
    Test::More::ok($clone, "dclone ok")
    Test::More::isnt( refaddr($Document), refaddr($clone), "Not the same object" )
    Test::More::is(ref($Document), ref($clone), "Same class")
    Test::More::is_deeply( $Document, $clone, "Deeply equal" )
    do
        my $isa = $clone->find_first(sub (@< @_) { @_[1]->content eq '@ISA'; })
        Test::More::ok($isa, "Found ISA var")
        Test::More::is($isa->parent->content, q|@ISA = (qw/File::Spec/);|, "Got parent ok")   # <-- this one fails
    



1
