#!/usr/bin/perl

BEGIN 
    $^OUTPUT_AUTOFLUSH  = 1
    $^WARNING = 1

use Test::More
use File::Spec::Functions ':ALL'
use Scalar::Util < qw(
	blessed
	reftype
	refaddr
	)

use Params::Util '_CODELIKE'

sub c_ok { is(
 refaddr(_CODELIKE(@_[0])),
 refaddr(@_[0]),
 "callable: @_[1]",
 ) }
sub nc_ok { is(
 _CODELIKE(@_[0]),
 undef,
 "not callable: @_[1]"
 ) }

my @callables = @: 
  "callable itself"                         => &_CODELIKE
  "a boring plain code ref"                 => sub {}

my @uncallables = @: 
  "undef"                                   => undef
  "a string"                                => "a string"
  "a number"                                => 19780720
  "a ref to code"                  => \(sub {})
  "a boring plain hash ref"                 => \$%
  'a class that builds from coderefs'       => "C::C"
  'a plain boring hash-based object'        => UC->new
  'a non-coderef blessed into CODE'         => (bless \$% => 'CODE')

plan tests => (nelems(@callables) + nelems(@uncallables)) / 2 + 1

# Import the function
ok( defined *_CODELIKE{CODE}, '_CODELIKE imported ok' )

while ( nelems(@callables) )
    my (@: $name, $object) =@:  splice @callables, 0, 2
    c_ok($object, $name)


while ( nelems(@uncallables) )
    my (@: $name, $object) =@:  splice @uncallables, 0, 2
    nc_ok($object, $name)






# callable: is a blessed code ref
package C::C
sub new { bless \ sub {} => shift; }





# uncallable: some boring object with no codey magic
package UC
sub new { bless \$% => shift; }
