#!/usr/bin/perl -w



use Params::Validate < qw(validate)
use Test::More tests => 13

do
    my @p = @:  foo => 'ClassCan' 

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'cancan' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR )

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/ )


do
    my @p = @:  foo => undef 
    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'baz' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'baz'/ )


do
    my $object = bless \$%, 'ClassCan'
    my @p = @:  foo => $object 

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'cancan' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR )

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/ )


do
    my @p = @:  foo => 'SubClass' 

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'cancan' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass->can(cancan)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/ )


do
    my $object = bless \$%, 'SubClass'
    my @p = @:  foo => $object 

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'cancan' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass object->can(cancan)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/ )


do
    my @p = @:  foo => \$% 
    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/, 'unblessed ref ->can' )

    @p = @:  foo => 27 
    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/, 'number can' )

    @p = @:  foo => 'A String' 
    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/, 'string can' )

    @p = @:  foo => undef 
    try {
        validate( @p,
                  (%:  foo => \(%:  can => 'thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/does not have the method: 'thingy'/, 'undef can' )



package ClassCan

sub can
    return 1 if @_[1] eq 'cancan'
    return 0


sub thingy { 1 }

package SubClass

use base 'ClassCan'


