#!/usr/bin/perl -w



use Params::Validate < qw(validate)
use Test::More tests => 13

do
    my @p = @:  foo => 'ClassISA' 

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'FooBar' ) ),
                  );
    }

    is( $^EVAL_ERROR, '', 'no error checking if ClassISA->isa(FooBar)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/ )


do
    my @p = @:  foo => undef 
    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'FooBar' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/was not a 'FooBar'/ )


do
    my @p = @:  foo => 'SubClass' 

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'ClassISA' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass->isa(ClassISA)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'FooBar' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass->isa(FooBar)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/ )


do
    my @p = @:  foo => bless \$%, 'SubClass' 

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'ClassISA' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass->isa(ClassISA)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'FooBar' ) ),
                  );
    }

    ok( ! $^EVAL_ERROR, 'SubClass->isa(FooBar)' )

    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }

    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/ )


do
    my @p = @:  foo => \$% 
    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/, 'unblessed ref ->isa' )

    @p = @:  foo => 27 
    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/, 'number isa' )

    @p = @:  foo => 'A String' 
    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/, 'string isa' )

    @p = @:  foo => undef 
    try {
        validate( @p,
                  (%:  foo => \(%:  isa => 'Thingy' ) ),
                  );
    }
    like( $^EVAL_ERROR->{?description}, qr/was not a 'Thingy'/, 'undef isa' )




package ClassISA

sub isa
    return 1 if @_[1] eq 'FooBar'
    return @_[0]->SUPER::isa(@_[1])


sub thingy { 1 }

package SubClass

use base 'ClassISA'


