

our @expect

use Params::Validate < qw(:all)

use Test::More

plan tests => 94

$^OUTPUT_AUTOFLUSH = 1

sub run_tests
    do
        # mandatory/optional
        try { sub1( foo => 'a', bar => 'b' ) }
        check()

        try { sub1( foo => 'a' ) }
        check()

        try { sub1() }
        check()

        try { sub1( foo => 'a', bar => 'b', baz => 'c' ) }
        check()

        try { sub2( foo => 'a', bar => 'b', baz => 'c' ) }
        check()

        try { sub2( foo => 'a', bar => 'b' ) }
        check()

        try { sub2a( foo => 'a', bar => 'b' ) }
        check()

        try { sub2a( foo => 'a' ) }
        check()


    do
        # simple types
        try { sub3( foo => 'a',
                    bar => \(@:  1, 2, 3 ),
                    baz => \(%:  a => 1 ),
                    quux => 'yadda',
                    brax => \(%:  < qw( a b c d ) ),
                    ) }
        check()

        try { sub3( foo => \(@: 'a'),
                    bar => \(@:  1, 2, 3 ),
                    baz => \(%:  a => 1 ),
                    quux => 'yadda',
                    brax => \(%:  < qw( a b c d ) ),
                    ) }
        check()

        try { sub3( foo => 'foobar',
                    bar => \(@:  1, 2, 3 ),
                    baz => \(%:  a => 1 ),
                    quux => 'yadda',
                    brax => \ qw( a b c d ),
                    ) }
        check()

        try { sub3( foo => 'foobar',
                    bar => \(%:  1, 2, 3, 4 ),
                    baz => \(%:  a => 1 ),
                    quux => 'yadda',
                    brax => 'a',
                    ) }
        check()


    do
        # funkier types
        my $foo = 'foobar'
        try { sub4( foo => \$foo,
                    baz => \*BAZZY,
                    quux => sub (@< @_) { 'a coderef' },
                    ) }
        check()

        try { sub4( foo => $foo,
                    baz => \*BAZZY,
                    quux => sub (@< @_) { 'a coderef' },
                    ) }
        check()

        try { sub4( foo => \$foo,
                    baz => \*BAZZY,
                    quux => \*CODEREF,
                    ) }
        check()

        # test HANDLE type
        try { sub4a( foo => \*HANDLE) }
        check()

        try { sub4a( foo => \(@: 'not a handle') ) }
        check()

        # test BOOLEAN type
        try { sub4b( foo => undef ) }
        check()

        try { sub4b( foo => 124125 ) }
        check()


    do
        # isa
        my ($x, $y, $z, $zz)
        my $foo = bless \$x, 'Foo'
        my $bar = bless \$y, 'Bar'
        my $baz = bless \$z, 'Baz'
        my $quux = bless \$zz, 'Quux'

        try { sub5( foo => $foo ) }
        check()
        try { sub5( foo => $bar ) }
        check()
        try { sub5( foo => $baz ) }
        check()

        try { sub6( foo => $foo ) }
        check()
        try { sub6( foo => $bar ) }
        check()
        try { sub7( foo => $baz ) }
        check()

        try { sub7( foo => $foo ) }
        check()
        try { sub7( foo => $bar ) }
        check()
        try { sub7( foo => $baz ) }
        check()

        try { sub8( foo => $foo ) }
        check()
        try { sub8( foo => $quux ) }
        check()


    do
        # can
        my ($x, $y, $z, $zz)
        my $foo = bless \$x, 'Foo'
        my $bar = bless \$y, 'Bar'
        my $baz = bless \$z, 'Baz'
        my $quux = bless \$zz, 'Quux'

        try { sub9( foo => $foo ) }
        check()
        try { sub9( foo => $quux ) }
        check()

        try { sub9a( foo => $foo ) }
        check()
        try { sub9a( foo => $bar ) }
        check()

        try { sub9b( foo => $baz ) }
        check()
        try { sub9b( foo => $quux ) }
        check()

        try { sub9c( foo => $bar ) }
        check()
        try { sub9c( foo => $quux ) }
        check()


    do
        # callbacks
        try { sub10( foo => 1 ) }
        check()
        try { sub10( foo => 19 ) }
        check()
        try { sub10( foo => 20 ) }
        check()

        try { sub11( foo => 1 ) }
        check()
        try { sub11( foo => 20 ) }
        check()
        try { sub11( foo => 0 ) }
        check()


    do
        # mix n' match
        try { sub12( foo => 1 ) }
        check()
        try { sub12( foo => \(@:  1, 2, 3 ) ) }
        check()
        try { sub12( foo => \(@:  1, 2, 3, 4, 5 ) ) }
        check()


    do
        # positional - 1
        try { sub13( 'a' ) }
        check()
        try { sub13( 'a', \(@:  1, 2, 3 ) ) }
        check()


    do
        # positional - 2
        my ($x, $y)
        my $foo = bless \$x, 'Foo'
        my $bar = bless \$y, 'Bar'
        try { sub14( 'a', \(@:  1, 2, 3 ), $foo ) }
        check()
        try { sub14( 'a', \(@:  1, 2, 3 ), $bar ) }
        check()


    do
        # hashref named params
        try { sub15( \(%:  foo => 1, bar => \(%:  a => 1 ) ) ) }
        check()

        try { sub15( \(%:  foo => 1 ) ) }
        check()


    do
        # positional - 3
        try { sub16( 1, 2, 3 ) }
        check()
        try { sub16( 1, 2 ) }
        check()
        try { sub16( 1 ) }
        check()
        try { sub16() }
        check()


    do
        # positional - 4
        try { sub17( 1, 2, 3 ) }
        check()
        try { sub17( 1, 2 ) }
        check()
        try { sub17( 1 ) }
        check()
        try { sub17() }
        check()


    do
        # positional - too few arguments supplied
        try { sub17a() }
        check()
        try { sub17a(1, 2) }
        check()
        try { sub17b() }
        check()
        try { sub17b(42, 2) }
        check()


    # validation_options
    do
        do
            package Foo
            Params::Validate::validation_options( ignore_case => 1 )

        try { Foo::sub18( FOO => 1 ) }
        check()
        try { sub18( FOO => 1 ) }
        check()


    do
        do
            package Foo
            validation_options( strip_leading => '-' )

        try { Foo::sub18( "-foo" => 1 ) }
        check()
        try { sub18( "-foo" => 1 ) }
        check()


    do
        do
            package Foo
            validation_options( allow_extra => 1 )

        my %ret = %:  < Foo::sub18( foo => 1, bar => 1 )
        check()
        ok(%ret{?foo} == 1)
        ok(%ret{?bar} == 1)
        try { sub18( foo => 1, bar => 1 ) }
        check()

        my @ret = Foo::sub19( 1, 2 )
        check()
        ok(@ret[0] == 1)
        ok(@ret[1] == 2)
        try { sub19( 1, 2 ) }
        check()

        validation_options( strip_leading => '-' )
        try { Foo::sub18( "-foo" => 1 ) }
        check()


    validation_options()

    do
        do
            package Foo
            validation_options( on_fail => sub (@< @_) { die "ERROR WAS: @_[0]" } )

        try { Foo::sub18( bar => 1 ) }
        check()
        try { sub18( bar => 1 ) }
        check()


    try { sub20( foo => undef ) }
    check()
    try { sub21( foo => undef ) }
    check()

    try { sub22( foo => \(@: 1) ) }
    check()
    try { sub22( foo => bless \(@: 1), 'object' ) }
    check()

    try { sub22a( ) }
    check()
    try { sub22a( foo => \(@: 1) ) }
    check()
    try { sub22a( foo => bless \(@: 1), 'object' ) }
    check()

    try { sub23( '1 element' ) }
    check()

    try { sub24( ) }
    check()
    try { sub24( '1 element' ) }
    check()
    try { sub24( bless \(@: 1), 'object' ) }
    check()

    try { sub25( 1 ) }
    check()

    try { sub26( foo => 1 ) }
    check()

    do
        my $fh = do { local *BAR; \*BAR }
        try { sub26( foo => 1, bar => $fh ) }
        check()



sub sub1
    validate( @_, (%:  foo => 1, bar => 1 ) )


sub sub2
    validate( @_, (%:  foo => 1, bar => 1, baz => 0 ) )


sub sub2a
    validate( @_, (%:  foo => 1, bar => \(%:  optional => 1 ) ) )


sub sub3
    validate( @_, %:  foo =>
                      \(%:  type => SCALAR )
                      bar =>
                      \(%:  type => ARRAYREF )
                      baz =>
                      \(%:  type => HASHREF )
                      quux =>
                      \(%:  type => SCALAR ^|^ ARRAYREF )
                      brax =>
                      \(%:  type => SCALAR ^|^ HASHREF )

              )


sub sub4
    validate( @_, %:  foo =>
                      \(%:  type => SCALARREF )
                      baz =>
                      \(%:  type => GLOBREF )
                      quux =>
                      \(%:  type => CODEREF )

              )


sub sub4a
    validate( @_, (%:  foo => \(%:  type => HANDLE ) ) )


sub sub4b
    validate( @_, (%:  foo => \(%:  type => BOOLEAN ) ) )


sub sub5
    validate( @_, (%:  foo => \(%:  isa => 'Foo' ) ) )


sub sub6
    validate( @_, (%:  foo => \(%:  isa => 'Bar' ) ) )


sub sub7
    validate( @_, (%:  foo => \(%:  isa => 'Baz' ) ) )


sub sub8
    validate( @_, (%:  foo => \(%:  isa => \(@:  'Foo', 'Yadda' ) ) ) )


sub sub9
    validate( @_, (%:  foo => \(%:  can => 'fooify') ) )


sub sub9a
    validate( @_, (%:  foo => \(%:  can => \(@:  'fooify', 'barify' ) ) ) )


sub sub9b
    validate( @_, (%:  foo => \(%:  can => \(@:  'barify', 'yaddaify' ) ) ) )


sub sub9c
    validate( @_, (%:  foo => \(%:  can => \(@:  'fooify', 'yaddaify' ) ) ) )


sub sub10
    validate( @_, %:  foo =>
                      \ %: callbacks =>
                           \%:  'less than 20' => sub (@< @_) { shift() +< 20 }
              )


sub sub11
    validate( @_, (%:  foo =>
                       \ %: callbacks =>
                            \%: 'less than 20' => sub (@< @_) { shift() +< 20 }
                                'more than 0'  => sub (@< @_) { shift() +> 0 }
              ) )


sub sub12
    validate( @_, (%:  foo =>
                       \ %: type => ARRAYREF
                            callbacks =>
                            \%:  '5 elements' => sub (@< @_) { nelems(shift()->@) == 5 }
              ) )


sub sub13
    validate_pos( @_,
                  \(%:  type => SCALAR ),
                  \(%:  type => ARRAYREF
                        callbacks =>
                        \%:  '5 elements' => sub (@< @_) { nelems(shift()->@) == 5 }
                  ) )


sub sub14
    validate_pos( @_,
                  \(%:  type => SCALAR ),
                  \(%:  type => ARRAYREF ),
                  \(%:  isa => 'Bar' ),
                  )


sub sub15
    validate( @_,
              (%:  foo => 1
                   bar => \%:  type => ARRAYREF
              ) )


sub sub16
    validate_pos( @_, 1, 0 )


sub sub17
    validate_pos( @_, (%:  type => SCALAR ), \(%:  type => SCALAR, optional => 1 ) )


do
    package Foo
    use Params::Validate
    sub sub18
        validate( @_, (%:  foo => 1 ) )


    sub sub19
        validate_pos( @_, 1 )



sub sub17a
    validate_pos( @_, 1, 1, 1, 0 )


sub sub17b
    validate_pos( @_,
                  \(%:  callbacks =>
                        \(%:  'less than 43' => sub (@< @_) { shift() +< 43 } )),
                  \(%:  type => SCALAR ),
                  1,
                  \(%: optional => 1))


sub sub18
    validate( @_, (%:  foo => 1 ) )


sub sub19
    validate_pos( @_, 1 )


sub sub20
    validate( @_, (%:  foo => \(%:  type => SCALAR ) ) )


sub sub21
    validate( @_, (%:  foo => \(%:  type => UNDEF ^|^ SCALAR ) ) )


sub sub22
    validate( @_, (%:  foo => \(%:  type => OBJECT ) ) )


sub sub22a
    validate( @_, (%:  foo => \(%:  type => OBJECT, optional => 1 ) ) )


sub sub23
    validate_pos( @_, 1 )


sub sub24
    validate_pos( @_, \(%:  type => OBJECT, optional => 1 ) )


sub sub25
    validate( @_, (%:  foo => 1 ) )


sub sub26
    validate( @_, (%:  foo =>
                       \(%:  type => SCALAR )
                       bar =>
                       \(%:  type => HANDLE, optional => 1 )
              ),
              )


do
    my $x = 0
    sub check
        my $expect = @expect[?$x++]

        my $line = (@: caller(0))[2]

        $expect
            ?? like( $^EVAL_ERROR && $^EVAL_ERROR->message,  qr/$expect/,
                     "exptected error" )
            !! ok( ! $^EVAL_ERROR, "no error was exptected" . ($^EVAL_ERROR && $^EVAL_ERROR->message) )
        $^EVAL_ERROR = ''



package Foo

use Params::Validate < qw(:all)

sub fooify {1}

package Bar

@Bar::ISA = @: 'Foo'

sub barify {1}

package Baz

@Baz::ISA = @: 'Bar'

sub bazify {1}

package Yadda

sub yaddaify {1}

package Quux

@Quux::ISA = @: 'Foo', 'Yadda'

sub quuxify {1}

1
