use Test::More
BEGIN { plan tests => 13 }

validate_with( params => (@:  'foo' ),
               spec => (@:  'SCALAR' ),
               )
ok( 1 )

try {
    validate_with( params => (%:  foo => 5
                                  bar => \$% ),
                   spec => (%:  foo => 'SCALAR'
                                bar => 'HASHREF' ),
                   );
}
ok( !$^EVAL_ERROR )

try {
    validate_with( params => $@,
                   spec => (@:  'SCALAR' ),
                   called => 'Yo::Mama',
                   );
}
if ( env::var('PERL_NO_VALIDATION') )
    ok( ! $^EVAL_ERROR )
else
    ok( $: $^EVAL_ERROR->{?description} =~ m/Yo::Mama/ )


do
    my %p
    try {
        %p = (%:  < validate_with( params => $@,
                                   spec => %:  a => \%: default => 3
                                               b => \%:  default => 'x'
                                   ) );
    }

    ok( exists %p{a} )
    is( %p{?a}, 3 )
    ok( exists %p{b} )
    is( %p{?b}, 'x' )


do
    my @p = validate_with( params => $@,
                           spec => (@:  \(%:  default => 3 )
                                        \(%:  default => 'x' ) ),
                           )

    is( @p[0], 3 )
    is( @p[1], 'x' )


do
    package Testing::X
    use Params::Validate < qw(:all)
    validation_options( allow_extra => 1 )

    try {
        validate_with( params => (@:  a => 1, b => 2, c => 3 ),
                       spec => (%:  a => 1, b => 1 ),
                       );
    }
    main::ok( ! $^EVAL_ERROR )

    try {
        validate_with( params => (@:  a => 1, b => 2, c => 3 ),
                       spec => (%:  a => 1, b => 1 ),
                       allow_extra => 0,
                       );
    }
    if ( env::var('PERL_NO_VALIDATION') )
        main::ok( ! $^EVAL_ERROR )
    else
        main::ok( $^EVAL_ERROR->{?description} =~ m/was not listed/ )
    


do
    # Bug 2791 on rt.cpan.org
    my %p
    try {
        my @p = (@:  \(%:  foo => 1 ) );
        %p = (%:  < validate_with( params => @p,
                                   spec   => (%:  foo => 1 )
                                   ) );
    }
    warn $^EVAL_ERROR if $^EVAL_ERROR
    ok( ! $^EVAL_ERROR )
    is( %p{?foo}, 1 )


1
