#!perl

##############################################################################
#     $URL: http://perlcritic.tigris.org/svn/perlcritic/trunk/Perl-Critic/t/10_userprofile.t $
#    $Date: 2008-09-02 11:43:48 -0500 (Tue, 02 Sep 2008) $
#   $Author: thaljef $
# $Revision: 2721 $
##############################################################################

use warnings

use English

use Perl::Critic::UserProfile

use Test::More tests => 41

#-----------------------------------------------------------------------------

our $VERSION = '1.093_01'

#-----------------------------------------------------------------------------

# Create profile from hash

do
    my %policy_params = %: keywords => 'Revision'
    my %profile_hash = %:  '-NamingConventions::ProhibitMixedCaseVars' => \(%: )
                           'Miscellanea::RequireRcsKeywords' => \%policy_params 

    my $up = Perl::Critic::UserProfile->new( profile => \%profile_hash )

    # Using short policy names
    is(
        $up->policy_is_enabled('Miscellanea::RequireRcsKeywords'),
        1,
        'Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('NamingConventions::ProhibitMixedCaseVars'),
        1,
        'NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Miscellanea::RequireRcsKeywords'),
        \%policy_params,
        'Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        1,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars'),
        1,
        'Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        \%policy_params,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Using bogus policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't enabled>,
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't disabled>,
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Bogus'),
        (%: ),
        q<Bogus Policy doesn't have any configuration.>,
        )


#-----------------------------------------------------------------------------
# Create profile from array

do
    my %policy_params = %: keywords => 'Revision'
    my @profile_array = @:  q{ [-NamingConventions::ProhibitMixedCaseVars] }
                            q{ [Miscellanea::RequireRcsKeywords]           }
                            q{ keywords = Revision                         }
        


    my $up = Perl::Critic::UserProfile->new( profile => \@profile_array )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Miscellanea::RequireRcsKeywords'),
        1,
        'Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('NamingConventions::ProhibitMixedCaseVars'),
        1,
        'NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        1,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars'),
        1,
        'Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Using bogus policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't enabled>,
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't disabled>,
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Bogus'),
        (%: ),
        q<Bogus Policy doesn't have any configuration.>,
        )


#-----------------------------------------------------------------------------
# Create profile from string

do
    my %policy_params = %: keywords => 'Revision'
    my $profile_string = <<'END_PROFILE'
[-NamingConventions::ProhibitMixedCaseVars]
[Miscellanea::RequireRcsKeywords]
keywords = Revision
END_PROFILE

    my $up = Perl::Critic::UserProfile->new( profile => \$profile_string )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Miscellanea::RequireRcsKeywords'),
        1,
        'Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('NamingConventions::ProhibitMixedCaseVars'),
        1,
        'NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        1,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars'),
        1,
        'Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Using bogus policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't enabled>,
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't disabled>,
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Bogus'),
        (%: ),
        q<Bogus Policy doesn't have any configuration.>,
        )


#-----------------------------------------------------------------------------
# Test long policy names

do
    my %policy_params = %: keywords => 'Revision'
    my $long_profile_string = <<'END_PROFILE'
[-Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars]
[Perl::Critic::Policy::Miscellanea::RequireRcsKeywords]
keywords = Revision
END_PROFILE

    my $up = Perl::Critic::UserProfile->new( profile => \$long_profile_string )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Miscellanea::RequireRcsKeywords'),
        1,
        'Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('NamingConventions::ProhibitMixedCaseVars'),
        1,
        'NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Now using long policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        1,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords is enabled.',
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars'),
        1,
        'Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseVars is disabled.',
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Miscellanea::RequireRcsKeywords'),
        %policy_params,
        'Perl::Critic::Policy::Miscellanea::RequireRcsKeywords got the correct configuration.',
        )

    # Using bogus policy names
    is(
        $up->policy_is_enabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't enabled>,
        )
    is(
        $up->policy_is_disabled('Perl::Critic::Policy::Bogus'),
        q{},
        q<Bogus Policy isn't disabled>,
        )
    is_deeply(
        $up->raw_policy_params('Perl::Critic::Policy::Bogus'),
        (%: ),
        q<Bogus Policy doesn't have any configuration.>,
        )


#-----------------------------------------------------------------------------
# Test exception handling

do
    my $code_ref = sub (@< @_) { return }
    dies_like(
        { Perl::Critic::UserProfile->new( profile => $code_ref ) },
        qr/CODE can not be used as a string/ms,
        'Invalid profile type',
        )

    try { Perl::Critic::UserProfile->new( profile => 'bogus' ) }
    like(
        $^EVAL_ERROR->message,
        qr/File [ ] 'bogus' [ ] does [ ] not [ ] exist/xms,
        'Invalid profile path',
        )

    my $invalid_syntax = '[Foo::Bar' # Missing "]"
    try { Perl::Critic::UserProfile->new( profile => \$invalid_syntax ) }
    like(
        $^EVAL_ERROR->message,
        qr/Syntax [ ] error [ ] at [ ] line/xms,
        'Invalid profile syntax',
        )

    $invalid_syntax = 'severity 2' # Missing "="
    try { Perl::Critic::UserProfile->new( profile => \$invalid_syntax ) }
    like(
        $^EVAL_ERROR->message,
        qr/Syntax [ ] error [ ] at [ ] line/xms,
        'Invalid profile syntax',
        )



#-----------------------------------------------------------------------------
# Test profile finding

do
    my $expected = 'foo'
    env::var('PERLCRITIC' ) = 'foo'
    my $got = Perl::Critic::UserProfile::_find_profile_path()
    is( $got, $expected, 'PERLCRITIC environment variable')


#-----------------------------------------------------------------------------

# ensure we run true if this test is loaded by
# t/10_userprofile.t_without_optional_dependencies.t
1

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
#   indent-tabs-mode: nil
#   c-indentation-style: bsd
# End:
# ex: set ts=8 sts=4 sw=4 tw=78 ft=perl expandtab shiftround :
