#!/usr/bin/perl -w

use Test::More tests => 33
use lib 't/lib'

BEGIN 
    use_ok( 'Pod::Coverage' )
    use_ok( 'Pod::Coverage::ExportOnly' )


my $obj = Pod::Coverage->new( package => 'Simple1')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 2/3, "Simple1 has 2/3rds coverage")

$obj = Pod::Coverage->new( package => 'Simple2')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 0.75, "Simple2 has 75\% coverage")

ok( eq_array(\ $obj->naked, \(@:  'naked' )), "naked isn't covered")

ok( eq_array(\ $obj->naked, \ $obj->uncovered), "naked is uncovered")

$obj = Pod::Coverage->new( package => 'Simple2', private => \(@:  'naked' ))
isa_ok( $obj, 'Pod::Coverage' )

is ( $obj->coverage, 1, "nakedness is a private thing" )

$obj = Pod::Coverage->new( package => 'Simple1', also_private => \(@:  'bar' ))
isa_ok( $obj, 'Pod::Coverage' )

is ( $obj->coverage, 1, "it's also a private bar" )

ok( eq_array(sort($obj->covered), (@:  'baz', 'foo' )), "those guys are covered" )

$obj = Pod::Coverage->new( package => 'Pod::Coverage')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1, "Pod::Coverage is covered" )

$obj = Pod::Coverage->new( package => 'Simple3')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1, 'Simple3 is covered' )

$obj = Pod::Coverage->new( package => 'Simple4')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1, "External .pod grokked" )

$obj = Pod::Coverage->new( package => 'Simple5')
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1, "Multiple docs per item works" )

$obj = Pod::Coverage->new( package => "Simple6")
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1/3, "Simple6 is 2/3rds with no extra effort" )

$obj = Pod::Coverage::ExportOnly->new( package => "Simple6")
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage, 1/2, "Simple6 is 50\% if you only check exports" )

$obj = Pod::Coverage->new( package => "Simple8")
isa_ok( $obj, 'Pod::Coverage' )

is( $obj->coverage,    undef, "can't deduce for Simple8" )
is( $obj->why_unrated, 'no public symbols defined', 'why is correct' )

$obj = Pod::Coverage->new( package => 'Earle' )
is( $obj->coverage, 1, "earle is covered" )
is( nelems($obj->covered), 2 )

$obj = Pod::Coverage->new( package => 'Args' )
is( $obj->coverage, 1, "Args is covered" )

$obj = Pod::Coverage->new( package => 'XS4ALL' )
is( $obj->coverage, 1, "XS4ALL is covered" )

$obj = Pod::Coverage->new( package => 'Fully::Qualified' )
is( $obj->coverage, 1, "Fully::Qualified is covered" )
