#!/usr/bin/perl

use warnings

use t::Util

use Scriptalicious

use Test::More tests => 17

my (@: $rc, @out) =  capture_err($^EXECUTABLE_NAME, "-Mlib=lib", "t/fork.pl", "-v")

is($rc, 0, "Command completed successfully")

my $out = join "", @out

like($out, qr/\(parent\)/, "Parent managed to use the timer")
like($out, qr/\(child\)/, "Child managed to use the timer")

# test that file descriptors can be fed in lots of different ways

slop our $testfile, "Hello, world!"
my $output = capture( "-in" => $testfile,
                      $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
like($output, qr/:.*Hello, world!/, "run -in => 'FILENAME'")

$output = capture( "-in" => sub (@< @_) { print $^STDOUT, "Hi there\n" },
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
like($output, qr/:.*Hi there/, "run -in => SUB")

open my $test, "<", "$testfile" or barf "damn! $^OS_ERROR"
$output = capture( "-in" => \$test->*,
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
like($output, qr/:.*Hello, world!/, "run -in => GLOB")
close $test

# output...
$output = capture( "-out" => $testfile,
                   "-in"  => sub (@< @_) { print $^STDOUT, "Loop this!\n" },
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
is($output, "", "run out => 'FILENAME' (no output from capture)")
$output = slurp $testfile
like($output, qr/:.*Loop this!/, "run -out => 'FILENAME'")

$output = capture( "-out" => sub (@< @_) { my $foo = readline $^STDIN;
    slop $testfile, $foo;
},
                   "-in"  => sub (@< @_) { print $^STDOUT, "slopslopslop\n" },
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
is($output, "", "run out => CODE (no output from capture)")
$output = slurp $testfile
like($output, qr/:.*slopslopslop/, "run -out => CODE")

open $test, ">", "$testfile" or barf $^OS_ERROR
$output = capture( "-out" => \$test->*,
                   "-in"  => sub (@< @_) { print $^STDOUT, "suckonthis!\n" },
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl")
is($output, "", "run out => GLOB (no output from capture)")
close $test
$output = slurp $testfile
like($output, qr/:.*suckonthis!/, "run -out => GLOB")

# explicit file descriptors...
slop $testfile, "Burp"
$output = capture( "-in4"  => $testfile,
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl", < qw(-i 4))
like($output, qr/Burp/, "-in4 => 'FILENAME'")

slop $testfile, "Burp"
$output = capture( "-in" => sub (@< @_) { print $^STDOUT, "It should be so easy!\n" },
                   "-out4"  => $testfile,
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl", < qw(-o 4))
is($output, "", "-out4 => 'FILENAME' (no output from capture)")
$output = slurp $testfile
like($output, qr/:.*easy!/, "run -out4 => 'FILENAME'")

# last out!
$output = capture( "-in5" => sub (@< @_) { print $^STDOUT, "slurpamunchalot\n" },
                   "-out4"  => sub (@< @_) { my $foo = readline $^STDIN;
                       slop $testfile, $foo },
                   $^EXECUTABLE_NAME, "-Mlib=lib", "t/loopback.pl", < qw(-o 4 -i 5))
is($output, "", "run -out4 => CODE, -in4 => CODE (no output from capture)")
$output = slurp $testfile
like($output, qr/:.*slurpamunchalot/, "run -out4 => CODE, -in4 => CODE")
