use Sub::Install
use Test::More tests => 16

use warnings

# These tests largely copied from Damian Conway's Sub::Installer tests.

do # Install a sub in a package...
    my $sub_ref = Sub::Install::install_sub(\(%:  code => &ok, as => 'ok1' ))

    ok(type::is_code($sub_ref), 'return value of first install_sub')

    is_deeply($sub_ref, &ok, 'it returns the correct code ref')

    ok1(1, 'installed sub runs')


do
    my $to_avail = eval "use Test::Output; 1"
    SKIP: do
        skip "can't run this test without Test::Output", 1 unless $to_avail
        Sub::Install::install_sub(\(%:  code => &ok, as => 'tmp_ok' ))

        my $expected_warning = <<'END_WARNING'
Subroutine main::tmp_ok redefined at t/install.t line 31
Prototype mismatch: sub main::tmp_ok ($;$) vs ($$;$) at t/install.t line 31
END_WARNING

        my $stderr = Test::Output::stderr_from(
                                               { Sub::Install::install_sub(\(%:  code => &is, as => 'tmp_ok' )) }
                                               )

        $stderr =~ s!\\!/!g
        is(
           $stderr,
           $expected_warning,
           "got expected warning",
           )

do # Install the same sub in the same package...
    my $redef = 0
    my $proto = 0

    local $^WARN_HOOK = sub($error)
        return ($redef = 1) if $error->message =~ m{Subroutine \S+ redef}
        # pass("warned as expected: $_[0]") if $_[0] =~ /redefined/;
        die "unexpected warning:  $($error->message)"
    

    my $sub_ref = Sub::Install::install_sub(\(%:  code => &is, as => 'ok1' ))

    ok($redef, 'correct redefinition warning went to $SIG{__WARN__}')

    ok(type::is_code($sub_ref), 'return value of second install_sub')

    is_deeply($sub_ref, &is, 'install2 returns correct code ref')

    ok1(1,1, 'installed sub runs (with new arguments)')


do # Install in another package...
    my $sub_ref = Sub::Install::install_sub(\(%: 
                                          code => &ok
                                          into => 'Other'
                                          as   => 'ok1'
                                          ))

    ok(type::is_code($sub_ref), 'return value of third install_sub')

    is_deeply($sub_ref, &ok, 'it returns the correct code ref')

    ok1(1,1, 'sub previously installed into main still runs properly')

    package Other;
    ok1(1,   'remotely installed sub runs properly')


do # cross-package installation
    sub Other::Another::foo { return @_[0] }

    my $sub_ref = Sub::Install::install_sub(\(%: 
                                          code => 'foo'
                                          from => 'Other::Another'
                                          into => 'Other::YetAnother'
                                          as   => 'return_lhs'
                                          ))

    isa_ok($sub_ref, 'CODE', 'return value of fourth install_sub')

    ok(
        $sub_ref->& &== &Other::Another::foo,
        'it returns the correct code ref'
        )

    is(
                  Other::Another->foo,
                  'Other::Another',
                  'the original code does what we want',
                  )

    is(
                     Other::YetAnother->return_lhs,
                     'Other::YetAnother',
                     'and the installed code works, too',
                     )

