use Sub::Install < qw(reinstall_sub)
use Test::More tests => 14

use warnings

# These tests largely copied from Damian Conway's Sub::Installer tests.

do # Install a sub in a package...

    my $sub_ref = reinstall_sub(\(%:  code => &ok, as => 'ok1' ))

    ok(type::is_code($sub_ref), 'return value of first install_sub')

    is_deeply($sub_ref, &Test::More::ok, 'it returned the right coderef')

    $sub_ref->(1, 'returned code ref runs')
    ok1(1, "reinstalled sub runs")


do
    my $to_avail = eval "use Test::Output; 1"
    SKIP: do
        skip "can't run this test without Test::Output", 1 unless $to_avail
        Sub::Install::reinstall_sub(\(%:  code => &ok, as => 'tmp_ok' ))

        my $expected_warning = <<'END_WARNING'
Prototype mismatch: sub main::tmp_ok ($;$) vs ($$;$) at t/reinstall.t line 32
END_WARNING

        my $stderr = Test::Output::stderr_from(
                                               { Sub::Install::reinstall_sub(\(%:  code => &is, as => 'tmp_ok' )) }
                                               )

        $stderr =~ s!\\!/!g
        is(
           $stderr,
           $expected_warning,
           "got expected warning",
           )


do # Install the same sub in the same package...
    my $proto = 0

    local $^WARN_HOOK = sub($error)
        die "unexpected warning:  $($error->message)"
    

    my $sub_ref = reinstall_sub(\(%:  code => &is, as => 'ok1' ))

    ok(type::is_code($sub_ref), 'return value of second install_sub')

    is_deeply($sub_ref, &Test::More::is, 'it returned the right coderef')

    $sub_ref->(1, 1, 'returned code ref runs')
    ok1(1,1, 'reinstalled sub reruns')


do # Install in another package...
    my $new_code = sub (@< @_) { ok(1, "remotely installed sub runs") }

    my $sub_ref = reinstall_sub(\(%: 
                              code => $new_code
                              into => 'Other'
                              as   => 'ok1'
                              ))

    ok(type::is_code($sub_ref), 'return value of third install_sub')

    is_deeply($sub_ref, $new_code, 'it returned the right coderef')

    ok1(1,1, 'reinstalled sub reruns')

    package Other;
    ok1()


dies_like( {
        my $arg = \(%:  code => sub {}, into => 'Other', as => 'ok1' );
        Sub::Install::_build_public_installer(&Sub::Install::_install_fatal)->($arg);
    },
    qr/redefine/, "(experimental fatal installer should croak)")
