#!/usr/bin/perl -w


use lib 't/lib'

use Test::More tests => 5

use File::Spec
use Test::Harness

do

    #todo_skip 'Harness compatibility incomplete', 5;
    #local $TODO = 'Harness compatibility incomplete';
    my $died

    sub prepare_for_death
        $died = 0
        return sub (@< @_) { $died = 1 }
    

    my $curdir = File::Spec->curdir
    my $sample_tests
        = env::var('PERL_CORE')
        ?? File::Spec->catdir( $curdir, 'lib', 'sample-tests' )
        !! File::Spec->catdir( $curdir, 't',   'sample-tests' )

    do
        try {
            local $^DIE_HOOK = prepare_for_death();
            _runtests( File::Spec->catfile( $sample_tests, "simple" ) );
        }
        ok( !$^EVAL_ERROR, "simple lives" )
        is( $died, 0, "Death never happened" )
    

    do
        try {
            local $^DIE_HOOK = prepare_for_death();
            _runtests( File::Spec->catfile( $sample_tests, "too_many" ) );
        }
        ok( $^EVAL_ERROR, "error OK" )
        ok( $^EVAL_ERROR->{?description} =~ m[Failed 1/1], "too_many dies" )
        is( $died, 1, "Death happened" )
    


sub _runtests
    my (@tests) = @_

    local env::var('PERL_TEST_HARNESS_DUMP_TAP' ) = 0
    local env::var('HARNESS_VERBOSE'            ) = 0
    local env::var('HARNESS_DEBUG'              ) = 0
    local env::var('HARNESS_TIMER'              ) = 0

    local $Test::Harness::Verbose = -9

    runtests(< @tests)


# vim:ts=4:sw=4:et:sta
