#!/usr/bin/perl -w

# Test that $^INCLUDE_PATH is propogated from the harness process to the test
# process.


use lib 't/lib'
use env

sub has_crazy_patch
    my $sentinel = 'blirpzoffle'
    local env::var('PERL5LIB' ) = $sentinel
    my $command = join ' ', map {qq{"$_"}}, @:  ( $^EXECUTABLE_NAME, '-e', 'print \*STDOUT, join q(:), $^INCLUDE_PATH' )
    my $path = `$command`
    my @got = @:  $path =~ m/($sentinel)/g 
    return (nelems @got) +> 1


use Test::More (
      $^OS_NAME eq 'VMS' ?? ( skip_all => 'VMS' )
      !! has_crazy_patch() ?? ( skip_all => 'Incompatible $^INCLUDE_PATH patch' )
      !! ( tests => 1 )
      )

use Data::Dumper
use Test::Harness

# Change $^INCLUDE_PATH so we ensure it's preserved.
use lib 'wibble'

# TODO: Disabled until we find out why it's breaking on Windows. It's
# not strictly a TODO because it seems pretty likely that it's a Windows
# problem rather than a problem with Test::Harness.

# Put a stock directory near the beginning.
# use lib $^INCLUDE_PATH[$#INC-2];

my $inc = Data::Dumper->new( \(@:  $^INCLUDE_PATH ) )->Terse(1)->Purity(1)->Dump

# The tail of $^INCLUDE_PATH is munged during core testing. We're only *really*
# interested in whether 'wibble' makes it anyway.
my $cmp_slice = env::var('PERL_CORE') ?? '[0..1]' !! ''

my $test_template = <<'END'
#!/usr/bin/perl %s

use Test::More tests => 2;

sub _strip_dups {
    my %%dups;
    # Drop '.' which sneaks in on some platforms
    my @r = grep { $_ ne '.' }, grep { !%%dups{+$_}++ }, @_;
    return @r%s;
}

# Make sure we did something sensible with PERL5LIB
like env::var(q[PERL5LIB]), qr{wibble};

do
 is_deeply(
    \(@: _strip_dups(<$^INCLUDE_PATH)),
    \@(: _strip_dups(<%s)),
    '$^INCLUDE_PATH propagated to test'
    ) or do
        diag join ",\n", < _strip_dups($^INCLUDE_PATH)
        diag '-----------------'
        diag join ",\n", < _strip_dups(< @(: %s ) )
END

open my $test, ">", "inc_check.t.tmp"
printf $test, $test_template, '', $cmp_slice, $inc, $inc
close $test

END { 1 while unlink 'inc_check.t.tmp'; }

for my $test ((@:  'inc_check.t.tmp') )
    my (@:  $tot, $failed, ... ) =  Test::Harness::execute_tests( tests => \(@: $test) )
    is $tot->{?bad}, 0

1
