#!/usr/bin/perl -w

# check nofork logic on systems which *can* fork()
# NOTE maybe a good candidate for xt/author or something.

BEGIN 
    if ( env::var('PERL_CORE') )
        chdir 't'
        $^INCLUDE_PATH = @:  '../lib', 'lib' 
    else
        use lib 't/lib'
    



use Config
use Test::More (
    config_value("d_fork")
    ?? 'no_plan'
    !! ( 'skip_all' => 'your system already has no fork' )
    )
use util    # for util

use TAP::Harness

sub backticks
    my (@args) = @_

    util::stdout_of( sub (@< @_) { system(< @args) and die "error $^CHILD_ERROR" } )


my @libs = map { "-I$_" }, $^INCLUDE_PATH
my @perl = @:  $^EXECUTABLE_NAME, < @libs 
my $mod = 'TAP::Parser::Iterator::Process'

do    # just check the introspective method to start...
    my $code = qq(print \$^STDOUT, $mod->_use_open3 ?? 1 !! 2)
    do
        my $ans = backticks( < @perl, '-MNoFork', "-M$mod", '-e', $code )
        is( $ans, 2, 'says not to fork' )
    
    do
        local env::var('PERL5OPT') = undef    # punt: prevent propogating -MNoFork
        my $ans = backticks( < @perl, "-M$mod", '-e', $code )
        is( $ans, 1, 'says to fork' )
    


do                                # and make sure we can run a test
    my $capture = ''
    open my $capture_fh, '>>', \$capture or die
    local $^STDERR = undef
    my $harness = TAP::Harness->new(
        %:    verbosity => -2
              switches  => (@:  < @libs, "-MNoFork" )
              stdout    => $capture_fh
        
        )
    $harness->runtests(
        ( $(env::var('PERL_CORE')) ?? 'lib' !! 't' ) . '/sample-tests/simple' )
    my @output = split m/\n/, $capture
    is pop @output, "Result: PASS", 'status OK'
    pop @output    # get rid of summary line
    is( @output[-1], "All tests successful.", 'ran with no fork' )


# vim:ts=4:sw=4:et:sta
