#!/usr/bin/perl -w

BEGIN 
    unshift $^INCLUDE_PATH, 't/lib'



use Test::More tests => 27

use File::Spec

use TAP::Parser::Source
use TAP::Parser::Source::Perl

my $test = File::Spec->catfile(
    ( env::var('PERL_CORE') ?? 'lib' !! 't' ), 'source_tests',
    'source'
    )

my $perl = $^EXECUTABLE_NAME

can_ok 'TAP::Parser::Source', 'new'
my $source = TAP::Parser::Source->new
isa_ok $source, 'TAP::Parser::Source'

can_ok $source, 'source'
try { $source->source("$perl -It/lib $test") }
ok((my $error = $^EVAL_ERROR),
   '... and calling it with a string should fail')
like $error->message, qr/^Argument to &source must be an array reference/,
     '... with an appropriate error message'
ok $source->source( \(@:  $perl, '-It/lib', '-w', $test ) ),
   '... and calling it with valid args should succeed'

can_ok $source, 'get_stream'
my $stream = $source->get_stream

isa_ok $stream, 'TAP::Parser::Iterator::Process',
       'get_stream returns the right object'
can_ok $stream, 'next'
is $stream->next, '1..1', '... and the first line should be correct'
is $stream->next, 'ok 1', '... as should the second'
ok !$stream->next, '... and we should have no more results'

can_ok 'TAP::Parser::Source::Perl', 'new'
$source = TAP::Parser::Source::Perl->new
isa_ok $source, 'TAP::Parser::Source::Perl', '... and the object it returns'

can_ok $source, 'source'
ok $source->source( \(@: $test) ),
   '... and calling it with valid args should succeed'

can_ok $source, 'get_stream'
$stream = $source->get_stream

isa_ok $stream, 'TAP::Parser::Iterator::Process',
       '... and the object it returns'
can_ok $stream, 'next'
is $stream->next, '1..1', '... and the first line should be correct'
is $stream->next, 'ok 1', '... as should the second'
ok !$stream->next, '... and we should have no more results'

# coverage test for TAP::PArser::Source

do

    # coverage for method get_steam

    my $source = TAP::Parser::Source->new()

    dies_like
        { $source->get_stream },
        qr/No command found!/, '...and it failed as expect'


do

    # coverage testing for error

    my $source = TAP::Parser::Source->new()

    my $error = $source->error

    is $error, undef, 'coverage testing for error()'

    $source->error('save me')

    $error = $source->error

    is $error, 'save me', '...and we got the expected message'


do

    # coverage testing for exit

    my $source = TAP::Parser::Source->new()

    my $exit = $source->exit

    is $exit, undef, 'coverage testing for exit()'

    $source->exit('save me')

    $exit = $source->exit

    is $exit, 'save me', '...and we got the expected message'

