# -*- perl -*-


our ($loaded)

$^OUTPUT_AUTOFLUSH = 1
print $^STDOUT, "1..72\n"

require Text::CSV_XS


############################################################################

package IO_Scalar   # IO::Scalar replacement, because IO::Scalar is not
# yet a Core module.

sub new($proto, ?$strRef)
    my($self)
    if (!$strRef)
        my($str) = ""
        $self = \$str
    elsif (ref($strRef) ne 'SCALAR')
        die "Expected scalar ref"
    else
        $self = \$strRef->$
    
    bless($self, (ref($proto) || $proto))
    $self


sub print ($self, @< @args)
    while (@args)
        my $str = shift(@args)
        if (defined($str))
            $self->$ .= $str
        
    
    1


sub getline ($self)
    my($result)
    my $ifs = $^INPUT_RECORD_SEPARATOR
    if (length($self->$) == 0)
        $result = undef
    elsif (defined($ifs)  &&  $self->$ =~ m/^(.*?$ifs)(.*)$/s)
        $result = $1
        $self->$ = $2
    else
        $result = $self->$
        $self->$ = ''
    
    $result


sub sref ($v)
    $v


sub Contents ($v)
    $v->sref->$


sub flush (?_)
    1


############################################################################


my($testNum) = 0
sub Test($result)
    $testNum++
    print($^STDOUT, ($result ?? "" !! "not "), "ok $testNum\n")
    $result

sub TestContents($csv, $fh, @< @input)
    Test($csv->combine(< @input)) or print $^STDOUT, "Failed to parse input"
    my $got = $fh->Contents()
    Test($csv->string() eq $got)
        or printf($^STDOUT, "Expected \%s, got \%s\n", < $csv->string(), $got)

sub TestPrintRead($csv, @< @input)
    my $fh = IO_Scalar->new()

    Test($csv->print($fh, \@input))
    TestContents($csv, $fh, < @input)
    Test($csv->getline($fh))
        or print($^STDOUT, "Failed to read.\n")
    Test(nelems($csv->fields()) == nelems(@input))
        or print($^STDOUT, "Expected \%d fields, got \%d\n",
        scalar($csv->fields()), scalar(nelems(@input)))
    for my $i ( 0 .. nelems(@input) -1 )
        Test($csv->fields[$i] eq @input[$i])
            or printf($^STDOUT, "Expected field $i to be '\%s', got '\%s'\n",
            @input[$i], $csv->fields[$i])
    

sub TestReadFailure($csv, $input)
    my $fh = IO_Scalar->new()
    if (!$fh->print($input)  ||  !$fh->flush())
        die "Error while creating input file: $^OS_ERROR"
    
    Test(!$csv->getline($fh))

sub TestRead($csv, $input, @< @expected)
    my $fh = IO_Scalar->new()
    if (!$fh->print($input)  ||  !$fh->flush())
        die "Error while creating input file: $^OS_ERROR"
    
    my $fields = $csv->getline($fh)
    Test($fields) or print($^STDOUT, "Failed to read\n")
    Test(nelems(@expected) == nelems($fields->@))
        or printf($^STDOUT, "Expected \%d fields, got \%d\n",
        scalar(nelems(@expected)), scalar($csv->fields()))
    for my $i ( 0 .. nelems(@expected) -1 )
        if (@expected[$i] ne $fields->[$i])
            printf($^STDOUT, "Field $i: Expected \%s, got \%s\n",
                @expected[$i], $fields->[$i])
        
    



my $csv = Text::CSV_XS->new()

my $fh = IO_Scalar->new()
Test(!$csv->print($fh, \(@: "abc", "def\007", "ghi")))
    or print $^STDOUT, "Bad character, but no failure\n"
TestPrintRead($csv, q(""))
TestPrintRead($csv, '', '')
TestPrintRead($csv, '', 'I said, "Hi!"', '')
TestPrintRead($csv, '"', 'abc')
TestPrintRead($csv, 'abc', '"')
TestPrintRead($csv, 'abc', 'def', 'ghi')
TestPrintRead($csv, "abc\tdef", 'ghi')
TestReadFailure($csv, '"abc')
    or print($^STDOUT, "Missing closing double-quote, but no failure\n")
TestReadFailure($csv, 'ab"c')
    or print($^STDOUT, "Double quote outside of double-quotes, but no failure.\n")
TestReadFailure($csv, '"ab"c"')
    or print($^STDOUT, "Bad character sequence, but no failure.\n")
TestReadFailure($csv, qq("abc\nc"))
    or print($^STDOUT, "Bad character, but no failure.\n")
TestRead($csv, q(","), ',')
TestRead($csv, qq("","I said,\t""Hi!""",""),
         '', qq(I said,\t"Hi!"), '')


# This test because of a problem with DBD::CSV

$fh = IO_Scalar->new()
$csv->{+binary} = 1
$csv->{+eol} = "\015\012"
Test($csv->print($fh, \(@: "id","name")))
    or print $^STDOUT, "Bad character, but no failure\n"
Test($csv->print($fh, \(@: 1, "Alligator Descartes")))
Test($csv->print($fh, \(@: "3", "Jochen Wiedmann")))
Test($csv->print($fh, \(@: 2, "Tim Bunce")))
Test($csv->print($fh, \(@: " 4", "Andreas Knig")))
Test($csv->print($fh, \(@: 5)))
my $contents
Test(($contents = $fh->Contents()) eq <<"CONTENTS")
id,name\015
1,"Alligator Descartes"\015
3,"Jochen Wiedmann"\015
2,"Tim Bunce"\015
" 4","Andreas Knig"\015
5\015
CONTENTS

my $fields
print $^STDOUT, "Retrieving data\n"
for my $i (0..5)
    Test($fields = $csv->getline($fh))
        and print $^STDOUT, "Row $i: $(dump::view($fields)) ($(join ' ',$fields->@))\n"

