#!/usr/local/bin/perl -w


use Test::More 
use Text::Diff 

my @A = map { "$_\n" }, qw( 1 2 3 4 ) 
my @B = map { "$_\n" }, qw( 1 2 3 5 ) 

sub _d($v) { diff \@A, \@B, \(%:  OUTPUT => $v ) }

sub slurp { open my $slurp, "<", "" . shift or die $^OS_ERROR ; return join "", (@:  ~< $slurp->*) }

my $expected  = _d undef 

my @tests = @: 
    sub (@< @_) { ok $expected =~ m/\n/ }

    sub (@< @_) { my $o ; _d sub (@< @_) { $o .= shift } ;  is $o,             $expected }

    sub (@< @_) { my @o ; _d \@o ;                  is join( "", @o ), $expected }

    sub (@< @_)
        open my $f, ">", "output.t.foo" or die $^OS_ERROR
        _d \$f->*
        close $f or die $^OS_ERROR
        is slurp("output.t.foo"), $expected
        unlink "output.t.foo" or warn $^OS_ERROR


    sub (@< @_)
        require IO::File
        my $fh = IO::File->new( "output.t.foo", '>' )
        _d $fh
        $fh = undef 
        is slurp("output.t.foo"), $expected
        unlink "output.t.foo" or warn $^OS_ERROR


    sub (@< @_) { ok 0 +< index( diff( \"\n", \"", \(%:  STYLE => "Table" ) ), "\\n" ) }

    # test for bug reported by Ilya Martynov <ilya@martynov.org>
    sub (@< @_) { is diff( \"", \"" ), "" }
    sub (@< @_) { is diff( \"A", \"A" ), "" }
     

plan tests => scalar nelems(@tests) 

for (@tests)
    $_->()
