package URI::urn  # RFC 2141

require URI
our @ISA=qw(URI)

use Carp < qw(carp)

our (%implementor)

sub _init
    my $class = shift
    my $self = $class->SUPER::_init(< @_)
    my $nid = $self->nid

    my $impclass = %implementor{?$nid}
    return $impclass->_urn_init($self, $nid) if $impclass

    $impclass = "URI::urn"
    if ($nid =~ m/^[A-Za-z\d][A-Za-z\d\-]*\z/)
        my $id = $nid
        # make it a legal perl identifier
        $id =~ s/-/_/g
        $id = "_$id" if $id =~ m/^\d/

        $impclass = "URI::urn::$id"

        unless (nelems(Symbol::fetch_glob("$($impclass)::ISA")->*->@))
            # Try to load it
            eval "require $impclass"
            die $^EVAL_ERROR if $^EVAL_ERROR && $^EVAL_ERROR->{?description} !~ m/Can\'t locate.*in \@INC/
            $impclass = "URI::urn" unless nelems(Symbol::fetch_glob("$($impclass)::ISA")->*->@)
        
    else
        carp("Illegal namespace identifier '$nid' for URN '$self'") if $^WARNING
    
    %implementor{+$nid} = $impclass

    return $impclass->_urn_init($self, $nid)


sub _urn_init($class, $self, $nid)
    bless $self, $class


sub _nid
    my $self = shift
    my $opaque = $self->opaque
    if (nelems(@_))
        my $v = $opaque
        my $new = shift
        $v =~ s/[^:]*/$new/
        $self->opaque($v)
    # XXX possible rebless
    
    $opaque =~ s/:.*//s
    return $opaque


sub nid  # namespace identifier
    my $self = shift
    my $nid = $self->_nid(< @_)
    $nid = lc($nid) if defined($nid)
    return $nid


sub nss  # namespace specific string
    my $self = shift
    my $opaque = $self->opaque
    if (nelems(@_))
        my $v = $opaque
        my $new = shift
        if (defined $new)
            $v =~ s/(:|\z).*/:$new/
        else
            $v =~ s/:.*//s
        
        $self->opaque($v)
    
    return undef unless $opaque =~ s/^[^:]*://
    return $opaque


sub canonical
    my $self = shift
    my $nid = $self->_nid
    my $new = $self->SUPER::canonical
    return $new if $nid !~ m/[A-Z]/ || $nid =~ m/%/
    $new = $new->clone if $new == $self
    $new->nid(lc($nid))
    return $new


1
