#!perl -w

use URI::file
use Test::More

my @tests =  @: 
    \(@:  "file",          "unix",       "win32",         "mac" )
    #----------------  ------------  ---------------  --------------
    \(@:  "file://localhost/foo/bar"
          "!/foo/bar",  "!\\foo\\bar",   "!foo:bar", )
    \(@:  "file:///foo/bar"
          "/foo/bar",   "\\foo\\bar",    "!foo:bar", )
    \(@:  "file:/foo/bar", "!/foo/bar",  "!\\foo\\bar",   "foo:bar", )
    \(@:  "foo/bar",       "foo/bar",    "foo\\bar",      ":foo:bar",)
    \(@:  "file://foo/bar","!//foo/bar", "!\\\\foo\\bar", "!foo:bar")
    \(@:  "file://a:/",    "!//a:/",     "!A:\\",         undef)
    \(@:  "file:///A:/",   "/A:/",       "A:\\",          undef)
    \(@:  "file:///",      "/",          "\\",            undef)
    \(@:  ".",             ".",          ".",             ":")
    \(@:  "..",            "..",         "..",            "::")
    \(@:  "\%2E",           "!.",         "!.",           ":.")
    \(@:  "../\%2E\%2E",     "!../..",     "!..\\..",      "::..")
    

my @os = (shift @tests)->@
shift @os  # file

my $num = nelems(@tests)
plan tests => $num + 2

for my $t ( @tests)
    my @t = $t->@
    my $file = shift @t
    my $err

    my $u = URI->new($file, "file")
    my $i = 0
    for my $os ( @os)
        my $f = $u->file($os)
        my $expect = @t[$i]
        $f = "<undef>" unless defined $f
        $expect = "<undef>" unless defined $expect
        my $loose
        $loose++ if $expect =~ s/^!//
        if ($expect ne $f)
            print $^STDOUT, "URI->new('$file', 'file')->file('$os') ne $expect, but $f\n"
            $err++
        
        if (defined(@t[$i]) && !$loose)
            my $u2 = URI::file->new(@t[$i], $os)
            unless ($u2->as_string eq $file)
                print $^STDOUT, "URI::file->new('@t[$i]', '$os') ne $file, but $u2\n"
                $err++
            
        
        $i++
    
    ok( ! $err )


my $u = URI::file->new_abs("t/file.t")
like $u->file, qr|t/file\.t$|
like $u->as_string, qr|file://.*t/file\.t|
