#!perl -w

use Test::More

plan tests => 47

use URI

my $foo = URI->new("Foo:opaque#frag")

ok( ref($foo) eq "URI::_foreign")

ok( $foo->as_string eq "Foo:opaque#frag")

# Try accessors
ok( $foo->_scheme eq "Foo" && $foo->scheme eq "foo")

ok( $foo->opaque eq "opaque")

ok( $foo->fragment eq "frag")

ok( $foo->canonical->as_string eq "foo:opaque#frag" )

# Try modificators
my $old = $foo->scheme("bar")

ok( $old eq "foo" && $foo->as_string eq "bar:opaque#frag")

$old = $foo->scheme("")
ok( $old eq "bar" && $foo->as_string eq "opaque#frag")

$old = $foo->scheme("foo")
$old = $foo->scheme(undef)

ok( $old eq "foo" && $foo->as_string eq "opaque#frag")

$foo->scheme("foo")


$old = $foo->opaque("xxx")
ok( $old eq "opaque" && $foo->as_string eq "foo:xxx#frag")

$old = $foo->opaque("")
ok( $old eq "xxx" && $foo->as_string eq "foo:#frag")

$old = $foo->opaque(" #?/")
$old = $foo->opaque(undef)
ok( $old eq "\%20\%23?/" && $foo->as_string eq "foo:#frag")

$foo->opaque("opaque")


$old = $foo->fragment("x")
ok( $old eq "frag" && $foo->as_string eq "foo:opaque#x")

$old = $foo->fragment("")
ok( $old eq "x" && $foo->as_string eq "foo:opaque#")

$old = $foo->fragment(undef)
ok( $old eq "" && $foo->as_string eq "foo:opaque")


# Compare
ok( $foo->eq("Foo:opaque") &&
    $foo->eq(URI->new("FOO:opaque")) &&
    $foo->eq("foo:opaque") )

ok( not ( $foo->eq("Bar:opaque") ||
    $foo->eq("foo:opaque#") ) )


# Try hierarchal unknown URLs

$foo = URI->new("foo://host:80/path?query#frag")

ok( $foo->as_string eq "foo://host:80/path?query#frag")

# Accessors
ok( $foo->scheme eq "foo")

ok( $foo->authority eq "host:80")

ok( $foo->path eq "/path")

ok( $foo->query eq "query")

ok( $foo->fragment eq "frag")

# Modificators
$old = $foo->authority("xxx")
ok( $old eq "host:80" && $foo->as_string eq "foo://xxx/path?query#frag")

$old = $foo->authority("")
ok( $old eq "xxx" && $foo->as_string eq "foo:///path?query#frag")

$old = $foo->authority(undef)
ok( $old eq "" && $foo->as_string eq "foo:/path?query#frag")

$old = $foo->authority("/? #;@&")
ok( !defined($old) && $foo->as_string eq "foo://\%2F\%3F\%20\%23;@&/path?query#frag")

$old = $foo->authority("host:80")
ok( $old eq "\%2F\%3F\%20\%23;@&" && $foo->as_string eq "foo://host:80/path?query#frag")


$old = $foo->path("/foo")
ok( $old eq "/path" && $foo->as_string eq "foo://host:80/foo?query#frag")

$old = $foo->path("bar")
ok( $old eq "/foo" && $foo->as_string eq "foo://host:80/bar?query#frag")

$old = $foo->path("")
ok( $old eq "/bar" && $foo->as_string eq "foo://host:80?query#frag")

$old = $foo->path(undef)
ok( $old eq "" && $foo->as_string eq "foo://host:80?query#frag")

$old = $foo->path("@;/?#")
ok( $old eq "" && $foo->as_string eq "foo://host:80/@;/\%3F\%23?query#frag")

$old = $foo->path("path")
ok( $old eq "/@;/\%3F\%23" && $foo->as_string eq "foo://host:80/path?query#frag")


$old = $foo->query("foo")
ok( $old eq "query" && $foo->as_string eq "foo://host:80/path?foo#frag")

$old = $foo->query("")
ok( $old eq "foo" && $foo->as_string eq "foo://host:80/path?#frag")

$old = $foo->query(undef)
ok( $old eq "" && $foo->as_string eq "foo://host:80/path#frag")

$old = $foo->query("/?&=# ")
ok( !defined($old) && $foo->as_string eq "foo://host:80/path?/?&=\%23\%20#frag")

$old = $foo->query("query")
ok( $old eq "/?&=\%23\%20" && $foo->as_string eq "foo://host:80/path?query#frag")

# Some buildup trics
$foo = URI->new("")
$foo->path("path")
$foo->authority("auth")

ok( $foo->as_string eq "//auth/path")

$foo = URI->new("", "http:")
$foo->query("query")
$foo->authority("auth")
ok( $foo->as_string eq "//auth?query")

$foo->path("path")
ok( $foo->as_string eq "//auth/path?query")

$foo = URI->new("")
$old = $foo->path("foo")
ok( $old eq "" && $foo->as_string eq "foo")

$old = $foo->path("bar")
ok( $old eq "foo" && $foo->as_string eq "bar")

$old = $foo->opaque("foo")
ok( $old eq "bar" && $foo->as_string eq "foo")

$old = $foo->path("")
ok( $old eq "foo" && $foo->as_string eq "")

$old = $foo->query("q")
ok( !defined($old) && $foo->as_string eq "?q")

