#!perl -w

print $^STDOUT, "1..22\n"

use URI

my $uri

$uri = URI->new("ldap://host/dn=base?cn,sn?sub?objectClass=*")

print $^STDOUT, "not " unless $uri->host eq "host"
print $^STDOUT, "ok 1\n"

print $^STDOUT, "not " unless $uri->dn eq "dn=base"
print $^STDOUT, "ok 2\n"

print $^STDOUT, "not " unless join("-",$uri->attributes) eq "cn-sn"
print $^STDOUT, "ok 3\n"

print $^STDOUT, "not " unless $uri->scope eq "sub"
print $^STDOUT, "ok 4\n"

print $^STDOUT, "not " unless $uri->filter eq "objectClass=*"
print $^STDOUT, "ok 5\n"

$uri = URI->new("ldap:")
$uri->dn("o=University of Michigan,c=US")

print $^STDOUT, "not " unless $uri->as_string eq "ldap:o=University\%20of\%20Michigan,c=US" &&
    $uri->dn eq "o=University of Michigan,c=US"
print $^STDOUT, "ok 6\n"

$uri->host("ldap.itd.umich.edu")
print $^STDOUT, "not " unless $uri->as_string eq "ldap://ldap.itd.umich.edu/o=University\%20of\%20Michigan,c=US"
print $^STDOUT, "ok 7\n"

# check defaults
print $^STDOUT, "not " unless $uri->_scope  eq "" &&
    $uri->scope   eq "base" &&
    $uri->_filter eq "" &&
    $uri->filter  eq "(objectClass=*)"
print $^STDOUT, "ok 8\n"

# attribute
$uri->attributes("postalAddress")
print $^STDOUT, "not " unless $uri->as_string eq "ldap://ldap.itd.umich.edu/o=University\%20of\%20Michigan,c=US?postalAddress"
print $^STDOUT, "ok 9\n"

# does attribute escapeing work as it should
$uri->attributes( <$uri->attributes, "foo", ",", "*", "?", "#", "\0")

print $^STDOUT, "not " unless join("-", $uri->attributes) eq "postalAddress-foo-,-*-?-#-\0"
print $^STDOUT, "ok 10\n"
$uri->attributes("")

$uri->scope("sub?#")
print $^STDOUT, "not " unless $uri->query eq "?sub\%3F\%23" &&
    $uri->scope eq "sub?#"
print $^STDOUT, "ok 11\n"
$uri->scope("")

$uri->filter("f=?,#")
print $^STDOUT, "not " unless $uri->query eq "??f=\%3F,\%23" &&
    $uri->filter eq "f=?,#"

$uri->filter("(int=\\00\\00\\00\\04)")
print $^STDOUT, "not " unless $uri->query eq "??(int=\%5C00\%5C00\%5C00\%5C04)"
print $^STDOUT, "ok 12\n"


print $^STDOUT, "ok 13\n"
$uri->filter("")

$uri->extensions("!bindname" => "cn=Manager,co=Foo")
my %ext = %:  < $uri->extensions 

print $^STDOUT, "not " unless $uri->query eq "???!bindname=cn=Manager\%2Cco=Foo" &&
    nelems(keys %ext) == 1 &&
    %ext{?"!bindname"} eq "cn=Manager,co=Foo"
print $^STDOUT, "ok 14\n"

$uri = URI->new("ldap://LDAP-HOST:389/o=University\%20of\%20Michigan,c=US?postalAddress?base?ObjectClass=*?FOO=Bar,bindname=CN\%3DManager\%CO\%3dFoo")

print $^STDOUT, "not " unless $uri->canonical->as_string eq "ldap://ldap-host/o=University\%20of\%20Michigan,c=US?postaladdress???foo=Bar,bindname=CN=Manager\%CO=Foo"
print $^STDOUT, "ok 15\n"

$uri = URI->new("ldaps://host/dn=base?cn,sn?sub?objectClass=*")

print $^STDOUT, "not " unless $uri->host eq "host"
print $^STDOUT, "ok 16\n"
print $^STDOUT, "not " unless $uri->port eq 636
print $^STDOUT, "ok 17\n"
print $^STDOUT, "not " unless $uri->dn eq "dn=base"
print $^STDOUT, "ok 18\n"

$uri = URI->new("ldapi://\%2Ftmp\%2Fldap.sock/????x-mod=-w--w----")
print $^STDOUT, "not " unless $uri->authority eq "\%2Ftmp\%2Fldap.sock"
print $^STDOUT, "ok 19\n"
print $^STDOUT, "not " unless $uri->un_path eq "/tmp/ldap.sock"
print $^STDOUT, "ok 20\n"

$uri->un_path("/var/x\@foo:bar/")
print $^STDOUT, "not " unless $uri->as_string eq "ldapi://\%2Fvar\%2Fx\%40foo\%3Abar\%2F/????x-mod=-w--w----"
print $^STDOUT, "ok 21\n"

%ext = %:  < $uri->extensions 
print $^STDOUT, "not " unless %ext{?"x-mod"} eq "-w--w----"
print $^STDOUT, "ok 22\n"

