#!perl -w

use Test::More

plan tests => 18

use URI
use URI::QueryParam

my $u = URI->new("http://www.sol.no?foo=4&bar=5&foo=5")

my $h = $u->query_form_hash
ok($h->{foo}->[0] eq "4" && $h->{foo}->[1] eq "5" && $h->{?bar} eq "5")

$u->query_form_hash(\(%:  a => 1, b => 2))
ok( $u->query eq "a=1&b=2" || $u->query eq "b=2&a=1" )

$u->query("a=1&b=2&a=3&b=4&a=5")
ok( nelems($u->query_param) == 2 && join(":", (@:  < $u->query_param)) eq "a:b" )

is_deeply( $u->query_param("a"), (@: 1,3,5) )

ok( $u->query_param(a => < 11 .. 14) )

ok( $u->query eq "a=11&b=2&a=12&b=4&a=13&a=14" )

ok( join(":", (@:  < $u->query_param(a => 11))) eq "11:12:13:14" )

ok( $u->query eq "a=11&b=2&b=4" )

ok( $u->query_param_delete("a") )

ok( $u->query eq "b=2&b=4" )

$u->query_param_append(a => 1, 3, 5)
$u->query_param_append(b => 6)

ok( $u->query eq "b=2&b=4&a=1&a=3&a=5&b=6" )

$u->query_param(a => \$@)  # same as $u->query_param_delete("a");

ok( $u->query eq "b=2&b=4&b=6" )

$u->query(undef)
$u->query_param(a => 1, 2, 3)
$u->query_param(b => 1)

ok( $u->query eq 'a=3&a=2&a=1&b=1' )

$u->query_param_delete('a')
$u->query_param_delete('b')

ok( ! $u->query )

ok( $u->as_string eq 'http://www.sol.no' )

$u->query(undef)
$u->query_param(a => 1, 2, 3)
$u->query_param(b => 1)

ok( $u->query eq 'a=3&a=2&a=1&b=1' )

$u->query_param('a' => \$@)
$u->query_param('b' => \$@)

ok( ! $u->query )

ok( $u->as_string eq 'http://www.sol.no' )
