#!perl -w

use Test::More

plan tests => 18

use URI ()
my $u = URI->new("", "http")
my @q

$u->query_form(a => 3, b => 4)

ok( $u->as_string eq "?a=3&b=4" )

$u->query_form(a => undef)
ok( $u->as_string eq "?a=" )

$u->query_form("a[=&+#] " => " [=&+#]")
ok( $u->as_string eq "?a\%5B\%3D\%26\%2B\%23\%5D+=+\%5B\%3D\%26\%2B\%23\%5D" )

@q = @:  < $u->query_form 
ok( join(":", (@:  < @q)) eq "a[=&+#] : [=&+#]" )

@q = @:  < $u->query_keywords 
ok( ! nelems(@q) )

$u->query_keywords("a", "b")
ok( $u->as_string eq "?a+b" )

$u->query_keywords(" ", "+", "=", "[", "]")
ok( $u->as_string eq "?\%20+\%2B+\%3D+\%5B+\%5D" )

@q = @:  < $u->query_keywords 
ok( join(":", (@:  < @q)) eq " :+:=:[:]" )

@q = @:  < $u->query_form 
ok( ! nelems(@q) )

$u->query(" +?=#")
ok( $u->as_string eq "?\%20+?=\%23" )

$u->query_keywords(\(@:  <qw(a b)))
ok( $u->as_string eq "?a+b" )

$u->query_keywords(\$@)
ok( $u->as_string eq "" )

$u->query_form(\(%:  a => 1, b => 2 ))
ok( $u->as_string eq "?a=1&b=2" || $u->as_string eq "?b=2&a=1" )

$u->query_form(\(@:  a => 1, b => 2 ))
ok( $u->as_string eq "?a=1&b=2" )

$u->query_form(\$%)
ok( $u->as_string eq "" )

$u->query_form(\(@: a => \(@:  <1..4)))
ok( $u->as_string eq "?a=1&a=2&a=3&a=4" )

$u->query_form(\$@)
ok( $u->as_string eq "" )

dies_like( sub (@< @_) { $u->query_form(a => \(%:  foo => 1 )); },
           qr/substitute expected a plain value/ )

__END__
# Some debugging while writing new tests
print "\@q='", join(":", @q), "'\n";
print "\$u='$u'\n";

