#!perl -w

print $^STDOUT, "1..11\n"

use URI

my $u = URI->new('sip:phone@domain.ext')
print $^STDOUT, "not " unless $u->user eq 'phone' &&
    $u->host eq 'domain.ext' &&
    $u->port eq '5060' &&
    $u->as_string eq 'sip:phone@domain.ext'
print $^STDOUT, "ok 1\n"

$u->host_port('otherdomain.int:9999')
print $^STDOUT, "not " unless $u->host eq 'otherdomain.int' &&
    $u->port eq '9999' &&
    $u->as_string eq 'sip:phone@otherdomain.int:9999'
print $^STDOUT, "ok 2\n"

$u->port('5060')
$u = $u->canonical
print $^STDOUT, "not " unless $u->host eq 'otherdomain.int' &&
    $u->port eq '5060' &&
    $u->as_string eq 'sip:phone@otherdomain.int'
print $^STDOUT, "ok 3\n"

$u->user('voicemail')
print $^STDOUT, "not " unless $u->user eq 'voicemail' &&
    $u->as_string eq 'sip:voicemail@otherdomain.int'
print $^STDOUT, "ok 4\n"

$u = URI->new('sip:phone@domain.ext?Subject=Meeting&Priority=Urgent')
print $^STDOUT, "not " unless $u->host eq 'domain.ext' &&
    $u->query eq 'Subject=Meeting&Priority=Urgent'
print $^STDOUT, "ok 5\n"

$u->query_form(Subject => 'Lunch', Priority => 'Low')
my @q = $u->query_form
print $^STDOUT, "not " unless $u->host eq 'domain.ext' &&
    $u->query eq 'Subject=Lunch&Priority=Low' &&
    nelems(@q) == 4 && "$(join ' ',@q)" eq "Subject Lunch Priority Low"
print $^STDOUT, "ok 6\n"

$u = URI->new('sip:phone@domain.ext;maddr=127.0.0.1;ttl=16')
print $^STDOUT, "not " unless $u->host eq 'domain.ext' &&
    $u->params eq 'maddr=127.0.0.1;ttl=16'
print $^STDOUT, "ok 7\n"

$u = URI->new('sip:phone@domain.ext?Subject=Meeting&Priority=Urgent')
$u->params_form(maddr => '127.0.0.1', ttl => '16')
my @p = $u->params_form
print $^STDOUT, "not " unless $u->host eq 'domain.ext' &&
    $u->query eq 'Subject=Meeting&Priority=Urgent' &&
    $u->params eq 'maddr=127.0.0.1;ttl=16' &&
    nelems(@p) == 4 && "$(join ' ',@p)" eq "maddr 127.0.0.1 ttl 16"

print $^STDOUT, "ok 8\n"

$u = URI->new_abs('sip:phone@domain.ext', 'sip:foo@domain2.ext')
print $^STDOUT, "not " unless $u->as_string eq 'sip:phone@domain.ext'
print $^STDOUT, "ok 9\n"

$u = URI->new('sip:phone@domain.ext')
print $^STDOUT, "not " unless $u->as_string eq $u->abs('http://www.cpan.org/')->as_string
print $^STDOUT, "ok 10\n"

print $^STDOUT, "not " unless $u->as_string eq $u->rel('http://www.cpan.org/')->as_string
print $^STDOUT, "ok 11\n"
