# $Id: XPathContext.pm 422 2002-11-08 17:10:30Z phish $

package XML::LibXML::XPathContext

our ($VERSION, @ISA, $USE_LIBXML_DATA_TYPES)

use Carp
use XML::LibXML
use XML::LibXML::NodeList

$VERSION = "1.65" # VERSION TEMPLATE: DO NOT CHANGE

# should LibXML XPath data types be used for simple objects
# when passing parameters to extension functions (default: no)
$USE_LIBXML_DATA_TYPES = 0

sub findnodes($self, $xpath, ?$node)

    my @nodes = $self->_guarded_find_call('_findnodes', $xpath, $node)

    return @nodes


sub find($self, $xpath, ?$node)

    my (@: $type, @< @params) =  $self->_guarded_find_call('_find', $xpath, $node)

    if ($type)
        return $type->new(< @params)
    
    return undef


sub findvalue
    my $self = shift
    return $self->find(< @_)->to_literal->value


sub _guarded_find_call($self, $method, $xpath, $node)

    my $prev_node
    if (ref($node))
        $prev_node = $self->getContextNode()
        $self->setContextNode($node)
    
    my @ret
    try {
        @ret = $self->?$method($xpath);
    }
    $self->_free_node_pool
    $self->setContextNode($prev_node) if ref($node)

    if ($^EVAL_ERROR)
        die $^EVAL_ERROR
    

    return @ret


sub registerFunction($self, $name, $sub)
    $self->registerFunctionNS($name, undef, $sub)
    return


sub unregisterNs($self, $prefix)
    $self->registerNs($prefix, undef)
    return


sub unregisterFunction($self, $name)
    $self->registerFunctionNS($name, undef, undef)
    return


sub unregisterFunctionNS($self, $name, $ns)
    $self->registerFunctionNS($name, $ns, undef)
    return


sub unregisterVarLookupFunc($self)
    $self->registerVarLookupFunc(undef, undef)
    return


# extension function perl dispatcher
# borrowed from XML::LibXSLT

sub _perl_dispatcher
    my $func = shift
    my @params = @_
    my @perlParams

    my $i = 0
    while (nelems(@params))
        my $type = shift(@params)
        if ($type eq 'XML::LibXML::Literal' or
            $type eq 'XML::LibXML::Number' or
            $type eq 'XML::LibXML::Boolean')
            my $val = shift(@params)
            unshift(@perlParams, $USE_LIBXML_DATA_TYPES ?? < $type->new($val) !! $val)
        elsif ($type eq 'XML::LibXML::NodeList')
            my $node_count = shift(@params)
            unshift(@perlParams, $type->new(splice(@params, 0, $node_count)))
        
    

    $func = "main::$func" unless type::is_code($func) || $func =~ m/(.+)::/
    $func = Symbol::fetch_glob($func)->*->& unless type::is_code($func)
    my $res = $func->(< @perlParams)
    return $res


1
