# $Id: 05text.t 618 2006-11-10 18:18:24Z pajas $

##
# this test checks the DOM Characterdata interface of XML::LibXML

use Test::More

BEGIN { plan tests => 32 };
use XML::LibXML

my $doc = XML::LibXML::Document->new()

do
    print $^STDOUT, "# 1. creation\n"
    my $foo = "foobar"
    my $textnode = $doc->createTextNode($foo)
    ok( $textnode )
    is( $textnode->nodeName(), '#text' )
    is( $textnode->nodeValue(), $foo )

    print $^STDOUT, "# 2. substring\n"
    my $tnstr = $textnode->substringData( 1,2 )
    is( $tnstr , "oo" )
    is( $textnode->nodeValue(), $foo )

    print $^STDOUT, "# 3. Expansion\n"
    $textnode->appendData( $foo )
    is( $textnode->nodeValue(), $foo . $foo )

    $textnode->insertData( 6, "FOO" )
    is( $textnode->nodeValue(), $foo."FOO".$foo )

    $textnode->setData( $foo )
    $textnode->insertData( 6, "FOO" )
    is( $textnode->nodeValue(), $foo."FOO" )
    $textnode->setData( $foo )
    $textnode->insertData( 3, "" )
    is( $textnode->nodeValue(), $foo )

    print $^STDOUT, "# 4. Removement\n"
    $textnode->deleteData( 1,2 )
    is( $textnode->nodeValue(), "fbar" )
    $textnode->setData( $foo )
    $textnode->deleteData( 1,10 )
    is( $textnode->nodeValue(), "f" )
    $textnode->setData( $foo )
    $textnode->deleteData( 10,1 )
    is( $textnode->nodeValue(), $foo )
    $textnode->deleteData( 1,0 )
    is( $textnode->nodeValue(), $foo )
    $textnode->deleteData( 0,0 )
    is( $textnode->nodeValue(), $foo )
    $textnode->deleteData( 0,2 )
    is( $textnode->nodeValue(), "obar" )

    print $^STDOUT, "# 5. Replacement\n"
    $textnode->setData( "test" )
    $textnode->replaceData( 1,2, "phish" )
    is( $textnode->nodeValue(), "tphisht" )
    $textnode->setData( "test" )
    $textnode->replaceData( 1,4, "phish" )
    is( $textnode->nodeValue(), "tphish" )
    $textnode->setData( "test" )
    $textnode->replaceData( 1,0, "phish" )
    is( $textnode->nodeValue(), "tphishest" )


    print $^STDOUT, "# 6. XML::LibXML features\n"
    $textnode->setData( "test" )

    $textnode->replaceDataString( "es", "new" )
    is( $textnode->nodeValue(), "tnewt" )

    $textnode->replaceDataRegEx( 'n(.)w', '$1s' )
    is( $textnode->nodeValue(), "test" )

    $textnode->setData( "blue phish, white phish, no phish" )
    $textnode->replaceDataRegEx( 'phish', 'test' )
    is( $textnode->nodeValue(), "blue test, white phish, no phish" )

    # replace them all!
    $textnode->replaceDataRegEx( 'phish', 'test', 'g' )
    is( $textnode->nodeValue(), "blue test, white test, no test" )

    # check if special chars are encoded properly
    $textnode->setData( "te?st" )
    $textnode->replaceDataString( "e?s", 'ne\w' )
    is( $textnode->nodeValue(), 'tne\wt' )

    # check if entities don't get translated
    $textnode->setData(q(foo&amp;bar))
    ok ( $textnode->getData eq q(foo&amp;bar) )


do
    print $^STDOUT, "# standalone test\n"
    my $node = XML::LibXML::Text->new("foo")
    ok($node)
    is($node->nodeValue, "foo" )


do
    print $^STDOUT, "# CDATA node name test\n"

    my $node = XML::LibXML::CDATASection->new("test")

    is( $node->string_value(), "test" )
    is( $node->nodeName(), "#cdata-section" )


do
    print $^STDOUT, "# Comment node name test\n"

    my $node = XML::LibXML::Comment->new("test")

    is( $node->string_value(), "test" )
    is( $node->nodeName(), "#comment" )


do
    print $^STDOUT, "# Document node name test\n"

    my $node = XML::LibXML::Document->new()

    is( $node->nodeName(), "#document" )

do
    print $^STDOUT, "# Document fragment node name test\n"

    my $node = XML::LibXML::DocumentFragment->new()

    is( $node->nodeName(), "#document-fragment" )

