# $Id: 06elements.t 624 2006-11-14 16:57:34Z pajas $

##
# this test checks the DOM element and attribute interface of XML::LibXML

use Test::More

BEGIN { plan tests => 187 };
use XML::LibXML

my $foo       = "foo"
my $bar       = "bar"
my $nsURI     = "http://foo"
my $prefix    = "x"
my $attname1  = "A"
my $attvalue1 = "a"
my $attname2  = "B"
my $attvalue2 = "b"
my $attname3  = "C"

my @badnames= @: "1A", "<><", "&", "-:"

print $^STDOUT, "# 1. bound node\n"
do
    my $doc = XML::LibXML::Document->new()
    my $elem = $doc->createElement( $foo )
    ok($elem)
    is($elem->tagName, $foo)

    do
        foreach my $name (  @badnames )
            try { $elem->setNodeName( $name ); }
            ok( $^EVAL_ERROR )

    $elem->setAttribute( $attname1, $attvalue1 )
    ok( $elem->hasAttribute($attname1) )
    is( $elem->getAttribute($attname1), $attvalue1)

    my $attr = $elem->getAttributeNode($attname1)
    ok($attr)
    is($attr->name, $attname1)
    is($attr->value, $attvalue1)

    $elem->setAttribute( $attname1, $attvalue2 )
    is($elem->getAttribute($attname1), $attvalue2)
    is($attr->value, $attvalue2)

    my $attr2 = $doc->createAttribute($attname2, $attvalue1)
    ok($attr2)

    $elem->setAttributeNode($attr2)
    ok($elem->hasAttribute($attname2) )
    is($elem->getAttribute($attname2),$attvalue1)

    my $tattr = $elem->getAttributeNode($attname2)
    ok($tattr->isSameNode($attr2))

    $elem->setAttribute($attname2, "")
    ok($elem->hasAttribute($attname2) )
    is($elem->getAttribute($attname2), "")

    $elem->setAttribute($attname3, "")
    ok($elem->hasAttribute($attname3) )
    is($elem->getAttribute($attname3), "")

    do
        foreach my $name (  @badnames )
            try {$elem->setAttribute( $name, "X" );}
            ok( $^EVAL_ERROR )


    print $^STDOUT, "# 1.1 Namespaced Attributes\n"

    $elem->setAttributeNS( $nsURI, $prefix . ":". $foo, $attvalue2 )
    ok( $elem->hasAttributeNS( $nsURI, $foo ) )
    ok( ! $elem->hasAttribute( $foo ) )
    ok( $elem->hasAttribute( $prefix.":".$foo ) )
    # warn $elem->toString() , "\n";
    $tattr = $elem->getAttributeNodeNS( $nsURI, $foo )
    ok($tattr)
    is($tattr->name, $foo)
    is($tattr->nodeName, $prefix .":".$foo)
    is($tattr->value, $attvalue2 )

    $elem->removeAttributeNode( $tattr )
    ok( !$elem->hasAttributeNS($nsURI, $foo) )


    # empty NS
    $elem->setAttributeNS( '', $foo, $attvalue2 )
    ok( $elem->hasAttribute( $foo ) )
    $tattr = $elem->getAttributeNode( $foo )
    ok($tattr)
    is($tattr->name, $foo)
    is($tattr->nodeName, $foo)
    ok(!defined($tattr->namespaceURI))
    is($tattr->value, $attvalue2 )

    ok($elem->hasAttribute($foo) == 1)
    ok($elem->hasAttributeNS(undef, $foo) == 1)
    ok($elem->hasAttributeNS('', $foo) == 1)

    $elem->removeAttributeNode( $tattr )
    ok( !$elem->hasAttributeNS('', $foo) )
    ok( !$elem->hasAttributeNS(undef, $foo) )

    # node based functions
    my $e2 = $doc->createElement($foo)
    $doc->setDocumentElement($e2)
    my $nsAttr = $doc->createAttributeNS( $nsURI.".x", $prefix . ":". $foo, $bar)
    ok( $nsAttr )
    $elem->setAttributeNodeNS($nsAttr)
    ok( $elem->hasAttributeNS($nsURI.".x", $foo) )
    $elem->removeAttributeNS( $nsURI.".x", $foo)
    ok( !$elem->hasAttributeNS($nsURI.".x", $foo) )

    # warn $elem->toString;
    print $^STDOUT, "# set attribute ".$prefix . ":". $attname1."\n"

    $elem->setAttributeNS( $nsURI, $prefix . ":". $attname1, $attvalue2 )
    # warn $elem->toString;


    $elem->removeAttributeNS("",$attname1)
    # warn $elem->toString;

    ok( ! $elem->hasAttribute($attname1) )
    ok( $elem->hasAttributeNS($nsURI,$attname1) )
    # warn $elem->toString;

    do
        foreach my $name (  @badnames )
            try {$elem->setAttributeNS( undef, $name, "X" );}
            ok( $^EVAL_ERROR )


print $^STDOUT, "# 2. unbound node\n"
do
    my $elem = XML::LibXML::Element->new($foo)
    ok($elem)
    is($elem->tagName, $foo)

    $elem->setAttribute( $attname1, $attvalue1 )
    ok( $elem->hasAttribute($attname1) )
    is( $elem->getAttribute($attname1), $attvalue1)

    my $attr = $elem->getAttributeNode($attname1)
    ok($attr)
    is($attr->name, $attname1)
    is($attr->value, $attvalue1)

    $elem->setAttributeNS( $nsURI, $prefix . ":". $foo, $attvalue2 )
    ok( $elem->hasAttributeNS( $nsURI, $foo ) )
    # warn $elem->toString() , "\n";
    my $tattr = $elem->getAttributeNodeNS( $nsURI, $foo )
    ok($tattr)
    is($tattr->name, $foo)
    is($tattr->nodeName, $prefix .":".$foo)
    is($tattr->value, $attvalue2 )

    $elem->removeAttributeNode( $tattr )
    ok( !$elem->hasAttributeNS($nsURI, $foo) )
# warn $elem->toString() , "\n";


print $^STDOUT, "# 3. Namespace handling\n"
print $^STDOUT, "# 3.1 Namespace switching\n"
do
    my $elem = XML::LibXML::Element->new($foo)
    ok($elem)

    my $doc = XML::LibXML::Document->new()
    my $e2 = $doc->createElement($foo)
    $doc->setDocumentElement($e2)
    my $nsAttr = $doc->createAttributeNS( $nsURI, $prefix . ":". $foo, $bar)
    ok( $nsAttr )

    $elem->setAttributeNodeNS($nsAttr)
    ok( $elem->hasAttributeNS($nsURI, $foo) )

    ok( not defined $nsAttr->ownerDocument)
# warn $elem->toString() , "\n";


print $^STDOUT, "# 3.2 default Namespace and Attributes\n"
do
    my $doc  = XML::LibXML::Document->new()
    my $elem = $doc->createElementNS( "foo", "root" )
    $doc->setDocumentElement( $elem )

    $elem->setNamespace( "foo", "bar" )

    $elem->setAttributeNS( "foo", "x:attr",  "test" )
    $elem->setAttributeNS( undef, "attr2",  "test" )

    is( $elem->getAttributeNS( "foo", "attr" ), "test" )
    is( $elem->getAttributeNS( "", "attr2" ), "test" )

    # warn $doc->toString;
    # actually this doesn't work correctly with libxml2 <= 2.4.23
    $elem->setAttributeNS( "foo", "attr2",  "bar" )
    is( $elem->getAttributeNS( "foo", "attr2" ), "bar" )
# warn $doc->toString;


print $^STDOUT, "# 4. Text Append and Normalization\n"

do
    my $doc = XML::LibXML::Document->new()
    my $t1 = $doc->createTextNode( "bar1" )
    my $t2 = $doc->createTextNode( "bar2" )
    my $t3 = $doc->createTextNode( "bar3" )
    my $e  = $doc->createElement("foo")
    $e->appendChild( $t1 )
    $e->appendChild( $t2 )
    $e->appendChild( $t3 )

    my @cn = $e->childNodes

    # this is the correct behaviour for DOM. the nodes are still
    # refered
    is( scalar( nelems(@cn) ), 3 )

    $e->normalize

    @cn = $e->childNodes
    is( scalar( nelems(@cn) ), 1 )

    ok(not defined $t2->parentNode)
    ok(not defined $t3->parentNode)



print $^STDOUT, "# 5. XML::LibXML extensions\n"
do
    my $plainstring = "foo"
    my $stdentstring= "$foo & this"

    my $doc = XML::LibXML::Document->new()
    my $elem = $doc->createElement( $foo )
    $doc->setDocumentElement( $elem )

    $elem->appendText( $plainstring )
    is( $elem->string_value , $plainstring )

    $elem->appendText( $stdentstring )
    is( $elem->string_value , $plainstring.$stdentstring )

    $elem->appendTextChild( "foo")
    $elem->appendTextChild( "foo" => "foo&bar" )

    my @cn = $elem->childNodes
    ok( nelems(@cn) )
    is( scalar(nelems(@cn)), 3 )
    ok( !@cn[1]->hasChildNodes)
    ok( @cn[2]->hasChildNodes)


print $^STDOUT, "# 6. XML::LibXML::Attr nodes\n"
do
    my $dtd = <<'EOF'
<!DOCTYPE root [
<!ELEMENT root EMPTY>
<!ATTLIST root fixed CDATA  #FIXED "foo">
<!ATTLIST root a:ns_fixed CDATA  #FIXED "ns_foo">
<!ATTLIST root name NMTOKEN #IMPLIED>
<!ENTITY ent "ENT">
]>
EOF
    my $ns = q(urn:xx)
    my $xml_nons = qq(<root foo="&quot;bar&ent;&quot;" xmlns:a="$ns"/>)
    my $xml_ns = qq(<root xmlns="$ns" xmlns:a="$ns" foo="&quot;bar&ent;&quot;"/>)

    for my $xml ((@: $xml_nons, $xml_ns))
        my $parser = XML::LibXML->new()
        $parser->complete_attributes(0)
        $parser->expand_entities(0)
        my $doc = $parser->parse_string($dtd.$xml)

        ok ($doc)
        my $root = $doc->getDocumentElement
        do
            my $attr = $root->getAttributeNode('foo')
            ok ($attr)
            is (ref($attr), 'XML::LibXML::Attr')
            ok ($root->isSameNode($attr->ownerElement))
            is ($attr->value, '"barENT"')
            is ($attr->serializeContent, '&quot;bar&ent;&quot;')
            is ($attr->toString, ' foo="&quot;bar&ent;&quot;"')
        
        do
            my $attr = $root->getAttributeNodeNS(undef,'foo')
            ok ($attr)
            is (ref($attr), 'XML::LibXML::Attr')
            ok ($root->isSameNode($attr->ownerElement))
            is ($attr->value, '"barENT"')
        
        # fixed values are defined
        is ($root->getAttribute('fixed'),'foo')
        SKIP:
            do
            skip 'skipping for libxml2 <= 2.6.27', 1 unless XML::LibXML::LIBXML_VERSION +>= 20627
            is($root->getAttributeNS($ns,'ns_fixed'),'ns_foo')
        
        is ($root->getAttribute('a:ns_fixed'),'ns_foo')

        is ($root->hasAttribute('fixed'),0)
        is ($root->hasAttributeNS($ns,'ns_fixed'),0)
        is ($root->hasAttribute('a:ns_fixed'),0)


        # but no attribute nodes correspond to them
        ok (!defined $root->getAttributeNode('a:ns_fixed'))
        ok (!defined $root->getAttributeNode('fixed'))
        ok (!defined $root->getAttributeNode('name'))
        ok (!defined $root->getAttributeNode('baz'))
        ok (!defined $root->getAttributeNodeNS($ns,'foo'))
        ok (!defined $root->getAttributeNodeNS($ns,'fixed'))
        ok (!defined $root->getAttributeNodeNS($ns,'ns_fixed'))
        ok (!defined $root->getAttributeNodeNS(undef,'fixed'))
        ok (!defined $root->getAttributeNodeNS(undef,'name'))
        ok (!defined $root->getAttributeNodeNS(undef,'baz'))
    

    for my $xml ((@: $xml_nons, $xml_ns))
        my $parser = XML::LibXML->new()
        $parser->complete_attributes(1)
        $parser->expand_entities(1)
        my $doc = $parser->parse_string($dtd.$xml)
        ok ($doc)
        my $root = $doc->getDocumentElement
        do
            my $attr = $root->getAttributeNode('foo')
            ok ($attr)
            is (ref($attr), 'XML::LibXML::Attr')
            ok ($root->isSameNode($attr->ownerElement))
            is ($attr->value, '"barENT"')
            is ($attr->serializeContent, '&quot;barENT&quot;')
            is ($attr->toString, ' foo="&quot;barENT&quot;"')
        
        # fixed values are defined
        is ($root->getAttribute('fixed'),'foo')
        is ($root->getAttributeNS($ns,'ns_fixed'),'ns_foo')
        is ($root->getAttribute('a:ns_fixed'),'ns_foo')

        # and attribute nodes are created
        do
            my $attr = $root->getAttributeNode('fixed')
            is (ref($attr), 'XML::LibXML::Attr')
            is ($attr->value,'foo')
            is ($attr->toString, ' fixed="foo"')
        
        do
            my $attr = $root->getAttributeNode('a:ns_fixed')
            is (ref($attr), 'XML::LibXML::Attr')
            is ($attr->value,'ns_foo')
        
        do
            my $attr = $root->getAttributeNodeNS($ns,'ns_fixed')
            is (ref($attr), 'XML::LibXML::Attr')
            is ($attr->value,'ns_foo')
            is ($attr->toString, ' a:ns_fixed="ns_foo"')
        

        ok (!defined $root->getAttributeNode('ns_fixed'))
        ok (!defined $root->getAttributeNode('name'))
        ok (!defined $root->getAttributeNode('baz'))
        ok (!defined $root->getAttributeNodeNS($ns,'foo'))
        ok (!defined $root->getAttributeNodeNS($ns,'fixed'))
        ok (!defined $root->getAttributeNodeNS(undef,'name'))
        ok (!defined $root->getAttributeNodeNS(undef,'baz'))
    

