use Test::More
BEGIN { plan tests => 37 }
use XML::LibXML
use IO::File
ok(1)

my $html = "example/test.html"

my $parser = XML::LibXML->new()
do
    my $doc = $parser->parse_html_file($html)
    ok($doc)


my $fh = IO::File->new($html) || die "Can't open $html: $^OS_ERROR"

my $string
do
    local $^INPUT_RECORD_SEPARATOR = undef
    $string = ~< $fh


seek($fh, 0, 0)

ok($string)

my $doc = $parser->parse_html_string($string)

ok($doc)

undef $doc

$doc = $parser->parse_html_fh($fh)

ok($doc)

$fh->close()

# parsing HTML's CGI calling links

my $strhref = <<EOHTML

<html>
    <body>
        <a href="http:/foo.bar/foobar.pl?foo=bar&bar=foo">
            foo
        </a>
        <p>test
    </body>
</html>
EOHTML

my $htmldoc

$parser->recover(1)
try {
    local $^WARN_HOOK = sub { };
    $htmldoc = $parser->parse_html_string( $strhref );
}

# ok( not $@ );
ok( $htmldoc )

print $^STDOUT, "parse_html_string with encoding...\n"
# encodings
do
    use Encode < qw(encode decode)
    use utf8

    my $utf_str = "ěščř"

    # w/o 'meta' charset, defaults to latin1
    $strhref = <<EOHTML
<html>
  <body>
    <p>$utf_str</p>
  </body>
</html>
EOHTML

    $htmldoc = $parser->parse_html_string( $strhref )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), Encode::decode('latin1', $utf_str))

    $htmldoc = $parser->parse_html_string( $strhref,
        \%: 
        encoding => 'UTF-8'
        
        )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)


    my $iso_str = Encode::encode('iso-8859-2', $strhref)
    $htmldoc = $parser->parse_html_string( $iso_str,
        \%: 
        encoding => 'iso-8859-2'
        
        )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    # w/ 'meta' charset
    $strhref = <<EOHTML
<html>
  <head>
    <meta http-equiv="Content-Type" content="text/html;
      charset=iso-8859-2">
  </head>
  <body>
    <p>$utf_str</p>
  </body>
</html>
EOHTML

    $htmldoc = $parser->parse_html_string( $strhref, \(%:  encoding => 'UTF-8' ))
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    $iso_str = Encode::encode('iso-8859-2', $strhref)
    $htmldoc = $parser->parse_html_string( $iso_str )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    $htmldoc = $parser->parse_html_string( $iso_str, \(%:  encoding => 'iso-8859-2'
                                                           URI => 'foo'
        ) )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)
    is($htmldoc->URI, 'foo')


print $^STDOUT, "parse example/enc_latin2.html...\n"
# w/ 'meta' charset
do
    use utf8
    my $utf_str = "ěščř"
    my $test_file = 'example/enc_latin2.html'
    my $fh

    $htmldoc = $parser->parse_html_file( $test_file )
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    $htmldoc = $parser->parse_html_file( $test_file, \(%:  encoding => 'iso-8859-2'
                                                           URI => 'foo'
        ))
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)
    is($htmldoc->URI, 'foo')

    open $fh, "<", $test_file
    $htmldoc = $parser->parse_html_fh( $fh )
    close $fh
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    open $fh, "<", $test_file
    $htmldoc = $parser->parse_html_fh( $fh, \(%:  encoding => 'iso-8859-2'
                                                  URI => 'foo'
        ))
    close $fh
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->URI, 'foo')
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    if (20627 +> XML::LibXML::LIBXML_VERSION)
        skip("skipping for libxml2 < 2.6.27", 2)
    else
        # translate to UTF8 on perl-side
        open $fh, '<:encoding(iso-8859-2)', $test_file
        $htmldoc = $parser->parse_html_fh( $fh, \(%:  encoding => 'UTF-8' ))
        close $fh
        ok( $htmldoc && $htmldoc->getDocumentElement )
        is($htmldoc->findvalue('//p/text()'), $utf_str)
    


print $^STDOUT, "parse example/enc2_latin2.html...\n"
# w/o 'meta' charset
do
    use utf8
    my $utf_str = "ěščř"
    my $test_file = 'example/enc2_latin2.html'
    my $fh

    $htmldoc = $parser->parse_html_file( $test_file, \(%:  encoding => 'iso-8859-2' ))
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    open $fh, "<", $test_file
    $htmldoc = $parser->parse_html_fh( $fh, \(%:  encoding => 'iso-8859-2' ))
    close $fh
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

    # translate to UTF8 on perl-side
    open $fh, '<:encoding(iso-8859-2)', $test_file
    $htmldoc = $parser->parse_html_fh( $fh, \(%:  encoding => 'UTF-8' ) )
    close $fh
    ok( $htmldoc && $htmldoc->getDocumentElement )
    is($htmldoc->findvalue('//p/text()'), $utf_str)

