use Test::More
BEGIN { plan tests => 19 }
use XML::LibXML
ok(1)

my $dtdstr
do
    local $^INPUT_RECORD_SEPARATOR = undef
    open(my $dtd, "<", 'example/test.dtd') || die $^OS_ERROR
    $dtdstr = ~< $dtd->*
    $dtdstr =~ s/\r//g
    $dtdstr =~ s/[\r\n]*$//
    close $dtd

ok($dtdstr)

do
    # parse a DTD from a SYSTEM ID
    my $dtd = XML::LibXML::Dtd->new('ignore', 'example/test.dtd')
    ok($dtd)
    my $newstr = $dtd->toString()
    $newstr =~ s/\r//g
    $newstr =~ s/^.*?\n//
    $newstr =~ s/\n^.*\Z//m
    is($newstr, $dtdstr)


do
    # parse a DTD from a string
    my $dtd = XML::LibXML::Dtd->parse_string($dtdstr)
    ok($dtd)


do
    # parse a DTD with a different encoding
    # my $dtd = XML::LibXML::Dtd->parse_string($dtdstr, "ISO-8859-1");
    # ok($dtd);
    1


do
    # validate with the DTD
    my $dtd = XML::LibXML::Dtd->parse_string($dtdstr)
    ok($dtd)
    my $xml = XML::LibXML->new->parse_file('example/article.xml')
    ok($xml)
    ok($xml->is_valid($dtd))
    try { $xml->validate($dtd) } # throws exception
    ok( !$^EVAL_ERROR )


do
    # validate a bad document
    my $dtd = XML::LibXML::Dtd->parse_string($dtdstr)
    ok($dtd)
    my $xml = XML::LibXML->new->parse_file('example/article_bad.xml')
    ok(!$xml->is_valid($dtd))
    dies_like( { $xml->validate($dtd) },
               qr/Element article content does not follow the DTD/)

    my $parser = XML::LibXML->new()
    ok($parser->validation(1))
    # this one is OK as it's well formed (no DTD)

    try{
        $parser->parse_file('example/article_bad.xml');
    }
    ok($^EVAL_ERROR)
    dies_like({ $parser->parse_file('example/article_internal_bad.xml') },
              qr/No declaration for element pub_date/ )


# this test fails under XML-LibXML-1.00 with a segfault because the
# underlying DTD element in the C libxml library was freed twice

my $parser = XML::LibXML->new()
my $doc = $parser->parse_file('example/dtd.xml')
my @a = $doc->getChildnodes
is(scalar(nelems(@a)),2)
undef @a
undef $doc

ok(1)

##
# Tests for ticket 2021
do
    my $dtd = XML::LibXML::Dtd->new("","")
    is( $dtd, undef )


do
    my $dtd = XML::LibXML::Dtd->new('', 'example/test.dtd')
    ok($dtd)

