use Test::More

BEGIN { plan tests => 52 }

use XML::LibXML
use XML::LibXML::SAX
use XML::LibXML::SAX::Parser
use XML::LibXML::SAX::Builder
use XML::SAX
use XML::SAX::ParserFactory
use IO::File
ok(1)

ok(XML::SAX->add_parser(q(XML::LibXML::SAX::Parser)))

local $XML::SAX::ParserPackage = 'XML::LibXML::SAX::Parser'

my $sax = SAXTester->new
ok($sax)

my $str = join('', IO::File->new("example/dromeds.xml")->getlines)
my $doc = XML::LibXML->new->parse_string($str)
ok($doc)

my $generator = XML::LibXML::SAX::Parser->new(Handler => $sax)
ok($generator)

$generator->generate($doc)

my $builder = XML::LibXML::SAX::Builder->new()
ok($builder)
my $gen2 = XML::LibXML::SAX::Parser->new(Handler => $builder)
my $dom2 = $gen2->generate($doc)
ok($dom2)

is($dom2->toString, $str)
# warn($dom2->toString);

########### XML::SAX Tests ###########
my $parser = XML::SAX::ParserFactory->parser(Handler => $sax)
ok($parser)
$parser->parse_uri("example/dromeds.xml")

$parser->parse_string(<<EOT)
<?xml version='1.0' encoding="US-ASCII"?>
<dromedaries one="1" />
EOT

$sax = SAXNSTester->new
ok($sax)

$parser->set_handler($sax)

$parser->parse_uri("example/ns.xml")

########### Namespace test ( empty namespaces ) ########

do
    my $h = "SAXNS2Tester"
    my $xml = "<a xmlns='xml://A'><b/></a>"
    my @tests = @: 
        sub (@< @_)
            XML::LibXML::SAX        ->new( Handler => $h )->parse_string( $xml )


        sub (@< @_)
            XML::LibXML::SAX::Parser->new( Handler => $h )->parse_string( $xml )



    for (@tests)
        $_->()


########### Error Handling ###########
do
    my $xml = '<a>Text</b>'

    my $handler = SAXErrorTester->new

    foreach my $pkg (qw(XML::LibXML::SAX::Parser XML::LibXML::SAX))
        undef $^EVAL_ERROR
        try {
            $pkg->new(Handler => $handler)->parse_string($xml);
        }
        ok($^EVAL_ERROR) # We got an error
    

    $handler = SAXErrorCallbackTester->new
    try { XML::LibXML::SAX->new(Handler => $handler )->parse_string($xml) }
    ok($^EVAL_ERROR) # We got an error
    ok( $handler->{?fatal_called} )



########### Helper class #############

package SAXTester
use Test::More

sub new
    my $class = shift
    return bless \$%, $class


sub start_document
    ok(1)


sub end_document
    ok(1)


sub start_element($self, $el)
    like($el->{?LocalName}, qr{^(dromedaries|species|humps|disposition|legs)$})
    foreach my $attr (keys $el->{?Attributes}->%) {
    # warn("Attr: $attr = $el->{Attributes}->{$attr}\n");
    }
# warn("start_element: $el->{Name}\n");


sub end_element($self, $el) {;
# warn("end_element: $el->{Name}\n");
}

sub characters($self, $chars) {;
# warn("characters: $chars->{Data}\n");
}

1

package SAXNSTester
use Test::More

sub new
    bless \$%, shift


sub start_element($self, $node)
    ok($node->{?NamespaceURI} =~ m/^urn:/)
# warn("start_element:\n", Dumper($node));


sub end_element($self, $node) {;
# warn("end_element: $node->{Name}\n");
}

sub start_prefix_mapping($self, $node)
    ok($node->{?NamespaceURI} =~ m/^(urn:camels|urn:mammals|urn:a)$/)
# warn("start_prefix_mapping:\n", Dumper($node));


sub end_prefix_mapping($self, $node)
    # warn("end_prefix_mapping:\n", Dumper($node));
    ok($node->{?NamespaceURI} =~ m/^(urn:camels|urn:mammals|urn:a)$/)


1

package SAXNS2Tester
use Test::More

#sub new {
#    my $class = shift;
#    return bless {}, $class;
#}

sub start_element($self, $elt)
    ok $elt->{?NamespaceURI} eq "xml://A"
        if $elt->{?Name} eq "b"


1

package SAXErrorTester
use Test::More

sub new
    bless \$%, shift


sub end_document
    print "End doc: $(join ' ',@_)\n"
    return 1 # Shouldn't be reached


package SAXErrorCallbackTester
use Test::More

sub fatal_error
    @_[0]->{+fatal_called} = 1


sub new
    bless \$%, shift


sub end_document
    print "End doc: $(join ' ',@_)\n"
    return 1 # Shouldn't be reached



1
