##
# $Id: 19encoding.t 485 2003-11-07 16:49:51Z pajas $
#
# This should test the XML::LibXML internal encoding/ decoding.
# Since most of the internal encoding code is depentend to
# the perl version the module is build for. only the encodeToUTF8() and
# decodeFromUTF8() functions are supposed to be general, while all the
# magic code is only available for more recent perl version (5.6+)
#
use Test::More
use bytes

BEGIN 
    my $tests        = 1
    my $basics       = 0
    my $magic        = 6

    $tests += $basics
    $tests += $magic

    if ( defined env::var('TEST_LANGUAGES') )
        if ( env::var('TEST_LANGUAGES') eq "all" )
            $tests += 2*$basics
            $tests += 2*$magic
        elsif ( env::var('TEST_LANGUAGES') eq "EUC-JP"
                    or env::var('TEST_LANGUAGES') eq "KOI8-R" )
            $tests += $basics
            $tests += $magic
        
    
    plan tests => $tests


use XML::LibXML::Common
use XML::LibXML
ok(1)


my $p = XML::LibXML->new()

# encoding tests
# ok there is the UTF16 test still missing

my $tstr_utf8       = 'test'
my $tstr_iso_latin1 = "tst"

my $domstrlat1 = q{<?xml version="1.0" encoding="iso-8859-1"?>
<tst>tst</tst>
}

do
    print $^STDOUT, "# magic encoding tests\n"

    my $dom_latin1 = XML::LibXML::Document->new('1.0', 'iso-8859-1')
    my $elemlat1   = $dom_latin1->createElement( $tstr_iso_latin1 )

    $dom_latin1->setDocumentElement( $elemlat1 )

    is( decodeFromUTF8( 'iso-8859-1' ,$elemlat1->toString()),
        "<$tstr_iso_latin1/>")
    is( $elemlat1->toString(0,1), "<$tstr_iso_latin1/>")

    my $elemlat2   = $dom_latin1->createElement( "l" )
    is( $elemlat2->toString(0,1), "<l/>")

    $elemlat1->appendText( $tstr_iso_latin1 )

    is( decodeFromUTF8( 'iso-8859-1' ,$elemlat1->string_value()),
        $tstr_iso_latin1)
    is( $elemlat1->string_value(1), $tstr_iso_latin1)

    is( $dom_latin1->toString(), $domstrlat1 )



exit(0) unless defined env::var('TEST_LANGUAGES')

if ( env::var('TEST_LANGUAGES') eq 'all' or env::var('TEST_LANGUAGES') eq "EUC-JP" )
    print $^STDOUT, "# japanese encoding (EUC-JP)\n"

    my $tstr_euc_jp     = ''
    my $domstrjp = q{<?xml version="1.0" encoding="EUC-JP"?>
<></>
}


    my $dom_euc_jp = XML::LibXML::Document->new('1.0', 'EUC-JP')
    my $elemjp = $dom_euc_jp->createElement( $tstr_euc_jp )


    is( decodeFromUTF8( 'EUC-JP' , $elemjp->nodeName()),
        $tstr_euc_jp )
    is( decodeFromUTF8( 'EUC-JP' ,$elemjp->toString()),
        "<$tstr_euc_jp/>")
    is( $elemjp->toString(0,1), "<$tstr_euc_jp/>")

    $dom_euc_jp->setDocumentElement( $elemjp )
    $elemjp->appendText( $tstr_euc_jp )

    is( decodeFromUTF8( 'EUC-JP' ,$elemjp->string_value()),
        $tstr_euc_jp)
    is( $elemjp->string_value(1), $tstr_euc_jp)

    is( $dom_euc_jp->toString(), $domstrjp )



if ( env::var('TEST_LANGUAGES') eq 'all' or env::var('TEST_LANGUAGES') eq "KOI8-R" )
    print $^STDOUT, "# cyrillic encoding (KOI8-R)\n"

    my $tstr_koi8r       = ''
    my $domstrkoi = q{<?xml version="1.0" encoding="KOI8-R"?>
<></>
}


    my ($dom_koi8, $elemkoi8)

    $dom_koi8 = XML::LibXML::Document->new('1.0', 'KOI8-R')
    $elemkoi8 = $dom_koi8->createElement( $tstr_koi8r )

    is( decodeFromUTF8( 'KOI8-R' ,$elemkoi8->nodeName()),
        $tstr_koi8r )

    is( decodeFromUTF8( 'KOI8-R' ,$elemkoi8->toString()),
        "<$tstr_koi8r/>")
    is( $elemkoi8->toString(0,1), "<$tstr_koi8r/>")

    $elemkoi8->appendText( $tstr_koi8r )

    is( decodeFromUTF8( 'KOI8-R' ,$elemkoi8->string_value()),
        $tstr_koi8r)
    is( $elemkoi8->string_value(1),
        $tstr_koi8r)
    $dom_koi8->setDocumentElement( $elemkoi8 )

    is( $dom_koi8->toString(),
        $domstrkoi )


