# $Id: 25relaxng.t 497 2004-02-26 18:52:37Z phish $

##
# Testcases for the RelaxNG interface
#

use Test::More


BEGIN 
    use XML::LibXML

    if ( XML::LibXML::LIBXML_VERSION +>= 20510 )
        plan tests => 8
    else
        plan tests => 0
        print($^STDOUT,  "# Skip No RNG Support compiled\n" )
    
;

if ( XML::LibXML::LIBXML_VERSION +>= 20510 )

    my $xmlparser = XML::LibXML->new()

    my $file         = "test/relaxng/schema.rng"
    my $badfile      = "test/relaxng/badschema.rng"
    my $validfile    = "test/relaxng/demo.xml"
    my $invalidfile  = "test/relaxng/invaliddemo.xml"


    print $^STDOUT, "# 1 parse schema from a file\n"
    do
        my $rngschema = XML::LibXML::RelaxNG->new( location => $file )
        ok ( $rngschema )

        try { $rngschema = XML::LibXML::RelaxNG->new( location => $badfile ); }
        ok( $^EVAL_ERROR )
    

    print $^STDOUT, "# 2 parse schema from a string\n"
    do
        open my $rngfile, "<", "$file"
        my $string = join "", @:  ~< $rngfile->*
        close $rngfile

        my $rngschema = XML::LibXML::RelaxNG->new( string => $string )
        ok ( $rngschema )

        open $rngfile, "<", "$badfile"
        $string = join "", @:  ~< $rngfile->*
        close $rngfile
        try { $rngschema = XML::LibXML::RelaxNG->new( string => $string ); }
        ok( $^EVAL_ERROR )
    

    print $^STDOUT, "# 3 parse schema from a document\n"
    do
        my $doc       = $xmlparser->parse_file( $file )
        my $rngschema = XML::LibXML::RelaxNG->new( DOM => $doc )
        ok ( $rngschema )

        $doc       = $xmlparser->parse_file( $badfile )
        try { $rngschema = XML::LibXML::RelaxNG->new( DOM => $doc ); }
        ok( $^EVAL_ERROR )
    

    print $^STDOUT, "# 4 validate a document\n"
    do
        my $doc       = $xmlparser->parse_file( $validfile )
        my $rngschema = XML::LibXML::RelaxNG->new( location => $file )

        my $valid = 0
        try { $valid = $rngschema->validate( $doc ); }
        is( $valid, 0 )

        $doc       = $xmlparser->parse_file( $invalidfile )
        $valid     = 0
        try { $valid = $rngschema->validate( $doc ); }
        ok ( $^EVAL_ERROR )
    

 # Version >= 20510 test 
