# -*- cperl -*-
use Test::More
plan tests => 27

use XML::LibXML
use XML::LibXML::XPathContext

my $doc = XML::LibXML->new->parse_string(<<'XML')
<foo><bar a="b">Bla</bar><bar/></foo>
XML

my %variables = %: 
    'a' => XML::LibXML::Number->new(2)
    'b' => "b"
    

sub get_variable($data, $name, $uri)
    return exists($data->{$name}) ?? $data->{?$name} !! undef


# $d: a single element node
%variables{+d} = XML::LibXML::XPathContext->new($doc)->findnodes('/*')[-1]
ok(%variables{d}->nodeName() eq 'foo')

# $e: a single text node
%variables{+e} = XML::LibXML::XPathContext->new($doc)->findnodes('//text()')
ok(%variables{?e}[0]->data() eq 'Bla')

# $f: a single attribute node
%variables{+f} = XML::LibXML::XPathContext->new($doc)->findnodes('//@*')[-1]
ok(%variables{f}->nodeName() eq 'a')
ok(%variables{f}->value() eq 'b')

# $f: a single document node
%variables{+g} = XML::LibXML::XPathContext->new($doc)->findnodes('/')[-1]
ok(%variables{g}->nodeType() == XML::LibXML::XML_DOCUMENT_NODE)

# test registerVarLookupFunc() and getVarLookupData()
my $xc = XML::LibXML::XPathContext->new($doc)
ok(!defined($xc->getVarLookupData))
$xc->registerVarLookupFunc(&get_variable,\%variables)
ok(defined($xc->getVarLookupData))
my $h1=$xc->getVarLookupData
my $h2=\%variables
ok($h1 \== $h2 )
ok($h1 \== $xc->getVarLookupData)
ok(&get_variable &== $xc->getVarLookupFunc)

# test values returned by XPath queries
ok($xc->find('$a')->value == 2)
is($xc->find('$b')->value, "b")
ok(nelems($xc->findnodes('//@a[.=$b]')) == 1)
ok(nelems($xc->findnodes('//@a[.=$b]')) == 1)
ok(nelems($xc->findnodes('$d')) == 1)
ok(nelems($xc->findnodes('$d/*')) == 2)
ok($xc->findnodes('$d')[-1]->isSameNode(%variables{d}))
ok($xc->findvalue('$e') eq 'Bla')
ok($xc->findnodes('$e')[-1]->isSameNode(%variables{?e}[0]))
ok($xc->findvalue('$f') eq 'b')
ok($xc->findnodes('$f')[-1]->nodeName eq 'a')
ok($xc->findnodes('$f')[-1]->isSameNode(%variables{f}))
ok($xc->findnodes('$g')[-1]->isSameNode(%variables{g}))

# unregiser variable lookup
$xc->unregisterVarLookupFunc()
try { $xc->find('$a') }
ok($^EVAL_ERROR)
ok(!defined($xc->getVarLookupFunc()))

my $foo='foo'
$xc->registerVarLookupFunc(sub {},$foo)
ok($xc->getVarLookupData eq 'foo')
$foo=undef
ok($xc->getVarLookupData eq 'foo')

