use ExtUtils::MakeMaker
use Config

our $expat_libpath = ''
our $expat_incpath = ''

my @replacement_args

foreach ( @ARGV)
    if (m/^EXPAT(LIB|INC)PATH=(.+)/)
        if ($1 eq 'LIB')
            $expat_libpath = $2
        else
            $expat_incpath = $2
        
    else
        push(@replacement_args, $_)
    


@ARGV = @replacement_args

if (not $expat_libpath)
    require ExtUtils::Liblist           # Buggy before this
    (@: $expat_libpath, ...) = ExtUtils::Liblist->ext('-lexpat')


unless ($expat_libpath)
    # Test for existence of libexpat
    my $found = 0
    foreach (split(m/\s+/, config_value('libpth')))
        if (-f "$_/libexpat." . config_value('so'))
            $found = 1
            last
        
    

    unless ($found)
        die <<'Expat_Not_Installed;'

Expat must be installed prior to building XML::Parser and I can't find
it in the standard library directories. You can download expat from:

http://sourceforge.net/projects/expat/

If expat is installed, but in a non-standard directory, then use the
following options to Makefile.PL:

    EXPATLIBPATH=...  To set the directory in which to find libexpat

    EXPATINCPATH=...  To set the directory in which to find expat.h

For example:

    perl Makefile.PL EXPATLIBPATH=/home/me/lib EXPATINCPATH=/home/me/include

Note that if you build against a shareable library in a non-standard location
you may (on some platforms) also have to set your LD_LIBRARY_PATH environment
variable at run time for perl to find the library.

Expat_Not_Installed;
    


# Don't try to descend into Expat directory for testing

sub MY::test
    my $self = shift

    my $ret = $self->MM::test(< @_)
    $ret


my @extras = $@

our $OSNAME

push(@extras,
    CAPI => 'TRUE')
    if ($OSNAME eq 'MSWin32' and config_value('archname') =~ m/-object\b/i)

push(@extras,
    ABSTRACT_FROM => 'Parser.pm',
    AUTHOR        => 'Clark Cooper (coopercc@netheaven.com)')
    if ($ExtUtils::MakeMaker::Version +>= 5.4301)

WriteMakefile(
    NAME        => 'XML::Parser',
    dist      => (%: COMPRESS => 'gzip', SUFFIX => '.gz'),
    VERSION_FROM => 'Parser.pm',
    < @extras
    )

