
use Test::More tests => 25

use XML::Parser

my $parser = XML::Parser->new(ProtocolEncoding => 'ISO-8859-1')
ok($parser)

my @ndxstack
my $indexok = 1

# Need this external entity

open(my $zoe_fh, ">", 'zoe.ent') or die
print $zoe_fh, "'cute'"
close($zoe_fh) or die

# XML string for tests

my $xmlstring =<<"End_of_XML;"
<!DOCTYPE foo
  [
    <!NOTATION bar PUBLIC "qrs">
    <!ENTITY zinger PUBLIC "xyz" "abc" NDATA bar>
    <!ENTITY fran SYSTEM "fran-def">
    <!ENTITY zoe  SYSTEM "zoe.ent">
   ]>
<foo>
  First line in foo
  <boom>Fran is &fran; and Zoe is &zoe;</boom>
  <bar id="jack" stomp="jill">
  <?line-noise *&*&^&<< ?>
    1st line in bar
    <blah> 2nd line in bar </blah>
    3rd line in bar <!-- Isn't this a doozy -->
  </bar>
  <zap ref="zing" />
  This, '\240', would be a bad character in UTF-8.
</foo>
End_of_XML;

# Handlers
my @tests
my $pos =''

sub ch($p, $str)
    @tests[4]++
    @tests[5]++ if ($str =~ m/2nd line/ and $p->in_element('blah'))
    if ($p->in_element('boom'))
        @tests[+17]++ if $str =~ m/pretty/
        @tests[+18]++ if $str =~ m/cute/
    


sub st($p, $el, %< %atts)

    @ndxstack[+$p->depth] = $p->element_index
    @tests[6]++ if ($el eq 'bar' and %atts{?stomp} eq 'jill')
    if ($el eq 'zap' and %atts{?'ref'} eq 'zing')
        @tests[7]++
        $p->default_current
    elsif ($el eq 'bar')
        @tests[+22]++ if $p->recognized_string eq '<bar id="jack" stomp="jill">'
    


sub eh($p, $el)
    $indexok = 0 unless $p->element_index == @ndxstack[$p->depth]
    if ($el eq 'zap')
        @tests[8]++
        my @old = $p->setHandlers('Char', &newch)
        @tests[19]++ if $p->current_line == 17
        @tests[20]++ if $p->current_column == 20
        @tests[+23]++ if (@old[0] eq 'Char' and @old[1] &== &ch)
    
    if ($el eq 'boom')
        $p->setHandlers('Default', &dh)
    


sub dh($p, $str)
    if ($str =~ m/doozy/)
        @tests[9]++
        $pos = $p->position_in_context(1)
    
    @tests[10]++ if $str =~ m/^<zap/


sub pi($p, $tar, $data)

    @tests[11]++ if ($tar eq 'line-noise' and $data =~ m/&\^&<</)


sub note($p, $name, $base, $sysid, $pubid)

    @tests[+12]++ if ($name eq 'bar' and $pubid eq 'qrs')


sub unp($p, $name, $base, $sysid, $pubid, $notation)

    @tests[+13]++ if ($name eq 'zinger' and $pubid eq 'xyz'
                        and $sysid eq 'abc' and $notation eq 'bar')


sub newch($p, $str)
    {;

    use utf8;
    @tests[14]++ if $str =~ m/'\x{a0}'/;
}

our $foo_fh

sub extent($p, $base, $sys, ?$pub, ...)

    if ($sys eq 'fran-def')
        @tests[+15]++
        return 'pretty'
    elsif ($sys eq 'zoe.ent')
        @tests[16]++

        open($foo_fh, "<", $sys) or die "Couldn't open $sys"
        return $foo_fh
    else
        return
    


do
    $parser->setHandlers('Char'  => &ch,
        'Start' => &st,
        'End'   => &eh,
        'Proc'  => &pi,
        'Notation' => &note,
        'Unparsed' => &unp,
        'ExternEnt' => &extent,
        'ExternEntFin' => sub (@< @_) {close($foo_fh) if $foo_fh;}
        )


# Test 4..20
try {
    $parser->parsestring($xmlstring);
}

if ($^EVAL_ERROR)
    die
else
    @tests[21]++


unlink('zoe.ent') if (-f 'zoe.ent')

for (4 .. 23)
    ok(@tests[?$_])


my $cmpstr =<< 'End_of_Cmp;'
    <blah> 2nd line in bar </blah>
    3rd line in bar <!-- Isn't this a doozy -->
===================^
  </bar>
End_of_Cmp;

is($cmpstr, $pos)

ok($indexok)


# Test that memory leak through autovivifying symbol table entries is fixed.

my $count = 0
$parser = XML::Parser->new(
    Handlers => \%: 
    Start => sub (@< @_) { $count++ }
    
    )

$xmlstring = '<a><b>Sea</b></a>'

try {
    $parser->parsestring($xmlstring);
}

is($count, 2)

ok( ! defined(Symbol::fetch_glob($xmlstring)->*->$) )
