
use Test::More tests => 4

use XML::Parser

use utf8

################################################################
# Check encoding

my $xmldec = "<?xml version='1.0' encoding='x-sjis-unicode' ?>\n"

my $docstring=<<"End_of_doc;"
<\x[8e]\x[83]>\x[90]\x[46]\x[81]\x[41]\x[98]\x[61]\x[81]\x[41]\x[99]\x[44]
</\x[8e]\x[83]>
End_of_doc;

my $doc = $xmldec . $docstring

my @bytes
my $lastel

sub text($xp, $data)

    push(@bytes, unpack('C*', $data))


sub start($xp, $el)

    $lastel = $el


my $p = XML::Parser->new(Handlers => \(%: Start => &start, Char => &text))

$p->parse($doc)

my $exptag = chr(0x7949)

my @expected = @: 0xe8, 0x89, 0xb2	# U+8272 beauty    0x9046
                  0xe3, 0x80, 0x81	# U+3001 comma     0x8141
                  0xe5, 0x92, 0x8c	# U+548C peace     0x9861
                  0xe3, 0x80, 0x81	# U+3001 comma     0x8141
                  0xe5, 0x83, 0x96	# U+50D6 joy       0x9944
                  0x0a

is($lastel, $exptag)

is_deeply(@bytes, @expected)

$lastel = ''

$p->parse($docstring, ProtocolEncoding => 'X-SJIS-UNICODE')

is($lastel, $exptag)

# Test the CP-1252 Win-Latin-1 mapping

$docstring = qq(<?xml version='1.0' encoding='WINDOWS-1252' ?>
<doc euro="\x[80]" lsq="\x[91]" rdq="\x[94]" />
)

my %attr

sub get_attr($xp, $el, @< @list)
    %attr = %:  < @list 


$p = XML::Parser->new(Handlers => \(%: Start => &get_attr))

$p->parse($docstring)

ok( not (   %attr{?euro} ne ( chr(0x20AC) )
    or %attr{?lsq}  ne ( chr(0x2018) )
    or %attr{?rdq}  ne ( chr(0x201D) ) ) )
