
use Test::More tests => 15

use XML::Parser

################################################################
# Check namespaces

my $docstring =<<'End_of_doc;'
<foo xmlns="urn:blazing-saddles"
     xmlns:bar="urn:young-frankenstein"
     bar:alpha="17">
 <zebra xyz="nothing"/>
 <tango xmlns=""
	xmlns:zoo="urn:high-anxiety"
        beta="blue"
        zoo:beta="green"
        bar:beta="red">
   <?nscheck?>
   <zoo:here/>
   <there/>
 </tango>
 <everywhere/>
</foo>
End_of_doc;

my $gname

sub init
    my $xp = shift
    $gname = $xp->generate_ns_name('alpha', 'urn:young-frankenstein')


sub start
    my $xp = shift
    my $el = shift

    if ($el eq 'foo')
        is($xp->namespace($el), 'urn:blazing-saddles')

        is(nelems($xp->new_ns_prefixes), 2)

        while (nelems(@_))
            my $att = shift
            my $val = shift
            if ($att eq 'alpha')
                ok($xp->eq_name($gname, $att))
                last
            
        
    elsif ($el eq 'zebra')
        is(nelems($xp->new_ns_prefixes), 0)

        is($xp->namespace($el), 'urn:blazing-saddles')
    elsif ($el eq 'tango')
        ok( ! $xp->namespace(@_[0]) )

        is(@_[0], @_[2])

        ok( ! $xp->eq_name(@_[0], @_[2]) )

        my $cnt = 0
        foreach ( $xp->new_ns_prefixes)
            $cnt++ if $_ eq '#default'
            $cnt++ if $_ eq 'zoo'
        

        is($cnt, 2)
    


sub end
    my $xp = shift
    my $el = shift

    if ($el eq 'zebra')
        is($xp->expand_ns_prefix('#default'), 'urn:blazing-saddles')
    elsif ($el eq 'everywhere')
        is($xp->namespace($el), 'urn:blazing-saddles')
    


sub proc
    my $xp = shift
    my $target = shift

    if ($target eq 'nscheck')
        is_deeply($xp->new_ns_prefixes, $@)

        my $cnt = 0
        foreach ( $xp->current_ns_prefixes)
            $cnt++ if $_ eq 'zoo'
            $cnt++ if $_ eq 'bar'
        

        is($cnt, 2)

        is($xp->expand_ns_prefix('bar'), 'urn:young-frankenstein')

        is($xp->expand_ns_prefix('zoo'), 'urn:high-anxiety')
    


my $parser = XML::Parser->new(ErrorContext => 2,
    Namespaces   => 1,
    Handlers     => \(%: Start => &start
                         End   => &end
                         Proc  => &proc
                         Init  => &init))

$parser->parse($docstring)
