
use Test::More tests => 11

use XML::Parser

my $internal_subset =<<'End_of_internal;'
[
  <!ENTITY % foo "IGNORE">
  <!ENTITY % bar "INCLUDE">
  <!ENTITY more SYSTEM "t/ext2.ent">
]
End_of_internal;

my $doc =<<"End_of_doc;"
<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE foo SYSTEM "t/foo.dtd"
$internal_subset>
<foo>Happy, happy
<bar>&joy;, &joy;</bar>
<ext/>
&more;
</foo>
End_of_doc;

my $gotinclude = 0
my $gotignore = 0
my $doctype_called = 0
my $internal_exists = 0
my $gotmore = 0

my $bartxt = ''

sub start($xp, $el, %< %atts)

    if ($el eq 'foo')
        ok( ! defined %atts{?top} )
        ok( defined(%atts{?zz}) )
    elsif ($el eq 'bar')
        is(%atts{xyz}, 'b')
    elsif ($el eq 'ext')
        is(%atts{type}, 'flag')
    elsif ($el eq 'more')
        $gotmore = 1
    


sub char($xp, $text, ...)

    $bartxt .= $text if $xp->current_element eq 'bar'


sub attl($xp, $el, $att, $type, $dflt, ?$fixed)

    $gotinclude = 1 if ($el eq 'bar' and $att eq 'xyz' and $dflt eq "'b'")
    $gotignore = 1 if ($el eq 'foo' and $att eq 'top' and $dflt eq '"hello"')


sub dtd($xp, $name, $sysid, $pubid, $internal)

    $doctype_called = 1
    $internal_exists = $internal


my $p = XML::Parser->new(ParseParamEnt => 1,
    ErrorContext  => 2,
    Handlers => \%: Start   => &start
                    Char    => &char
                    Attlist => &attl
                    Doctype => &dtd
    
    )

$p->parse($doc)

ok( $gotmore )

use utf8
is($bartxt, (chr(0x50d6). ", " . chr(0x50d6)))

ok($gotinclude)

ok( ! $gotignore )

ok($doctype_called)

ok($internal_exists)

$doc =~ s/[\s\n]+\[[^]]*\][\s\n]+//m

$p->setHandlers(Start =>
    sub ($xp,$el,%< %atts)
        if ($el eq 'foo')
            ok(defined(%atts{?zz}))
    )

$p->parse($doc)
