# $Id: DocumentLocator.pm,v 1.3 2005/10/14 20:31:20 matt Exp $

package XML::SAX::DocumentLocator

sub new($class, $pubmeth, $sysmeth, $linemeth, $colmeth, $encmeth, $xmlvmeth)
    return bless \(%: 
        pubmeth => $pubmeth
        sysmeth => $sysmeth
        linemeth => $linemeth
        colmeth => $colmeth
        encmeth => $encmeth
        xmlvmeth => $xmlvmeth
        ), $class


my %methods = %:
    "pubmeth" => \*PublicId
    "sysmeth" => \*SystemId
    "linemeth" => \*LineNumber
    "colmeth" => \*ColumnNumber
    "encmeth" => \*Encoding
    "xmlvmeth" => \*XMLVersion


for my $key (keys %methods)
    %methods{$key}->* = sub (@< @_)
        my $self = shift
        my $method = $self->{$key}
        return $method->($key)
    


1
__END__

=head1 NAME

XML::SAX::DocumentLocator - Helper class for document locators

=head1 SYNOPSIS

  my $locator = XML::SAX::DocumentLocator->new(
      sub { $object->get_public_id },
      sub { $object->get_system_id },
      sub { $reader->current_line },
      sub { $reader->current_column },
      sub { $reader->get_encoding },
      sub { $reader->get_xml_version },
  );

=head1 DESCRIPTION

This module gives you a tied hash reference that calls the
specified closures when asked for PublicId, SystemId,
LineNumber and ColumnNumber.

It is useful for writing SAX Parsers so that you don't have
to constantly update the line numbers in a hash reference on
the object you pass to set_document_locator(). See the source
code for XML::SAX::PurePerl for a usage example.

=head1 API

There is only 1 method: C<new>. Simply pass it a list of
closures that when called will return the PublicId, the
SystemId, the LineNumber, the ColumnNumber, the Encoding 
and the XMLVersion respectively.

The closures are passed a single parameter, the key being
requested. But you're free to ignore that.

=cut

