use Test::More
plan tests => 8

use File::Path
use Fatal < qw(open close chdir mkpath)
use XML::SAX
use XML::SAX::ParserFactory

is(nelems(XML::SAX->parsers->@), 0)
ok(XML::SAX->add_parser(q(XML::SAX::PurePerl)))
is(nelems(XML::SAX->parsers->@), 1)

rmtree('t/lib', 0, 0)
mkpath('t/lib', 0, 0777)

push $^INCLUDE_PATH, 't/lib'
write_file('t/lib/TestParserPackage.pm', <<'EOT')
package TestParserPackage
use XML::SAX::Base

our @ISA = qw(XML::SAX::Base)

sub new($class)
    return bless \$%, $class

sub supported_features
    return @: 'http://axkit.org/sax/frobnosticating'

1
EOT

my $parser

# Test we can get TestParserPackage out
write_file('t/lib/SAX.ini', 'ParserPackage = TestParserPackage')
$parser = XML::SAX::ParserFactory->parser()
is(ref($parser), "TestParserPackage", "Parser was not TestParserPackage")

# Test we can get XML::SAX::PurePerl out
write_file('t/lib/SAX.ini', 'ParserPackage = XML::SAX::PurePerl')
$parser = XML::SAX::ParserFactory->parser()
is(ref($parser), "XML::SAX::PurePerl", "Parser was not XML::SAX::PurePerl")

# Test we can ask for a frobnosticating parser, but not get it
write_file('t/lib/SAX.ini', 'http://axkit.org/sax/frobnosticating = 1')
$parser = XML::SAX::ParserFactory->parser()
is(ref($parser), "XML::SAX::PurePerl", "Parser was not PurePerl (frobnosticating)")

# Test we can ask for a frobnosticating parser, and get it
XML::SAX->add_parser('TestParserPackage')
$parser = XML::SAX::ParserFactory->parser()
is(ref($parser), "TestParserPackage", "Parser was not TestParserPackage (frobnosticating)")

# Test we can get a namespaces parser
write_file('t/lib/SAX.ini', 'http://xml.org/sax/features/namespaces = 1')
$parser = XML::SAX::ParserFactory->parser()
is(ref($parser), "XML::SAX::PurePerl", "Parser was not PurePerl (namespaces)")

sub write_file($file, $data)
    open(my $fh, ">", "$file")
    print $fh, $data, "\n"
    close $fh

