# $Id: Makefile.PL,v 1.8 2006/10/03 01:02:20 grantm Exp $

use ExtUtils::MakeMaker

my $make_params = \%:
    'NAME'          => 'XML::Simple'
    'VERSION_FROM'  => 'lib/XML/Simple.pm'
    'DISTNAME'      => 'XML-Simple'
    'PREREQ_PM'     => %:
        'Test::Simple'          => 0.41
    'dist'          => (%:  COMPRESS => 'gzip --best', SUFFIX => 'gz' )
    'AUTHOR'        => 'Grant McLean <grantm@cpan.org>'
    'ABSTRACT_FROM' => 'lib/XML/Simple.pm'


print $^STDOUT, "Checking installed modules ...\n"

if ( try { require XML::SAX } && ! $^EVAL_ERROR )
    require XML::SAX::ParserFactory
    my $default_parser = ref(XML::SAX::ParserFactory->parser())
    if ($default_parser eq 'XML::SAX::PurePerl')
        my $version = XML::SAX->VERSION
        if($version +> 0.12  and  $version +< 0.15)
            die <<"EOF"
=============================================================================

  Fatal error: Your default XML parser (XML::SAX::PurePerl) is broken.

  There are known bugs in the PurePerl parser included with version 0.13
  and 0.14 of XML::SAX.  The XML::Simple tests will fail with this parser.

  One way to avoid the problem is to install XML::SAX::Expat - it will
  install itself as the system default XML parser and then you will be able
  to install XML::Simple successfully.  XML::SAX::Expat is also much faster
  than XML::SAX::PurePerl so you probably want it anyway.

=============================================================================
EOF

    elsif ($default_parser eq 'XML::SAX::RTF')
        die <<"EOF"
=============================================================================

  Fatal error: Your default XML parser (XML::SAX::RTF) is broken.

  The XML::SAX:RTF module is installed as the default XML parser on your
  system.  This is a bug - although the module does generate SAX events,
  it does not parse XML and should not register itself as an XML parser.

  One way to avoid the problem is to install XML::SAX::Expat - it will
  register itself as the system default XML parser and then you will be
  able to install XML::Simple successfully.

  Another solution would be to locate the XML/SAX/ParserDetails.ini file
  and edit it to completely remove the section beginning [XML::SAX::RTF].

=============================================================================
EOF

    print $^STDOUT, "XML::SAX is installed, it will be used by the test suite\n"
    $make_params->{PREREQ_PM}{+'XML::SAX'} = 0
    $make_params->{PREREQ_PM}{+'XML::NamespaceSupport'} = 1.04
elsif ( try { require XML::Parser } && ! $^EVAL_ERROR )
    print $^STDOUT, "XML::Parser is installed, it will be used by the test suite\n"
    $make_params->{PREREQ_PM}{+'XML::Parser'} = 0
else
    print $^STDOUT, "You don't have either XML::SAX or XML::Parser installed!\n"
    $make_params->{PREREQ_PM}{+'XML::SAX'} = 0
    $make_params->{PREREQ_PM}{+'XML::NamespaceSupport'} = 1.04
    # Hopefully the following line can be removed after next XML::SAX release
    $make_params->{PREREQ_PM}{+'XML::SAX::Expat'} = 0


try { require Storable }
if($^EVAL_ERROR)
    print $^STDOUT, "Storable is not installed ... caching functions will not be available\n"



WriteMakefile(< $make_params->%)
