# $Id: A_XMLParser.t,v 1.1 2004/02/29 09:49:18 grantm Exp $
# vim: syntax=perl

use Test::More
use IO::File
use File::Spec

$^WARNING = 1


# Initialise filenames and check they're there

my $XMLFile = File::Spec->catfile('t', 'test1.xml')  # t/test1.xml

unless(-e $XMLFile)
    plan skip_all => 'Test data missing'


try { require XML::Parser; }
unless($^INCLUDED{?'XML/Parser.pm'})
    plan skip_all => 'no XML::Parser'


plan tests => 14

use XML::Simple

my $last_warning = ''
my $opt


# Use environment variable to set preferred parser

env::var('XML_SIMPLE_PREFERRED_PARSER' ) = 'XML::Parser'


# Try using a SAX-only option

do
    local($^WARN_HOOK) = &warn_handler

    $opt = XMLin('<x y="z" />', nsexpand => 1)


isnt($last_warning, '', "Parsing caused warning (as expected)")
like($last_warning, qr/'nsexpand' option requires XML::SAX/,
     'Message contained expected text')
is_deeply($opt, (%: y => 'z'), "Parsing was successful")


# Check for deprecation warning

do
    local($^WARN_HOOK) = &warn_handler

    $^EVAL_ERROR = ''
    $last_warning = ''
    $opt = try { XMLin('<x y="z" />', ParserOpts => \(@:  ParseParamEnt => 1 )) }


isnt($last_warning, '', "Using ParserOpts caused warning (as expected)")
like($last_warning, qr/'ParserOpts' is deprecated/,
     'Message contained expected text')
is_deeply($opt, (%: y => 'z'), "Parsing was successful")


# Check it doesn't happen if warnings disabled

do
    local($^WARN_HOOK) = &warn_handler

    $^EVAL_ERROR = ''
    $last_warning = ''
    local($^WARNING) = 0
    $opt = try { XMLin('<x y="z" />', ParserOpts => \(@:  ParseParamEnt => 1 )) }


is($last_warning, '', "ParserOpts warning uppressed successfully")
is_deeply($opt, (%: y => 'z'), "Parsing was successful")



# Try parsing a string

$^EVAL_ERROR = ''
$opt = try {
    XMLin(q(<opt name1="value1" name2="value2"></opt>));
}

my $expected = %: 
    name1 => 'value1'
    name2 => 'value2'
    

is($^EVAL_ERROR, '', "No error when parsing")
is_deeply($opt, $expected, 'matches expectations (attributes)')


# Try parsing a named external file

$^EVAL_ERROR = ''
$opt = XMLin($XMLFile)
ok(1, "XML::Parser didn't choke on named external file")
is_deeply($opt, (%: 
          location => 't/test1.xml'
          ), 'and contents parsed as expected')


# Try parsing from an IO::Handle

$^EVAL_ERROR = ''
my $fh = IO::File->new()
$XMLFile = File::Spec->catfile('t', '1_XMLin.xml')  # t/1_XMLin.xml
$fh->open($XMLFile) || die "$^OS_ERROR"
$opt = XMLin($fh)
ok(1, "XML::Parser didn't choke on an IO::File object")
is($opt{location}, 't/1_XMLin.xml', 'and it parsed the right file')


exit(0)

sub warn_handler
    $last_warning = @_[0]->message

