# This -*- perl -*- script makes the Makefile

#--- Distribution section ---

our $NAME     = 'Net'
our $DISTNAME = "libnet"
our $VERSION  = "1.22"
our $config_pm = "Net/libnet.cfg"

use ExtUtils::MakeMaker
use ExtUtils::Manifest < qw(maniread)

#--- Installation check

sub chk_version($pkg,$wanted,?$msg)

    local($^OUTPUT_AUTOFLUSH) = 1
    print $^STDOUT, "Checking for $pkg..."

    try { my $p; ($p = $pkg . ".pm") =~ s#::#/#g; require $p; }

    my $version = Symbol::fetch_glob("$($pkg)::VERSION")->*->$
    my $vstr =  $version ?? "found v" . $version !! "not found"
    my $vnum = $version || 0

    print $^STDOUT, $vnum +>= $wanted ?? "ok\n" !! " " . $vstr . "\n"

    $vnum +>= $wanted


sub MY::post_initialize($self)

    #--- Create Net::Config

    my $libnet_cfg = "libnet.cfg"

    # Use %INC and ExtUtils::MakeMaker to determine how this machine
    # maps package names to path names

    foreach (keys $^INCLUDED)
        last if ($config_pm = $_) =~ s/^ExtUtils(.)MakeMaker.pm/Net$($1)libnet.cfg/
    

    system(($^OS_NAME eq 'VMS' ?? 'mcr '!! ()),$^EXECUTABLE_NAME, 'libnetcfg', '-c', (env::var('PERL_MM_USE_DEFAULT') ?? '-d' !! ()), )
        unless -f $libnet_cfg

    $self->{PM}{+$libnet_cfg} = $self->catfile('$(INST_LIBDIR)',$config_pm)

    ""


#--- Check for Socket

chk_version(Socket => '1.30') or
    warn  "\n"
    . "*** For Net::Cmd to work you require version 1.30, or later, of\n"
    . "    Socket.pm from CPAN/modules/by-module/Socket/Socket-x.x.tar.gz\n\n"

chk_version('IO::Socket' => '1.05') or
    warn  "\n"
    . "*** For Net::Cmd to work you require version 1.05, or later, of\n"
    . "    IO/Socket.pm from CPAN/modules/by-module/IO/IO-x.x.tar.gz\n\n"

if ($^OS_NAME eq 'os390')
    chk_version('Convert::EBCDIC' => '0.06') or
        warn  "\n"
        . "*** For Net::Cmd to work on $^OS_NAME version 0.06, or later, of\n"
        . "    Convert::EBCDIC is required, which can be found at"
        . "    CPAN/modules/by-module/Convert/Convert-EBCDIC-x.x.tar.gz\n\n"


#--- Write the Makefile

WriteMakefile(
    VERSION     => $VERSION,
    DISTNAME    => $DISTNAME,
    NAME        => $NAME,
    'realclean' => (%: FILES => $config_pm),
    PREREQ_PM   => (%: 
    Socket     => 1.3
    'IO::Socket' => 1.05
    ),
    EXE_FILES => (@:  'libnetcfg' ),
    dist => (%:  DIST_DEFAULT => 'mydist', ),
    AUTHOR    => 'Graham Barr <gbarr@pobox.com>',
    ABSTRACT  => 'Collection of Network protocol modules',
    )

sub MY::postamble
    return <<'POSTAMBLE'

mydist : distmeta distsignature tardist

POSTAMBLE


