#!./perl

BEGIN 
    if (!eval "require Socket")
        print $^STDOUT, "1..0 # no Socket\n"; exit 0
    



use Cwd

use Test::More tests => 20

require Net::Netrc
ok( 1, 'should be able to require() Net::Netrc safely' )
ok( exists $^INCLUDED{'Net/Netrc.pm'}, 'should be able to use Net::Netrc' )

SKIP: do
    skip('incompatible stat() handling for OS', 4), next SKIP
        if ($^OS_NAME =~ m/os2|win32|macos|cygwin/i)

    my $warn
    local $^WARN_HOOK = sub (@< @_)
        $warn = shift
    

    # add write access for group/other
    chmod 0666, 't/test-net.cfg' or die "Failed changing file permissions: $^OS_ERROR"

    ok( !defined(Net::Netrc::_readrc_file(undef, 't/test-net.cfg')),
        '_readrc() should not read world-writable file' )
    ok( scalar($warn->message =~ m/^Bad permissions:/),
        '... and should warn about it' )

    TODO: do
        todo_skip('fix stat override', 2)

        # the owner field should still not match
        if ($^UID)
            ok( !defined(Net::Netrc::_readrc()),
                '_readrc() should not read file owned by someone else' )
            ok( scalar($warn->message =~ m/^Not owner:/),
                '... and should warn about it' )
        else
            skip("testing as root",2)
        
    


chmod 0600, 't/test-net.cfg' or die "Failed changing permission: $^OS_ERROR"

# this curious mix of spaces and quotes tests a regex at line 79 (version 2.11)

# having set several lines and the uid, this should succeed
is( Net::Netrc::_readrc_file(undef, 't/test-net.cfg'), 1,
    '_readrc() should succeed now' )

# on 'foo', the login is 'nigol'
is( Net::Netrc->lookup('foo')->{?login}, 'nigol',
              'lookup() should find value by host name' )

# on 'foo' with login 'l2', the password is 'p2'
is( Net::Netrc->lookup('foo', 'l2')->{?password}, 'p2',
              'lookup() should find value by hostname and login name' )

# the default password is 'p3', as later declarations have priority
is( Net::Netrc->lookup()->{?password}, 'p3',
              'lookup() should find default value' )

# lookup() ignores the login parameter when using default data
is( Net::Netrc->lookup('default', 'baz')->{?password}, 'p3',
              'lookup() should ignore passed login when searching default' )

# lookup() goes to default data if hostname cannot be found in config data
is( Net::Netrc->lookup('abadname')->{?login}, 'baz',
              'lookup() should use default for unknown machine name' )

# now test these accessors
my $instance = bless(\$%, 'Net::Netrc')
for my $accessor (qw( login account password ))
    is( $instance->?$accessor(), undef,
        "$accessor() should return undef if $accessor is not set" )
    $instance->{+$accessor} = $accessor
    is( $instance->?$accessor(), $accessor,
        "$accessor() should return value when $accessor is set" )


# and the three-for-one accessor
is( nelems( $instance->lpa()), 3,
    'lpa() should return login, password, account')
is( join(' ', $instance->lpa), 'login password account',
    'lpa() should return appropriate values for l, p, and a' )
