# This -*- perl -*- script writes the Makefile for libwww-perl
# $Id: Makefile.PL,v 1.76 2005/11/03 17:26:39 gisle Exp $

use ExtUtils::MakeMaker < qw(WriteMakefile prompt)
use Config

#--- Configuration section ---

my @programs_to_install = qw(lwp-request lwp-mirror lwp-rget lwp-download)
my @request_aliases     = qw(GET HEAD POST)

#--- End Configuration - You should not have to change anything below this line


# Allow us to suppress all program installation with the -n (library only)
# option.  This is for those that don't want to mess with the configuration
# section of this file.
use Getopt::Std
our ($opt_n)
unless (getopts("n"))
    die "Usage: $^PROGRAM_NAME [-n]\n"

@programs_to_install = $@ if $opt_n || grep { m/^LIB=/ }, @ARGV

# Check if we should try to run tests that talk to ourself
system(qq("$^EXECUTABLE_NAME" talk-to-ourself))
if ($^CHILD_ERROR)
    if ($^CHILD_ERROR != 1)
        die "talk-to-ourself test failed unexpected"
    
    print $^STDOUT, <<EOT

You appear to have a misconfigured system, so I will disable tests
that try to talk HTTP to a local server.
EOT
    unlink("t/CAN_TALK_TO_OURSELF")
else
    open(my $can_talk_to_ourself, ">", "t/CAN_TALK_TO_OURSELF") || die "Can't create CAN_TALK_TO_OURSELF: $^OS_ERROR"
    close($can_talk_to_ourself)


# Check if we should try to run the live tests
open(my $changes, "<", "Changes") || die "Can't open Changes: $^OS_ERROR"
my $release_date
while ( ~< $changes->*)
    if (m/^(\d{4}-\d{2}-\d{2})\D/)
        $release_date = $1
        last
    

close($changes)
die "Can't figure out release date" unless $release_date
#print "Release date: $release_date\n";

my $some_time_ago = sprintf "\%04d-\%02d-\%02d", <
    sub (@< @_) { ( (@: @_[5]+1900, @_[4]+1, @_[3] ))}->(localtime(time - 45 * 24*60*60))
if (($some_time_ago cmp $release_date) +< 0)
    # Check if we have internet connection
    require IO::Socket
    my $s = IO::Socket::INET->new(PeerAddr => "www.google.com:80",
        Timeout  => 10,
        )
    if ($s)
        # XXX could try to send a GET to it???
        close($s)

        print $^STDOUT, <<EOT

You appear to be directly connected to the Internet.  I have some tests
that tries to access some sites on the net to verify that the new HTTP/1.1
support works as it should.

EOT

        if (prompt("Do you want to enable these tests?", "y") =~ m/^y/i)
            open(my $enabled, ">", "t/live/ENABLED") || die "Can't enable: $^OS_ERROR"
            close($enabled)

            # Figure out if the compress lib works and signal that with
            # a file for the test suite to find.  We don't want the
            # test script to do this 'require' itself because we want
            # to test that the module loads it on demand as it should.
            try {
                require Compress::Zlib;
                Compress::Zlib->VERSION(1.10);
                open(my $zlib_ok, ">", "t/live/ZLIB_OK") || die "Can't create ZLIB_OK: $^OS_ERROR";
                print $zlib_ok, "$Compress::Zlib::VERSION\n";
                close($zlib_ok);
            }
            if ($^EVAL_ERROR)
                #warn $@;
                unlink("t/live/ZLIB_OK")
            
        else
            unlink("t/live/ENABLED")
        
    


if (nelems(@programs_to_install))
    print $^STDOUT, <<EOT

This package comes with some sample programs that I can try
to install in $(config_value("installscript")).

   Note that you can avoid these questions by passing
   the '-n' option to 'Makefile.PL'.

EOT
    my @tmp
    for ( @programs_to_install)
        if (prompt("Do you want to install $_?", "y") =~ m/^y/)
            push(@tmp, $_)
        
    
    @programs_to_install = @tmp


if (grep( {$_ eq 'lwp-request' }, @programs_to_install) && nelems(@request_aliases))
    print $^STDOUT, <<EOT

The lwp-request program will use the name it is invoked with to
determine what HTTP method to use.  I can set up alias for the most
common HTTP methods.  These alias are also installed in
$(config_value("installscript")).

EOT
    my @tmp
    for my $alias ( @request_aliases)
        my $default = "n"
        if (prompt("Do you want to install the $alias alias?", $default) =~ m/^y/)
            push(@tmp, $alias)
        
    
    @request_aliases = @tmp
else
    @request_aliases = $@


# Check for non-standard modules that are used by this library.
$^OUTPUT_AUTOFLUSH = 1
my $missing_modules = 0

print $^STDOUT, "\nChecking for URI..........."
try {
    require URI;
    URI->VERSION(1.10);
}
if ($^EVAL_ERROR)
    print $^STDOUT, " failed\n"
    $missing_modules++
    print $^STDOUT, <<EOT
$^EVAL_ERROR
The URI module must be installed.  WWW without URIs would not
be that great :-)

EOT
    sleep(2)  # Don't hurry too much
else
    print $^STDOUT, " ok\n"

print $^STDOUT, "Checking for HTML::Parser.."
try {
    require HTML::HeadParser;
    HTML::Parser->VERSION(3.33);
}
if ($^EVAL_ERROR)
    print $^STDOUT, " failed\n"
    $missing_modules++
    print $^STDOUT, <<EOT
$^EVAL_ERROR
The HTML::Parser is needed to extract correct base URI information from
HTML so that we can resolve relative links correctly.  The HTML::Form
module also need HTML::TokeParser to work.

EOT
    sleep(2)  # Don't hurry too much
else
    print $^STDOUT, " ok\n"


print $^STDOUT, "Checking for MIME::Base64.."
try {
    require MIME::Base64;
#MIME::Base64->VERSION('2.00');
}
if ($^EVAL_ERROR)
    print $^STDOUT, " failed\n"
    $missing_modules++
    print $^STDOUT, <<EOT
$^EVAL_ERROR
The Base64 encoding is used in authentication headers in HTTP.

EOT
    sleep(2)  # Don't hurry too much
else
    print $^STDOUT, " ok\n"


print $^STDOUT, "Checking for Net::FTP......"
try {
    require Net::FTP;
    Net::FTP->VERSION('2.58');
}
if ($^EVAL_ERROR)
    print $^STDOUT, " failed\n"
    $missing_modules++
    print $^STDOUT, <<EOT
$($^EVAL_ERROR->message)
The libwww-perl library normally use the Net::FTP module when
accessing ftp servers.  You would have to install this package or
configure your application to use a proxy server for making ftp
requests work.  Net::FTP is part of the 'libnet' distribution.

EOT
    sleep(2)  # Don't hurry too much
else
    print $^STDOUT, " ok\n"


print $^STDOUT, "Checking for Digest::MD5 .."
try {
    require Digest::MD5;
}
if ($^EVAL_ERROR)
    print $^STDOUT, " failed\n"
    $missing_modules++
    print $^STDOUT, <<EOT
$^EVAL_ERROR
The Digest::MD5 library is needed if you want to be able use the
experimental "Digest Access Authentication" scheme.  Since very few
servers implement this authentication scheme, you should normally not
worry too much about this.

EOT
else
    print $^STDOUT, " ok\n"


print $^STDOUT, <<EOT if $missing_modules
The missing modules can be obtained from CPAN.  Visit
<URL:http://www.perl.com/CPAN/> to find a CPAN site near you.

EOT

print $^STDOUT, "\n"

if (nelems(@request_aliases))
    require File::Copy
    for ( @request_aliases)
        File::Copy::copy("bin/lwp-request", "bin/$_") || die "Can't copy bin/$_"
        chmod(0755, "bin/$_")
        push(@programs_to_install, $_)
    



# Ok, now it is time to really generate the Makefile

WriteMakefile(
   NAME          => 'LWP',
   DISTNAME      => 'libwww-perl',
   VERSION_FROM  => 'lib/LWP.pm',
   EXE_FILES     => (map { "bin/$_" }, @programs_to_install),
   PREREQ_PM     => (%:  'URI'              => "1.10"
                         'MIME::Base64'     => "2.1"
                         'Net::FTP'         => "2.58"
                         'HTML::Parser'     => "3.33"
                         'Digest::MD5'      => 0
                         'Compress::Zlib'   => "1.10"
   ),
   'clean'       => (%:  FILES => join(" ", map { "bin/$_" }, @request_aliases) ),
   'dist'        => (%:  COMPRESS => 'gzip -9f', SUFFIX => 'gz', ),
   )



package MY

# What happens when we say 'make test'
sub test
    q(
TEST_VERBOSE=0

test: all
	$(FULLPERL) t/TEST $(TEST_VERBOSE)

)



# Determine things that should *not* be installed
sub libscan($self, $path)
    return '' if $path =~ m/\.(pl|dtd|sgml)$/
    return '' if $path =~ m:\bCVS/:
    return '' if $path =~ m/~$/
    $path


# Pass libwww-perl version number to pod2man
sub manifypods
    my $self = shift
    my $ver = $self->{?VERSION} || ""
    local($_) = $self->SUPER::manifypods(< @_)
    s/pod2man\s*$/pod2man --release libwww-perl-$ver/m
    $_

