#
# $Id: mailto.pm,v 1.12 2004/05/21 08:56:15 gisle Exp $
#
# This module implements the mailto protocol.  It is just a simple
# frontend to the Unix sendmail program except on MacOS, where it uses
# Mail::Internet.

package LWP::Protocol::mailto

require LWP::Protocol
require HTTP::Request
require HTTP::Response
require HTTP::Status

use Carp

our (@ISA, $SENDMAIL)

@ISA = qw(LWP::Protocol)

unless ($SENDMAIL = env::var('SENDMAIL'))
    for my $sm (qw(/usr/sbin/sendmail
		   /usr/lib/sendmail
		   /usr/ucblib/sendmail
		  ))
        if (-x $sm)
            $SENDMAIL = $sm
            last


    die "Can't find the 'sendmail' program" unless $SENDMAIL


sub request
    my (@: $self, $request, $proxy, $arg, $size, ...) =  @_

    my ($mail, $addr) if $^OS_NAME eq "MacOS"
    my @text = $@ if $^OS_NAME eq "MacOS"

    my $sendmail_fh

    # check proxy
    if (defined $proxy)
        return HTTP::Response->new( HTTP::Status::RC_BAD_REQUEST(),
            'You can not proxy with mail')


    # check method
    my $method = $request->method

    if ($method ne 'POST')
        return HTTP::Response->new( HTTP::Status::RC_BAD_REQUEST(),
            'Library does not allow method ' .
            "$method for 'mailto:' URLs")


    # check url
    my $url = $request->url

    my $scheme = $url->scheme
    if ($scheme ne 'mailto')
        return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
            "LWP::Protocol::mailto::request called for '$scheme'")

    if ($^OS_NAME eq "MacOS")
        try {
            require Mail::Internet;
        }
        if($^EVAL_ERROR)
            return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
                "You don't have MailTools installed")

        unless (env::var('SMTPHOSTS'))
            return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
                "You don't have SMTPHOSTS defined")

    else
        unless (-x $SENDMAIL)
            return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
                "You don't have $SENDMAIL")


    if ($^OS_NAME eq "MacOS")
        $mail = Mail::Internet->new or
            return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
            "Can't get a Mail::Internet object")
    else
        open($sendmail_fh, "|-", "$SENDMAIL -oi -t") or
            return HTTP::Response->new( HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
            "Can't run $SENDMAIL: $^OS_ERROR")

    if ($^OS_NAME eq "MacOS")
        $addr = $url->encoded822addr
    else
        $request = $request->clone  # we modify a copy
        my @h = $url->headers  # URL headers override those in the request
        while (nelems(@h))
            my $k = shift @h
            my $v = shift @h
            next unless defined $v
            if (lc($k) eq "body")
                $request->content($v)
            else
                $request->push_header($k => $v)



    if ($^OS_NAME eq "MacOS")
        $mail->add(To => $addr)
        $mail->add( <split(m/[:\n]/,$request->headers_as_string))
    else
        print $sendmail_fh, $request->headers_as_string
        print $sendmail_fh, "\n"

    my $content = $request->content
    if (defined $content)
        my $contRef = ref($content) ?? $content !! \$content
        if (ref($contRef) eq 'SCALAR')
            if ($^OS_NAME eq "MacOS")
                @text = split("\n",$contRef->$)
                foreach ( @text)
                    $_ .= "\n"
            else
                print $sendmail_fh, $contRef->$

        elsif (ref($contRef) eq 'CODE')
            # Callback provides data
            my $d
            if ($^OS_NAME eq "MacOS")
                my $stuff = ""
                while (length($d = $contRef->()))
                    $stuff .= $d
                @text = split("\n",$stuff)
                foreach ( @text)
                    $_ .= "\n"
            else
                print $sendmail_fh, $d

    if ($^OS_NAME eq "MacOS")
        $mail->body(\@text)
        unless ($mail->smtpsend)
            return HTTP::Response->new(HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
                "Mail::Internet->smtpsend unable to send message to <$addr>")

    else
        unless (close($sendmail_fh))
            my $err = $^OS_ERROR ?? "$^OS_ERROR" !! "Exit status $^CHILD_ERROR"
            return HTTP::Response->new(HTTP::Status::RC_INTERNAL_SERVER_ERROR(),
                "$SENDMAIL: $err")


    my $response = HTTP::Response->new(HTTP::Status::RC_ACCEPTED(),
        "Mail accepted")
    $response->header('Content-Type', 'text/plain')
    if ($^OS_NAME eq "MacOS")
        $response->header('Server' => "Mail::Internet $Mail::Internet::VERSION")
        $response->content("Message sent to <$addr>\n")
    else
        $response->header('Server' => $SENDMAIL)
        my $to = $request->header("To")
        $response->content("Message sent to <$to>\n")


    return $response


1
