
use Test::More tests => 22

use HTTP::Request::Common

my $r = GET 'http://www.sn.no/'
info $r->as_string

ok($r->method eq "GET" and $r->url->as_string eq "http://www.sn.no/")

$r = HEAD "http://www.sn.no/",
          If_Match => 'abc',
          From => 'aas@sn.no'
info( $r->as_string )

ok($r->method eq "HEAD" and $r->url->eq("http://www.sn.no"))

ok($r->header('If-Match') eq "abc" and $r->header("from") eq "aas\@sn.no")

$r = PUT "http://www.sn.no",
         Content => 'foo'
info( $r->as_string )

ok($r->method eq "PUT" and $r->uri->host eq "www.sn.no")

ok(not defined($r->header("Content")))

ok($r->content_ref->$ eq "foo" and
    $r->content eq "foo")

#--- Test POST requests ---

$r = POST "http://www.sn.no", \(@: foo => 'bar;baz'
                                   baz => \(@:  <qw(a b c))
                                   foo => 'zoo=&'
                                   "space " => " + "
          ),
          bar => 'foo'
info( $r->as_string )

ok($r->method eq "POST" and
    $r->content_type eq "application/x-www-form-urlencoded" and
    $r->content_length == 58 and
    $r->header("bar") eq "foo")

ok($r->content eq "foo=bar\%3Bbaz&baz=a&baz=b&baz=c&foo=zoo\%3D\%26&space+=+\%2B+")

$r = POST "mailto:gisle\@aas.no",
          Subject => "Heisan",
          Content_Type => "text/plain",
          Content => "Howdy\n"
info( $r->as_string )

ok($r->method eq "POST" and
    $r->header("Subject") eq "Heisan" and
    $r->content eq "Howdy\n" and
    $r->content_type eq "text/plain")

#
# POST for File upload
#
my $file = "test-$^PID"
open(my $fh, ">", "$file") or die "Can't create $file: $^OS_ERROR"
print $fh, "foo\nbar\nbaz\n"
close($fh)

$r = POST 'http://www.perl.org/survey.cgi',
          Content_Type => 'form-data',
          Content      => \@:  name  => 'Gisle Aas'
                               email => 'gisle@aas.no'
                               gender => 'm'
                               born   => '1964'
                               file   => \(@: $file)
          
info( $r->as_string )

unlink($file) or warn "Can't unlink $file: $^OS_ERROR"

ok($r->method eq "POST" and
    $r->url->path eq "/survey.cgi" and
    $r->content_type eq "multipart/form-data" and
    $r->header('Content_type') =~ m/boundary="?([^"]+)"?/)
my $boundary = $1

my $c = $r->content
$c =~ s/\r//g
my @c = @:  < split(m/--\Q$boundary/, $c) 
info( "@c[5]\n" )

ok( nelems(@c) == 7 and @c[6] =~ m/^--\n/ )  # 5 parts + header & trailer

ok( @c[2] =~ m/^Content-Disposition:\s*form-data;\s*name="email"/m and
    @c[2] =~ m/^gisle\@aas.no$/m )

ok( @c[5] =~ m/^Content-Disposition:\s*form-data;\s*name="file";\s*filename="$file"/m and
    @c[5] =~ m/^Content-Type:\s*text\/plain$/m and
    @c[5] =~ m/^foo\nbar\nbaz/m )

$r = POST 'http://www.perl.org/survey.cgi',
          \(@:  file => \(@:  undef, "xxx", Content_type => "text/html", Content => "<h1>Hello, world!</h1>" )),
          Content_type => 'multipart/form-data'
info( $r->as_string )

if($^OS_NAME eq "MacOS")
    ok( $r->content =~ m/^--\S+\015\012Content-Disposition:\s*form-data;\s*name="file";\s*filename="xxx"/m and
        $r->content =~ m/^\012Content-Type: text\/html/m and
        $r->content =~ m/^\012<h1>Hello, world/m )
else
    ok( $r->content =~ m/^--\S+\015\012Content-Disposition:\s*form-data;\s*name="file";\s*filename="xxx"/m and
        $r->content =~ m/^Content-Type: text\/html/m and
        $r->content =~ m/^<h1>Hello, world/m )



$r = POST 'http://www.perl.org/survey.cgi',
          Content_type => 'multipart/form-data',
          Content => \@:  file => \(@:  undef, undef, Content => "foo")
info( $r->as_string )

unlike( $r->content, qr/filename=/ )


# The POST routine can now also take a hash reference.
my %hash = %: foo => 42, bar => 24
$r = POST 'http://www.perl.org/survey.cgi', \%hash
info(  $r->as_string )
ok( $r->content =~ m/foo=42/ &&
    $r->content =~ m/bar=24/ &&
    $r->content_type eq "application/x-www-form-urlencoded" &&
    $r->content_length == 13 )


#
# POST for File upload
#
use HTTP::Request::Common < qw($DYNAMIC_FILE_UPLOAD)

$file = "test-$^PID"
open($fh, ">", "$file") or die "Can't create $file: $^OS_ERROR"
for (1..1000)
    print $fh, < (@: "a") x 26

close($fh)

$DYNAMIC_FILE_UPLOAD++
$r = POST 'http://www.perl.org/survey.cgi',
          Content_Type => 'form-data',
          Content      => \@:  name  => 'Gisle Aas'
                               email => 'gisle@aas.no'
                               gender => 'm'
                               born   => '1964'
                               file   => \(@: $file)
          

ok( $r->method eq "POST" and
    $r->url->path eq "/survey.cgi" and
    $r->content_type eq "multipart/form-data" and
    $r->header('Content_type') =~ m/boundary="?([^"]+)"?/ and
    type::is_code($r->content) )
$boundary = $1

ok( length($boundary) +> 10 )

my $code = $r->content
my $chunk
my @chunks
while (defined($chunk = $code->()) && length $chunk)
    push(@chunks, $chunk)


unlink($file) or warn "Can't unlink $file: $^OS_ERROR"

$_ = join("", (@:  < @chunks))

info( int(nelems(@chunks)), " chunks, total size is ", length($_), " bytes\n" )

# should be close to expected size and number of chunks
ok( abs(nelems(@chunks) - 15 +< 3) and
    abs(length($_) - 26589) +< 20 )

$r = POST 'http://www.example.com'
ok( $r->as_string eq <<EOT )
POST http://www.example.com
Content-Length: 0
Content-Type: application/x-www-form-urlencoded

EOT

$r = POST 'http://www.example.com', Content_Type => 'form-data', Content => \$@
ok( $r->as_string eq <<EOT )
POST http://www.example.com
Content-Length: 0
Content-Type: multipart/form-data; boundary=none

EOT

$r = POST 'http://www.example.com', Content_Type => 'form-data'
#print $r->as_string;
ok( $r->as_string eq <<EOT )
POST http://www.example.com
Content-Length: 0
Content-Type: multipart/form-data

EOT

