use LWP::MediaTypes

use Test::More

require URI::URL

my $url1 = URI::URL->new( 'http://www/foo/test.gif?search+x#frag')
my $url2 = URI::URL->new( 'http:test')

my $pwd if $^OS_NAME eq "MacOS"

my $file
unless ($^OS_NAME eq "MacOS")
    $file = "/etc/passwd"
    -r $file or $file = "./README"
else
    require Mac::Files
    $pwd = `pwd`
    chomp($pwd)
    my $dir = Mac::Files::FindFolder( <Mac::Files::kOnSystemDisk(), <
                                      Mac::Files::kDesktopFolderType())
    chdir($dir)
    $file = "README"
    open(my $readme, ">","$file") or die "Unable to open $file"
    print $readme, "This is a dummy file for LWP testing purposes\n"
    close $readme
    open($readme, ">","/dev/null") or die "Unable to open /dev/null"
    print $readme, "This is a dummy file for LWP testing purposes\n"
    close $readme


my @tests =
    @: 
 \(@: "/this.dir/file.html" => "text/html",)
 \(@: "test.gif.htm"        => "text/html",)
 \(@: "test.txt.gz"         => "text/plain", "gzip")
 \(@: "gif.foo"             => "application/octet-stream",)
 \(@: "lwp-0.03.tar.Z"      => "application/x-tar", "compress")
 \(@: $file		        => "text/plain",)
 \(@: "/random/file"        => "application/octet-stream",)
 \(@: ($^OS_NAME eq 'VMS'?? "nl:" !! "/dev/null") => "text/plain",)
 \(@: $url1	        	=> "image/gif",)
 \(@: $url2	        	=> "application/octet-stream",)
 \(@: "x.ppm.Z.UU"		=> "image/x-portable-pixmap","compress","x-uuencode",)
    

plan tests => nelems(@tests)*2 + 3

if (env::var('HOME') and -f "$(env::var('HOME'))/.mime.types")
    warn "
The MediaTypes test might fail because you have a private ~/.mime.types file
If you get a failed test, try to move it away while testing.
"



my $testno = 1
for ( @tests)
    my (@: $file, $expectedtype, @< @expectedEnc) =  $_->@
    my (@: $type, @< @enc) =  guess_media_type($file)
    $type = "undef" unless defined $type
    is($type, $expectedtype)
    is_deeply(@enc, @expectedEnc)


my @imgSuffix = media_suffix('image/*')
print $^STDOUT, "Image suffixes: $(join ' ',@imgSuffix)\n"

print $^STDOUT, "\n"
require HTTP::Response
my $r = HTTP::Response->new( 200, "Document follows")
guess_media_type("file.tar.gz.uu", $r)
print $^STDOUT, $r->as_string

is($r->content_type, "application/x-tar")

my $enc = $r->header("Content-Encoding")
is($enc, "gzip, x-uuencode")

#
use LWP::MediaTypes < qw(add_type add_encoding)
add_type("x-world/x-vrml", < qw(wrl vrml))
add_encoding("x-gzip" => "gz")
add_encoding(rot13 => "r13")

my @x = guess_media_type("foo.vrml.r13.gz")
#print "@x\n";
is_deeply(@x,qw|x-world/x-vrml rot13 x-gzip|)

#print LWP::MediaTypes::_dump();

if($^OS_NAME eq "MacOS")
    unlink "README"
    unlink "/dev/null"
    chdir($pwd)


