
use HTTP::Request
use HTTP::Negotiate

use Test::More

plan tests => 5


#  ID       QS     Content-Type             Encoding     Char-Set      Lang    Size
my $variants =
 \ @: 
   \ @: 'var1',  0.950, 'text/plain',       \(@: 'uuencode'
                                                 'compress'), 'iso-8859-2', 'se',    400
   \ @: 'var2',  1.000, 'text/html;version=2.0', 'gzip',      'iso-8859-1', 'en',   3000
   \ @: 'var3',  0.333, 'image/gif',            undef,        undef,        undef, 43555


       # First we try a request with not accept headers
my $request = HTTP::Request->new( 'GET', 'http://localhost/')
my @a = choose($variants, $request)
show_res(< @a)
is_deeply(\@a, \@: \(@: 'var2' => 1)
                   \(@: 'var1' => 0.95)
                   \@: 'var3' => 0.333

          )


$a = choose($variants, $request)[0]->[0]
is($a, 'var2')

#------------------

$request = HTTP::Request->new( 'GET', 'http://localhost/')
$request->header('Accept', 'text/plain; q=0.55, image/gif; mbx=10000')
$request->push_header('Accept', 'text/*; q=0.25')
$request->header('Accept-Language', 'no, en')
$request->header('Accept-Charset', 'iso-8859-1')
$request->header('Accept-Encoding', 'gzip')

@a = choose($variants, $request)
show_res(< @a)
is_deeply(\@a, \@: \(@: 'var2' => 0.249975)
                   \(@: 'var1' => 0)
                   \@: 'var3' => 0

          )

$variants = \@: 
  \(@: 'var-en', undef, 'text/html', undef, undef, 'en', undef)
  \(@: 'var-de', undef, 'text/html', undef, undef, 'de', undef)
  \(@: 'var-ES', undef, 'text/html', undef, undef, 'ES', undef)
  \(@: 'provoke-warning',  undef, undef, undef, undef, 'x-no-content-type', undef)
    

$HTTP::Negotiate::DEBUG=1
env::var('HTTP_ACCEPT_LANGUAGE' ) = 'DE,en,fr;Q=0.5,es;q=0.1'

$a = choose($variants)[0]->[0]

is($a, 'var-de')


$variants = \@: 
  \(@:  'Canadian English' => 1.0, 'text/html', undef, undef, 'en-CA', undef )
  \(@:  'Generic English'  => 1.0, 'text/html', undef, undef, 'en',    undef )
  \(@:  'Non-Specific'     => 1.0, 'text/html', undef, undef, undef,   undef )
    

env::var('HTTP_ACCEPT_LANGUAGE' ) = 'en-US'
$a = choose($variants)[0]->[0]
is($a, 'Generic English')

#------------------

sub show_res
    print $^STDOUT, "-------------\n"
    for ( @_)
        printf $^STDOUT, "\%-6s \%.3f\n", < $_->@
    
    print $^STDOUT, "-------------\n"

