#!perl -w

use Test::More

plan tests => 18

use HTML::Form

my (@: $form) =  HTML::Form->parse(<<"EOT", "http://example.com")
<form>
<input type="hidden" name="hidden_1">

<input type="checkbox" name="checkbox_1" value="c1_v1" CHECKED>
<input type="checkbox" name="checkbox_1" value="c1_v2" CHECKED>
<input type="checkbox" name="checkbox_2" value="c2_v1" CHECKED>

<select name="multi_select_field" multiple="1">
 <option> 1
 <option> 2
 <option> 3
</select>
</form>
EOT

my @warn
$^WARN_HOOK = sub (@< @_) { push(@warn, shift->message) }

# list names
is(j( <$form->param), "hidden_1:checkbox_1:checkbox_2:multi_select_field")

# get
is(j( <$form->param('checkbox_1')), 'c1_v1:c1_v2')
is(j( <$form->param('checkbox_2')), 'c2_v1')
is(j( <$form->param('multi_select_field')), '')
is(j( <$form->param('unknown')), '')
ok(!nelems(@warn))

# set
$form->param('hidden_1', 'x')
ok(nelems(@warn) && @warn[0] =~ m/^Input 'hidden_1' is readonly/)
@warn = $@
is(j( <$form->param('hidden_1')), 'x')

try {
    $form->param('checkbox_1', 'foo');
}
ok($^EVAL_ERROR)
is(j( <$form->param('checkbox_1')), 'c1_v1:c1_v2')

$form->param('checkbox_1', 'c1_v2')
is(j( <$form->param('checkbox_1')), 'c1_v2')
$form->param('checkbox_1', 'c1_v2')
is(j( <$form->param('checkbox_1')), 'c1_v2')
$form->param('checkbox_1', \$@)
is(j( <$form->param('checkbox_1')), '')
$form->param('checkbox_1', \(@: 'c1_v2', 'c1_v1'))
is(j( <$form->param('checkbox_1')), 'c1_v1:c1_v2')
$form->param('checkbox_1', \$@)
is(j( <$form->param('checkbox_1')), '')
$form->param('checkbox_1', 'c1_v2', 'c1_v1')
is(j( <$form->param('checkbox_1')), 'c1_v1:c1_v2')

$form->param('multi_select_field', 3, 2)
is(j( <$form->param('multi_select_field')), "2:3")

print $^STDOUT, "# Done\n"
ok(!nelems(@warn))

sub j
    join(":", (@:  < @_))

