#
# Test retrieving a file with a 'file://' URL,
#

if ($^OS_NAME eq "MacOS")
    print $^STDOUT, "1..0\n"
    exit


use Test::More

# First locate some suitable tmp-dir.  We need an absolute path.
my $TMPDIR = undef
for ((@: "/tmp/", "/var/tmp", "/usr/tmp", "/local/tmp"))
    if (open(my $test, ">", "$_/test-$^PID"))
        close($test)
        unlink("$_/test-$^PID")
        $TMPDIR = $_
        last
    

$TMPDIR ||= env::var('TEMP') if $^OS_NAME eq 'MSWin32'
unless ($TMPDIR)
    # Can't run any tests
    print $^STDOUT, "1..0\n"
    print $^STDOUT, "ok 1\n"
    exit

$TMPDIR =~ s|\\|/|g

plan tests => 2

use LWP::Simple
require LWP::Protocol::file

my $orig = "$TMPDIR/lwp-orig-$^PID"          # local file
my $copy = "$TMPDIR/lwp-copy-$^PID" 	    # downloaded copy

# First we create the original
open(my $out_fh, ">", "$orig") or die "Cannot open $orig: $^OS_ERROR"
binmode($out_fh)
for (1..100)
    print $out_fh, "This is line $_ of $orig\n"

close($out_fh)


# Then we make a test using getprint(), so we need to capture stdout
open ($out_fh, ">", "$copy") or die "Cannot open $copy: $^OS_ERROR"
do
    local $^STDOUT = undef
    open $^STDOUT, ">&", $out_fh or die "Failed open: $^OS_ERROR"

    # do the retrieval
    getprint("file://localhost" . ($orig =~ m|^/| ?? $orig !! "/$orig"))

    close($out_fh)


# read and compare the files
open(my $in, "<", $orig) or die "Cannot open '$orig': $^OS_ERROR"
undef($^INPUT_RECORD_SEPARATOR)
my $origtext = ~< $in->*
close($in)
open($in, "<", $copy) or die "Cannot open '$copy': $^OS_ERROR"
undef($^INPUT_RECORD_SEPARATOR)
my $copytext = ~< $in->*
close($in)

unlink($copy)

is($origtext, $copytext)


# Test getstore() function

getstore("file:$orig", $copy)

# Take a look at the new copy
open($in, "<", $copy) or die "Cannot open '$copy': $^OS_ERROR"
undef($^INPUT_RECORD_SEPARATOR)
$copytext = ~< $in->*
close($in)

unlink($orig)
unlink($copy)

is($origtext, $copytext)
