
use Test::More

if ($^OS_NAME eq "MacOS")
    print $^STDOUT, "1..0\n"
    exit(0)


unless (-f "CAN_TALK_TO_OURSELF")
    print $^STDOUT, "1..0 # Skipped: Can't talk to ourself (misconfigured system)\n"
    exit


# Hm, this should really use Test.pm, but not worth changing over, really.

use bytes
$^OUTPUT_AUTOFLUSH = 1                         # autoflush

require IO::Socket # make sure this work before we try to make a HTTP::Daemon

# First we make ourself a daemon in another process
my $D = shift(@ARGV) || ''
my $daemon
if ($D eq 'daemon')

    require HTTP::Daemon

    my $d = HTTP::Daemon->new(Timeout => 10)

    print $^STDOUT, "Please to meet you at: <URL:", $d->url, ">\n"
    open($^STDOUT, ">", $^OS_NAME eq 'VMS'?? "nl: " !! "/dev/null")

    while (my $c = $d->accept)
        my $r = $c->get_request
        if ($r)
            my $p = $r->url->path_segments[1]
            my $func = \Symbol::fetch_glob(lc("httpd_" . $r->method . "_$p"))->*
            if (exists $func->*->&)
                $func->*->($c, $r)
            else
                $c->send_error(404)
        $c = undef             # close connection
    
    print $^STDERR, "HTTP Server terminated\n"
    exit
else
    use Config
    my $perl = config_value('perlpath')
    $perl = $^EXECUTABLE_NAME if $^OS_NAME eq 'VMS' or -x $^EXECUTABLE_NAME and $^EXECUTABLE_NAME =~ m,^([a-z]:)?/,i
    open($daemon, "-|", "$perl local/http-get.t daemon") or die "Can't exec daemon: $^OS_ERROR"


plan tests => 21


my $greeting = ~< $daemon->*
$greeting =~ m/(<[^>]+>)/

require URI
my $base = URI->new($1)
sub url
    my $u = URI->new(< @_)
    $u = $u->abs(@_[1]) if nelems(@_) +> 1
    $u->as_string


print $^STDOUT, "# Will access HTTP server at $($base->as_string)\n"

require LWP::UserAgent
require HTTP::Request
my $ua = LWP::UserAgent->new()
$ua->agent("Mozilla/0.01 " . $ua->agent)
$ua->from('gisle@aas.no')
$ua->cookie_jar(\$%)

#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Bad request...\n"
my $res = $ua->get(
    url("/not_found", $base),
    'X-Foo' => "Bar",
    )

ok( $res->is_error
      and $res->code == 404
    and $res->message =~ m/not\s+found/i )
# we also expect a few headers
ok( $res->server and $res->date )

#----------------------------------------------------------------
info "------------Testing: Simple echo..."
sub httpd_get_echo($c, $req)
    $c->send_basic_header(200)
    print $c, "Content-Type: text/plain\015\012"
    $c->send_crlf
    print $c, $req->as_string


$res = $ua->get(
    url("/echo/path_info?query", $base),
    Accept => 'text/html',
    Accept => 'text/plain; q=0.9',
    Accept => 'image/*',
    Long_text => 'This is a very long header line
which is broken between
more than one line.',
    X_Foo => "Bar",

    )
#print $res->as_string;

ok( $res->is_success
    and  $res->code == 200 && $res->message eq "OK" )

$_ = $res->content
my @accept = (@:  m/^Accept:\s*(.*)/mg )

#print "$_\n";

ok( m/^From:\s*gisle\@aas\.no$/m
      and m/^Host:/m
      and nelems(@accept) == 3
      and m/^Accept:\s*text\/html/m
      and m/^Accept:\s*text\/plain/m
      and m/^Accept:\s*image\/\*/m
      and m/^Long-Text:\s*This.*broken between/m
      and m/^X-Foo:\s*Bar$/m
    and m/^User-Agent:\s*Mozilla\/0.01/m )

#----------------------------------------------------------------
info "------------Testing: Send file..."

my $file = "test-$^PID.html"
sub _write_file
    open(my $fh, ">", "$file") or die "Can't create $file: $^OS_ERROR"
    binmode $fh or die "Can't binmode $file: $^OS_ERROR"
    print $fh, <<EOT
<html><title>En prve</title>
<h1>Dette er en testfil</h1>
Jeg vet ikke hvor stor fila behver  vre heller, men dette
er sikkert nok i massevis.
EOT
    close($fh)
    print $^STDOUT, "# ", -s $file, " bytes written to $file\n"
    return


sub httpd_get_file($c, $r)
    my %form = %:  < $r->url->query_form 
    my $file = %form{?'name'}
    $c->send_file_response($file)


_write_file()

$res = $ua->get( url("/file?name=$file", $base) )

info $res->as_string

ok( $res->is_success
      and $res->content_type eq 'text/html'
      and $res->content_length == 147
    and $res->content =~ m/ vre/ )



do

    my $content
    $res = $ua->get( url("/file?name=$file", $base),
        ':content_cb'     => sub (@< @_) { $content .= @_[0]; return; },
        )
    #print $res->as_string;

    ok( $res->is_success
          and $res->content_type eq 'text/html'
          and $res->content_length == 147
          and defined $content
          and ! $res->content # No content, because callback
        and $content =~ m/ vre/ )



unlink($file)



# Then try to list current directory
$res = $ua->get( url("/file?name=.", $base) )
#print $res->as_string;
ok( $res->code == 501 ) # NYI


#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Check redirect...\n"
sub httpd_get_redirect($c, ...)
    $c->send_redirect("/echo/redirect")


$res = $ua->get( url("/redirect/foo", $base) )
#print $res->as_string;

ok( $res->is_success
    and $res->content =~ m|/echo/redirect| )
ok( $res->previous->is_redirect
    and $res->previous->code == 301 )

# Let's test a redirect loop too
sub httpd_get_redirect2 { shift->send_redirect("/redirect3/") }
sub httpd_get_redirect3 { shift->send_redirect("/redirect2/") }

$res = $ua->get(url("/redirect2", $base))
#print $res->as_string;
ok( $res->is_redirect
    and $res->header("Client-Warning") =~ m/loop detected/i )
my $i = 0
while ($res->previous)
    $i++
    $res = $res->previous

ok( $i == 7 )

sub httpd_get_redirect_file { shift->send_redirect("file:/etc/passwd") }
$res = $ua->get(url("/redirect_file/", $base))
#print $res->as_string;
ok( $res->is_redirect
    and $res->header("Client-Warning") =~ m/can't redirect to a file:/i )


#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Check basic authorization...\n"
sub httpd_get_basic($c, $r)
    #print STDERR $r->as_string;
    my (@: ?$u,?$p) =  $r->authorization_basic || $@
    if (defined($u) && $u eq 'ok 13' && $p eq 'xyzzy')
        $c->send_basic_header(200)
        print $c, "Content-Type: text/plain"
        $c->send_crlf
        $c->send_crlf
        $c->print("$u\n")
    else
        $c->send_basic_header(401)
        $c->print("WWW-Authenticate: Basic realm=\"libwww-perl\"\015\012")
        $c->send_crlf
    


do
    package MyUA; our @ISA= @:  <qw(LWP::UserAgent) 
    sub get_basic_credentials($self, $realm, $uri, $proxy)
        if ($realm eq "libwww-perl" && $uri->rel($base)->as_string eq "basic")
            return ( (@: "ok 13", "xyzzy" ))
        else
            return undef
        
    


do
    my $that_url = url("/basic", $base)

    $res = MyUA->new->get( $that_url )
    #print $res->as_string;

    my $host_port = $res->request->uri->host_port

    ok($res->is_success)
    is($res->content, "ok 13\n")

    # Let's try with a $ua that does not pass out credentials
    $res = $ua->get( $that_url )
    ok( $res->code == 401 )


    print $^STDOUT, "# Host port: $host_port\n"

    # Let's try to set credentials for this realm
    $ua->credentials($host_port, "libwww-perl", "ok 13", "xyzzy")

    $res = $ua->get( $that_url )

    ok( $res->is_success )

    # Then illegal credentials
    $ua->credentials($host_port, "libwww-perl", "user", "passwd")
    $res = $ua->get( $that_url )
    ok( $res->code == 401 )


#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Check proxy...\n"
sub httpd_get_proxy($c,$r)
    if ($r->method eq "GET" and
        $r->url->scheme eq "ftp")
        $c->send_basic_header(200)
        $c->send_crlf
    else
        $c->send_error
    


$ua->proxy(ftp => $base->as_string)

$res = $ua->get( "ftp://ftp.perl.com/proxy" )
#print $res->as_string;
ok( $res->is_success )

#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Check POSTing...\n"
sub httpd_post_echo($c,$r)
    $c->send_basic_header
    $c->print("Content-Type: text/plain")
    $c->send_crlf
    $c->send_crlf
    $c->print($r->as_string)


$res = $ua->post(
    url("/echo/foo", $base),
    \(@: 'foo' => 'bar', 'bar' => 'test'),
    )
#print $res->as_string;

$_ = $res->content
ok( $res->is_success
      and m/^Content-Length:\s*16$/mi
      and m/^Content-Type:\s*application\/x-www-form-urlencoded$/mi
    and m/^foo=bar&bar=test/m )


do

    my $content

    $res = $ua->post(
        url("/echo/foo", $base),
        \(@: 'foo' => 'bar', 'bar' => 'test'),
        ':content_cb'     => sub (@< @_) { $content .= @_[0]; return; },
        )

    $_ = $content
    ok( $res->is_success
          and m/^Content-Length:\s*16$/mi
          and m/^Content-Type:\s*application\/x-www-form-urlencoded$/mi
          and m/^foo=bar&bar=test/m
        and ! $res->content )


#----------------------------------------------------------------
print $^STDOUT, "#------------Testing: Terminating server...\n"
sub httpd_get_quit($c, ...)
    $c->send_error(503, "Bye, bye")
    exit                       # terminate HTTP server


$res = $ua->get( url("/quit", $base) )
ok( $res->code == 503 and $res->content =~ m/Bye, bye/ )

