if ($^OS_NAME eq "MacOS")
    print $^STDOUT, "1..0\n"
    exit(0)


unless (-f "CAN_TALK_TO_OURSELF")
    print $^STDOUT, "1..0 # Skipped: Can't talk to ourself (misconfigured system)\n"
    exit


$^OUTPUT_AUTOFLUSH = 1 # autoflush
use bytes

require IO::Socket  # make sure this work before we try to make a HTTP::Daemon

# First we make ourself a daemon in another process
my $D = shift(@ARGV) || ''
my $daemon
if ($D eq 'daemon')

    require HTTP::Daemon

    my $d = HTTP::Daemon->new(Timeout => 10)

    print $^STDOUT, "Please to meet you at: <URL:", $d->url, ">\n"
    open($^STDOUT, ">", $^OS_NAME eq 'VMS'?? "nl: " !! "/dev/null")

    while (my $c = $d->accept)
        my $r = $c->get_request
        if ($r)
            my $p = $r->url->path_segments[1]
            my $func = \Symbol::fetch_glob(lc("httpd_" . $r->method . "_$p"))->*
            if (exists $func->*->&)
                $func->*->($c, $r)
            else
                $c->send_error(404)
            
        
        $c = undef  # close connection
    
    print $^STDERR, "HTTP Server terminated\n"
    exit
else
    use Config
    my $perl = config_value('perlpath')
    $perl = $^EXECUTABLE_NAME if $^OS_NAME eq 'VMS' or -x $^EXECUTABLE_NAME and $^EXECUTABLE_NAME =~ m,^([a-z]:)?/,i
    open($daemon, "-|", "$perl local/http.t daemon") or die "Can't exec daemon: $^OS_ERROR"


print $^STDOUT, "1..18\n"


my $greeting = ~< $daemon->*
$greeting =~ m/(<[^>]+>)/

require URI
my $base = URI->new($1)
sub url
    my $u = URI->new(< @_)
    $u = $u->abs(@_[1]) if nelems(@_) +> 1
    $u->as_string


print $^STDOUT, "Will access HTTP server at $($base->as_string)\n"

require LWP::UserAgent
require HTTP::Request
my $ua = LWP::UserAgent->new()
$ua->agent("Mozilla/0.01 " . $ua->agent)
$ua->from('gisle@aas.no')

#----------------------------------------------------------------
print $^STDOUT, "Bad request...\n"
my $req = HTTP::Request->new( GET => url("/not_found", $base))
$req->header(X_Foo => "Bar")
my $res = $ua->request($req)

print $^STDOUT, "not " unless $res->is_error
                                and $res->code == 404
    and $res->message =~ m/not\s+found/i
print $^STDOUT, "ok 1\n"
# we also expect a few headers
print $^STDOUT, "not " if !$res->server and !$res->date
print $^STDOUT, "ok 2\n"

#----------------------------------------------------------------
print $^STDOUT, "Simple echo...\n"
sub httpd_get_echo($c, $req)
    $c->send_basic_header(200)
    print $c, "Content-Type: message/http\015\012"
    $c->send_crlf
    print $c, $req->as_string


$req = HTTP::Request->new( GET => url("/echo/path_info?query", $base))
$req->push_header(Accept => 'text/html')
$req->push_header(Accept => 'text/plain; q=0.9')
$req->push_header(Accept => 'image/*')
$req->push_header(':foo_bar' => 1)
$req->if_modified_since(time - 300)
$req->header(Long_text => 'This is a very long header line
which is broken between
more than one line.')
$req->header(X_Foo => "Bar")

$res = $ua->request($req)
#print $res->as_string;

print $^STDOUT, "not " unless $res->is_success
    and  $res->code == 200 && $res->message eq "OK"
print $^STDOUT, "ok 3\n"

$_ = $res->content
my @accept = @:  m/^Accept:\s*(.*)/mg 

print $^STDOUT, "not " unless m/^From:\s*gisle\@aas\.no$/m
                                and m/^Host:/m
                                and nelems(@accept) == 3
                                and m/^Accept:\s*text\/html/m
                                and m/^Accept:\s*text\/plain/m
                                and m/^Accept:\s*image\/\*/m
                                and m/^If-Modified-Since:\s*\w{3},\s+\d+/m
                                and m/^Long-Text:\s*This.*broken between/m
                                and m/^Foo-Bar:\s*1$/m
                                and m/^X-Foo:\s*Bar$/m
    and m/^User-Agent:\s*Mozilla\/0.01/m
print $^STDOUT, "ok 4\n"

#----------------------------------------------------------------
print $^STDOUT, "Send file...\n"

my $file = "test-$^PID.html"
open(my $fh, ">", "$file") or die "Can't create $file: $^OS_ERROR"
binmode $fh or die "Can't binmode $file: $^OS_ERROR"
print $fh, <<EOT
<html><title>En prve</title>
<h1>Dette er en testfil</h1>
Jeg vet ikke hvor stor fila behver  vre heller, men dette
er sikkert nok i massevis.
EOT
close($fh)

sub httpd_get_file($c, $r)
    my %form = %:  < $r->url->query_form 
    my $file = %form{?'name'}
    $c->send_file_response($file)
    unlink($file) if $file =~ m/^test-/


$req = HTTP::Request->new( GET => url("/file?name=$file", $base))
$res = $ua->request($req)
#print $res->as_string;

print $^STDOUT, "not " unless $res->is_success
                                and $res->content_type eq 'text/html'
                                and $res->content_length == 147
    and $res->content =~ m/ vre/
print $^STDOUT, "ok 5\n"


# A second try on the same file, should fail because we unlink it
$res = $ua->request($req)
#print $res->as_string;
print $^STDOUT, "not " unless $res->is_error
    and $res->code == 404   # not found
print $^STDOUT, "ok 6\n"

# Then try to list current directory
$req = HTTP::Request->new( GET => url("/file?name=.", $base))
$res = $ua->request($req)
#print $res->as_string;
print $^STDOUT, "not " unless $res->code == 501   # NYI
print $^STDOUT, "ok 7\n"


#----------------------------------------------------------------
print $^STDOUT, "Check redirect...\n"
sub httpd_get_redirect($c, ...)
    $c->send_redirect("/echo/redirect")


$req = HTTP::Request->new( GET => url("/redirect/foo", $base))
$res = $ua->request($req)
#print $res->as_string;

print $^STDOUT, "not " unless $res->is_success
    and $res->content =~ m|/echo/redirect|
print $^STDOUT, "ok 8\n"
print $^STDOUT, "not " unless $res->previous->is_redirect
    and $res->previous->code == 301
print $^STDOUT, "ok 9\n"

# Let's test a redirect loop too
sub httpd_get_redirect2 { shift->send_redirect("/redirect3/") }
sub httpd_get_redirect3 { shift->send_redirect("/redirect2/") }

$req->url(url("/redirect2", $base))
$ua->max_redirect(5)
$res = $ua->request($req)
#print $res->as_string;
print $^STDOUT, "not " unless $res->is_redirect
    and $res->header("Client-Warning") =~ m/loop detected/i
print $^STDOUT, "ok 10\n"
my $i = 0
while ($res->previous)
    $i++
    $res = $res->previous


print $^STDOUT, "not " unless $i == 5
print $^STDOUT, "ok 11\n"

#----------------------------------------------------------------
print $^STDOUT, "Check basic authorization...\n"
sub httpd_get_basic($c, $r)
    #print STDERR $r->as_string;
    my (@: ?$u,?$p) =  $r->authorization_basic || $@
    if (defined($u) && $u eq 'ok 12' && $p eq 'xyzzy')
        $c->send_basic_header(200)
        print $c, "Content-Type: text/plain"
        $c->send_crlf
        $c->send_crlf
        $c->print("$u\n")
    else
        $c->send_basic_header(401)
        $c->print("WWW-Authenticate: Basic realm=\"libwww-perl\"\015\012")
        $c->send_crlf
    


do
    package MyUA; our @ISA= @:  <qw(LWP::UserAgent) 
    sub get_basic_credentials($self, $realm, $uri, $proxy)
        if ($realm eq "libwww-perl" && $uri->rel($base)->as_string eq "basic")
            return ( (@: "ok 12", "xyzzy" ))
        else
            return undef
        
    

$req = HTTP::Request->new( GET => url("/basic", $base))
$res = MyUA->new->request($req)
#print $res->as_string;

print $^STDOUT, "not " unless $res->is_success
print $^STDOUT, $res->content

# Let's try with a $ua that does not pass out credentials
$res = $ua->request($req)
print $^STDOUT, "not " unless $res->code == 401
print $^STDOUT, "ok 13\n"

# Let's try to set credentials for this realm
$ua->credentials($req->url->host_port, "libwww-perl", "ok 12", "xyzzy")
$res = $ua->request($req)
print $^STDOUT, "not " unless $res->is_success
print $^STDOUT, "ok 14\n"

# Then illegal credentials
$ua->credentials($req->url->host_port, "libwww-perl", "user", "passwd")
$res = $ua->request($req)
print $^STDOUT, "not " unless $res->code == 401
print $^STDOUT, "ok 15\n"


#----------------------------------------------------------------
print $^STDOUT, "Check proxy...\n"
sub httpd_get_proxy($c,$r)
    if ($r->method eq "GET" and
        $r->url->scheme eq "ftp")
        $c->send_basic_header(200)
        $c->send_crlf
    else
        $c->send_error
    


$ua->proxy(ftp => $base->as_string)
$req = HTTP::Request->new( GET => "ftp://ftp.perl.com/proxy")
$res = $ua->request($req)
#print $res->as_string;
print $^STDOUT, "not " unless $res->is_success
print $^STDOUT, "ok 16\n"

#----------------------------------------------------------------
print $^STDOUT, "Check POSTing...\n"
sub httpd_post_echo($c,$r)
    $c->send_basic_header
    $c->print("Content-Type: text/plain")
    $c->send_crlf
    $c->send_crlf

    # Do it the hard way to test the send_file
    open(my $tmp_fh, ">", "tmp$^PID") || die
    binmode($tmp_fh)
    print $tmp_fh, $r->as_string
    close($tmp_fh) || die

    $c->send_file("tmp$^PID")

    unlink("tmp$^PID")


$req = HTTP::Request->new( POST => url("/echo/foo", $base))
$req->content_type("application/x-www-form-urlencoded")
$req->content("foo=bar&bar=test")
$res = $ua->request($req)
#print $res->as_string;

$_ = $res->content
print $^STDOUT, "not " unless $res->is_success
                                and m/^Content-Length:\s*16$/mi
                                and m/^Content-Type:\s*application\/x-www-form-urlencoded$/mi
    and m/^foo=bar&bar=test$/m
print $^STDOUT, "ok 17\n"

#----------------------------------------------------------------
print $^STDOUT, "Terminating server...\n"
sub httpd_get_quit($c, ...)
    $c->send_error(503, "Bye, bye")
    exit  # terminate HTTP server


$req = HTTP::Request->new( GET => url("/quit", $base))
$res = $ua->request($req)

print $^STDOUT, "not " unless $res->code == 503 and $res->content =~ m/Bye, bye/
print $^STDOUT, "ok 18\n"

