
use Test::More

unless (-f "CAN_TALK_TO_OURSELF")
    plan skip_all => "Can't talk to ourself (misconfigured system)"


require IO::Socket # make sure this work before we try to make a HTTP::Daemon

# First we make ourself a daemon in another process
my $D = shift(@ARGV) || ''
my $daemon
if ($D eq 'daemon')

    require HTTP::Daemon

    my $d = HTTP::Daemon->new(Timeout => 10)

    print $^STDOUT, "Please to meet you at: <URL:", $d->url, ">\n"
    open($^STDOUT, ">", $^OS_NAME eq 'VMS'?? "nl: " !! "/dev/null")

    while (my $c = $d->accept)
        my $r = $c->get_request
        if ($r)
            my $p = $r->url->path_segments[1]
            my $func = \Symbol::fetch_glob(lc("httpd_" . $r->method . "_$p"))->*
            if (exists $func->*->&)
                $func->*->($c, $r)
            else
                $c->send_error(404)
        $c = undef             # close connection

    print $^STDERR, "HTTP Server terminated\n"
    exit
else
    use Config
    my $perl = config_value('perlpath')
    $perl = $^EXECUTABLE_NAME if $^OS_NAME eq 'VMS' or -x $^EXECUTABLE_NAME and $^EXECUTABLE_NAME =~ m,^([a-z]:)?/,i
    open($daemon, "-|", "$perl local/net-http.t daemon") or die "Can't exec daemon: $^OS_ERROR"


plan tests => 5

require URI

my $greeting = ~< $daemon->*
$greeting =~ m/(<[^>]+>)/
my $base = URI->new($1)
sub url
    my $u = URI->new(< @_)
    $u = $u->abs(@_[1]) if nelems(@_) +> 1
    $u->as_string


print $^STDOUT, "# Will access HTTP server at $($base->as_string)\n"

sub httpd_get_basic($c, ...)
    $c->send_basic_header(200)
    $c->print("Content-Type: text/plain")
    $c->send_crlf
    $c->send_crlf
    $c->print("test response")
    $c->close

use Net::HTTP

my $con = Net::HTTP->new(Host => $base->host, PeerPort => $base->port) || die "no conn?: $^OS_ERROR"
$con->write_request(GET => "/basic")
my (@: $code, $message, %< $headers) = $con->read_response_headers()
is( $code, 200)
my $body = ""
my $res
while ($res = $con->read_entity_body(my $buffer))
    $body .= $buffer
is( $res, 0 )
is($body, "test response")

#----------------------------------------------------------------
info "------------Testing: Terminating server..."
sub httpd_get_quit($c, ...)
    $c->send_error(503, "Bye, bye")
    exit                       # terminate HTTP server

$con = Net::HTTP->new(Host => $base->host, PeerPort => $base->port) || die "no conn?: $^OS_ERROR"
$con->write_request(GET => "/quit")
$con->read_response_headers()
$body = ""
while ($res = $con->read_entity_body(my $buffer))
    $body .= $buffer
is( $res, 0 )
like( $body, qr/Bye, bye/ )

