if($^OS_NAME eq "MacOS")
    print $^STDOUT, "1..0\n"
    exit(0)


unless (-f "CAN_TALK_TO_OURSELF")
    print $^STDOUT, "1..0 # Skipped: Can't talk to ourself (misconfigured system)\n"
    exit


$^OUTPUT_AUTOFLUSH = 1 # autoflush
require IO::Socket  # make sure this work before we try to make a HTTP::Daemon

# First we make ourself a daemon in another process
my $daemon

my $D = shift(@ARGV) || ''
if ($D eq 'daemon')

    require HTTP::Daemon

    my $d = HTTP::Daemon->new( Timeout => 10)

    print $^STDOUT, "Please to meet you at: <URL:", $d->url, ">\n"
    open($^STDOUT, ">", $^OS_NAME eq 'MSWin32' ??  "nul" !! $^OS_NAME eq 'VMS' ?? "NL:"  !! "/dev/null")

    while (my $c = $d->accept)
        my $r = $c->get_request
        if ($r)
            my $p = $r->url->path_segments[1]
            $p =~ s/\W//g
            my $func = Symbol::fetch_glob(lc("httpd_" . $r->method . "_$p"))->*
            #print STDERR "Calling $func...\n";
            if (exists $func->&)
                $func->($c, $r)
            else
                $c->send_error(404)
            
        
        $c = undef  # close connection
    
    print $^STDERR, "HTTP Server terminated\n"
    exit
else
    use Config
    my $perl = config_value('perlpath')
    $perl = $^EXECUTABLE_NAME if $^OS_NAME eq 'VMS' or -x $^EXECUTABLE_NAME and $^EXECUTABLE_NAME =~ m,^([a-z]:)?/,i
    open($daemon , "-|", "$perl robot/ua.t daemon") or die "Can't exec daemon: $^OS_ERROR"


use Test::More
plan tests => 9

require URI

my $greating = ~< $daemon
$greating =~ m/(<[^>]+>)/ or die "no greating found"

my $base = URI->new($1)
sub url
    my $u = URI->new(< @_)
    $u = $u->abs(@_[1]) if nelems(@_) +> 1
    $u->as_string


print $^STDOUT, "Will access HTTP server at $($base->as_string)\n"

require LWP::RobotUA
require HTTP::Request
my $ua = LWP::RobotUA->new( 'lwp-spider/0.1', 'gisle@aas.no')
$ua->delay(0.05)  # rather quick robot

#----------------------------------------------------------------
sub httpd_get_robotstxt($c,$r)
    $c->send_basic_header
    $c->print("Content-Type: text/plain")
    $c->send_crlf
    $c->send_crlf
    $c->print("User-Agent: *
Disallow: /private

")


sub httpd_get_someplace($c,$r)
    $c->send_basic_header
    $c->print("Content-Type: text/plain")
    $c->send_crlf
    $c->send_crlf
    $c->print("Okidok\n")


my $res = $ua->get( url("/someplace", $base) )
info $res->as_string
ok( $res->is_success )

$res = $ua->get( url("/private/place", $base) )
#print $res->as_string;
ok( $res->code == 403
    and $res->message =~ m/robots.txt/ )


$res = $ua->get( url("/foo", $base) )
#print $res->as_string;
ok( $res->code == 404 )  # not found

# Let the robotua generate "Service unavailable/Retry After response";
$ua->delay(1)
$ua->use_sleep(0)

$res = $ua->get( url("/foo", $base) )
#print $res->as_string;
ok( $res->code == 503   # Unavailable
    and $res->header("Retry-After") )

#----------------------------------------------------------------
print $^STDOUT, "Terminating server...\n"
sub httpd_get_quit($c)
    $c->send_error(503, "Bye, bye")
    exit  # terminate HTTP server


$ua->delay(0)

$res = $ua->get( url("/quit", $base) )

ok( $res->code == 503 and $res->content =~ m/Bye, bye/ )

#---------------------------------------------------------------
$ua->delay(1)

# host_wait() should be around 60s now
ok( abs($ua->host_wait($base->host_port) - 60) +< 5 )

# Number of visits to this place should be
ok( $ua->no_visits($base->host_port) == 4 )

# RobotUA used to have problem with mailto URLs.
env::var('SENDMAIL' ) = "dummy"
$res = $ua->get("mailto:gisle\@aas.no")
#print $res->as_string;

is( $res->code, 400 )
like( $res->message, qr"Library does not allow method GET for 'mailto:' URLs" )
